package AntTasks;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.tools.ant.Project;

/**
 * Removes the "required" annotation from java source files.
 * This is necessary for an incompatibility between XJC in JDK 1.8 (which creates that annotation) 
 * and java compilers of older JDKs (which don't understand that annotation).
 * 
 * @author Copyright (c) 2017 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class FileEnhanceHtmlLinksForMultiLanguage extends FileAction {

	@Override
	public String getName() {
		return "adapting";
	}

	/**
	 * @param srcFilename
	 *            the file that will be changed, so this file is source and
	 *            destination.
	 * @param destFilename
	 *            ignored.
	 */
	@Override
	public void execute(String srcFilename, String destFilename) {
        try {
        	System.out.println("Testing one two: " + srcFilename);
        	final File file = new File(srcFilename);
        	final String[] nameparts = file.getName().split("_");
        	final String lang = nameparts[3] + "_" + nameparts[4];
        	final BufferedReader is = new BufferedReader(new FileReader(srcFilename));
        	final ArrayList<String> lines = new ArrayList<String>();
            String oneLine;
            is.readLine();
            while ((oneLine = is.readLine()) != null) {
            	for (String pattern : PATTERNS) {
            		oneLine = oneLine.replaceAll(pattern, pattern + lang + "_");
            	}
        		lines.add(oneLine);
            }
            is.close();
        	final BufferedWriter os = new BufferedWriter(new FileWriter(srcFilename));
        	for (final String line : lines) {
                os.write(line);
                os.newLine();
            }
            os.close();
        } catch (final IOException e) {
            log(e.toString(), Project.MSG_ERR);
        }
	}

	private final String[] PATTERNS = new String[] { 
			" href=\"#", 
			" id=\"",
			" name=\"", 
	};
	
    public static void main(String[] args) {
    	FileEnhanceHtmlLinksForMultiLanguage task = new FileEnhanceHtmlLinksForMultiLanguage();
        task.executeMain(args);
    }
}
