package de.ped.advent.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.advent.logic.Advent;

/**
 * General unit tests for the {@link Advent} class.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach
 */
public class AdventTest {

	@Test
	public void testParseParam() {
		testParseParam(Advent.PARAM_KEY_DATE);
		testParseParam(Advent.PARAM_KEY_DEMO);
		assertEquals(Advent.PARAM_VAL_FOR_LEONIE, Advent.parseParam(Advent.PARAM_KEY_FOR, null));
		assertEquals(Advent.PARAM_VAL_FOR_LEONIE, Advent.parseParam(Advent.PARAM_KEY_FOR, ""));
		assertEquals(Advent.PARAM_VAL_FOR_LEONIE, Advent.parseParam(Advent.PARAM_KEY_FOR, Advent.PARAM_VAL_FOR_LEONIE));
		assertEquals(Advent.PARAM_VAL_FOR_FAMILY, Advent.parseParam(Advent.PARAM_KEY_FOR, Advent.PARAM_VAL_FOR_FAMILY));
		assertEquals(Advent.PARAM_VAL_FOR_FAMILY, Advent.parseParam(Advent.PARAM_KEY_FOR, "egal"));
	}

	private void testParseParam(final String key) {
		final String y = Integer.toString(Advent.DEFAULT_YEAR);
		assertEquals(4, y.length());
		assertEquals(null, Advent.parseParam(key, null));
		assertEquals(null, Advent.parseParam(key, ""));
		assertEquals("2014-12-24", Advent.parseParam(key, "2014-12-24"));
		assertEquals("2014-12-24", Advent.parseParam(key, "2014-24"));
		assertEquals("2014-12-24", Advent.parseParam(key, "20141224"));
		assertEquals("2014-12-24", Advent.parseParam(key, "201424"));
		assertEquals(y + "-12-24", Advent.parseParam(key, "24"));
		assertEquals(y + "-12-05", Advent.parseParam(key, "05"));
	}

	@Test
	public void testGetContentForDate() {
		final Advent advent = new Advent();
		{
			final String s = advent.getContent("2013-12-01", Advent.PARAM_VAL_FOR_LEONIE, "");
			assertNotNull(s);
			assertNotEquals(Advent.DEFAULT_CONTENT, s);
		}
		assertEquals(Advent.DEFAULT_CONTENT, advent.getContent("2013-11-30", Advent.PARAM_VAL_FOR_LEONIE, ""));
		{
			String res = advent.getContent("2013-12-21", Advent.PARAM_VAL_FOR_LEONIE, "");
			assertTrue(res.contains("<object"));
			assertTrue(res.contains("1.2"));
		}
	}

	private void assertNotEquals(String expected, String result) {
		assertFalse(expected.equals(result));
	}

	@Test
	public void testGetDateFormatted() {
		final Advent advent = new Advent();
		assertEquals("01. Dezember 2013", advent.getDateFormatted("2013-12-01"));
	}

	@Test
	public void testIsOpen() {
		final Advent advent = new Advent();
		assertEquals(false, advent.isOpen(null, null));
		assertEquals(false, advent.isOpen("2100-12-02", null));
		assertEquals(true,  advent.isOpen("2013-12-02", null));
		assertEquals(true,  advent.isOpen("2013-12-01", "2013-12-01"));
		assertEquals(true,  advent.isOpen("2013-12-01", "2013-12-02"));
		assertEquals(false, advent.isOpen("2013-12-02", "2013-12-01"));
		assertEquals(false, advent.isOpen("2000", null));
		assertEquals(false, advent.isOpen("2013", null));
		assertEquals(false, advent.isOpen("2100", null));
	}

	@Test
	public void testHref() {
		final String ret = Advent.href("2016", "leonie", "");
		assert(0 <= ret.indexOf("&amp;"));
	}

	@Test
	public void testLoadStory() {
		final Advent advent = new Advent();
		final String vItem = "2015-12-05";
		final String vFor = "leonie";
		final int vPart = 1;
		String content = advent.loadStory(vItem, vFor, vPart);
		assertNotNull(content);
		assertFalse(content.contains("Exception"));
		final String expContent = "Testing testing one two three";
		advent.saveStory(vItem, vFor, Integer.toString(vPart), expContent);
		content = advent.loadStory(vItem, vFor, vPart);
		assertNotNull(content);
		assertEquals(expContent, content);
	}
}
