package de.ped.atari8bit.logic;

/**
 * What kinds of disk density formats exist for Atari XL disks.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum Density {
	/** Single density (Atari 810) */
	SINGLEDENS(SIOConstants.SIO_SSSD_SECTOR_NUM, SIOConstants.SIO_SD_SECTOR_SIZE, 0x12, Density.DENSITY_FM, 0x00),
	/** Enhanced density (Atari 1050) */
	ENHANCDENS(SIOConstants.SIO_SSED_SECTOR_NUM, SIOConstants.SIO_SD_SECTOR_SIZE, 0x1a, Density.DENSITY_MFM, DiskImage.D_ENHDENS),
	/** Double density (Percom standard) */
	DOUBLEDENS(SIOConstants.SIO_SSSD_SECTOR_NUM, SIOConstants.SIO_DD_SECTOR_SIZE, 0x12, Density.DENSITY_MFM, DiskImage.D_DBLDENS);

	public static final int DENSITY_FM = 0x00;
	public static final int DENSITY_MFM = 0x04;

	public final int sectors;
	public final int bytesPerSector;
	public final int sectorsPerTrack;
	public final int density;
	public final int deviceStatus;

	private Density(
			final int sectors, 
			final int bytesPerSector, 
			final int sectorsPerTrack, 
			final int density, 
			final int deviceStatus) {
		this.sectors = sectors;
		this.bytesPerSector = bytesPerSector;
		this.sectorsPerTrack = sectorsPerTrack;
		this.density = density;
		this.deviceStatus = deviceStatus;
	}
}