package de.ped.atari8bit.logic;

/**
 * Several Atari OS SIO (serial input/output) constants.
 * 
 * For documentation of the SIO command bytes and protocols,
 * see the Atari Personal Computer System Operating System User's Manual,
 * Atari Part No. CO16555.
 * relevant sections referenced in this code are:
 * <dl>
 *   <dt>
 * chapter 5 -- I/O subsystems
 *   </dt>
 *   <dd>
 *					Serial bus I/O (SIO)
 *   </dd>
 *   <dt>
 * chapter 9 -- Adding new device handlers/peripherals
 *   </dt>
 *   <dd>
 *					Handler/SIO interface
 *   </dd>
 *   <dd>
 *					Serial I/O bus characteristics and protocol
 *   </dd>
 * </dl>
 * 
 * @author Copyright (c) 2002 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class SIOConstants {
	public static final int SIO_WAIT_COMPLETE = 1500; // min=250us, max=255sec. (ch. 9 of CO16555)
	public static final int SIO_WAIT_DATA_ACK =    0; //850 // min=850us, max=16ms       "       "
	public static final int SIO_WAIT_CMD_ACK  =    0; //10  // min=0us  , max=16ms       "       "

	public static final int SIO_COMMAND_SIZE    =  	 5;
	public static final int SIO_STATUS_SIZE	    =    4;
	public static final int SIO_MAX_DISKS       =    9;
	public static final int SIO_SD_SECTOR_SIZE  =  128;
	public static final int SIO_DD_SECTOR_SIZE  =  256;
	public static final int SIO_SSSD_SECTOR_NUM =  720;
	public static final int SIO_SSED_SECTOR_NUM = 1040;
	public static final int SIO_MAX_SECTOR_SIZE = 1024;
}
