package de.ped.conquest.gui;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.util.Vector;
import java.awt.GridBagConstraints;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import de.ped.conquest.logic.Game;
import de.ped.conquest.logic.Player;
import de.ped.tools.gui.JIconComboBox;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.PropertyElement;

/**
 * Lets the user select the parameters of the new game.
 * 
 * @author Copyright (c) 2001 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionDialog extends PedJDialog {
	private static final long serialVersionUID = 1L;
	
	public NewGameSelectionDialog(final ConquestMainWindow parent) {
		super(parent, parent);
		setTitle(getMessages().getString("GamePropertiesWindow.Title"));
		choiceBoards = getMainWindow().createComboBox(parent.getBoardFactory().getNameKeysOfBoardTemplates());
		choicePlayers = new Vector<JIconComboBox<Object>>(Game.NUMBER_OF_PLAYERS);
		for (int i = 0; i < choicePlayers.capacity(); i++) {
			choicePlayers.add(getMainWindow().createComboBox(Player.getStrategies(), false));
		}
		setPropertyElements(new PropertyElement[] {
				new PropertyElement("GameProperties.Board", choiceBoards),
				new PropertyElement("Game.Player0", choicePlayers.get(0)),
				new PropertyElement("Game.Player1", choicePlayers.get(1)),
		});
		
		setResizable(false);
		setTitlePanel(parent.getMessages().getString("GamePropertiesWindow.Heading"));
		final JPanel mainPanel = createMainPanel();
		final GridBagLayout gridbag = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainPanel.setLayout(gridbag);
		addAllPropertyElementsTo(mainPanel, c);

		add(mainPanel, BorderLayout.CENTER);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);
		addHelpButton("Help.Properties");
		finishLayout();
	}

	public int getChosenBoard() {
		return choiceBoards.getSelectedIndex();
	}

	public int getChosenStrategy(final int playerIndex) {
		switch(playerIndex) {
		case 0:
		case 1:
			return choicePlayers.get(playerIndex).getSelectedIndex();
		default:
			throw new IllegalArgumentException();
		}
	}

	public void setChosenBoard(final int boardIndex) {
		choiceBoards.setSelectedIndex(boardIndex);
	}

	public void setChosenStrategy(final int playerIndex, final int strategyIndex) {
		choicePlayers.get(playerIndex).setSelectedIndex(strategyIndex);
	}

	public void enterDialog(final int boardIndex, final int player0strategy, final int player1strategy) {
		setChosenBoard(boardIndex);
		setChosenStrategy(0, player0strategy);
		setChosenStrategy(1, player1strategy);
		enterDialog();
	}

	private JIconComboBox<Object> choiceBoards;
	private Vector<JIconComboBox<Object>> choicePlayers;
}
