package de.ped.conquest.logic;

import de.ped.tools.PlayfieldPosition;

/**
 * Represents a move of a player.
 * Uses the constant object pattern.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GameMove {

	/** The game state the move starts from. */
	public final GameState fromState;

	/** The starting position of the move. */
	public final PlayfieldPosition start;

	/** The ending position of the move. */
	public final PlayfieldPosition end;

	/** 
	 * The distance of the move.
	 * Either {@link Game#DISTANCE_GROW} or {@link Game#DISTANCE_JUMP}.
	 */
	public final int distance;

	/**
	 * Sets all attributes.
	 * 
	 * @param fromState
	 *            the game state the move starts from
	 * @param start
	 *            the starting position of the move
	 * @param end
	 *            the ending position of the move
	 * @param distance
	 *            the distance of the move. Either {@link Game#DISTANCE_GROW} or
	 *            {@link Game#DISTANCE_JUMP}
	 */
	public GameMove(
			final GameState fromState, 
			final PlayfieldPosition start, 
			final PlayfieldPosition end, 
			final int distance) {
		this.fromState = fromState;
		this.start    = start;
		this.end      = end;
		this.distance = distance;
	}

	@Override
	public String toString() {
		String arrow = "?>";
		if (2 == distance) {
			arrow = "=>";
		} else if (1 == distance) {
			arrow = "->";
		}
		return fromState.toString() + start.toString() + arrow + end.toString();
	}
}