package de.ped.conquest.logic;

import de.ped.tools.PlayfieldPosition;

/**
 * Implements a strategy that relies completely on the user to enter the move.
 * 
 * @author Copyright (c) 2001 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class HumanPlayer extends Player {
	static String STRATEGY_NAME = Player.STRATEGY_NAME_HUMAN;

	private PlayfieldPosition clickedPos;

	@Override
	public boolean needsPositionOnClick() {
		return true;
	}

	/**
	 * The user has clicked into a playfield. What happens now? This player
	 * needs to have a closer look and remember where the click had happened.
	 * 
	 * @param pos
	 *            the playfield position that has been clicked
	 */
	@Override
	public void onClick(final PlayfieldPosition pos) {
		clickedPos = pos;
		super.onClick(pos);
	}

	@Override
	protected PlayfieldPosition suggestStartPos() {
		return clickedPos;
	}

	@Override
	protected PlayfieldPosition suggestEndPos() {
		return clickedPos;
	}

	@Override
	public String getPlayerTypeName() {
		return STRATEGY_NAME;
	}
}
