package de.ped.conquest.logic;

/**
 * Simple data type to represent the value of a playfield on the board.
 * The following values are useful: 
 * Non-field (e.g., outside of the board, holes),
 * empty, player1, player2.
 * A playfield does not know its neighbors or the topology, that's the job of the board.
 * 
 * @author Copyright (c) 2001 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Playfield {
	/**
	 * Code for a blocked field, that none of the players can move onto. 
	 */
	public static final int WALL = 1;

	/**
	 * Code for a free field that any player can move to, but none has.
	 */
	public static final int FREE = 0;

	/**
	 * Code for a field owned by player 1.
	 */
	private static final int OWNED1 = 2;

	/**
	 * @param idx
	 *            index of the player, starts at 0.
	 * @return the code for a field owned by player <em>idx</em>
	 */
	public static int owned(final int idx) {
		if (!((0 == idx) || (1 == idx))) {
			throw new ArrayIndexOutOfBoundsException();
		}
		return OWNED1 + idx;
	}

	/**
	 * @param val
	 *            a coding for the content of the playfield
	 * @return <code>true</code> if <em>val</em> is a legal coding for the content
	 *         of the playfield
	 */
	public static boolean isValid(final int val) {
		return ((val >= 0) && val <= owned(1));
	}

	public static String toString(final int val) {
		final String chars = "#.12";
		try {
			return chars.substring(val, val + 1);
		} catch (final StringIndexOutOfBoundsException ex) {
			return "?";
		}
	}
}
