package de.ped.conquest.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.conquest.logic.Board;
import de.ped.conquest.logic.BoardFactory;
import de.ped.conquest.logic.Game;
import de.ped.conquest.logic.GameState;
import de.ped.conquest.logic.Player;
import de.ped.tools.log.Logger;

/**
 * Automated unit tests for the Conquest project.
 * 
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GameTest {
	Logger logger = Logger.getLogger(this.getClass());

	/**
	 * Plays a whole game from beginning to end.
	 * 
	 * @param sb
	 *            the board to play
	 * @param playerType1
	 *            the player type for player 1
	 * @param playerType2
	 *            the player type for player 2
	 */
	public void play(final Board sb, final int playerType1, final int playerType2) {
		final Game game = new Game(sb, playerType1, playerType2);
		GameState state = game.getGameState();
		while (Game.StateOfMove.GAMEOVER != game.getStateOfMove()) {
			assertEquals(Game.StateOfMove.STARTPOS, game.getStateOfMove());
			game.getCurrentPlayer().onClick(null);
			assertEquals(Game.StateOfMove.ENDPOS, game.getStateOfMove());
			game.getCurrentPlayer().onClick(null);
			final GameState nextState = game.getGameState();
			assertNotSame(state, nextState);
			state = nextState;
		}
	}

	@Test
	public void testGame() {
		final BoardFactory boardFactory = new BoardFactory();
		final int templates = boardFactory.getNameKeysOfBoardTemplates().length;
		for (int i = 0; i < templates; i++) {
			final Board sb = boardFactory.createBoard(i);
			BoardTest.validateBoard(sb);
			play(sb, Player.PLAYER_TYPE_COMPUTER_SIMPLE, Player.PLAYER_TYPE_COMPUTER_RANDOM);
		}
	}
}
