package de.ped.deinbac.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import de.ped.deinbac.logic.DeinbacProperties;
import de.ped.tools.BorderMode;
import de.ped.tools.ResourceFinder;
import de.ped.tools.SliderDef;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.FormattedTextFieldInputDoubleVerifier;
import de.ped.tools.gui.FormattedTextFieldInputIntegerVerifier;
import de.ped.tools.gui.FormattedTextFieldInputVerifier;
import de.ped.tools.gui.JIconComboBox;
import de.ped.tools.gui.JSliderWithTextFieldPanel;
import de.ped.tools.gui.PropertyElement;
import de.ped.tools.gui.Wizard;
import de.ped.tools.gui.WizardPanelDescriptor;

/**
 * The panel to show all the game options for starting a new game.
 * 
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionPanelDeinbacProps extends WizardPanelDescriptor {
	public static final String ID = "GameProperties.Deinbac";

	final NewGameSelectionPaneller paneller;

	public NewGameSelectionPanelDeinbacProps(final Wizard wizard) {
		super(wizard);
		paneller = new NewGameSelectionPaneller(wizard, true);
		final JPanel panel = new JPanel();
		choiceMapBorderMode = ((ApplicationMainWindow) wizard.getParent()).createComboBox(
				(Enum<?>[]) new BorderMode[] { BorderMode.ALL, BorderMode.NONE, }, "GameProperties.Deinbac.MapBorderMode", BorderMode.KEY,
				ResourceFinder.Folder.COMMON);
		final SliderDef sizeSliderDef = new SliderDef(4, 2000, 4);
		final FormattedTextFieldInputVerifier nonNegativeIntegerVerifier = new FormattedTextFieldInputIntegerVerifier(
				wizard, 0, FormattedTextFieldInputIntegerVerifier.MAX_DEFAULT);
		final FormattedTextFieldInputVerifier ppmDoubleVerifier = new FormattedTextFieldInputDoubleVerifier(
				wizard, 0.0, FACTOR_PPM);
		sliderXsize = new JSliderWithTextFieldPanel(wizard, sizeSliderDef, "GameProperties.MapSizeX", ResourceFinder.Folder.COMMON);
		sliderYsize = new JSliderWithTextFieldPanel(wizard, sizeSliderDef, "GameProperties.MapSizeY", ResourceFinder.Folder.COMMON);
		textInitialManaToGrowPerMFields.setInputVerifier(ppmDoubleVerifier);
		textManaToGrowEachRoundPerMFields.setInputVerifier(ppmDoubleVerifier);
		textManaToGrowEachRoundInCockaignePerMFields.setInputVerifier(ppmDoubleVerifier);
		textManaEnergy.setInputVerifier(nonNegativeIntegerVerifier);
		textMaxEnergy.setInputVerifier(nonNegativeIntegerVerifier);
		textInitialEnergy.setInputVerifier(nonNegativeIntegerVerifier);
		textEnergyForSplit.setInputVerifier(nonNegativeIntegerVerifier);
		textAgeForSplit.setInputVerifier(nonNegativeIntegerVerifier);
		textEnergyForSex.setInputVerifier(nonNegativeIntegerVerifier);
		textAgeForSex.setInputVerifier(nonNegativeIntegerVerifier);
		textInitialNumberOfBacterions.setInputVerifier(nonNegativeIntegerVerifier);
		textTrendChartFrequency.setInputVerifier(nonNegativeIntegerVerifier);
		setPropertyElements(new PropertyElement[] {
				new PropertyElement("GameProperties.Deinbac.World", null, new PropertyElement[] { 
						new PropertyElement("GameProperties.MapSizeX", sliderXsize), 
						new PropertyElement("GameProperties.MapSizeY", sliderYsize), 
						new PropertyElement("GameProperties.Deinbac.MapBorderMode", choiceMapBorderMode),
						new PropertyElement("GameProperties.Deinbac.InitialManaToGrowPerMFields", textInitialManaToGrowPerMFields),
						new PropertyElement("GameProperties.Deinbac.ManaToGrowEachRoundPerMFields", textManaToGrowEachRoundPerMFields),
						new PropertyElement("GameProperties.Deinbac.ManaToGrowEachRoundInCockaignePerMFields", textManaToGrowEachRoundInCockaignePerMFields),
						new PropertyElement("GameProperties.Deinbac.InitialNumberOfBacterions", textInitialNumberOfBacterions),
				}),
				new PropertyElement("GameProperties.Deinbac.Bacterion", null, new PropertyElement[] { 
						new PropertyElement("GameProperties.Deinbac.ManaEnergy", textManaEnergy),
						new PropertyElement("GameProperties.Deinbac.MaxEnergy", textMaxEnergy),
						new PropertyElement("GameProperties.Deinbac.InitialEnergy", textInitialEnergy),
						new PropertyElement("GameProperties.Deinbac.EnergyForSplit", textEnergyForSplit),
						new PropertyElement("GameProperties.Deinbac.AgeForSplit", textAgeForSplit),
						new PropertyElement("GameProperties.Deinbac.EnergyForSex", textEnergyForSex),
						new PropertyElement("GameProperties.Deinbac.AgeForSex", textAgeForSex),
				}),
				new PropertyElement("GameProperties.Deinbac.Statistics", null, new PropertyElement[] { 
						new PropertyElement("GameProperties.Deinbac.TrendChartFrequency", textTrendChartFrequency),
				}),
		});
		final GridBagLayout gridbag = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		panel.setLayout(gridbag);
		wizard.addPropertyElementsTo(getAllPropertyElements(), panel, c);

		setPanelComponent(new JScrollPane(panel));
		setPanelId(ID);
		setBackPanelId(NewGameSelectionPanelGameType.ID);
		setNextPanelId(FINISH_ID);
	}

	private static void setValue(final JFormattedTextField tf, final Object value) {
		tf.setValue(value);
		tf.setColumns(5);
		tf.setHorizontalAlignment(JTextField.RIGHT);
	}

	public double FACTOR_PPM = 1000000.0;

	@Override
	public void readUIFieldsIntoProperties() {
		final DeinbacProperties model = getModel();
		model.setXsize(sliderXsize.getValue());
		model.setYsize(sliderYsize.getValue());
		model.setWorldTorus(choiceMapBorderMode.getSelectedIndex() > 0);
		model.setInitialManaToGrowPerField(((Double) (textInitialManaToGrowPerMFields.getValue())).doubleValue() / FACTOR_PPM);
		model.setManaToGrowEachRoundPerField(((Double) (textManaToGrowEachRoundPerMFields.getValue())).doubleValue() / FACTOR_PPM);
		model.setManaToGrowEachRoundInCockaignePerField(((Double) (textManaToGrowEachRoundInCockaignePerMFields.getValue())).doubleValue() / FACTOR_PPM);
		model.setManaEnergy(((Integer) (textManaEnergy.getValue())).intValue());
		model.setMaxEnergy(((Integer) (textMaxEnergy.getValue())).intValue());
		model.setInitialEnergy(((Integer) (textInitialEnergy.getValue())).intValue());
		model.setAgeForSplit(((Integer) (textAgeForSplit.getValue())).intValue());
		model.setEnergyForSplit(((Integer) (textEnergyForSplit.getValue())).intValue());
		model.setAgeForSex(((Integer) (textAgeForSex.getValue())).intValue());
		model.setEnergyForSex(((Integer) (textEnergyForSex.getValue())).intValue());
		model.setInitialNumberOfBacterions(((Integer) (textInitialNumberOfBacterions.getValue())).intValue());
		model.setTrendChartFrequency(((Integer) (textTrendChartFrequency.getValue())).intValue());
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		final DeinbacProperties model = getModel();
		sliderXsize.setValue(model.getXsize());
		sliderYsize.setValue(model.getYsize());
		choiceMapBorderMode.setSelectedIndex(model.isWorldTorus() ? 1 : 0);
		setValue(textInitialManaToGrowPerMFields, new Double(model.getInitialManaToGrowPerField() * FACTOR_PPM));
		setValue(textManaToGrowEachRoundPerMFields, new Double(model.getManaToGrowEachRoundPerField() * FACTOR_PPM));
		setValue(textManaToGrowEachRoundInCockaignePerMFields, new Double(model.getManaToGrowEachRoundInCockaignePerField() * FACTOR_PPM));
		setValue(textManaEnergy, new Integer(model.getManaEnergy()));
		setValue(textMaxEnergy, new Integer(model.getMaxEnergy()));
		setValue(textInitialEnergy, new Integer(model.getInitialEnergy()));
		setValue(textAgeForSplit, new Integer(model.getAgeForSplit()));
		setValue(textEnergyForSplit, new Integer(model.getEnergyForSplit()));
		setValue(textAgeForSex, new Integer(model.getAgeForSex()));
		setValue(textEnergyForSex, new Integer(model.getEnergyForSex()));
		setValue(textInitialNumberOfBacterions, new Integer(model.getInitialNumberOfBacterions()));
		setValue(textTrendChartFrequency, new Integer(model.getTrendChartFrequency()));
	}

	public DeinbacProperties getModel() {
		return ((NewGameSelectionWizard) getWizard()).getModel().getDeinbacProperties();
	}
	private final JSliderWithTextFieldPanel sliderXsize;
	private final JSliderWithTextFieldPanel sliderYsize;
	private JIconComboBox<?> choiceMapBorderMode;
	private JFormattedTextField textInitialManaToGrowPerMFields = new JFormattedTextField();
	private JFormattedTextField textManaToGrowEachRoundPerMFields = new JFormattedTextField();
	private JFormattedTextField textManaToGrowEachRoundInCockaignePerMFields = new JFormattedTextField();
	private JFormattedTextField textManaEnergy = new JFormattedTextField();
	private JFormattedTextField textInitialEnergy = new JFormattedTextField();
	private JFormattedTextField textMaxEnergy = new JFormattedTextField();
	private JFormattedTextField textAgeForSplit = new JFormattedTextField();
	private JFormattedTextField textEnergyForSplit = new JFormattedTextField();
	private JFormattedTextField textAgeForSex = new JFormattedTextField();
	private JFormattedTextField textEnergyForSex = new JFormattedTextField();
	private JFormattedTextField textInitialNumberOfBacterions = new JFormattedTextField();
	private JFormattedTextField textTrendChartFrequency = new JFormattedTextField();
}
