package de.ped.deinbac.gui;

import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.ped.deinbac.logic.GameProperties;
import de.ped.deinbac.logic.SimulationType;
import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PropertyElement;
import de.ped.tools.gui.Wizard;
import de.ped.tools.gui.WizardPanelDescriptor;

/**
 * For the new game selection wizard, this is the first panel to be shown. 
 * It allows the user to pick between a tutorial or a conventional game.
 * 
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionPanelGameType extends WizardPanelDescriptor implements ChangeListener {
	public static final String ID = "GameProperties.SimulationType";

	public NewGameSelectionPanelGameType(final Wizard wizard) {
		super(wizard);
		final ApplicationMainWindow mainWindow = (ApplicationMainWindow) wizard.getParent();

		final JPanel panel = new JPanel();
		final BoxLayout layout = new BoxLayout(panel, BoxLayout.PAGE_AXIS);
		panel.setLayout(layout);
		final ButtonGroup group = new ButtonGroup();
		final ArrayList<String> keys = new ArrayList<String>();
		for (SimulationType t : SimulationType.values()) {
			keys.add(t.key);
		}
		panel.add(Box.createVerticalGlue());
		final PropertyElement[] elems = new PropertyElement[keys.size()];
		buttons = new JRadioButton[keys.size()];
		for (int i = 0; i < keys.size(); i++) {
			final JRadioButton button = mainWindow.createRadioButton(new I18NStringWithFillIns(keys.get(i), null, null));
			buttons[i] = button;
			group.add(button);
			panel.add(button);
			button.addChangeListener(this);
			elems[i] = new PropertyElement(keys.get(i), buttons[i]);
		}
		setPropertyElements(elems);
		panel.add(Box.createVerticalGlue());
		setPanelComponent(panel);
		setPanelId(ID);
	}

	@Override
	public void stateChanged(ChangeEvent event) {
		update();
	}

	@Override
	protected void updateNavDescriptors() {
		final int idx = findSelectedButtonIndex();
		if (0 <= idx) {
			setNextPanelId(SimulationType.values()[idx].targetPanelKey);
		} else {
			setNextPanelId(null);
		}
	}

	private int findSelectedButtonIndex() {
		int idx = -1;
		for (int i = 0; i < buttons.length; i++) {
			if (buttons[i].isSelected()) {
				idx = i;
				break;
			}
		}
		return idx;
	}

	@Override
	public void readUIFieldsIntoProperties() {
		final int idx = findSelectedButtonIndex();
		if (0 <= idx) {
			getModel().setSimulation(SimulationType.values()[idx]);
		}
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		for (int i = 0; i < buttons.length; i++) {
			buttons[i].setSelected(false);
		}
		buttons[getModel().getSimulation().ordinal()].setSelected(true);
	}

	public GameProperties getModel() {
		return ((NewGameSelectionWizard) getWizard()).getModel();
	}

	private final JRadioButton[] buttons;
}