package de.ped.deinbac.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import de.ped.deinbac.logic.LifeProperties;
import de.ped.deinbac.logic.LifeProperties.Pattern;
import de.ped.tools.BorderMode;
import de.ped.tools.ResourceFinder;
import de.ped.tools.SliderDef;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.JIconComboBox;
import de.ped.tools.gui.JSliderWithTextFieldPanel;
import de.ped.tools.gui.PropertyElement;
import de.ped.tools.gui.Wizard;
import de.ped.tools.gui.WizardPanelDescriptor;

/**
 * The panel to show all the game options for starting a new game.
 * 
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionPanelLifeProps extends WizardPanelDescriptor {
	public static final String ID = "GameProperties.Life";

	final NewGameSelectionPaneller paneller;

	public NewGameSelectionPanelLifeProps(final Wizard wizard) {
		super(wizard);
		paneller = new NewGameSelectionPaneller(wizard, true);
		final JPanel panel = new JPanel();
		final SliderDef sizeSliderDef = new SliderDef(4, 2000, 4);
		sliderXsize = new JSliderWithTextFieldPanel(wizard, sizeSliderDef, "GameProperties.MapSizeX", ResourceFinder.Folder.COMMON);
		sliderYsize = new JSliderWithTextFieldPanel(wizard, sizeSliderDef, "GameProperties.MapSizeY", ResourceFinder.Folder.COMMON);
		final ApplicationMainWindow applicationMainWindow = (ApplicationMainWindow) wizard.getParent();
		choiceMapBorderMode = applicationMainWindow.createComboBox(
				(Enum<?>[]) new BorderMode[] { BorderMode.ALL, BorderMode.NONE, }, "GameProperties.Life.MapBorderMode", BorderMode.KEY,
				ResourceFinder.Folder.COMMON);
		choicePattern = applicationMainWindow.createComboBox(Pattern.values(), "GameProperties.Life.Pattern", null, null);
		freePattern = new JTextArea(15, 50);
		setPropertyElements(new PropertyElement[] {
				new PropertyElement("GameProperties.MapSizeX", sliderXsize), 
				new PropertyElement("GameProperties.MapSizeY", sliderYsize), 
				new PropertyElement("GameProperties.Life.MapBorderMode", choiceMapBorderMode),
				new PropertyElement("GameProperties.Life.Pattern", choicePattern),
				new PropertyElement("GameProperties.Life.Pattern.FREE_TEXT.Field", freePattern),
		});
		final GridBagLayout gridbag = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		panel.setLayout(gridbag);
		wizard.addPropertyElementsTo(getAllPropertyElements(), panel, c);

		setPanelComponent(new JScrollPane(panel));
		setPanelId(ID);
		setBackPanelId(NewGameSelectionPanelGameType.ID);
		setNextPanelId(FINISH_ID);
	}

	@Override
	public void readUIFieldsIntoProperties() {
		final LifeProperties model = getModel();
		model.setXsize(sliderXsize.getValue());
		model.setYsize(sliderYsize.getValue());
		model.setWorldTorus(choiceMapBorderMode.getSelectedIndex() > 0);
		final int idx = choicePattern.getSelectedIndex();
		model.setPattern(Pattern.values()[idx]);
		model.setFreePattern(freePattern.getText());
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		final LifeProperties model = getModel();
		sliderXsize.setValue(model.getXsize());
		sliderYsize.setValue(model.getYsize());
		choiceMapBorderMode.setSelectedIndex(model.isWorldTorus() ? 1 : 0);
		final Pattern pattern = model.getPattern();
		choicePattern.setSelectedIndex(pattern.ordinal());
		freePattern.setText(model.getFreePattern());
	}

	public LifeProperties getModel() {
		return ((NewGameSelectionWizard) getWizard()).getModel().getLifeProperties();
	}

	private final JSliderWithTextFieldPanel sliderXsize;
	private final JSliderWithTextFieldPanel sliderYsize;
	private JIconComboBox<?> choiceMapBorderMode;
	private JIconComboBox<?> choicePattern;
	private JTextArea freePattern;
}
