package de.ped.deinbac.logic;

import de.ped.tools.Assert;

/** 
 * Contains all properties that remain constant for a run. 
 * 
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LifeProperties implements Cloneable {
	/** In pattern representing character streams, the representation of a dead cell */
	public static final char DEAD_CELL = 'o';
	/** In pattern representing character streams, the representation of an alive cell */
	public static final char ALIVE_CELL = 'b';
	/** In pattern representing character streams, the representation of a line end */
	public static final char LINE_DELIM = '$';
	/** In pattern representing character streams, the representation of a pattern end */
	public static final char PATTERN_DELIM = '!';

	public enum Pattern {
		GLIDER(		"bbb$boo$obo!"),
		R_PENTOMINO("obb$bbo$obo!"),
		ACORN(		"obooooo$ooobooo$bboobbb!"),
		B_HEPTOMINO("bobb$bbbo$oboo!"),
		LOAFER(
					"obboobobb$"
				+	"booboobbo$"
				+	"obobooooo$"
				+	"ooboooooo$"
				+	"oooooooob$"
				+	"oooooobbb$"
				+	"ooooobooo$"
				+	"ooooooboo$"
				+	"ooooooobb!"
				),
		COPPERHEAD(
					"obboobbo$"
				+	"ooobbooo$"
				+	"ooobbooo$"
				+	"boboobob$"
				+	"boooooob$"
				+	"oooooooo$"
				+	"boooooob$"
				+	"obboobbo$"
				+	"oobbbboo$"
				+	"oooooooo$"
				+	"ooobbooo$"
				+	"ooobbooo!"
					),
		WING_SPACESHIP(
					"oooobbooooooooooooooooo$"
				+	"ooooboboooooooooooooooo$"
				+	"ooooboooooooooooooooooo$"
				+	"ooooooobooooooooooooooo$"
				+	"bbboooobobooooooooooooo$"
				+	"booooooboobooobbboooooo$"
				+	"obooooooobbooboobbooooo$"
				+	"ooobbbooooooobooooobboo$"
				+	"ooooooooooooobboooobbbo$"
				+	"oooobobooobbbooooooooob$"
				+	"ooooobbooboobooooooboob$"
				+	"ooooooooobooooooooobooo$"
				+	"ooooooooobboooooooobobo$"
				+	"oooooobbboooooooooooooo$"
				+	"ooooobooboooooooooooooo$"
				+	"ooooobooooooooooooooooo$"
				+	"ooooobboooooooooooooooo$"
				+	"ooooooboooooooooooooooo$"
				+	"ooooooooooooooooooooooo$"
				+	"ooooooobbobbboooooooooo$"
				+	"ooooooobboooooooooooooo$"
				+	"ooooooooboooboooooooooo$"
				+	"ooooooooobboooooooooooo!"
				),
		GOSPER_GLIDER_GUN(
					"oooooooooooooooooooooooobooooooooooo$"
				+ 	"oooooooooooooooooooooobobooooooooooo$"
				+ 	"oooooooooooobboooooobboooooooooooobb$"
				+ 	"oooooooooooboooboooobboooooooooooobb$"
				+ 	"bboooooooobooooobooobboooooooooooooo$"
				+ 	"bboooooooobooobobboooobobooooooooooo$"
				+ 	"oooooooooobooooobooooooobooooooooooo$"
				+ 	"oooooooooooboooboooooooooooooooooooo$"
				+ 	"oooooooooooobboooooooooooooooooooooo!"
				),
		METHUSELAH_8x32_001(
					"ooobbbobbobboobbbbobooobbbbbobob$"
				+	"oooboobbobbooobobboobbboboooooob$"
				+	"ooobbooooboobbbbobbooooobbbbbbbb$"
				+	"bobbbboobbbbboobooobbbobbbooobob$"
				+	"ooboooobbobbboooboboobbbooobbobb$"
				+	"bbbboobobobbbooooobobobooooooooo$"
				+	"boooobbbbbbobbbbobbobbbobbboooob$"
				+	"ooobbbbobbboboobbbooooobbbboobob!"
				),
		METHUSELAH_16x16_001(
					"bbbbobbboboobbbb$"
				+	"bobbbooooobbboob$"
				+	"bbobbooobooobobo$"
				+	"ooobbboboboboobb$"
				+	"obbbobbobooboobo$"
				+	"oobobbboboobboob$"
				+	"bboboooobbbooboo$"
				+	"oobobboboobbobob$"
				+	"oboboobooooooooo$"
				+	"booboooooboobbob$"
				+	"bbbobooobboobooo$"
				+	"bbbboobbbboobobb$"
				+	"obobbbobbobbobbb$"
				+	"bbbbbbbobbbobbbo$"
				+	"obobboobbboobbbo$"
				+	"bobbobbbooooobbo!"
				),
		METHUSELAH_ZZ_REPLICATOR_STDIN_001(
					"41bo$40b3o$39b2obo5bo$39b3o5b3o$40b2o4bo2b2o3b3o$46b3o4bo2bo$56bo$56b"
				+	"o$56bo$40b3o12bo$40bo2bo$40bo$40bo$41bo7$38b3o$38bo2bo11bo$38bo13b3o$"
				+	"38bo12b2obo$38bo12b3o$39bo12b2o3$35b3o$34b5o$34b3ob2o7b2o5bo2bo$37b2o"
				+	"7b4o8bo$46b2ob2o3bo3bo$48b2o5b4o3$20bo$21bo$b2o13bo4bo32b3o$4o13b5o34b"
				+	"o$2ob2o51bo$2b2o51bo$36bo$37bo$21b2o10bo3bo$22b2o10b4o15b2o$21b2o27b3o"
				+	"b2o$21bo28b5o$51b3o2$22b2o$13b4o4b4o$12bo3bo4b2ob2o$b5o10bo6b2o$o4bo9b"
				+	"o$5bo$4bo!"
				),
		FREE_TEXT(""
				);
		public final String spec;
		private Pattern(final String spec) {
			this.spec = spec;
		}
	};

	public LifeProperties() {
		setRulesetStr("B3/S23");
	}

	@Override
	public LifeProperties clone() {
		LifeProperties ret = null;
		try {
			ret = (LifeProperties) super.clone();
		} catch (final CloneNotSupportedException e) {
			Assert.assertTrue(false);
		}
		return ret;
	}

	public int getXsize() {
		return xsize;
	}
	public void setXsize(final int xsize) {
		this.xsize = xsize;
	}
	public int getYsize() {
		return ysize;
	}
	public void setYsize(final int ysize) {
		this.ysize = ysize;
	}
	public boolean isWorldTorus() {
		return isWorldTorus;
	}
	public void setWorldTorus(final boolean isWorldTorus) {
		this.isWorldTorus = isWorldTorus;
	}
	public Pattern getPattern() {
		return pattern;
	}
	public void setPattern(final Pattern pattern) {
		this.pattern = pattern;
	}
	/** @return the free pattern to pick if {@link Pattern#FREE_TEXT} is selected as pattern. */
	public String getFreePattern() {
		return freePattern;
	}
	/** @param pattern the free pattern to start with if {@link Pattern#FREE_TEXT} is selected as pattern. */
	public void setFreePattern(final String pattern) {
		this.freePattern = pattern;
	}
	public String getRulesetStr() {
		return convertRuleset(ruleset);
	}
	public void setRulesetStr(final String ruleset) {
		this.ruleset = convertRuleset(ruleset);
	}
	/**
	 * @return the ruleset as bitmask, with a 1 in a bit meaning the center cell will survive / being born.
	 * 		The bits 0 (LSB) to 9 define birth mode (the center cell is empty), 
	 * 		while bits 16 to 16+9 mark survival mode (the center cell is already alive).
	 * 		So e.g., bit 0 set to 1 means that if the center cell is empty and has 0 alive neighboring cells, 
	 * 		the center cell gets to life.
	 * 		Another example: Standard Conway ruleset is B3/S23, so as bitmask this is 
	 * 		0b00000000000011000000000000001000.
	 * 
	 */
	public int getRuleset() {
		return ruleset;
	}
	public void setRuleset(final int ruleset) {
		this.ruleset = ruleset;
	}

	private static int convertRuleset(String ruleset) {
		int ret = 0;
		boolean isSurvivalMode = true;
		for (int i = 0; i < ruleset.length(); i++) {
			char c = ruleset.charAt(i);
			if ('S' == c) {
				isSurvivalMode = true;
			} else if ('B' == c) {
				isSurvivalMode = false;
			} else if ('/' == c) {
				isSurvivalMode = !isSurvivalMode;
			} else if (Character.isDigit(c)) {
				ret |= getRulesetBit(isSurvivalMode, Character.digit(c, 10));
			}
		}
		return ret;
	}
	public static int getRulesetBit(final boolean isSurvivalMode, final int numberOfNeighbors) {
		return (1 << (numberOfNeighbors + (isSurvivalMode ? 16 : 0)));
	}
	private static String convertRuleset(final int ruleset) {
		final StringBuffer b = new StringBuffer();
		for (int mode = 0; mode < 32; mode += 16) {
			b.append(mode == 0 ? "B" : "/S");
			for (int bit = 0; bit <= 8; bit++) {
				if (0 != (ruleset & getRulesetBit((0 == mode), bit))) {
					b.append(bit);
				}
			}
		}
		return b.toString();
	}

	/** The x-size of the world. Must be divisible by 32. */
	private int xsize = 320;
	/** The y-size of the world. Must be divisible by 1. */
	private int ysize = 200;
	/** Is the world a torus - compared to having border walls? */
	private boolean isWorldTorus = true;

	private Pattern pattern = Pattern.WING_SPACESHIP;
	/** The free pattern to pick if {@link Pattern#FREE_TEXT} is selected as pattern. */
	private String freePattern;
	private int ruleset;
}
