package de.ped.deinbac.test;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import de.ped.deinbac.logic.Bacterion;
import de.ped.deinbac.logic.DeinbacProperties;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Unit tests for the {@link Bacterion} class.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class BacterionTest {

	public static final double EPS = 1E-15;

	@Before
	public void setup() {
		Logger.getRootLogger().setLogLevel(Level.WARN);
	}

	@Test
	public void testPropertiesClone() {
		final DeinbacProperties p = new DeinbacProperties();
		final DeinbacProperties c = (DeinbacProperties)p.clone();
		assertEquals(p.getAgeForSex(), c.getAgeForSex());
		assertEquals(p.getAgeForSplit(), c.getAgeForSplit());
		assertEquals(p.getEnergyForSex(), c.getEnergyForSex());
		assertEquals(p.getEnergyForSplit(), c.getEnergyForSplit());
		assertEquals(p.getTrendChartFrequency(), c.getTrendChartFrequency());
		assertEquals(p.getInitialEnergy(), c.getInitialEnergy());
		assertEquals(p.getInitialManaToGrow(), c.getInitialManaToGrow());
		assertEquals(p.getInitialNumberOfBacterions(), c.getInitialNumberOfBacterions());
		assertEquals(p.getManaEnergy(), c.getManaEnergy());
		assertEquals(p.getManaToGrowEachRound(), c.getManaToGrowEachRound(), EPS);
		assertEquals(p.getManaToGrowEachRoundInCockaigne(), c.getManaToGrowEachRoundInCockaigne(), EPS);
		assertEquals(p.getMaxEnergy(), c.getMaxEnergy());
		assertEquals(p.getXsize(), c.getXsize());
		assertEquals(p.getYsize(), c.getYsize());
		assertEquals(p.isGenealogyGraph(), c.isGenealogyGraph());
		assertEquals(p.isWorldTorus(), c.isWorldTorus());
	}

	private void assertPlausible(final Bacterion b, final DeinbacProperties p) {
		assertTrue(b.getX() >= 0);
		assertTrue(b.getX() < p.getXsize());
		assertTrue(b.getY() >= 0);
		assertTrue(b.getY() < p.getYsize());
		assertTrue(Bacterion.isInsideWorldBorders(b.getX(), b.getY(), p));
		assertTrue(b.getEnergy() > 0);
	}

	@Test
	public void testConstructorProps() {
		final DeinbacProperties p = new DeinbacProperties();
		final Bacterion b = new Bacterion(p);
		assertPlausible(b, p);
	}

	@Test
	public void testConstructorSplit() {
		final DeinbacProperties p = new DeinbacProperties();
		final Bacterion b = new Bacterion(p);
		final Bacterion c = new Bacterion(b, 50);
		assertPlausible(c, p);
	}

	@Test
	public void testConstructorSex() {
		final DeinbacProperties p = new DeinbacProperties();
		final Bacterion b = new Bacterion(p);
		final Bacterion c = new Bacterion(p);
		final Bacterion d = new Bacterion(b, c, 100);
		assertPlausible(d, p);
	}

	@Test
	public void testDie() {
		final DeinbacProperties p = new DeinbacProperties();
		final Bacterion b = new Bacterion(p);
		b.die(150);
		assertTrue(b.getEnergy() < 0);
	}
}
