package de.ped.deinbac.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.deinbac.logic.LifeGame;
import de.ped.deinbac.logic.LifeProperties;
import de.ped.deinbac.logic.LifeProperties.Pattern;

public class LifeGameTest {

	class TestData3x3 {
		public final boolean isTorus;
		public final String start;
		public final String expected;
		public TestData3x3(final boolean isTorus, final String start, final String expected) {
			this.isTorus = isTorus;
			this.start = start;
			this.expected = expected;
		}
	}

	public TestData3x3[] dataSetBorder = {
			new TestData3x3(false, "ooo$ooo$ooo!", "ooo$ooo$ooo!"),
			new TestData3x3(false, "ooo$obo$ooo!", "ooo$ooo$ooo!"),
			new TestData3x3(false, "boo$obo$oob!", "ooo$obo$ooo!"),
			new TestData3x3(true, "ooo$ooo$ooo!", "ooo$ooo$ooo!"),
			new TestData3x3(true, "ooo$obo$ooo!", "ooo$ooo$ooo!"),
			new TestData3x3(true, "boo$obo$oob!", "bbb$bbb$bbb!"),
			new TestData3x3(true, "bbb$bbb$bbb!", "ooo$ooo$ooo!"),
	};

	@Test
	public void testMiniBoards() {
		final LifeProperties props = new LifeProperties();
		props.setXsize(3);
		props.setYsize(3);
		props.setPattern(null);
		LifeGame.init();
		if (null != LifeGame.preCalculated) {
			assertEquals(0b0000, LifeGame.preCalculated[0b0101]);

			assertEquals(0b0000, LifeGame.preCalculated[0b0000]);
			assertEquals(0b0000, LifeGame.preCalculated[0b0001]);
			assertEquals(0b0000, LifeGame.preCalculated[0b0010]);
			assertEquals(0b0000, LifeGame.preCalculated[0b0011]);
			assertEquals(0b0000, LifeGame.preCalculated[0b0100]);

			assertEquals(0b0000, LifeGame.preCalculated[0b0110]);
			assertEquals(0b0010, LifeGame.preCalculated[0b0111]);
			assertEquals(0b0000, LifeGame.preCalculated[0b1000]);
			assertEquals(0b0000, LifeGame.preCalculated[0b1001]);
			assertEquals(0b0000, LifeGame.preCalculated[0b1010]);
			assertEquals(0b0000, LifeGame.preCalculated[0b1011]);
			assertEquals(0b0000, LifeGame.preCalculated[0b1100]);
			assertEquals(0b0000, LifeGame.preCalculated[0b1101]);
			assertEquals(0b0100, LifeGame.preCalculated[0b1110]);
			assertEquals(0b0110, LifeGame.preCalculated[0b1111]);
		}
		for (TestData3x3 data : dataSetBorder) {
			props.setWorldTorus(data.isTorus);
			final LifeGame game = new LifeGame(props);
			for (int y = 0; y < props.getYsize(); y++) {
				for (int x = 0; x < props.getXsize(); x++) {
					final char c = data.start.charAt(y * (1 + props.getXsize()) + x);
					assertTrue(data.start + "(" + x + "," + y + ")", (c == LifeProperties.ALIVE_CELL) || (c == LifeProperties.DEAD_CELL));
					game.setPlayfieldAt(x, y, (c == LifeProperties.ALIVE_CELL));
				}
			}
			final boolean result = game.step();
			final String eb = game.getBoardString(true);
			assertEquals(data.start, data.expected, eb);
			assertEquals(data.start, (!data.start.contentEquals(data.expected)), result);
		}
	}

	@Test
	public void testLongRun() {
		final LifeProperties props = new LifeProperties();
		props.setXsize(500);
		props.setYsize(500);
		props.setPattern(Pattern.WING_SPACESHIP);
		props.setWorldTorus(false);
		for (LifeGame.ImplementationMode mode : LifeGame.ImplementationMode.values()) {
			LifeGame.implMode = mode;
			long initStartTime = System.currentTimeMillis();
			LifeGame.init();
			long initEndTime = System.currentTimeMillis();
			final LifeGame game = new LifeGame(props);
			long startTime = System.currentTimeMillis();
			for (int i = 0; i < 1000; i++) {
				game.step();
			}
			long endTime = System.currentTimeMillis();
			System.out.println("For " + mode.name() + " init took " + (initEndTime - initStartTime) + "ms and long run duration took " + (endTime - startTime) + "ms");
		}
	}
}
