package de.ped.dsatool.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.SwingConstants;

import de.ped.tools.Assert;

/**
 * An icon showing an arrow.
 * 
 * @author Copyright (c) 2007 Peter Diefenbach (peter@pdiefenbach.de)
 */
class ArrowIcon implements Icon {
	/** The width and height of the icon in pixels. */
	public static final int W = 16;

	/** The number of points of the polygon. */
	private static final int POINTS = 4;

	/** The x-coordinates of the polygon drawing the arrow pointing to the east. */
	private static final int[] XPOINTS = new int[]{W/4, W/4, W*3/4, W*3/4};
	/** The y-coordinates of the polygon drawing the arrow pointing to the east. */
	private static final int[] YPOINTS = new int[]{ -1,   W,   W/2, W/2 - 1};

	/** The x-coordinates of the polygon drawing this arrow. */
	private int[] xPoints = new int[POINTS];
	/** The y-coordinates of the polygon drawing this arrow. */
	private int[] yPoints = new int[POINTS];

	/**
	 * Creates the icon showing an arrow.
	 * @param direction the direction, defined by {@link javax.swing.SwingConstants}
	 */
	public ArrowIcon(final int direction) {
		Assert.assertEquals(POINTS, XPOINTS.length);
		Assert.assertEquals(POINTS, YPOINTS.length);
		switch (direction) {
		case SwingConstants.WEST:
			for (int i = 0; i < POINTS; i++) {
				xPoints[i] = W - XPOINTS[i];
				yPoints[i] = YPOINTS[i];
			}
			break;
		case SwingConstants.EAST:
			for (int i = 0; i < POINTS; i++) {
				xPoints[i] = XPOINTS[i];
				yPoints[i] = YPOINTS[i];
			}
			break;
		case SwingConstants.NORTH:
			for (int i = 0; i < POINTS; i++) {
				xPoints[i] = YPOINTS[i];
				yPoints[i] = W - XPOINTS[i];
			}
			break;
		case SwingConstants.SOUTH:
			for (int i = 0; i < POINTS; i++) {
				xPoints[i] = YPOINTS[i];
				yPoints[i] = XPOINTS[i];
			}
			break;
		default:
			throw new IllegalArgumentException();
		}
	}

	@Override
	public void paintIcon(final Component c, final Graphics g, final int x, final int y) {
		if (c.isEnabled()) {
			g.setColor(c.getForeground());
		} else {
			g.setColor(Color.gray);
		}
		g.translate(x, y);
		g.fillPolygon(xPoints, yPoints, POINTS);
		g.translate(-x, -y); // Restore Graphics object
	}

	@Override
	public int getIconHeight() {
		return W;
	}

	@Override
	public int getIconWidth() {
		return W;
	}
}
