package de.ped.dsatool.gui;

import javax.swing.JComboBox;

import java.util.Iterator;

import de.ped.dsatool.dsa.generated.SeasonType;
import de.ped.dsatool.logic.Weather;

/**
 * A choice box that offers all weather situations of Aventuria
 *
 * @author Copyright (c) 1999 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ChoiceWeather extends JComboBox<String> {
	private static final long serialVersionUID = 1L;

	private SeasonType seasonMemory;

	/**
	 * Standard Constructor.
	 */
	public ChoiceWeather() {
		// Make this field definitely large enough for all entries:
		final Iterator<String> it = Weather.getAllWeatherNames().iterator();
		while (it.hasNext()) {
			addItem(it.next());
		}
	}

	public void saveSize() {
		setPreferredSize(getSize());
	}

	/**
	 * Adapts the possible weather situations to the given season.
	 * 
	 * @param weather
	 *            the weather object the season of its date is evaluated
	 * @return <code>true</code>, iff the contents of the choice box has changed (because the season changed)
	 */
	public boolean adapt(final Weather weather) {
		if ((null == seasonMemory) || (!(weather.getDate().season().equals(seasonMemory)))) {
			seasonMemory = weather.getDate().season();	// We really need a copy, to identify changes in the original season.
			removeAllItems();
			final String[] names = weather.getWeatherNames();
			for (int i = 0; i < names.length; i++) {
				addItem(names[i]);
			}
			return true;
		}
		return false;
	}
}
