package de.ped.dsatool.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.JTextField;

import de.ped.tools.CollectionUtil;
//import de.ped.tools.Assert;
import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.log.Logger;
import de.ped.dsatool.dsa.generated.MonthType;
import de.ped.dsatool.logic.DSA;
import de.ped.dsatool.logic.DSAEnv;
import de.ped.dsatool.logic.Model;
import de.ped.dsatool.logic.PlantsModel;
import de.ped.dsatool.logic.Terrain;

/**
 * The dialog that asks the user for input for a botany search -
 * like Aventurian region, terrain, or success of the hero's test for his/her botany talent.
 *
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlantsDialog extends PedJDialog implements ActionListener, FocusListener, ListSelectionListener {
	private static final long serialVersionUID = 1L;
	
	private static Logger logger = DSATool.getLogger();

	/**
	 * Constructor.
	 * @param parent the parent window
	 * @param plantsModel the underlying model
	 */
	public PlantsDialog(final ApplicationMainWindow parent, final PlantsModel plantsModel) {
		super(parent, parent);
		model = plantsModel;
		final Messages messages = getMessages();
		setTitle(messages.getText("Action.CreatePlants"));
		setTitlePanel(messages.getText("PlantsDialog.Title")); 
		labelRegion  = parent.createLabel("PlantsDialog.Region");
		labelTerrain = parent.createLabel("PlantsDialog.Terrains");
		labelMonth   = parent.createLabel("PlantsDialog.Month");
		labelBotany  = parent.createLabel("PlantsDialog.Botany");
		checkBoxIsToFilterForDate      = createCheckBox("PlantsDialog.IsToFilterFor.Date");
		checkBoxIsToTestForTerrainProb = createCheckBox("PlantsDialog.IsToFilterFor.TerrainProb");
		checkBoxIsToTestForBotany      = createCheckBox("PlantsDialog.IsToFilterFor.Botany");

		// Eingabefeld:
		final JPanel mainPanel = createMainPanel();
		final GridBagLayout gridbag = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainPanel.setLayout(gridbag);
		{
			c.gridwidth = 1;
			gridbag.setConstraints(labelRegion, c);
			mainPanel.add(labelRegion);
		}
		{
			choiceRegion = new ChoiceRegion();
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(choiceRegion);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			labelRegion.setLabelFor(choiceRegion);
			mainPanel.add(p);
		}
		{
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEADING));
			p.add(labelTerrain);
			c.gridwidth = 1;
			gridbag.setConstraints(labelTerrain, c);
			mainPanel.add(p);
		}
		{
			listTerrains.setLayoutOrientation(JList.VERTICAL_WRAP);
			listTerrains.addListSelectionListener(this);
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(listTerrains);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			labelTerrain.setLabelFor(listTerrains);
			mainPanel.add(p);
		}
		{
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(checkBoxIsToTestForTerrainProb);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			mainPanel.add(p);
		}
		{
			checkBoxIsToFilterForDate.addActionListener(this);
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(checkBoxIsToFilterForDate);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			mainPanel.add(p);
		}
		{
			c.gridwidth = 1;
			gridbag.setConstraints(labelMonth, c);
			mainPanel.add(labelMonth);
		}
		{
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(choiceMonth);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			labelMonth.setLabelFor(choiceMonth);
			mainPanel.add(p);
		}
		{
			checkBoxIsToTestForBotany.addActionListener(this);
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(checkBoxIsToTestForBotany);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			mainPanel.add(p);
		}
		{
			c.gridwidth = 2;
			gridbag.setConstraints(labelBotany, c);
			mainPanel.add(labelBotany);
		}
		{
			textBotany.setColumns(5);
			final JPanel p = new JPanel();
			p.setLayout(new FlowLayout(FlowLayout.LEFT));
			p.add(textBotany);
			c.gridwidth = GridBagConstraints.REMAINDER; //end row
			gridbag.setConstraints(p, c);
			labelBotany.setLabelFor(textBotany);
			mainPanel.add(p);
		}

		getContentPane().add(new JScrollPane(mainPanel), BorderLayout.CENTER);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);
		addHelpButton("Action.CreatePlants");
		finishLayout();
		updateUIFieldsBasedOnProperties();
		updateView();
	}

	private JCheckBox createCheckBox(final String key) {
		final Messages messages = DSAEnv.instance().messages();
		final JCheckBox box = new JCheckBox(messages.getText(key));
		final String toolTip = messages.getShortDescription(key);
		if (!CollectionUtil.isNullOrEmpty(toolTip)) {
			box.setToolTipText(messages.getShortDescription(key));
		}
		return box;
	}
	
	private void updateView() {
		final boolean isToTestForBotany = checkBoxIsToTestForBotany.isSelected();
		textBotany.setEnabled(isToTestForBotany);
		labelBotany.setEnabled(isToTestForBotany);
		final boolean isToFilterForDate = checkBoxIsToFilterForDate.isSelected();
		choiceMonth.setEnabled(isToFilterForDate);
		labelMonth.setEnabled(isToFilterForDate);
		if (listTerrains.isSelectionEmpty()) {
			getButtonOk().setEnabled(false);
		} else {
			getButtonOk().setEnabled(true);
		}
	}

	public PlantsModel getModel() {
		return model;
	}

	/**
	 * Invoked when an action occurs.
	 * @param e the event to handle
	 */
	@Override
	public void actionPerformed(final ActionEvent e) {
		logger.debug(e.paramString());
		final Object source = e.getSource();
		if ((source == getButtonOk()) || (source == textBotany)) {
			// Pressing return in a text field should be handled as pressing the OK button
			readUIFieldsIntoProperties();
			onOk();
		} else if (source == getButtonCancel()) {
			onCancel();
		} else if ((source == checkBoxIsToTestForBotany) || (source == checkBoxIsToFilterForDate)) {
			updateView();
		}
	}

	@Override
	public void focusGained(final FocusEvent e) {
		// Nothing to do
	}

	@Override
	public void focusLost(final FocusEvent e) {
		if (e.getSource() == textBotany) {
			// If the user changes the year, we must act immediatly
			// to check if the year is valid (some calendars don't have a year 0)
			readUIFieldsIntoProperties();
			updateUIFieldsBasedOnProperties();
			repaint();
		}
	}

	/** 
	 * Handles the state changes of lists. 
	 * @param e the event to handle
	 */
	@Override
	public void valueChanged(final ListSelectionEvent e) {
		if (e.getSource() == listTerrains) {
			// Has the user finished manipulating the state of the list?
			if (e.getValueIsAdjusting() == false) {
				updateView();
			}
		}
	}

	@Override
	public void readUIFieldsIntoProperties() {
		model.setRegion(choiceRegion.getSelectedRegion());
		model.setTerrainIDs(listTerrains.getSelectedIndices());
		model.setToTestForTerrainProbability(checkBoxIsToTestForTerrainProb.isSelected());
		model.setToTestForBotany(checkBoxIsToTestForBotany.isSelected());
		model.setTestForBotanySuccess(Integer.parseInt(textBotany.getText()));
		model.setToFilterForDate(checkBoxIsToFilterForDate.isSelected());
		final MonthType month = (MonthType)DSA.findBOByIDnum(DSA.instance().getMonth(), choiceMonth.getSelectedIndex());
		model.setMonth(month);
		model.recalculate();
		updateUIFieldsBasedOnProperties();
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		choiceRegion.setSelectedRegion(model.getRegion());
		listTerrains.setSelectedIndices(model.getTerrainIDs());
		checkBoxIsToTestForTerrainProb.setSelected(model.isToTestForTerrainProbability());
		checkBoxIsToTestForBotany.setSelected(model.isToTestForBotany());
		textBotany.setText(Integer.toString(model.getTestForBotanySuccess()));
		checkBoxIsToFilterForDate.setSelected(model.isToFilterForDate());
		choiceMonth.setSelectedIndex(model.getMonth().getIDnum());
	}

	private JLabel       labelRegion;
	private ChoiceRegion choiceRegion;
	private JLabel       labelTerrain;
	private JList<String> listTerrains = new JList<String>(Terrain.getAllStrings());
	private JCheckBox    checkBoxIsToTestForTerrainProb;
	private JCheckBox    checkBoxIsToTestForBotany;
	private JCheckBox    checkBoxIsToFilterForDate;
	private JComboBox<String> choiceMonth = new JComboBox<String>(Model.getAllMonthStrings());
	private JLabel       labelMonth;
	private JTextField   textBotany = new JTextField();
	private JLabel       labelBotany;
	private PlantsModel model;
}
