package de.ped.dsatool.logic;

import de.ped.dsatool.dsa.generated.PersonType;
import de.ped.dsatool.dsa.generated.BuildingType;
import de.ped.tools.Assert;

/**
 * The concept of buildings for towns.
 * This is the abstract root class for single shops, hotels, bars, temples, and all other buildings.
 *
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Stadt- und Dorfentwicklung" (p.20..23)</p>
 *
 * @author Copyright (c) 1998 Peter Diefenbach (peter@pdiefenbach.de)
 */
public abstract class Building extends BuildingType {
	/**
	 * Default constructor.
	 */
	protected Building() {
		// Nothing to do
	}

	/**
	 * Get-method for property.
	 * Overload this method, if the title of the most important inhabitant is more specific,
	 * as for example "bartender".
	 * 
	 * @return The correct title for the person being the chief of this building
	 */
	public String getChiefTitle() {
		return DSAEnv.instance().messages().getText("Building.Chief");
	}

	@Override
	public void setChief(Object o) {
		Assert.assertTrue(o instanceof PersonType);
		super.setChief(o);
	}
}
