package de.ped.dsatool.logic;

/**
 * DSA Character template.
 * 
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class CharacterTemplate {

	/**
	 * Internal table entry for defining the talent defaults for the different character types.
	 * 
	 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class TalentEntry {
		public final CharacterTalents Talent;
		public final CharacterAttribute[] Probe;
		public final int[] Wert;
		public final String Behinderung;

		public TalentEntry(
				final CharacterTalents Talent,
				final String ProbeStr,
				final int[] Werte,
				final String Behinderung) {
			this.Talent = Talent;
			this.Probe = CharacterAttribute.testCoding(ProbeStr);
			this.Wert = Werte;
			this.Behinderung = Behinderung;
		}
	};

	/**
	 * Internal table entry for defining the spell defaults for the different character types.
	 * 
	 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class ZauberEntry {
		public final CharacterSpell	Zauber;
		public final CharacterType Ursprung;
		public final int[] Wert;
		public final CharacterAttribute[] Probe;
		
		public ZauberEntry(
				CharacterSpell Zauber,
				CharacterType Ursprung,
				int[] Wert,
				String ProbeStr) {
			this.Zauber = Zauber;
			this.Ursprung = Ursprung;
			this.Wert = Wert;
			this.Probe = CharacterAttribute.testCoding(ProbeStr);
		}
	};

	/**
	 * Internal data about magical academy entries
	 * 
	 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class AkademieEntry {
		final String Name;
		final CharacterSpellGroup Gebiet;
		final CharacterSpellColor Richtung;
		final CharacterSpellValue[] Sprueche; // = new int[14][2];

		public AkademieEntry(
				final String	Name,
				final CharacterSpellGroup Gebiet,
				final CharacterSpellColor Richtung,
				final CharacterSpellValue[] Sprueche) {
			this.Name = Name;
			this.Gebiet = Gebiet;
			this.Richtung = Richtung;
			this.Sprueche = Sprueche;
		}
	};

	private static final int[][][] EigBoundTabelle = new int[][][] {
		  /* MU/AG   KL/HA   CH/RA   GE/GG   KK/TA	*/
		  {	{11,13},{ 6,13},{ 6,13},{13,13},{ 6,13},	/* Gaukler		*/
			{ 8, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{11,13},{12,13},{ 6,13},{ 6,13},{ 6,13},	/* Händler		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 7, 9},{ 2, 9}},
		  {	{12,13},{ 6,13},{ 6,13},{ 6,13},{12,13},	/* Krieger		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 6,13},{ 6,13},{12,13},{11,13},{ 6,13},	/* Moha			*/
			{ 7, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 6,13},{ 6,13},{ 6,13},{11,13},{11,13},	/* Nivese		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 8, 9}},
		  {	{13,13},{ 6,13},{ 6,13},{ 6,13},{ 6,13},	/* Novadi		*/
			{ 2, 9},{ 2, 9},{ 7, 9},{ 2, 9},{ 2, 9}},
		  {	{13,13},{ 6,13},{ 6,13},{13,13},{ 6,13},	/* Streuner		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{11,13},{ 6,13},{ 6,13},{ 6,13},{13,13},	/* Thorwaler	*/
			{ 7, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 6,13},{ 6,13},{ 6,13},{12,13},{12,13},	/* Zwerg		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 8, 9},{ 2, 9}},
		  {	{ 6,13},{ 6,13},{ 6,13},{ 6,13},{ 6,13},	/* Bürger		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{13,13},{12,13},{ 7,13},{ 7,13},	/* Magier		*/
			{ 2, 4},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{13,13},{ 7,13},{12,13},{ 7,13},	/* Auelf		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 4},{ 2, 9}},
		  {	{ 7,13},{12,13},{ 7,13},{13,13},{ 7,13},	/* Waldelf		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 4},{ 2, 9}},
		  {	{ 7,13},{13,13},{ 7,13},{13,13},{ 7,13},	/* Firnelf		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 4},{ 2, 9}},
		  {	{12,13},{ 7,13},{13,13},{ 7,13},{ 7,13},	/* Hexe			*/
			{ 2, 9},{ 2, 4},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{ 7,13},{13,13},{13,13},{ 7,13},	/* Schelm		*/
			{ 2, 9},{ 2, 4},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{13,13},{12,13},{ 7,13},{ 7,13},{ 7,13},	/* Druide		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 4}},
		  /* MU/AG   KL/HA   CH/RA   GE/GG   KK/TA	*/
		  {	{13,13},{12,13},{12,13},{ 7,13},{ 7,13},	/* Praios-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{12,13},{ 7,13},{12,13},{11,13},{ 7,13},	/* Rondra-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{12,13},{12,13},{11,13},{ 7,13},	/* Efferd-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{12,13},{13,13},{11,13},{ 7,13},	/* Travia-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{11,13},{13,13},{ 7,13},{ 7,13},{ 7,13},	/* Boron-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 3}},
		  {	{ 7,13},{13,13},{12,13},{ 7,13},{ 7,13},	/* Hesinde-Gew.	*/
			{ 2, 3},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{12,13},{12,13},{ 7,13},{ 7,13},{12,13},	/* Firun-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{12,13},{12,13},{12,13},{ 7,13},	/* Tsa-Gew.		*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{12,13},{11,13},{13,13},{ 7,13},	/* Phex-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{12,13},{13,13},{11,13},{ 7,13},	/* Peraine-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{12,13},{ 7,13},{12,13},{12,13},	/* Ingerimm-Gew.*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  {	{ 7,13},{11,13},{13,13},{12,13},{ 7,13},	/* Rahja-Gew.	*/
			{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9},{ 2, 9}},
		  /* MU/AG   KL/HA   CH/RA   GE/GG   KK/TA	*/
	};

	static final TalentEntry[]	TalentTabelle = new TalentEntry[] {
		/*																			  Ga Hä Kr Mo Ni No St Th Zw Bü|Ma AE WE FE He Sc Dr|PraRonEffTraBorHesFirTsaPhePerIngRah*/
		new TalentEntry(CharacterTalents.TA_Boxen,				"MU,GE,KK", new int[]{ 2, 2, 4,-1, 0, 1, 2, 3,-2, 0, 0,-1,-1, 0,-2, 2,-2, 0, 3, 2, 0, 1, 0, 2, 2, 2, 1, 1, 2	},"-RS"),
		new TalentEntry(CharacterTalents.TA_Ringen,				"MU,GE,KK", new int[]{ 3, 1, 2, 2, 3, 2, 3, 3, 3, 0, 1, 2, 2, 3, 2, 3, 2, 0, 1, 2, 0, 1, 1, 2, 3, 3, 1, 2, 2	},"-RS"),
		new TalentEntry(CharacterTalents.TA_Hruruzat,			"MU,GE,KK", new int[]{-4,-7,-5, 3,-5,-4,-4,-6,-7,-8,-7,-5,-6,-7,-6,-7,-7,-7,-7,-7,-7,-7,-7,-7,-5,-6,-7,-7,-6	},"-RS"),
		new TalentEntry(CharacterTalents.TA_Aexte,				"MU,GE,KK", new int[]{ 0, 0, 2, 1, 1, 0, 1, 3, 4,-1,-5,-1,-1,-2,-5,-7,-6,-3, 1,-4,-3,-3,-4,-4,-4,-4,-4, 1,-4	},"-(RS-2)"),
		new TalentEntry(CharacterTalents.TA_HiebwaffenScharf,	"MU,GE,KK", new int[]{ 1, 2, 4, 0, 3, 4, 1, 3, 2,-1,-4, 3, 3, 3,-5,-5,-7,-2, 3, 0,-2,-2,-4,-2,-4,-4,-2, 2,-2	},"-(RS-2)"),
		new TalentEntry(CharacterTalents.TA_HiebwaffenStumpf,	"MU,GE,KK", new int[]{ 2, 3, 2, 3, 2, 2, 2, 3, 3, 0, 2, 2, 3, 2, 3, 1, 2, 3, 2, 0, 0, 2, 1, 0, 0, 2, 0, 2, 0	},"-(RS-2)"),
		new TalentEntry(CharacterTalents.TA_Schwerter,			"MU,GE,KK", new int[]{ 1, 1, 5, 0, 3, 1, 1, 1, 2,-1,-4, 3, 3, 2,-5,-5,-7, 1, 6,-2,-4, 1, 1,-4,-4,-2,-3,-2,-3	},"-(RS-2)"),
		new TalentEntry(CharacterTalents.TA_Stichwaffen,		"MU,GE,KK", new int[]{ 3, 1, 2, 2, 1, 1, 4, 0,-2,-3, 2, 3, 3, 3, 2, 0, 0, 0, 1, 3, 0, 2, 1, 2,-2, 2, 0,-1, 1	},"-(RS-1)"),
		new TalentEntry(CharacterTalents.TA_Schusswaffen,		"KL,GE,KK", new int[]{ 0,-2, 3, 1, 4, 0,-2,-3,-2,-4,-3, 4, 4, 2, 0,-3, 1,-2, 1,-4,-4,-2,-1, 5,-4,-2,-3,-3,-4	},""),
		new TalentEntry(CharacterTalents.TA_Wurfwaffen,			"KL,GE,KK", new int[]{ 3,-3, 0, 2, 1, 1, 3, 2, 2,-4,-5, 2, 2, 4, 2, 2, 1, 2, 0, 3, 0,-2,-3, 3,-4, 0,-3,-3, 1	},"-(RS-3)"),
		new TalentEntry(CharacterTalents.TA_Faehrtensuche,		"KL,KL,GE", new int[]{-3, 1,-1, 4, 3, 1,-2,-5, 0,-3,-5, 3, 4, 2, 2,-3, 2,-4,-1,-4,-3,-4,-3, 4,-3,-5,-2,-4,-5	},""),
		new TalentEntry(CharacterTalents.TA_Fallenstellen,		"KL,GE,KK", new int[]{-3, 0,-3, 2, 3, 0,-2,-4, 0,-5,-5, 2, 4, 3,-3,-3, 0,-4,-3,-4,-3,-4,-3, 2,-4,-5,-2,-4,-5	},""),
		new TalentEntry(CharacterTalents.TA_FischenAngeln,		"KL,GE,KK", new int[]{-3, 0,-3, 3, 4,-6, 0, 4,-1,-2,-3, 3, 0, 4,-1,-3, 0,-3,-2, 4, 0,-3,-3, 3,-2,-2, 1,-3,-4	},""),
		new TalentEntry(CharacterTalents.TA_FesselnEntfesseln,	"GE,GE,KK", new int[]{ 5,-3,-3, 3, 2, 2, 2, 3, 2,-1, 0, 1, 1, 0, 2, 4, 2,-3,-1, 3,-3,-3,-2,-1,-1, 3,-3,-1, 3	},"RS-3"),
		new TalentEntry(CharacterTalents.TA_Gefahreninstinkt,	"MU,KL,CH", new int[]{ 1,-1, 0, 5, 2, 1, 3,-3, 2, 0, 1, 1, 2, 2, 2, 0, 2, 0, 1, 1,-1, 1, 1, 2, 2, 3, 0, 0, 1	},""),
		new TalentEntry(CharacterTalents.TA_Pflanzenkunde,		"MU,KL,KL", new int[]{-3, 1,-3, 4, 3, 1,-2,-5, 0,-1, 3, 2, 4,-2, 5,-4, 5, 0,-2, 0, 2, 3, 4, 2, 4,-1, 6,-2, 2	},""),
		new TalentEntry(CharacterTalents.TA_Orientierung,		"KL,KL,GE", new int[]{-1, 2, 1, 3, 3, 4, 0, 4, 1,-1,-2, 2, 3, 5, 2, 0, 2, 2, 1, 4, 0, 1, 1, 3, 0, 1, 1, 1, 0	},""),
		new TalentEntry(CharacterTalents.TA_Tierkunde,			"MU,KL,CH", new int[]{-2, 1,-2, 3, 4, 1,-2,-2,-3,-1,-2, 2, 2, 1, 3,-3, 4,-1, 0, 1, 1,-2, 0, 3, 0,-2, 0,-3,-2	},""),
		new TalentEntry(CharacterTalents.TA_Wettervorhersage,	"KL,CH,CH", new int[]{-3, 1,-4, 0, 2,-2,-3, 4,-4,-3,-3, 2, 1, 4, 3,-1, 3, 3,-2, 5,-1, 0, 0, 3, 0,-4,-1,-3,-4	},""),
		new TalentEntry(CharacterTalents.TA_Wildnisleben,		"MU,KL,GE", new int[]{-2, 1, 2, 3, 5, 2,-2,-3, 0,-2,-4, 2, 4, 3, 2,-2, 3,-4, 1,-1,-2,-4,-4, 4,-2,-4, 2,-2,-4	},""),
		new TalentEntry(CharacterTalents.TA_Akrobatik,			"MU,GE,KK", new int[]{ 5,-2, 1, 3, 0, 0, 1, 0,-2,-3,-3, 1, 2, 0,-2, 4,-2, 0, 1, 0,-1,-1, 0,-1, 0, 1, 0,-1, 1	},"RS*2"),
		new TalentEntry(CharacterTalents.TA_Fliegen,			"MU,KL,KK", new int[]{ 1, 0,-1,-3,-2, 0,-2,-3,-5,-5,-1,-1,-2,-2, 3, 0,-2, 0,-2,-2,-2,-2,-1,-2,-1, 0,-1,-2,-1	},"RS-4"),
		new TalentEntry(CharacterTalents.TA_Klettern,			"MU,GE,KK", new int[]{ 5, 1, 1, 5, 1, 0, 3, 3, 4, 1, 1, 2, 5, 0, 0, 3, 2, 1, 0, 1, 0, 0, 1, 1, 0, 3, 0, 0, 0	},"RS"),
		new TalentEntry(CharacterTalents.TA_Reiten,				"CH,GE,KK", new int[]{ 3, 4, 4,-3, 1, 7, 1,-2,-5, 0, 1, 2, 0,-2,-1,-3,-1, 3, 5, 0, 1, 1, 1, 1, 2, 1, 1, 1, 1	},"RS-4"),
		new TalentEntry(CharacterTalents.TA_RudernSegeln,		"GE,KK,KK", new int[]{-3, 1, 0,-2, 0,-5,-2, 6,-4,-1,-2, 1,-1, 4,-3,-4,-3, 0, 0, 4, 0, 1,-1, 1, 1,-2, 1, 0,-1	},""),
		new TalentEntry(CharacterTalents.TA_Schleichen,			"MU,KL,GE", new int[]{ 4, 0, 0, 5, 3, 0, 4,-1, 0,-1, 0, 3, 4, 5, 2, 4, 3,-1,-1,-1, 0, 2, 0, 2, 2, 4, 0, 0, 2	},"RS"),
		new TalentEntry(CharacterTalents.TA_Schwimmen,			"MU,GE,KK", new int[]{ 1, 1, 2, 0, 1,-4, 0, 5,-5, 0, 0, 2,-2, 0, 1, 0, 1, 1, 2, 4, 1, 0, 0, 2, 1, 0, 1, 0, 1	},"RS"),
		new TalentEntry(CharacterTalents.TA_SichVerstecken,		"MU,KL,GE", new int[]{ 4, 0,-2, 4, 2, 0, 4,-2, 3, 0, 0, 3, 5, 5, 2, 3, 0,-3,-3, 0, 1, 0, 0, 1, 2, 3, 0, 0, 2	},"RS"),
		new TalentEntry(CharacterTalents.TA_Sinnesschaerfe,		"KL,CH,GE", new int[]{ 1, 0, 0, 4, 3, 3, 2, 2, 4, 0, 3, 4, 5, 6, 3, 1, 3, 2, 1, 2, 1, 2, 4, 4, 1, 2, 1, 2, 3	},""),
		new TalentEntry(CharacterTalents.TA_Springen,			"MU,GE,KK", new int[]{ 5,-2, 1, 2, 0, 1, 2, 2,-4,-2,-1, 1, 3, 1, 0, 3, 0, 1, 2, 2, 0, 0, 0, 1, 0, 1, 0, 0, 1	},"RS*2"),
		new TalentEntry(CharacterTalents.TA_Abrichten,			"MU,KL,CH", new int[]{ 3, 0,-2, 0, 2, 2,-3,-4,-3, 0,-2, 1, 2, 1, 2,-3, 2, 0,-1,-2, 1,-3,-3, 0,-2,-3,-3,-4,-4	},""),
		new TalentEntry(CharacterTalents.TA_Alchimie,			"MU,KL,KL", new int[]{ 0,-1,-5,-7,-6,-4,-2,-6, 2,-2, 6,-5,-6,-7, 0,-5, 0,-2,-4,-4,-3, 3, 5,-4, 2, 0, 3, 2, 1	},""),
		new TalentEntry(CharacterTalents.TA_Bekehren,			"KL,CH,CH", new int[]{-5,-2,-5,-5,-5,-4,-2,-5,-5,-3,-2,-1,-1,-3,-2,-5,-7, 7, 2, 3, 7, 1, 2, 1, 4, 3, 2, 3, 5	},""),
		new TalentEntry(CharacterTalents.TA_Bergbau,			"KL,GE,KK", new int[]{-6,-6,-6,-8,-7,-8,-6,-6, 7,-6,-5,-7,-7,-7,-7,-5,-7,-7,-7,-7,-7,-5,-4,-7,-7,-7,-7, 2,-7	},""),
		new TalentEntry(CharacterTalents.TA_Betoeren,			"MU,CH,CH", new int[]{ 2, 0,-2, 3, 0, 1, 4, 0,-2, 0, 0, 3, 2, 2, 4, 1,-3, 1, 0, 0, 1,-3, 0,-2, 0, 2, 0, 0, 5	},""),
		new TalentEntry(CharacterTalents.TA_Bogenbau,			"KL,GE,KK", new int[]{-2,-2, 2, 1, 4, 0, 0,-3, 0,-3,-5, 4, 4, 3,-5,-7,-5, 0, 0,-2,-2,-2,-1, 3,-3,-5,-3, 2,-5	},""),
		new TalentEntry(CharacterTalents.TA_Fahrzeuglenken,		"MU,CH,GE", new int[]{ 0, 3, 0,-5, 4, 0, 0,-3,-3, 0, 0, 0,-2, 2,-2, 0,-2, 4, 3, 0,-1, 0, 1, 2,-1, 1,-1, 0,-1	},""),
		new TalentEntry(CharacterTalents.TA_Falschspiel,		"MU,KL,GE", new int[]{ 4, 1, 0,-6,-2, 2, 6, 0, 0, 0, 0, 0, 0, 0, 3, 4,-1,-7,-2,-1,-5,-2,-2,-4,-4, 3,-4,-2, 2	},"RS-2"),
		new TalentEntry(CharacterTalents.TA_Feilschen,			"MU,KL,CH", new int[]{ 1, 5,-1,-3, 1, 3, 3,-2, 0, 1, 1,-1,-2,-3, 2, 1, 0, 0,-1,-1, 0,-2, 0,-2,-1, 4,-1, 2, 1	},""),
		new TalentEntry(CharacterTalents.TA_Gassenwissen,		"MU,KL,CH", new int[]{ 3, 2, 0,-6,-4,-2, 7,-2,-4, 1, 0,-3,-6,-7,-2, 4,-2,-2,-1,-1, 0, 0, 0,-3, 2, 4, 0, 1, 2	},""),
		new TalentEntry(CharacterTalents.TA_Gluecksspiel,		"MU,MU,CH", new int[]{ 3, 2, 2, 0, 0, 2, 3, 3,-5, 1, 0, 1, 0, 3, 0, 3, 0,-2, 2, 1,-2, 2, 2,-1,-1, 5,-1, 2, 4	},""),
		new TalentEntry(CharacterTalents.TA_HeilkundeGift,		"MU,KL,CH", new int[]{ 0, 1,-1, 5, 0, 0, 3,-1, 2,-1, 2, 2, 3, 0, 4,-1, 3, 4,-1, 1, 2, 3, 4, 2, 4, 1, 4, 2, 3	},""),
		new TalentEntry(CharacterTalents.TA_HeilkundeKrankh,	"KL,KL,CH", new int[]{ 0, 2, 0, 2, 3, 1, 1, 1, 3, 1, 2, 4, 4, 2, 3,-1, 4, 3, 0, 1, 4, 2, 4, 2, 4, 0, 6,-1, 1	},""),
		new TalentEntry(CharacterTalents.TA_HeilkundeSeele,		"KL,CH,CH", new int[]{ 2, 0,-3, 4, 0, 1,-1,-3, 1, 1, 2, 3, 4, 3, 1, 2, 2, 0,-3, 0, 5, 4, 1,-3, 4, 2, 3,-1, 5	},""),
		new TalentEntry(CharacterTalents.TA_HeilkundeWunden,	"KL,CH,GE", new int[]{ 2, 1, 2, 2, 2, 2, 3, 2, 3, 2, 2, 3, 3, 4, 1, 1, 2, 1, 2, 2, 2, 0, 1, 3, 6, 3, 4, 3, 1	},""),
		new TalentEntry(CharacterTalents.TA_Holzbearbeitung,	"KL,GE,KK", new int[]{ 0, 0, 0, 2, 3,-2, 0, 4, 0, 1, 0, 3, 4, 5, 1,-1, 3, 0,-1, 1, 1, 1, 0, 1,-1, 0,-1, 3,-1	},""),
		new TalentEntry(CharacterTalents.TA_Kochen,				"MU,KL,GE", new int[]{ 1, 2, 0, 0, 3, 0, 2,-1, 0, 2, 1, 2, 2, 1, 3, 0, 2, 1, 0, 1, 6, 1, 2, 0, 0, 0, 3, 0, 1	},""),
		new TalentEntry(CharacterTalents.TA_LesenSchreiben,		"KL,KL,GE", new int[]{-4, 4, 2,-4,-3, 0, 0, 1, 0, 0, 6, 1, 0, 0, 0, 0, 0, 5, 4, 4, 4, 5, 6, 4, 4, 4, 4, 5, 4	},""),
		new TalentEntry(CharacterTalents.TA_Luegen,				"MU,MU,CH", new int[]{ 4, 4, 0,-3, 0, 3, 5,-1,-2, 2, 3, 3, 3, 3, 3, 6, 2,-7,-3, 0,-3,-2, 0,-2, 1, 4, 0, 0, 2	},""),
		new TalentEntry(CharacterTalents.TA_MalenZeichnen,		"KL,GE,GE", new int[]{ 0, 0, 0,-3, 0, 1, 1, 0, 0, 0, 0, 2, 1, 2, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 2, 2	},""),
		new TalentEntry(CharacterTalents.TA_Mechanik,			"KL,KL,GE", new int[]{ 2, 1, 0,-5, 0, 0, 0, 1, 4, 0, 1,-1,-2,-3,-3, 0,-3, 0, 1, 0, 0, 1, 3, 1, 0, 1, 0, 4, 0	},""),
		new TalentEntry(CharacterTalents.TA_Menschenkenntnis,	"KL,CH,CH", new int[]{ 2, 3, 0, 2, 2, 1, 3, 0,-3, 1, 2, 0,-1, 2, 2,-2, 1, 1, 0, 2, 5, 6, 0,-1, 0, 3, 0, 0, 4	},""),
		new TalentEntry(CharacterTalents.TA_Musizieren,			"KL,CH,GE", new int[]{ 4, 0, 0, 0, 3, 1, 1, 1,-2, 0,-2, 4, 3, 3, 2, 1, 0, 1, 2, 1, 2, 1, 1, 0, 1, 1, 1, 0, 4	},"RS-4"),
		new TalentEntry(CharacterTalents.TA_Prophezeien,		"MU,CH,CH", new int[]{ 0,-1,-3, 0,-2, 3,-2,-2, 0,-2, 0, 1, 1, 3, 4,-3, 3, 3, 1, 1, 1, 3, 1, 2, 3, 0, 1, 0, 1	},""),
		new TalentEntry(CharacterTalents.TA_Rechnen,			"KL,KL,GE", new int[]{ 0, 4, 2, 0, 1, 1, 1, 2, 2, 1, 5, 1, 1, 1, 0, 0, 0, 3, 1, 2, 2, 2, 5, 2, 2, 3, 2, 5, 2	},""),
		new TalentEntry(CharacterTalents.TA_Ruestungsbau,		"KL,GE,KK", new int[]{-3,-3, 1,-5,-4,-4,-4, 0, 3,-4,-7,-1,-3,-4,-7,-7,-7,-1, 3,-4,-4,-4,-2,-1,-4,-4,-4, 2,-4	},""),
		new TalentEntry(CharacterTalents.TA_Schaetzen,			"MU,KL,CH", new int[]{ 2, 4, 0, 0, 1, 2, 3, 0, 5, 1, 1,-2,-2,-2,-1,-1, 0, 0, 0, 0, 2, 0, 2,-1, 1, 3, 0, 3, 2	},""),
		new TalentEntry(CharacterTalents.TA_SichVerkleiden,		"MU,CH,GE", new int[]{ 5, 0,-2, 0, 0, 0, 5,-2,-6, 1, 0,-2,-3,-4, 3, 5,-2,-4,-2,-1, 0,-1, 0,-2, 2, 3,-1,-1, 4	},""),
		new TalentEntry(CharacterTalents.TA_SprachenKennen,		"KL,KL,CH", new int[]{ 2, 4, 0,-3,-2,-1, 3, 0,-2, 0, 6, 2, 1, 0, 0, 0, 0, 4, 2, 3, 3, 3, 6, 1, 2, 3, 2, 2, 2	},""),
		new TalentEntry(CharacterTalents.TA_Tanzen,				"CH,CH,GE", new int[]{ 5, 0, 0, 2, 0, 2, 2, 0,-4, 1,-1, 4, 4, 3, 4, 0,-2, 1, 0, 1, 2, 0, 0,-1, 1, 1, 1,-1, 5	},"RS"),
		new TalentEntry(CharacterTalents.TA_Taschendiebstahl,	"MU,KL,GE", new int[]{ 2,-1,-2, 0,-1, 0, 5,-2, 0,-2,-2,-2,-3,-4,-2, 3,-4,-7,-2,-3,-4,-4,-3,-4,-3, 1,-4,-3, 0	},"RS*2"),
		new TalentEntry(CharacterTalents.TA_Waffenschmieden,	"KL,GE,KK", new int[]{-5,-4,-2,-5,-1,-3,-4, 0, 6,-4,-6,-4,-4,-5,-7,-7,-7,-3, 1,-4,-6,-4,-5,-3,-7,-5,-5, 2,-6	},""),
		new TalentEntry(CharacterTalents.TA_Zechen,				"KL,GE,KK", new int[]{ 2, 2, 3,-5, 0,-2, 3, 6, 5, 0, 0, 0, 0,-3, 0, 4, 2, 0, 3, 1, 5, 1, 0, 0, 0, 1, 0, 2, 4	},""),
		new TalentEntry(CharacterTalents.TA_Zwergennase,		"KL,CH,GE", new int[]{ 0, 0,-2, 0, 0, 0, 1,-3, 6, 0, 1,-1,-2,-4, 1, 1, 1, 1,-1,-1,-1, 0, 1,-2,-1, 2,-1, 2, 0	},""),
	};

	static final ZauberEntry[]	ZauberTabelle = new ZauberEntry[] {
		/*                                                                                                 Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_BeherrschungenBrechen,	CharacterType.TypDruide,	new int[]{ -2,  0, -1, -2, -4,-10,  2},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_BewegungenStoeren,		CharacterType.TypMagier,	new int[]{ -4, -5, -5, -4, -8, -5,-10},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_DestructiboArcanitas,		CharacterType.TypMagier,	new int[]{ -6,-12,-14,-16,-10,-16,-10},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_GardianumParadei,			CharacterType.TypMagier,	new int[]{ -1,-10,-10,-11, -9,-10, -8},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_HellsichtTrueben,			CharacterType.TypDruide,	new int[]{ -3, -3, -6, -6, -2,  0,  2},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_IllusionenZerstoeren,		CharacterType.TypMagier,	new int[]{ -5, -7, -8, -9, -7, -6, -9},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_KampfzauberStoeren,		CharacterType.TypAuelf,		new int[]{ -1,  1, -1, -3, -8,-15, -9},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_VerstaendigungStoeren,	CharacterType.TypFirnelf,	new int[]{ -4, -2, -2,  0, -4, -6, -1},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_VerwandlungenBeenden,		CharacterType.TypHexe,		new int[]{ -8,-10,-11,-12, -5,-15,-12},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_BandUndFessel,			CharacterType.TypWaldelf,	new int[]{ -6, -4,  0, -2, -6, -8, -6},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_Bannbaladin,				CharacterType.TypAuelf,		new int[]{  0,  4,  2,  0,-10, -8,-10},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_BoeserBlick,				CharacterType.TypDruide,	new int[]{ -6,-10,-10,-10, -6,-15,  3},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_EigneAengste,				CharacterType.TypMagier,	new int[]{-15,-20,-20,-20,-20,-20,-15},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_ErinnerungVerlasse,		CharacterType.TypMagier,	new int[]{ -8,-11,-11,-11,-11,-11, -8},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_GrosseGier,				CharacterType.TypHexe,		new int[]{ -9, -9, -9, -9,  1, -6, -6},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_GrosseVerwirrung,			CharacterType.TypDruide,	new int[]{-12,-10,-10,-10, -8, -8,  2},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Halluzination,			CharacterType.TypDruide,	new int[]{ -5, -7, -7, -7, -5, -7,  0},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Haselbusch,				CharacterType.TypWaldelf,	new int[]{-10,  1,  5, -1, -1,-10, -4},"CH,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_HerrUeberDasTierreich,	CharacterType.TypDruide,	new int[]{-10, -5, -5, -7, -8,-10,  2},"MU,MU,CH"),
		new ZauberEntry(CharacterSpell.ZS_Horriphobus,				CharacterType.TypMagier,	new int[]{ -2,-12,-12,-12,-10,-20, -6},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Lachkrampf,				CharacterType.TypSchelm,	new int[]{-15,-12,-12,-12,-12,  3,-20},"CH,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_MagischerRaub,			CharacterType.TypDruide,	new int[]{-10,-15,-15,-15,-15,-12, -4},"MU,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_RespondamiVeritar,		CharacterType.TypMagier,	new int[]{ -2, -6, -6, -6, -6,-10, -4},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Sanftmut,					CharacterType.TypHexe,		new int[]{-10, -4, -2, -4,  2, -5, -8},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Schabernack,				CharacterType.TypSchelm,	new int[]{-12,-10,-10,-10, -8,  3,-20},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_SchwarzerSchrecken,		CharacterType.TypMagier,	new int[]{-10,-12,-12,-12,-12,-12,-10},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Somnigravis,				CharacterType.TypWaldelf,	new int[]{ -3, -1,  1,  0, -6, -8, -8},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Zwingtanz,				CharacterType.TypDruide,	new int[]{-15,-15,-15,-15,-12,-12,  1},"MU,KL,CH"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_FurorBlut,				CharacterType.TypMagier,	new int[]{ -5,-12,-15,-15,-12,-20,-10},"MU,MU,CH"),
		new ZauberEntry(CharacterSpell.ZS_GeisterAustreiben,		CharacterType.TypDruide,	new int[]{ -7, -8, -9,-10, -7, -7, -3},"MU,MU,CH"),
		new ZauberEntry(CharacterSpell.ZS_GeisterBeschwoeren,		CharacterType.TypDruide,	new int[]{ -5, -6, -7, -8, -5, -5, -2},"MU,MU,CH"),
		new ZauberEntry(CharacterSpell.ZS_HeptagonUndKroetenei,		CharacterType.TypMagier,	new int[]{ -8,-15,-20,-20,-10,-20,-12},"MU,MU,CH"),
		new ZauberEntry(CharacterSpell.ZS_KommKoboldKomm,			CharacterType.TypSchelm,	new int[]{-15,-10,-10,-10,-10,  2,-12},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Kraehenruf,				CharacterType.TypHexe,		new int[]{-12,-12,-12,-12,  2,-20, -8},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_MeisterMindererGeister,	CharacterType.TypSchelm,	new int[]{ -7,-10,-11,-12, -6,  2, -8},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_SkelettariusKryptaduft,	CharacterType.TypMagier,	new int[]{ -6,-20,-20,-20,-15,-20,-12},"MU,MU,CH"),
		new ZauberEntry(CharacterSpell.ZS_WeisseMaehn,				CharacterType.TypAuelf,		new int[]{-12, -2, -5,-10,-12,-20,-20},"KL,CH,CH"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_Axxeleratus,				CharacterType.TypWaldelf,	new int[]{ -6, -2,  2, -2, -8, -6,-10},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_DurchMarmorstein,			CharacterType.TypSchelm,	new int[]{-12,-12,-12,-12,-12,  1,-12},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_ForamenForaminor,			CharacterType.TypMagier,	new int[]{  0, -6, -7, -8, -6,  0, -8},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_Hexenholz,				CharacterType.TypHexe,		new int[]{ -5, -5, -5, -5,  2, -5, -5},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Klickeradomms,			CharacterType.TypSchelm,	new int[]{ -6, -5, -5, -5, -5,  3, -6},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_MotoricusMotilitich,		CharacterType.TypMagier,	new int[]{ -2, -6, -5, -4, -6, -3,-12},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Nackedei,					CharacterType.TypSchelm,	new int[]{-15,-15,-15,-15,-15,  2,-15},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_SpurlosTrittlos,			CharacterType.TypWaldelf,	new int[]{-10, -2, -1, -2, -5, -7, -5},"KL,GE,GE"),
		new ZauberEntry(CharacterSpell.ZS_TransversalisTeleport,	CharacterType.TypMagier,	new int[]{-10,-15,-15,-12,-18,-12,-20},"KL,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_UeberEisUndUeberSchnee,	CharacterType.TypFirnelf,	new int[]{-15, -7, -6, -2,-15,-15,-15},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Verschwindibus,			CharacterType.TypSchelm,	new int[]{ -5, -6, -7, -8,-10,  0,-12},"KL,CH,GE"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_AengsteLindern,			CharacterType.TypHexe,		new int[]{ -2, -3, -3, -3,  2,-10, -6},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Balsamsalabunde,			CharacterType.TypAuelf,		new int[]{  2,  5,  4,  2, -8, -4, -8},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Hexenspeichel,			CharacterType.TypHexe,		new int[]{-20,-20,-20,-20,  5,-20,-20},"CH,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_KlarumPurumKraeutersud,	CharacterType.TypMagier,	new int[]{ -2, -4, -4, -4, -8,-10, -6},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_RuheKoerperRuheGeist,		CharacterType.TypFirnelf,	new int[]{-10, -3, -1,  1,-10,-15,-10},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_TiereBesprechen,			CharacterType.TypHexe,		new int[]{-10, -2,  0, -2,  3,-10, -8},"MU,KL,CH"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_AdleraugUndLuchsenohr,	CharacterType.TypAuelf,		new int[]{ -7,  2,  1,  1, -4, -7, -4},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_AnaluesArcanstruktur,		CharacterType.TypMagier,	new int[]{ -4,-10,-10,-10,-10,-15,-10},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_EigenschaftenLesen,		CharacterType.TypAuelf,		new int[]{ -4,  2,  0, -2, -4, -7,-10},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_ExposamiCreatur,			CharacterType.TypWaldelf,	new int[]{-10,  1,  3,  2,  0, -7, -4},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_InDeinTrachten,			CharacterType.TypFirnelf,	new int[]{ -1, -1, -2,  0,-12,-12,-12},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_OdemArcanumSenserei,		CharacterType.TypAuelf,		new int[]{  3,  3,  1, -1, -1, -5, -3},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Penetrizzel,				CharacterType.TypMagier,	new int[]{  0, -2, -4, -2, -8, -2, -8},"KL,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_Sensibar,					CharacterType.TypAuelf,		new int[]{  0,  2, -1,  1, -5, -4, -8},"KL,CH,CH"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_AurisNasusOculus,			CharacterType.TypMagier,	new int[]{ -5,-12,-13,-14,-13, -7,-13},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_Blendwerk,				CharacterType.TypSchelm,	new int[]{ -8,-12,-13,-14,-13, -5,-14},"CH,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_ChamaelioniMimikry,		CharacterType.TypWaldelf,	new int[]{-10, -1,  2,  0, -7, -5, -7},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_DuplicatusDoppelpein,		CharacterType.TypMagier,	new int[]{  0, -8, -8, -8, -6, -2, -6},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_HarmloseGestalt,			CharacterType.TypHexe,		new int[]{ -6, -6, -7, -8,  2, -3, -6},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_Hexenknoten,				CharacterType.TypHexe,		new int[]{-15,-15,-15,-15,  0,-12,-10},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Koboldgeschenk,			CharacterType.TypSchelm,	new int[]{ -7, -7, -7, -7, -7,  3,-10},"CH,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_Schelmenmaske,			CharacterType.TypSchelm,	new int[]{-17,-18,-19,-20,-12, -3,-16},"CH,CH,GE"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_BlitzDichFind,			CharacterType.TypWaldelf,	new int[]{  2,  4,  5,  4,  0,  0,  1},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_Ecliptifactus,			CharacterType.TypMagier,	new int[]{ -6,-10,-11,-12,-12,-20, -8},"MU,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_Eisenrost,				CharacterType.TypMagier,	new int[]{ -2, -4, -5, -6, -8, -3, -3},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_FulminictusDonnerkeil,	CharacterType.TypAuelf,		new int[]{  0,  4,  2,  0,-10,-20, -8},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Hoellenpein,				CharacterType.TypMagier,	new int[]{ -8,-11,-11,-11,-11,-11, -8},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_Ignifaxius,				CharacterType.TypMagier,	new int[]{ -3,-10,-10,-10,-10,-20,-12},"KL,GE,GE"),
		new ZauberEntry(CharacterSpell.ZS_Plumbumbarum,				CharacterType.TypMagier,	new int[]{ -1, -7, -7, -7,-10, -4, -7},"CH,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Radau,					CharacterType.TypHexe,		new int[]{-20,-20,-20,-20,  2,-20,-20},"MU,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_SaftKraftMonstermacht,	CharacterType.TypMagier,	new int[]{ -3, -8, -7, -6,-15,-20,-10},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_ScharfesAug,				CharacterType.TypWaldelf,	new int[]{ -4, -2,  0, -1, -8,-15, -8},"KL,KL,GE"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_DasSinnenFremderWesen,	CharacterType.TypWaldelf,	new int[]{ -5, -1,  1, -2,  0, -8,  0},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_ElfenFreundeHoert,		CharacterType.TypFirnelf,	new int[]{-10,  1,  3,  5,-10,-10,-10},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Hexenblick,				CharacterType.TypHexe,		new int[]{-20,-20,-20,-20,  5,-20,-20},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Nekropathia,				CharacterType.TypMagier,	new int[]{ -6,-10, -9, -8, -8,-15, -6},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_ObjectumStumm,			CharacterType.TypMagier,	new int[]{ -7, -8, -9,-10,-12, -9, -8},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_UnitatioGeistesbund,		CharacterType.TypFirnelf,	new int[]{  0, -2, -2,  3,  0,-20,-10},"KL,CH,KK"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_AdlerWolf,				CharacterType.TypWaldelf,	new int[]{-10, -6, -4, -5, -7,-15, -8},"MU,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_ArcanoPsychostabilis,		CharacterType.TypMagier,	new int[]{ -2, -4, -4, -4, -3, -8, -8},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Armatrutz,				CharacterType.TypAuelf,		new int[]{  0,  4,  3,  2, -8,-12, -8},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Druidenrache,				CharacterType.TypDruide,	new int[]{-20,-20,-20,-20,-20,-20,  3},"MU,MU,KK"),
		new ZauberEntry(CharacterSpell.ZS_Erleuchten,				CharacterType.TypMagier,	new int[]{  0, -1, -1, -1, -1, -8, -3},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Ermutigen,				CharacterType.TypFirnelf,	new int[]{ -1,  0,  0,  1, -3, -8, -1},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_Ertuechtigen,				CharacterType.TypMagier,	new int[]{  0,  0,  0,  0, -3, -8, -3},"KL,CH,GE"),
		new ZauberEntry(CharacterSpell.ZS_Feuerbann,				CharacterType.TypHexe,		new int[]{-10, -8, -8,-12,  2, -8, -6},"MU,MU,KK"),
		new ZauberEntry(CharacterSpell.ZS_FluchDerPestilenz,		CharacterType.TypDruide,	new int[]{-12,-15,-15,-15,-10,-20, -2},"MU,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_InSeeUndFluss,			CharacterType.TypAuelf,		new int[]{-15, -2, -4, -4,-10,-15,-10},"MU,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_LangerLulatsch,			CharacterType.TypSchelm,	new int[]{-15,-15,-15,-15,-15,  0,-15},"CH,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_ParalueParalein,			CharacterType.TypMagier,	new int[]{ -5,-15,-15,-15,-15,-20,-15},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_SalanderMutanderer,		CharacterType.TypMagier,	new int[]{-10,-15,-15,-15,-12,-15,-15},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Schmuecken,				CharacterType.TypMagier,	new int[]{  0, -1, -1, -1,  0, -8, -5},"KL,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_SchwarzUndRot,			CharacterType.TypMagier,	new int[]{ -7,-10,-11,-12, -9,-15, -8},"MU,CH,CH"),
		new ZauberEntry(CharacterSpell.ZS_Staerken,					CharacterType.TypFirnelf,	new int[]{ -1, -1, -1,  0, -3, -8, -3},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_VisibiliVanitar,			CharacterType.TypAuelf,		new int[]{ -4,  2,  1,  1,-10, -5,-12},"KL,KL,GE"),
											  		/*Ma  AE  WE  FE  He  Sc  Dr*/
		new ZauberEntry(CharacterSpell.ZS_Abvenenum,				CharacterType.TypWaldelf,	new int[]{ -2, -2,  0, -4, -4, -8, -6},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_AeolitusWindgebraus,		CharacterType.TypFirnelf,	new int[]{  0,  1,  1,  3, -4, -6, -4},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_Arcanovi,					CharacterType.TypMagier,	new int[]{ -3, -8, -9,-10, -6, -8, -8},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_BrenneToterStoff,			CharacterType.TypMagier,	new int[]{ -7,-10,-10,-10,-10,-10, -8},"MU,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_ClaudibusClavistibor,		CharacterType.TypMagier,	new int[]{  0, -5, -6, -7, -8, -4, -8},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Dunkelheit,				CharacterType.TypDruide,	new int[]{ -3, -5, -6, -7, -5, -5, -1},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_FlimFlamFunkel,			CharacterType.TypFirnelf,	new int[]{  3,  5,  5,  6,  0,  0,  0},"KL,KL,GE"),
		new ZauberEntry(CharacterSpell.ZS_HartesSchmelze,			CharacterType.TypMagier,	new int[]{ -6,-10,-10,-10,-10,-10, -8},"MU,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_MetamorphoGletscherkalt,	CharacterType.TypFirnelf,	new int[]{-20,-10,-10,  0,-20,-20,-20},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_NihilatioGravitas,		CharacterType.TypMagier,	new int[]{-10,-13,-14,-12,-15,-12,-15},"KL,KK,KK"),
		new ZauberEntry(CharacterSpell.ZS_ObjectumFix,				CharacterType.TypMagier,	new int[]{ -6, -9, -7, -9, -8,-10,-10},"KL,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_ReversalisRevidum,		CharacterType.TypMagier,	new int[]{ -8,-15,-15,-15,-15,-20,-12},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_SilentiumSilentille,		CharacterType.TypAuelf,		new int[]{ -6,  0, -2, -4, -6,-10, -6},"KL,KL,CH"),
		new ZauberEntry(CharacterSpell.ZS_SolidiridFarbenspiel,		CharacterType.TypFirnelf,	new int[]{-10, -8, -8, -4,-10,-15,-12},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_Sturmgebruell,			CharacterType.TypFirnelf,	new int[]{-15, -6, -6, -2,-12,-20,-15},"KL,CH,KK"),
		new ZauberEntry(CharacterSpell.ZS_WeheWalleNebula,			CharacterType.TypWaldelf,	new int[]{-10, -2,  0, -3,-10,-15,-10},"KL,GE,KK"),
		new ZauberEntry(CharacterSpell.ZS_WeichesErstarre,			CharacterType.TypMagier,	new int[]{ -6,-10,-10,-10,-10,-10, -8},"MU,KL,KK"),
		new ZauberEntry(CharacterSpell.ZS_ZagibuUbigaz,				CharacterType.TypSchelm,	new int[]{ -6, -7, -8, -9, -7,  0, -3},"KL,CH,GE"),
	};

	static class CharacterSpellValue {
		public final CharacterSpell spell;
		public final int value;
		public CharacterSpellValue(final CharacterSpell spell, final int value) {
			this.spell = spell;
			this.value = value;
		}
	}
	
	static final AkademieEntry[]	Akademie = new AkademieEntry[] {
		new AkademieEntry(
			"Schule der Austreibung zu Perricum",
			CharacterSpellGroup.ZG_Antimagie,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_IllusionenZerstoeren, 2),
					new CharacterSpellValue(CharacterSpell.ZS_BeherrschungenBrechen, 2),
					new CharacterSpellValue(CharacterSpell.ZS_VerwandlungenBeenden,	2),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben, 2),
					new CharacterSpellValue(CharacterSpell.ZS_DestructiboArcanitas,	2),
					new CharacterSpellValue(CharacterSpell.ZS_HellsichtTrueben, 2),
					new CharacterSpellValue(CharacterSpell.ZS_ReversalisRevidum, 2)
			}),
		new AkademieEntry(
			"Halle der Antimagie zu Kuslik",
			CharacterSpellGroup.ZG_Antimagie,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_HellsichtTrueben,		2),
					new CharacterSpellValue(CharacterSpell.ZS_BewegungenStoeren,		2),
					new CharacterSpellValue(CharacterSpell.ZS_VerstaendigungStoeren,	2),
					new CharacterSpellValue(CharacterSpell.ZS_GardianumParadei,		2),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben,		2),
					new CharacterSpellValue(CharacterSpell.ZS_ArcanoPsychostabilis,	2),
					new CharacterSpellValue(CharacterSpell.ZS_AengsteLindern,			2),
			}),
		new AkademieEntry(
			"Akademie der magischen Rüstung zu Gareth",
			CharacterSpellGroup.ZG_Antimagie,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Armatrutz,				2),
					new CharacterSpellValue(CharacterSpell.ZS_VisibiliVanitar,			2),
					new CharacterSpellValue(CharacterSpell.ZS_GardianumParadei,		2),
					new CharacterSpellValue(CharacterSpell.ZS_KampfzauberStoeren,		2),
					new CharacterSpellValue(CharacterSpell.ZS_IllusionenZerstoeren,	2),
					new CharacterSpellValue(CharacterSpell.ZS_ArcanoPsychostabilis,	2),
					new CharacterSpellValue(CharacterSpell.ZS_BlitzDichFind,			2),
			}),
		new AkademieEntry(
			"Bannakademie zu Ysilia",
			CharacterSpellGroup.ZG_Antimagie,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_IllusionenZerstoeren,	2),
					new CharacterSpellValue(CharacterSpell.ZS_VerwandlungenBeenden,	2),
					new CharacterSpellValue(CharacterSpell.ZS_DestructiboArcanitas,	2),
					new CharacterSpellValue(CharacterSpell.ZS_BeherrschungenBrechen,	2),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben,		2),
					new CharacterSpellValue(CharacterSpell.ZS_HellsichtTrueben,		1),
					new CharacterSpellValue(CharacterSpell.ZS_KampfzauberStoeren,		1),
					new CharacterSpellValue(CharacterSpell.ZS_ArcanoPsychostabilis,	1),
					new CharacterSpellValue(CharacterSpell.ZS_GardianumParadei,		1),
			}),
		new AkademieEntry(
			"Schule des seienden Scheins zu Zorgan",
			CharacterSpellGroup.ZG_Illusion,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_DuplicatusDoppelpein,	3),
					new CharacterSpellValue(CharacterSpell.ZS_AurisNasusOculus,		3),
					new CharacterSpellValue(CharacterSpell.ZS_Schelmenmaske,			3),
					new CharacterSpellValue(CharacterSpell.ZS_Dunkelheit,				1),
					new CharacterSpellValue(CharacterSpell.ZS_VisibiliVanitar,			1),
					new CharacterSpellValue(CharacterSpell.ZS_HarmloseGestalt,			1),
					new CharacterSpellValue(CharacterSpell.ZS_IllusionenZerstoeren,	1),
					new CharacterSpellValue(CharacterSpell.ZS_AdleraugUndLuchsenohr,	1),
			}),
		new AkademieEntry(
			"Akademie der Erscheinungen zu Grangor",
			CharacterSpellGroup.ZG_Illusion,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_AurisNasusOculus,		3),
					new CharacterSpellValue(CharacterSpell.ZS_IllusionenZerstoeren,	3),
					new CharacterSpellValue(CharacterSpell.ZS_DuplicatusDoppelpein,	2),
					new CharacterSpellValue(CharacterSpell.ZS_Schelmenmaske,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Ecliptifactus,			2),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterBeschwoeren,		2),
			}),
		new AkademieEntry(
			"Akademie Schwert und Stab zu Beilunk",
			CharacterSpellGroup.ZG_Kampfzauber,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_BlitzDichFind,			2),
					new CharacterSpellValue(CharacterSpell.ZS_FulminictusDonnerkeil,	2),
					new CharacterSpellValue(CharacterSpell.ZS_Plumbumbarum,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Ignifaxius,				2),
					new CharacterSpellValue(CharacterSpell.ZS_ScharfesAug,				2),
					new CharacterSpellValue(CharacterSpell.ZS_Armatrutz,				2),
					new CharacterSpellValue(CharacterSpell.ZS_Staerken,				2),
			}),
		new AkademieEntry(
			"Halle des vollendeten Kampfes zu Bethana",
			CharacterSpellGroup.ZG_Kampfzauber,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Ignifaxius,				2),
					new CharacterSpellValue(CharacterSpell.ZS_FulminictusDonnerkeil,	2),
					new CharacterSpellValue(CharacterSpell.ZS_BlitzDichFind,			2),
					new CharacterSpellValue(CharacterSpell.ZS_SaftKraftMonstermacht,	2),
					new CharacterSpellValue(CharacterSpell.ZS_Armatrutz,				2),
					new CharacterSpellValue(CharacterSpell.ZS_UnitatioGeistesbund,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Balsamsalabunde,			2),
			}),
		new AkademieEntry(
			"Kampfseminar Andergast",
			CharacterSpellGroup.ZG_Kampfzauber,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Eisenrost,				3),
					new CharacterSpellValue(CharacterSpell.ZS_Ecliptifactus,			3),
					new CharacterSpellValue(CharacterSpell.ZS_Plumbumbarum,			1),
					new CharacterSpellValue(CharacterSpell.ZS_ScharfesAug,				1),
					new CharacterSpellValue(CharacterSpell.ZS_FulminictusDonnerkeil,	1),
					new CharacterSpellValue(CharacterSpell.ZS_BlitzDichFind,			1),
					new CharacterSpellValue(CharacterSpell.ZS_SilentiumSilentille,		1),
					new CharacterSpellValue(CharacterSpell.ZS_KampfzauberStoeren,		1),
					new CharacterSpellValue(CharacterSpell.ZS_Ermutigen,				1),
					new CharacterSpellValue(CharacterSpell.ZS_FlimFlamFunkel,			1),
			}),
		new AkademieEntry(
			"Akademie der Verformungen zu Lowangen",
			CharacterSpellGroup.ZG_VerwandlungLebewesen,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_VisibiliVanitar,			2),
					new CharacterSpellValue(CharacterSpell.ZS_ParalueParalein,			2),
					new CharacterSpellValue(CharacterSpell.ZS_SalanderMutanderer,		2),
					new CharacterSpellValue(CharacterSpell.ZS_AdlerWolf,				2),
					new CharacterSpellValue(CharacterSpell.ZS_InSeeUndFluss,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Ertuechtigen,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Armatrutz,				2),
			}),
		new AkademieEntry(
			"Halle der Metamorphosen zu Kuslik",
			CharacterSpellGroup.ZG_VerwandlungLebewesen,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_SalanderMutanderer,		4),
					new CharacterSpellValue(CharacterSpell.ZS_ArcanoPsychostabilis,	3),
					new CharacterSpellValue(CharacterSpell.ZS_AdlerWolf,				3),
					new CharacterSpellValue(CharacterSpell.ZS_VisibiliVanitar,			1),
					new CharacterSpellValue(CharacterSpell.ZS_ParalueParalein,			1),
					new CharacterSpellValue(CharacterSpell.ZS_Erleuchten,				1),
					new CharacterSpellValue(CharacterSpell.ZS_Schmuecken,				1),
			}),
		new AkademieEntry(
			"Schule des Wandelbaren zu Tuzak",
			CharacterSpellGroup.ZG_VerwandlungLebewesen,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_AdlerWolf,				6),
					new CharacterSpellValue(CharacterSpell.ZS_SalanderMutanderer,		2),
					new CharacterSpellValue(CharacterSpell.ZS_ParalueParalein,			2),
					new CharacterSpellValue(CharacterSpell.ZS_SchwarzUndRot,			1),
					new CharacterSpellValue(CharacterSpell.ZS_ArcanoPsychostabilis,	1),
					new CharacterSpellValue(CharacterSpell.ZS_Ermutigen,				1),
					new CharacterSpellValue(CharacterSpell.ZS_Ecliptifactus,			1),
			}),
		new AkademieEntry(
			"Pentagramm-Akademie zu Rashdul",
			CharacterSpellGroup.ZG_Beschwoerung,		CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_FurorBlut,				3),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterBeschwoeren,		3),
					new CharacterSpellValue(CharacterSpell.ZS_HeptagonUndKroetenei,	3),
					new CharacterSpellValue(CharacterSpell.ZS_Nekropathia,				2),
					new CharacterSpellValue(CharacterSpell.ZS_SkelettariusKryptaduft,	2),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben,		1),
			}),
		new AkademieEntry(
			"Dunkle Halle der Geister zu Brabak",
			CharacterSpellGroup.ZG_Beschwoerung,		CharacterSpellColor.Schwarz,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_GeisterBeschwoeren,		3),
					new CharacterSpellValue(CharacterSpell.ZS_SkelettariusKryptaduft,	3),
					new CharacterSpellValue(CharacterSpell.ZS_Nekropathia,				3),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben,		1),
					new CharacterSpellValue(CharacterSpell.ZS_FurorBlut,				1),
					new CharacterSpellValue(CharacterSpell.ZS_HeptagonUndKroetenei,	1),
					new CharacterSpellValue(CharacterSpell.ZS_Ecliptifactus,			1),
					new CharacterSpellValue(CharacterSpell.ZS_UnitatioGeistesbund,		1),
			}),
		new AkademieEntry(
			"Schule des Direkten Weges zu Gerasim",
			CharacterSpellGroup.ZG_Bewegung,		CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_ForamenForaminor,		2),
					new CharacterSpellValue(CharacterSpell.ZS_ClaudibusClavistibor,	2),
					new CharacterSpellValue(CharacterSpell.ZS_MotoricusMotilitich,		2),
					new CharacterSpellValue(CharacterSpell.ZS_TransversalisTeleport,	2),
					new CharacterSpellValue(CharacterSpell.ZS_SpurlosTrittlos,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Axxeleratus,				2),
					new CharacterSpellValue(CharacterSpell.ZS_SolidiridFarbenspiel,	2),
			}),
		new AkademieEntry(
			"Akademie der Geistreisen zu Belhanka",
			CharacterSpellGroup.ZG_Bewegung,		CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_TransversalisTeleport,	4),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumStumm,			3),
					new CharacterSpellValue(CharacterSpell.ZS_DurchMarmorstein,		3),
					new CharacterSpellValue(CharacterSpell.ZS_ElfenFreundeHoert,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Verschwindibus,			2),
			}),
		new AkademieEntry(
			"Halle des Windes zu Olport",
			CharacterSpellGroup.ZG_VerwandlungUnbelebtes,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_AeolitusWindgebraus,		4),
					new CharacterSpellValue(CharacterSpell.ZS_Sturmgebruell,			4),
					new CharacterSpellValue(CharacterSpell.ZS_WeheWalleNebula,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Dunkelheit,				2),
					new CharacterSpellValue(CharacterSpell.ZS_SolidiridFarbenspiel,	2),
			}),
		new AkademieEntry(
			"Akademie von Licht und Dunkelheit zu Nostria",
			CharacterSpellGroup.ZG_VerwandlungUnbelebtes,	CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_FlimFlamFunkel,			3),
					new CharacterSpellValue(CharacterSpell.ZS_Ecliptifactus,			3),
					new CharacterSpellValue(CharacterSpell.ZS_WeheWalleNebula,			2),
					new CharacterSpellValue(CharacterSpell.ZS_SilentiumSilentille,		2),
					new CharacterSpellValue(CharacterSpell.ZS_NihilatioGravitas,		2),
					new CharacterSpellValue(CharacterSpell.ZS_ClaudibusClavistibor,	1),
					new CharacterSpellValue(CharacterSpell.ZS_ZagibuUbigaz,			1),
			}),
		new AkademieEntry(
			"Schule der variablen Form zu Mirham",
			CharacterSpellGroup.ZG_VerwandlungUnbelebtes,	CharacterSpellColor.Schwarz,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Arcanovi,				2),
					new CharacterSpellValue(CharacterSpell.ZS_HartesSchmelze,			2),
					new CharacterSpellValue(CharacterSpell.ZS_WeichesErstarre,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Abvenenum,				2),
					new CharacterSpellValue(CharacterSpell.ZS_ZagibuUbigaz,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Ecliptifactus,			2),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumFix,				2),
			}),
		new AkademieEntry(
			"Akademie \257Drachenei\256 zu Khunchom",
			CharacterSpellGroup.ZG_VerwandlungUnbelebtes,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Arcanovi,				4),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumFix,				4),
					new CharacterSpellValue(CharacterSpell.ZS_Abvenenum,				1),
					new CharacterSpellValue(CharacterSpell.ZS_MotoricusMotilitich,		1),
					new CharacterSpellValue(CharacterSpell.ZS_ClaudibusClavistibor,	1),
					new CharacterSpellValue(CharacterSpell.ZS_ForamenForaminor,		1),
					new CharacterSpellValue(CharacterSpell.ZS_FlimFlamFunkel,			1),
					new CharacterSpellValue(CharacterSpell.ZS_DestructiboArcanitas,	1),
			}),
		new AkademieEntry(
			"Halle des Quecksilbers zu Festum",
			CharacterSpellGroup.ZG_VerwandlungUnbelebtes,	CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_SilentiumSilentille,		2),
					new CharacterSpellValue(CharacterSpell.ZS_NihilatioGravitas,		2),
					new CharacterSpellValue(CharacterSpell.ZS_FlimFlamFunkel,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Dunkelheit,				2),
					new CharacterSpellValue(CharacterSpell.ZS_AeolitusWindgebraus,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Abvenenum,				1),
					new CharacterSpellValue(CharacterSpell.ZS_MetamorphoGletscherkalt,	1),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumFix,				1),
					new CharacterSpellValue(CharacterSpell.ZS_ClaudibusClavistibor,	1),
			}),
		new AkademieEntry(
			"Informations-Institut zu Rommilys",
			CharacterSpellGroup.ZG_Hellsicht,				CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_InDeinTrachten,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Penetrizzel,				2),
					new CharacterSpellValue(CharacterSpell.ZS_AdleraugUndLuchsenohr,	2),
					new CharacterSpellValue(CharacterSpell.ZS_EigenschaftenLesen,		2),
					new CharacterSpellValue(CharacterSpell.ZS_OdemArcanumSenserei,		1),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				1),
					new CharacterSpellValue(CharacterSpell.ZS_AnaluesArcanstruktur,	1),
					new CharacterSpellValue(CharacterSpell.ZS_RespondamiVeritar,		1),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumStumm,			1),
					new CharacterSpellValue(CharacterSpell.ZS_Bannbaladin,				1),
			}),
		new AkademieEntry(
			"Schule der Hellsicht zu Thorwal",
			CharacterSpellGroup.ZG_Hellsicht,				CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Penetrizzel,				2),
					new CharacterSpellValue(CharacterSpell.ZS_ExposamiCreatur,			2),
					new CharacterSpellValue(CharacterSpell.ZS_InDeinTrachten,			2),
					new CharacterSpellValue(CharacterSpell.ZS_AdleraugUndLuchsenohr,	2),
					new CharacterSpellValue(CharacterSpell.ZS_AnaluesArcanstruktur,	2),
					new CharacterSpellValue(CharacterSpell.ZS_EigenschaftenLesen,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				1),
					new CharacterSpellValue(CharacterSpell.ZS_OdemArcanumSenserei,		1),
			}),
		new AkademieEntry(
			"Akademie des Magischen Wissens zu Methumis",
			CharacterSpellGroup.ZG_Hellsicht,				CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_OdemArcanumSenserei,		4),
					new CharacterSpellValue(CharacterSpell.ZS_AnaluesArcanstruktur,	4),
					new CharacterSpellValue(CharacterSpell.ZS_Penetrizzel,				2),
					new CharacterSpellValue(CharacterSpell.ZS_AdleraugUndLuchsenohr,	2),
					new CharacterSpellValue(CharacterSpell.ZS_EigenschaftenLesen,		1),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				1),
			}),
		new AkademieEntry(
			"Akademie der Geistigen Kraft zu Fasar",
			CharacterSpellGroup.ZG_Beherrschung,			CharacterSpellColor.Schwarz,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Bannbaladin,				2),
					new CharacterSpellValue(CharacterSpell.ZS_Horriphobus,				2),
					new CharacterSpellValue(CharacterSpell.ZS_Halluzination,			2),
					new CharacterSpellValue(CharacterSpell.ZS_RespondamiVeritar,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Somnigravis,				2),
					new CharacterSpellValue(CharacterSpell.ZS_BeherrschungenBrechen,	2),
					new CharacterSpellValue(CharacterSpell.ZS_BandUndFessel,			1),
					new CharacterSpellValue(CharacterSpell.ZS_MagischerRaub,			1),
			}),
		new AkademieEntry(
			"Halle der Macht zu Lowangen",
			CharacterSpellGroup.ZG_Beherrschung,			CharacterSpellColor.Schwarz,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Bannbaladin,				3),
					new CharacterSpellValue(CharacterSpell.ZS_Horriphobus,				3),
					new CharacterSpellValue(CharacterSpell.ZS_RespondamiVeritar,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Somnigravis,				2),
					new CharacterSpellValue(CharacterSpell.ZS_BandUndFessel,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				2),
			}),
		new AkademieEntry(
			"Akademie der Herrschaft zu Elenvina",
			CharacterSpellGroup.ZG_Beherrschung,			CharacterSpellColor.Weiss,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Bannbaladin,				4),
					new CharacterSpellValue(CharacterSpell.ZS_RespondamiVeritar,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				2),
					new CharacterSpellValue(CharacterSpell.ZS_InDeinTrachten,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Somnigravis,				2),
					new CharacterSpellValue(CharacterSpell.ZS_Horriphobus,				1),
					new CharacterSpellValue(CharacterSpell.ZS_BandUndFessel,			1),
			}),
		new AkademieEntry(
			"Schule der Beherrschung zu Neersand",
			CharacterSpellGroup.ZG_Beherrschung,			CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Bannbaladin,				5),
					new CharacterSpellValue(CharacterSpell.ZS_Horriphobus,				2),
					new CharacterSpellValue(CharacterSpell.ZS_Halluzination,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Somnigravis,				2),
					new CharacterSpellValue(CharacterSpell.ZS_BeherrschungenBrechen,	2),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				1),
			}),
		new AkademieEntry(
			"Halle des Lebens zu Norburg",
			CharacterSpellGroup.ZG_Heilung,					CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Balsamsalabunde,			4),
					new CharacterSpellValue(CharacterSpell.ZS_KlarumPurumKraeutersud,	4),
					new CharacterSpellValue(CharacterSpell.ZS_RuheKoerperRuheGeist,	4),
					new CharacterSpellValue(CharacterSpell.ZS_AengsteLindern,			3),
					new CharacterSpellValue(CharacterSpell.ZS_EigenschaftenLesen,		3),
					new CharacterSpellValue(CharacterSpell.ZS_TiereBesprechen,			1),
					new CharacterSpellValue(CharacterSpell.ZS_Abvenenum,				1),
					new CharacterSpellValue(CharacterSpell.ZS_Hexenspeichel,			1),
			}),
		new AkademieEntry(
			"Anatomische Akademie zu Vinsalt",
			CharacterSpellGroup.ZG_Heilung,					CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Balsamsalabunde,			3),
					new CharacterSpellValue(CharacterSpell.ZS_KlarumPurumKraeutersud,	3),
					new CharacterSpellValue(CharacterSpell.ZS_RuheKoerperRuheGeist,	3),
					new CharacterSpellValue(CharacterSpell.ZS_AengsteLindern,			3),
					new CharacterSpellValue(CharacterSpell.ZS_EigenschaftenLesen,		3),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben,		2),
					new CharacterSpellValue(CharacterSpell.ZS_OdemArcanumSenserei,		2),
					new CharacterSpellValue(CharacterSpell.ZS_Staerken,				1),
					new CharacterSpellValue(CharacterSpell.ZS_Ertuechtigen,			1),
			}),
		new AkademieEntry(
			"Seminar der elfischen Verständigung zu Donnerbach",
			CharacterSpellGroup.ZG_Verstaendigung,			CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_ElfenFreundeHoert,		3),
					new CharacterSpellValue(CharacterSpell.ZS_UnitatioGeistesbund,		2),
					new CharacterSpellValue(CharacterSpell.ZS_InDeinTrachten,			2),
					new CharacterSpellValue(CharacterSpell.ZS_DasSinnenFremderWesen,	2),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				2),
					new CharacterSpellValue(CharacterSpell.ZS_RuheKoerperRuheGeist,	1),
					new CharacterSpellValue(CharacterSpell.ZS_Abvenenum,				1),
					new CharacterSpellValue(CharacterSpell.ZS_Balsamsalabunde,			1),
			}),
		new AkademieEntry(
			"Seminar der natürlichen Heilung zu Donnerbach",
			CharacterSpellGroup.ZG_Heilung,					CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Balsamsalabunde,			3),
					new CharacterSpellValue(CharacterSpell.ZS_RuheKoerperRuheGeist,	3),
					new CharacterSpellValue(CharacterSpell.ZS_Abvenenum,				2),
					new CharacterSpellValue(CharacterSpell.ZS_ElfenFreundeHoert,		2),
					new CharacterSpellValue(CharacterSpell.ZS_UnitatioGeistesbund,		1),
					new CharacterSpellValue(CharacterSpell.ZS_DasSinnenFremderWesen,	1),
					new CharacterSpellValue(CharacterSpell.ZS_InDeinTrachten,			1),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				1),
			}),
		new AkademieEntry(
			"Kreis der Einfühlung in den nördlichen Salamanderstein",
			CharacterSpellGroup.ZG_Verstaendigung,			CharacterSpellColor.Grau,
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_ElfenFreundeHoert,		5),
					new CharacterSpellValue(CharacterSpell.ZS_UnitatioGeistesbund,		4),
					new CharacterSpellValue(CharacterSpell.ZS_Sensibar,				4),
					new CharacterSpellValue(CharacterSpell.ZS_InDeinTrachten,			3),
					new CharacterSpellValue(CharacterSpell.ZS_DasSinnenFremderWesen,	3),
					new CharacterSpellValue(CharacterSpell.ZS_ExposamiCreatur,			2),
			}),
		new AkademieEntry(
			"Academia der Hohen Magie zu Punin",
			null, CharacterSpellColor.Grau,	/* Kein Spezialgebiet!!	*/
			new CharacterSpellValue[] {
					new CharacterSpellValue(CharacterSpell.ZS_Arcanovi,				4),
					new CharacterSpellValue(CharacterSpell.ZS_ReversalisRevidum,		4),
					new CharacterSpellValue(CharacterSpell.ZS_UnitatioGeistesbund,		3),
					new CharacterSpellValue(CharacterSpell.ZS_HeptagonUndKroetenei,	2),
					new CharacterSpellValue(CharacterSpell.ZS_OdemArcanumSenserei,		2),
					new CharacterSpellValue(CharacterSpell.ZS_AnaluesArcanstruktur,	2),
					new CharacterSpellValue(CharacterSpell.ZS_DestructiboArcanitas,	2),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumFix,				2),
					new CharacterSpellValue(CharacterSpell.ZS_ObjectumStumm,			2),
					new CharacterSpellValue(CharacterSpell.ZS_Penetrizzel,				2),
					new CharacterSpellValue(CharacterSpell.ZS_GeisterAustreiben,		1),
					new CharacterSpellValue(CharacterSpell.ZS_GardianumParadei,		1),
					new CharacterSpellValue(CharacterSpell.ZS_BandUndFessel,			1),
			}),
	};
}

