package de.ped.dsatool.logic;

/**
 * The concept of character types.
 *
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum CharacterType {
	TypGaukler	("Gaukler", "Gauklerin", 0, Category.Normal, false),
	TypHaendler	("Händler", "Händlerin", 0, Category.Normal, false),
	TypKrieger	("Krieger", "Kriegerin", 0, Category.Normal, false),
	TypMoha		("Moha", "Moha", 0, Category.Normal, true),
	TypNivese	("Nivese", "Nivesin", 0, Category.Normal, true),
	TypNovadi	("Novadi", "Novadin", 0, Category.Normal, true),
	TypStreuner	("Streuner", "Streunerin", 0, Category.Normal, false),
	TypThorwaler("Thowaler", "Thowalerin", 0, Category.Normal, true),
	TypZwerg	("Zwerg", "Zwergin", 0, Category.Normal, true),
	TypBuerger	("Bürger", "Bürgerin", 0, Category.Normal, true),

	TypMagier	("Magier", "Magierin", 0, Category.Magisch, false),
	TypAuelf	("Auelf", "Auelfin", 0, Category.Magisch, true),
	TypWaldelf	("Waldelf", "Waldelfin", 0, Category.Magisch, true),
	TypFirnelf	("Firnelf", "Firnelfin", 0, Category.Magisch, true),
	TypHexe		("Hexe", "Hexer", 1, Category.Magisch, true),
	TypSchelm	("Schelm", "Schelmin", 0, Category.Magisch, true),
	TypDruide	("Druide", "Druidin", 0, Category.Magisch, false),

	TypGewPra	("Praios-Geweihter", "Praios-Geweihte", 0, Category.Geweiht, false),
	TypGewRon	("Rondra-Geweihter", "Rondra-Geweihte", 0, Category.Geweiht, false),
	TypGewEff	("Efferd-Geweihter", "Efferd-Geweihte", 0, Category.Geweiht, false),
	TypGewTra	("Travia-Geweihter", "Travia-Geweihte", 0, Category.Geweiht, false),
	TypGewBor	("Boron-Geweihter", "Boron-Geweihte", 0, Category.Geweiht, false),
	TypGewHes	("Hesinde-Geweihter", "Hesinde-Geweihte", 0, Category.Geweiht, false),
	TypGewFir	("Firun-Geweihter", "Firun-Geweihte", 0, Category.Geweiht, false),
	TypGewTsa	("Tsa-Geweihter", "Tsa-Geweihte", 0, Category.Geweiht, false),
	TypGewPhe	("Phex-Geweihter", "Phex-Geweihte", 0, Category.Geweiht, false),
	TypGewPer	("Peraine-Geweihter", "Peraine-Geweihte", 0, Category.Geweiht, false),
	TypGewIng	("Ingerimm-Geweihter", "Ingerimm-Geweihte", 0, Category.Geweiht, false),
	TypGewRah	("Rahja-Geweihter", "Rahja-Geweihte", 0, Category.Geweiht, false);

	/**
	 * The categories of character types.
	 *
	 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static enum Category {
		/** Helden des Schwarzen Auges */
		Normal("normal"),
		/** Magie des Schwarzen Auges */
		Magisch("magisch"),
		/** Götter des Schwarzen Auges */
		Geweiht("geweiht");

		public final String name;
		private Category(final String name) {
			this.name = name;
		}
	};

	public static final int TypNormalMax = TypBuerger.ordinal() + 1;
	public static final int TypMagieMax  = TypDruide.ordinal() + 1;

	public final String nameMale;
	public final String nameFemale;
	public int genericGender;
	public final Category category;
	public final boolean volk;

	CharacterType(
			final String nameMale, 
			final String nameFemale, 
			final int genericGender, 
			final Category category, 
			final boolean volk) {
		this.nameMale = nameMale;
		this.nameFemale = nameFemale;
		this.category = category;
		this.volk = volk;
	}
}
