package de.ped.dsatool.logic;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import de.ped.dsatool.dsa.generated.CalendarType;
import de.ped.tools.log.Logger;

/**
 * In Aventuria the years are named following different calendars.
 * 
 * @author Copyright (c) 2006 Peter Diefenbach (peter@pdiefenbach.de),
 */
public abstract class DSACalendar {
	protected static Logger getLogger() {
		return DSAEnv.instance().logger();
	}

	public static List<CalendarType> findAll() {
		return DSA.instance().getCalendar();
	}

	public static CalendarType findByIDnum(final int id) {
		return (CalendarType) (DSA.findBOByIDnum(findAll(), id));
	}

	public static CalendarType findByName(final String name) {
		return (CalendarType) (DSA.findBOByName(findAll(), name));
	}

	public static CalendarType findByID(final String id) {
		return (CalendarType) (DSA.findBOByID(findAll(), id));
	}

	/**
	 * Finds the default calendar - in this case it is Hal.
	 * 
	 * @return the default calendar.
	 */
	public static CalendarType findDefault() {
		final List<CalendarType> calendars = DSA.instance().getCalendar();
		CalendarType calendar = (CalendarType)DSA.findBOByID(calendars, DEFAULT_CALENDAR_ID);
		if ((null == calendar) && (!calendars.isEmpty())) {
			getLogger().info("Calendar " + DEFAULT_CALENDAR_ID + " not found - taking an arbitrary one instead.");
			calendar = (CalendarType)calendars.get(0);
		}
        getLogger().trace("Default calendar is " + calendar.getName() + " with" + (calendar.isHasYearZero() ? "" : "out") + " year zero.");
		return calendar;
	}

	/** The ID of the default calendar. */
	public static final String DEFAULT_CALENDAR_ID = "CALENDAR_BOSPARAN";

	private static Vector<String> allNames = null;

	/**
	 * @return A vector containing strings describing each object. Useful e.g. for ComboBoxes.
	 */
	public static Vector<String> getAllStrings() {
		if (null == allNames) {
			// First initialize it all, then assign to static object, to avoid multithreading problems.
			final Vector<String> an = new Vector<String>();
			final Iterator<CalendarType> it = DSA.instance().getCalendar().iterator();
			while (it.hasNext()) {
				final CalendarType calendar = (CalendarType)it.next();
				an.add(calendar.getName());
			}
			allNames = an;
		}
		return allNames;
	}

	private static HashMap<CalendarType, DSADateCalendarTwelve> calendarObjects;

	public static HashMap<CalendarType, DSADateCalendarTwelve> getCalendars() {
		if (null == calendarObjects) {
			// First initialize it all, then assign to static object, to avoid
			// multithreading problems.
			final HashMap<CalendarType, DSADateCalendarTwelve> co = new HashMap<CalendarType, DSADateCalendarTwelve>();
			final List<CalendarType> cals = DSACalendar.findAll();
			for (final CalendarType cal : cals) {
				co.put(cal, new DSADateCalendarTwelve(cal.getID()));
			}
			calendarObjects = co;
		}
		return calendarObjects;
	}

	public static List<DSADateCalendar> createAllCalendarsFor(final DSADate date) {
		final LinkedList<DSADateCalendar> ret = new LinkedList<DSADateCalendar>();
		final List<CalendarType> cals = DSACalendar.findAll();
		for (final CalendarType cal : cals) {
			ret.add(new DSADateCalendarTwelve(date, cal.getID()));
		}
		ret.add(new DSADateCalendarThorwal(date));
		ret.add(new DSADateCalendarNovadi(date));
		ret.add(new DSADateCalendarNorbards(date));
		ret.add(new DSADateCalendarGjalskerland(date));
		ret.add(new DSADateCalendarDwarfs(date));
		ret.add(new DSADateCalendarOrcs(date));
		ret.add(new DSADateCalendarSaurians(date));
		ret.add(new DSADateCalendarMyranisch(date));
		ret.add(new DSADateCalendarGregorian(date));
		return ret;
	}
}
