package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar or days counting practice in Aventuria
 *  as used by dwarfs.
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarDwarfs extends DSADateCalendar {
	/**
	 * Creates a calendar showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar()
	 */
	public DSADateCalendarDwarfs() {
		super();
		init();
	}

	/**
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate)
	 */
	public DSADateCalendarDwarfs(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Zwergisch");
		setDaysFromYear0ToBF(-YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF);
		setHasYear0(true);
		setDaysPerYear(DSADateCalendar.DAYS_PER_YEAR_BF);
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		final int monthdiv = MathUtil.divisio(jday, DAYS_PER_MONTH);
		final int monthmod = MathUtil.modulo(jday, DAYS_PER_MONTH);
		s.append(TextFormatter.format(monthmod + 1, 2));
		s.append(". ");
		s.append(doc.tf().formatIfPlainText(monthNames[monthdiv], 13));
		s.append(" (");
		s.append(TextFormatter.format(monthdiv + 1, 2));
		s.append('.');
		s.append(") ");
		s.append(getYearString());
		return (s.toString());
	}

	public String getYearString() {
		final StringBuffer s = new StringBuffer();
		final int year = getYear();
		if (-1 == year) {
			s.append(TextFormatter.format(-year, 5));
			s.append(' ');
			s.append("Jahr vor " + DSAEnv.CALENDAR_DWARF_YEAR_ZERO_NAME);
		} else if (0 == year) {
			s.append("im Jahr ");
		} else if (1 == year) {
			s.append(TextFormatter.format(year, 5));
			s.append(' ');
			s.append("Jahr nach ");
		} else if (year < 0) {
			s.append(TextFormatter.format(-year, 5));
			s.append(' ');
			s.append("Jahre vor ");
		} else {
			s.append(TextFormatter.format(year, 5));
			s.append(' ');
			s.append("Jahre nach ");
		}
		s.append(DSAEnv.CALENDAR_DWARF_YEAR_ZERO_NAME);
		return s.toString();
	}

	public static final int YEAR_ZERO_BF_IS = 0;

	public static final int DAYS_PER_MONTH = 30;

	public static final String[] monthNames = new String[] {
			"Sommermond", 
			"Hitzemond", 
			"Regenmond", 
			"Weinmond", 
			"Nebelmond", 
			"Dunkelmond", 
			"Frostmond", 
			"Neugeburt", 
			"Marktmond", 
			"Saatmond", 
			"Feuer- (oder Feier)mond",
			"Brautmond",
			"Drachentag",
	};
}
