package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar or days counting practice in Aventuria
 *  as used in Gjalskerland.
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarGjalskerland extends DSADateCalendar {

	/** 
	 * Creates a calendar showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar() 
	 */
	public DSADateCalendarGjalskerland() {
		super();
		init();
	}

	/** 
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate) 
	 */
	public DSADateCalendarGjalskerland(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Gjalskerland");
		setDaysFromYear0ToBF(YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF + YEAR_OFFSET_IN_DAYS);
		setHasYear0(false);
		setDaysPerYear(DAYS_PER_MONTH * MONTHS_PER_YEAR);
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		final int monthdiv = MathUtil.divisio(jday, DAYS_PER_MONTH);
		final int monthmod = MathUtil.modulo(jday, DAYS_PER_MONTH);
		s.append(TextFormatter.format(monthmod + 1, 2));
		s.append(". ");
		s.append("Tag der Geburt");
		s.append(' ');
		s.append("im");
		s.append(' ');
		s.append(doc.tf().formatIfPlainText(monthNames[monthdiv], 20));
		s.append(" (");
		s.append(TextFormatter.format(monthdiv + 1, 2));
		s.append(") ");
		s.append(getYearString());
		return (s.toString());
	}

	public String getYearString() {
		return getYearString(getYear(), "gS", "v.gS");
	}

	/** 
	 * Die Jahreszählung erfolgt nach dem ersten Tag der Geburt im Mond des Blutes im Jahr 1 nach der großen Schlacht (gS). 
	 * Das Datum entspricht dem 19. Ingerimm 1370 v. BF. 
	 */
	public static final int YEAR_ZERO_BF_IS = -1370 - 1;

	public static final int YEAR_OFFSET_IN_DAYS = 10 * 30 + 19;

	public static final int DAYS_PER_MONTH = DSADate.MOON_MONTH_DAYS;

	public static final int MONTHS_PER_YEAR = 13;

	public static final int YEARS_PER_ERA = 364;

	public static final String[] monthNames = DSAEnv.CALENDAR_GJALSKERLAND_MONTH_NAMES;
}
