package de.ped.dsatool.logic;

import java.text.DateFormat;

import de.ped.tools.TextDocumentBuilder;

/**
 * A class representing the calendar or days counting practice in Aventuria,
 *  based on the translation from the Gregorian calendar used on earth.
 * <p><b>See:</b>
 * <ul>
 *   <li><a href="http://www.wiki-aventurica.de/index.php?title=Kalender">DSA Wiki Kalender</a></li>
 *   <li><a href="http://www.dasschwarzeauge.de/index.php?id=49">Aventurische Zeitrechnung auf dasschwarzeauge.de</a></li>
 *   <li>"Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</li>
 * </ul>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarGregorian extends DSADateCalendar {
	/** 
	 * Creates a date representing Praios 1st, 0 BF.
	 * @see DSADate#DSADate() 
	 */
	public DSADateCalendarGregorian() {
		super();
	}

	/** 
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 */
	public DSADateCalendarGregorian(final DSADate date) {
		super(date);
	}

	@Override
	public String getName() {
		return "Erde (Gregorianisch)";
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		return getDateFormat().format(getDSADate().toEarthDate());
	}

	@Override
	public String getContentText(final TextDocumentBuilder doc) {
		return getHeadingText(doc);
	}

	@Override
	public String toString() {
		return getDSADate().toEarthDate().toString();
	}

	private DateFormat df;

	protected DateFormat getDateFormat() {
		if (null == df) {
			df = DateFormat.getDateInstance(DateFormat.FULL);
		}
		return df;
	}
}
