package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar or days counting practice in Aventuria
 *  as used on the continent Myranor.
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarMyranisch extends DSADateCalendar {
	/** 
	 * Creates a calendar showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar() 
	 */
	public DSADateCalendarMyranisch() {
		super();
		init();
	}

	/** 
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate) 
	 */
	public DSADateCalendarMyranisch(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Myranisch");
		setDaysFromYear0ToBF(-YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF);
		setHasYear0(true);
		setDaysPerYear(DSADateCalendar.DAYS_PER_YEAR_BF);
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		if (360 <= jday) {
			s.append(TextFormatter.format(jday - 360 + 1, 1));
			s.append(". ");
			s.append("Sondertag");
		} else {
			s.append(TextFormatter.format(getDay() + 1, 1));
			s.append(". ");
			s.append("Tag der");
			s.append(' ');
			s.append(TextFormatter.format(getNone() + 1, 1));
			s.append(". ");
			s.append("None der");
			s.append(' ');
			s.append(TextFormatter.format(getOctade() + 1, 1));
			s.append(". ");
			s.append("Oktade");
		}
		s.append(getYearString());
		return (s.toString());
	}

	public int getDay() {
		final int jday = getJDay() - 1;
		return getPart(jday, 1, DAYS_PER_NONE);
	}

	public int getNone() {
		final int jday = getJDay() - 1;
		return getPart(jday, DAYS_PER_NONE, NONES_PER_OCTADE);
	}

	public int getOctade() {
		final int jday = getJDay() - 1;
		return getPart(jday, DAYS_PER_NONE * NONES_PER_OCTADE, getDaysPerYear());
	}

	public String getYearString() {
		final StringBuffer s = new StringBuffer();
		final int year = getYear();
		s.append(TextFormatter.format(year, 5));
		s.append(' ');
		s.append("IZ");
		return s.toString();
	}

	public int getPart(final int value, final int divisor, final int modulo) {
		if (0 == modulo) {
			return MathUtil.divisio(value, divisor);
		} else {
			return MathUtil.divisio(MathUtil.modulo(value, divisor * modulo), divisor);
		}
	}

	public static final int YEAR_ZERO_BF_IS = 3747;

	public static final int DAYS_PER_NONE = 9;

	public static final int NONES_PER_OCTADE = 5;

	public static final int DAYS_PER_MONTH = DAYS_PER_NONE * NONES_PER_OCTADE;
}
