package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar or days counting practice in Aventuria
 *  as used by Norbards.
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarNorbards extends DSADateCalendar {
	/** 
	 * Creates a calendar showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar() 
	 */
	public DSADateCalendarNorbards() {
		super();
		init();
	}

	/** 
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate) 
	 */
	public DSADateCalendarNorbards(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Norbardisch");
		setDaysFromYear0ToBF(YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF + YEAR_OFFSET_IN_DAYS);
		setHasYear0(false);
		setDaysPerYear(DAYS_PER_MONTH * MONTHS_PER_YEAR);
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		final int monthdiv = MathUtil.divisio(jday, DAYS_PER_MONTH);
		final int monthmod = MathUtil.modulo(jday, DAYS_PER_MONTH);
		s.append(TextFormatter.format(monthmod + 1, 2));
		s.append(". ");
		s.append("Tag");
		s.append(' ');
		s.append("im");
		s.append(' ');
		s.append(TextFormatter.format(monthdiv + 1, 3));
		s.append(". ");
		s.append("Mondmonat");
		s.append(' ');
		s.append("im");
		s.append(' ');
		s.append(TextFormatter.format(getYear(), 5));
		s.append(". ");
		s.append("Uh'Jun");
		s.append(" (just guessing!)");
		return (s.toString());
	}

	/** This is just guessing. */
	public static final int YEAR_ZERO_BF_IS = 0;

	/** This is just guessing. */
	public static final int YEAR_OFFSET_IN_DAYS = 0;

	public static final int DAYS_PER_MONTH = 28;

	public static final int MONTHS_PER_YEAR = 100;
}
