package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar or days counting practice in Aventuria
 *  as used by Novadi.
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarNovadi extends DSADateCalendar {
	/** 
	 * Creates a calendar showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar() 
	 */
	public DSADateCalendarNovadi() {
		super();
		init();
	}

	/** 
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate) 
	 */
	public DSADateCalendarNovadi(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Novadisch");
		setDaysFromYear0ToBF(YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF + YEAR_OFFSET_IN_DAYS);
		setHasYear0(false);
		setDaysPerYear(DSADateCalendar.DAYS_PER_YEAR_BF);
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		final int rastdiv = MathUtil.divisio(jday, DAYS_PER_MONTH);
		final int rastmod = MathUtil.modulo(jday, DAYS_PER_MONTH);
		if (DAYS_PER_MONTH - 1 == rastmod) { // Rastullahellah
			s.append(TextFormatter.format(rastdiv + 1, 1));
			s.append(". ");
			s.append("Rastullahellah");
		} else {
			final int weekday = MathUtil.modulo(rastmod, DAYS_PER_WEEK);
			final int week = MathUtil.divisio(rastmod, DAYS_PER_WEEK) + rastdiv * WEEKS_PER_MONTH;
			s.append(TextFormatter.format(weekday + 1, 1));
			s.append(". ");
			s.append("Tag");
			s.append(" (");
			s.append(doc.tf().formatIfPlainText(weekdayNames[weekday], 12));
			s.append(") ");
			s.append("im");
			s.append(' ');
			s.append(TextFormatter.format(week + 1, 2));
			s.append(". ");
			s.append("Gottesnamen");
		}
		s.append(' ');
		s.append(getYearString());
		return (s.toString());
	}

	/**
	 * Am 23. Boron 760 BF offenbarte sich Rastullah, das ist Jahr 1 Rastullah.
	 */
	public static final int YEAR_ZERO_BF_IS = 760 - 1;

	public static final int YEAR_OFFSET_IN_DAYS = 4 * 30 + 23 - 1;

	public static final int DAYS_PER_WEEK = 9;

	public static final int WEEKS_PER_MONTH = 8;

	public static final int DAYS_PER_MONTH = DAYS_PER_WEEK * WEEKS_PER_MONTH + 1;

	public static final String[] weekdayNames = new String[] {
			"Hellah",
			"Orhima",
			"Shimja",
			"Rhondara",
			"Heschinja",
			"Dschella",
			"Marhibo",
			"Khabla ",
			"Amm el-Thona",
	};

	public String getYearString() {
		return getYearString(getYear(), "Rastullah (n. d. O.)", "vor Rastullahs Erscheinen (v. d. O.)");
	}
}
