package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar or days counting practice in Aventuria
 *  as used by Orcs.
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarOrcs extends DSADateCalendar {
	/** 
	 * Creates a calendar showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar() 
	 */
	public DSADateCalendarOrcs() {
		super();
		init();
	}

	/** 
	 * Creates a view following this calendar to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate) 
	 */
	public DSADateCalendarOrcs(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Orkisch");
		final int daysPerYear = DSADate.DAYS_BETWEEN_LUNAR_ECLIPSES;
		final int offset = REFERENCE_DAY_YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF
				+ REFERENCE_DAY_YEAR_OFFSET_IN_DAYS - daysPerYear * REFERENCE_YEAR;
		getLogger().debug("Orc calendar offset is " + offset);
		setDaysFromYear0ToBF(offset);
		setHasYear0(true);
		setDaysPerYear(daysPerYear);
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		final int monthdiv = MathUtil.divisio(jday, DAYS_PER_MONTH);
		final int monthmod = MathUtil.modulo(jday, DAYS_PER_MONTH);
		s.append(TextFormatter.format(monthmod + 1, 2));
		s.append(". ");
		s.append("Tag");
		s.append(" ");
		s.append("im");
		s.append(' ');
		s.append(TextFormatter.format(monthdiv + 1, 3));
		s.append(". ");
		s.append("Mond");
		s.append(' ');
		s.append('(');
		s.append(doc.tf().formatIfPlainText(getDSADate().getDaysSinceLastLunarEclipse() + 1, 5));
		s.append(". ");
		s.append("Tag)");
		s.append(' ');
		s.append("im");
		s.append(' ');
		s.append(getYearString());
		return (s.toString());
	}

	/** 18. Efferd 1029 BF = erster Tag des Großen Jahres 2001 */
	public static final int REFERENCE_DAY_YEAR_ZERO_BF_IS = 1029;

	public static final int REFERENCE_DAY_YEAR_OFFSET_IN_DAYS = 2 * 30 + 18 - 1;

	public static final int REFERENCE_YEAR = 2001;

	public static final int MONTHS_PER_YEAR = DSADate.MOON_MONTHS_BETWEEN_LUNAR_ECLIPSES;

	public static final int DAYS_PER_MONTH = DSADate.MOON_MONTH_DAYS;

	public String getYearString() {
		final StringBuffer s = new StringBuffer();
		s.append(TextFormatter.format(getYear(), 5));
		s.append(". ");
		s.append("Jahr des Tairach");
		return s.toString();
	}
}
