package de.ped.dsatool.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextFormatter;

/**
 * A class representing the calendar used in Thorwal. 
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Der Aventurische Kalender" (p. 60..63)</p>
 *
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateCalendarThorwal extends DSADateCalendar {
	/** 
	 * Creates a calendar based on Jurgas landing, showing a date representing Praios 1st, 0 BF.
	 * @see DSADateCalendar#DSADateCalendar() 
	 */
	public DSADateCalendarThorwal() {
		super();
		init();
	}

	/** 
	 * Creates a view following the calendar based on Jurgas landing to the given date.
	 * 
	 * @param date
	 *            the DSA date to be represented using this calendar
	 * @see DSADateCalendar#DSADateCalendar(DSADate) 
	 */
	public DSADateCalendarThorwal(final DSADate date) {
		super(date);
		init();
	}

	protected void init() {
		setName("Thorwalsch (Jurgas Landung)");
		setDaysFromYear0ToBF(-YEAR_ZERO_BF_IS * DSADateCalendar.DAYS_PER_YEAR_BF);
		setHasYear0(false);
		setDaysPerYear(DSADateCalendar.DAYS_PER_YEAR_BF);
	}

	/** Das Jahr 1 JL entspricht 1627 v. BF. */
	public static final int YEAR_ZERO_BF_IS = 1627;

	public static final int DAYS_PER_MONTH = 30;

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		final StringBuffer s = new StringBuffer();
		final int jday = getJDay() - 1;
		final int monthdiv = MathUtil.divisio(jday, DAYS_PER_MONTH);
		final int monthmod = MathUtil.modulo(jday, DAYS_PER_MONTH);
		s.append(TextFormatter.format(monthmod + 1, 2));
		s.append(". ");
		s.append(doc.tf().formatIfPlainText(monthNames[monthdiv], 13));
		s.append(" (");
		s.append(TextFormatter.format(monthdiv + 1, 2));
		s.append('.');
		s.append(") ");
		s.append(getYearString());
		return (s.toString());
	}

	public String getYearString() {
		return getYearString(getYear(), "JL", "v.JL");
	}

	public static final String[] monthNames = new String[] {
			"Midsonnmond", 
			"Kornmond", 
			"Heimamond", 
			"Schlachtmond", 
			"Sturmmond", 
			"Frostmond", 
			"Grimfrostmond", 
			"Goimond", 
			"Friskenmond", 
			"Eimond", 
			"Faramond", 
			"Vinmond",
			"Hranngar-Tag",
	};
}
