package de.ped.dsatool.logic;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.Messages;

/**
 * The system tried to set a year 0 for a calendar that has no year 0.
 * @see DSADateCalendar#setJDayYear(int, int)
 *
 * @author Copyright (c) 1999 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSADateYearZeroException extends DSAException {
	private static final long serialVersionUID = 1L;

	public DSADateYearZeroException() {
		super("Year 0 not supported by the chosen calendar!");
		calendarName = null;
	}

	public DSADateYearZeroException(final String s) {
		super("Year 0 not supported by the chosen calendar: " + s);
		calendarName = s;
	}

	@Override
	public String getLocalizedMessage() {
		final Messages messages = DSAEnv.instance().messages();
		String ret;
		if (null == calendarName) {
			ret = messages.getText("DSADateYearZeroException.Message.Without");
		} else {
			ret = messages.getText(new I18NStringWithFillIns("DSADateYearZeroException.Message.Default", null,
					new String[] { calendarName, }));
		}
		return ret;
	}

	private final String calendarName;
}
