package de.ped.dsatool.logic;

import de.ped.tools.ApplicationEnvironment;
import de.ped.tools.ApplicationParameter;
import de.ped.tools.Messages;
import de.ped.tools.ResourceFinder;
import de.ped.tools.TextDocumentType;
import de.ped.tools.log.Logger;

/**
 * DSA tool specific implementation of the {@link ApplicationEnvironment} interface.
 * Normally, this interface is fulfilled by UI applications; but we also need it for the web interface.
 *
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSAEnv implements ApplicationEnvironment {

	/** Only defined here to be in charset ISO-8859-1 */
	public static final String DEFAULT_TOWN_NAME = "Brtigerdorf";
	/** Only defined here to be in charset ISO-8859-1 */
	public static final String APP_SHORT_DESCRIPTION = "Spielleiterhilfe fr berlandabenteuer";
	/** Only defined here to be in charset ISO-8859-1 */
	public static final String CALENDAR_DWARF_YEAR_ZERO_NAME = "der zweiten Dmonenschlacht";
	/** Only defined here to be in charset ISO-8859-1 */
	public static final String[] CALENDAR_GJALSKERLAND_MONTH_NAMES = new String[] {
			"Mond des Blutes", 
			"Siegesmond", 
			"Trauermond", 
			"Mond des Bundes", 
			"Mond des Schaffens", 
			"Mond des Sturms", 
			"Mond der Stille", 
			"Mond des Hungers", 
			"Mond der Jagd", 
			"Gttermond", 
			"Mond des Odn", 
			"Mond der Niederkunft", 
			"Ahnenmond",     
	};
	/** Only defined here to be in charset ISO-8859-1 */
	public static final String[] CALENDAR_SAURION_SECTION_NAMES = new String[] {
			"des Waran", 
			"des Schlingers", 
			"der Schlange", 
			"des Salamanders", 
			"des Drachen", 
			"der Flugechse", 
			"der Schildkrte", 
			"des Chamleons", 
			"der Seeschlange",
			"des Krokodils", 
	};
	
	/**
	 * Implements the Singleton pattern from the Gang of Four.
	 * 
	 * @return the singleton instance
	 */
	public static synchronized DSAEnv instance() {
		if (null == instance) {
			instance = new DSAEnv();
		}
		return instance;
	}

	private static DSAEnv instance;

	private DSAEnv() {
		// Nothing to do, just defined to fulfill Singleton pattern
	}

	@Override
	public ApplicationParameter findParameter(String key) {
		return null;
	}

	@Override
	public synchronized ResourceFinder resources() {
		if (null == resourceFinder) {
			resourceFinder = new ResourceFinder("de/ped/dsatool/resources");
		}
		return resourceFinder;
	}

	@Override
	public synchronized Messages messages() {
		if (null == messages) {
			messages = new Messages(
					"de.ped.dsatool.logic.messages", 
					TextDocumentType.CHARSET_ISO_8859_1,
					"DSATool", "v1.08", 
					"Copyright (c) 1998-2024 Peter Diefenbach (peter@pdiefenbach.de)",
					APP_SHORT_DESCRIPTION,
					"For DSA dates, weather, random day events, creation of small towns,\n"
							+ "Aventurian names, tests for botany, and ideas for quests");
		}
		return messages;
	}

	@Override
	public boolean isExitable() {
		return false;
	}

	@Override
	public boolean isSilent() {
		return false;
	}

	public synchronized Logger logger() {
		if (null == logger) {
			final Logger lg = Logger.getRootLogger();
			lg.removeAppender(Logger.DEFAULT_STRING_WRITER_APPENDER);
			logger = lg;
		}
		return logger;
	}

	public synchronized DSA dsa() {
		if (null == dsa) {
			final DSA inst = new DSA();
			DSA.init(inst);
			dsa = inst;
		}
		return dsa;
	}

	private ResourceFinder resourceFinder;

	private Messages messages;

	private Logger logger;
	
	private DSA dsa;
}
