package de.ped.dsatool.logic;

import de.ped.dsatool.dsa.generated.LanguageDistributionType;
import de.ped.dsatool.dsa.generated.ObjectFactory;

/**
 * Factory for all DSA classes.
 * Has to be a separate class because JAXB ObjectFactory offers no JAXB-independent interface or abstract class.
 * 
 * @author Copyright (c) 2006 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DSAObjectFactory {
	/** The JAXB generated object factory. If <code>null</code>, not using JAXB but stand-alone implementation. */
	protected ObjectFactory objectFactory;

	/**
	 * @param isUsingJAXB if <code>true</code>, uses JAXB classes, otherwise uses stand-alone implementation.
	 */
	public DSAObjectFactory(final boolean isUsingJAXB) {
		if (isUsingJAXB) {
			objectFactory = new ObjectFactory();
		}
	}

	/**
	 * @return if <code>true</code>, uses JAXB classes, otherwise uses stand-alone implementation.
	 */
	public boolean isUsingJAXB() {
		return (null != objectFactory);
	}

	public LanguageDistributionType createLanguageDistributionType() {
		if (null != objectFactory) {
			try {
				return objectFactory.createLanguageDistributionType();
			} catch (final Exception e) {
				e.printStackTrace();
				return null;
			}
		} else {
			// return new LanguageDistributionTypeImpl();
			return null;
		}
	}
}
