package de.ped.dsatool.logic;

import de.ped.dsatool.dsa.generated.DeityType;
import de.ped.dsatool.dsa.generated.LanguageType;
import de.ped.dsatool.dsa.generated.SexType;

/**
 * DSA Character.
 * 
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class HeroCharacter {
	/** Dateiname des Helden	*/
	String filename;
	/** DSA-Name				*/
	String name;
	String geburtsort;
	CharacterBodyPartColor haarfarbe;
	CharacterBodyPartColor augenfarbe;
	/** Männlich/weiblich?	*/
	SexType geschlecht;
	/** Heldentyp (Beruf)	*/
	CharacterType typus;
	/** Volk des Helden		*/
	CharacterType volk;
	/** Magisch begabt?		*/
	int magisch;
	/** Geburt: Tag im Monat	*/
	int geburtstag;
	/** Geburt: Monat (Gott)	*/
	int geburtsmonat;
	/** Heldengröße in cm	*/
	int groesse;
	/** Heldengewicht in kg	*/
	int gewicht;
	/** Stufe des Helden		*/
	int ST;
	/** Ausdauer				*/
	int AU;
	/** Magieresistenz		*/
	int MR;
	/** Lebensenergie		*/
	int LE;
	/**  Obere Grenze		*/
	int LEperm;
	/** Astralenergie		*/
	int AE;
	/**  Obere Grenze		*/
	int AEperm;
	/** Karmaenergie			*/
	int KE;
	/**  Obere Grenze		*/
	int KEperm;
	/** Muttersprache		*/
	LanguageType muttersprache;
	/** Gewählte Gottheit	*/
	DeityType gottheit;
	/** Bei Magiern Akademie	*/
	int akademie;
	/** Hexe aus dem Ei?		*/
	boolean eihexe;
	/** Bei Hexen Gefährte	*/
	int gefaehrte;
	/** Eigenschaften Stufe 0*/
	int[]	wurfeig = new int[CharacterAttribute.values().length];
	/** Eigenschaften jetzt	*/
	int[] eigenschaft = new int[CharacterAttribute.values().length];
	/** Die AT/PA-Werte		*/
	int[][] kampfwert = new int[CharacterTalents.KT_Max][2];
	/** Die Talentwerte		*/
	int[] talent = new int[CharacterTalents.values().length];
	/** Die Zauberfähigkeiten*/
	int[] zauber = new int[CharacterSpell.values().length];
	/** Sprachen kennen	*/
	int[] sprache = new int[Language.findAll().size()];
	/** Abenteuerpunkte		*/
	long AP;
	/** Geld in Kreuzern		*/
	long vermoegen;

	public HeroCharacter() {
		for (int i = 0; i < talent.length; i++) {
			talent[i] = -20;
		}
		for (int i = 0; i < zauber.length; i++) {
			zauber[i] = -20;
		}
	}

	/**
	/*	Das Geburtsdatum eines Helden auswürfeln:
	 */
	public void wurf_geburtstag() {
		final int[] monthsForRollLookup = new int[] { 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 7, 8, 8, 8, 9, 10, 11, 11 };
		geburtstag = (Dice.roll(1, 20, -1) / 4) * 6 + Dice.roll(1, 6);
		geburtsmonat = monthsForRollLookup[Dice.roll(1, 20, -1)];
	}

	/**
	 * When the hero character type is set, the hero can get initialized in many parts
	 * without needing to as the user, like:
	 * sex, hair color, size, weight, wealth, life and magic energy.
	 */
	void setheldtyp() {	
		int			w;
		CharacterBodyPartColor d;
		long		g;
		switch (typus) {
		case TypGaukler:
			LE = 30;
			AE =  0;
			groesse = 148 + Dice.roll(2,20);
			gewicht = groesse - 110;
			w = Dice.roll(1, 20);
			if 		(w<15)	g=   500L;	/*  5 S	*/
			else if (w<20)	g= 10000L;	/* 10 D	*/
			else			g= 30000L;	/* 30 D	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Schwarz;
			else if (w< 9)	d = CharacterBodyPartColor.Blond;
			else if (w<11)	d = CharacterBodyPartColor.Weissblond;
			else if (w<15)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<17)	d = CharacterBodyPartColor.Braun;
			else if (w<20)	d = CharacterBodyPartColor.Rot;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypHaendler:
	  		LE = 30;
			AE =  0;
			groesse = 160 + Dice.roll(2,20);
			gewicht = groesse - 110;
			w = Dice.roll(1, 20);
			if 		(w< 6)	g=  1000L;	/* 10 S	*/
			else if (w<10)	g=  2000L;	/* 20 S	*/
			else if (w<13)	g=  5000L;	/* 50 S	*/
			else if (w<17)	g= 10000L;	/* 10 D	*/
			else if (w<20)	g= 30000L;	/* 30 D	*/
			else			g=100000L;	/*100 D	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<10)	d = CharacterBodyPartColor.Schwarz;
			else if (w<11)	d = CharacterBodyPartColor.Weissblond;
			else if (w<15)	d = CharacterBodyPartColor.Blond;
			else if (w<19)	d = CharacterBodyPartColor.Braun;
			else if (w<20)	d = CharacterBodyPartColor.Rot;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypKrieger:
	  		LE = 30;
			AE =  0;
			groesse = 163 + Dice.roll(2,20);
			gewicht = groesse - 100;
			w = Dice.roll(1, 20);
			if 		(w< 3)	g=   100L;	/*  1 S	*/
			else if (w< 5)	g=  2000L;	/* 20 S	*/
			else if (w< 8)	g=  3000L;	/* 30 S	*/
			else if (w<11)	g=  5000L;	/* 50 S	*/
			else if (w<19)	g= 20000L;	/* 20 D	*/
			else			g=100000L;	/*100 D	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<10)	d = CharacterBodyPartColor.Schwarz;
			else if (w<11)	d = CharacterBodyPartColor.Weissblond;
			else if (w<15)	d = CharacterBodyPartColor.Blond;
			else if (w<19)	d = CharacterBodyPartColor.Braun;
			else if (w<20)	d = CharacterBodyPartColor.Rot;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypMoha:
	  		LE = 30;
			AE =  0;
			groesse = 152 + Dice.roll(2,20);
			gewicht = groesse - 110;
			muttersprache = Language.findByID("LANGUAGE_MOHISH");
			w = Dice.roll(1, 20);
			if 		(w< 2)	g=   100L;	/*  1 S	*/
			else			g=     0L;	/*  0 S	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Schwarz;
			else if (w<20)	d = CharacterBodyPartColor.Blauschwarz;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypNivese:
	  		LE = 30;
			AE =  0;
			groesse = 156 + Dice.roll(2,20);
			gewicht = groesse - 110;
			muttersprache = Language.findByID("LANGUAGE_NIVESISH");
			w = Dice.roll(1, 20);
			if 		(w< 5)	g=  1000L;	/* 10 S	*/
			else if (w<17)	g=  2000L;	/* 20 S	*/
			else if (w<20)	g=  5000L;	/* 50 S	*/
			else			g= 10000L;	/* 10 D	*/
			w = Dice.roll(1, 20);
			if 		(w<16)	d = CharacterBodyPartColor.Kupferrot;
			else if (w<17)	d = CharacterBodyPartColor.Blond;
			else if (w<19)	d = CharacterBodyPartColor.Braun;
			else if (w<20)	d = CharacterBodyPartColor.Dunkelblond;
			else			d = CharacterBodyPartColor.Schwarz;
			break;
		case TypNovadi:
	  		LE = 30;
			AE =  0;
			groesse = 156 + Dice.roll(2,20);
			gewicht = groesse - 110;
			muttersprache = Language.findByID("LANGUAGE_NOVADISH");
			w = Dice.roll(1, 20);
			if 		(w< 8)	g=   500L;	/*  5 S	*/
			else if (w<17)	g=  1000L;	/* 10 S	*/
			else if (w<19)	g= 20000L;	/* 20 D	*/
			else if (w<20)	g= 40000L;	/* 40 D	*/
			else			g=100000L;	/*100 D	*/
			w = Dice.roll(1, 20);
			if 		(w<19)	d = CharacterBodyPartColor.Schwarz;
			else if (w<20)	d = CharacterBodyPartColor.Blond;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypStreuner:
	  		LE = 30;
			AE =  0;
			groesse = 154 + Dice.roll(2,20);
			gewicht = groesse - 110;
			w = Dice.roll(1, 20);
			if 		(w<11)	g=     0L;	/*  0 S	*/
			else if (w<15)	g=   200L;	/*  2 S	*/
			else if (w<18)	g=   200L;	/*  2 S	*/
			else if (w<19)	g=   500L;	/*  5 S	*/
			else			g=  1000L;	/* 10 S	*/
			w = Dice.roll(1, 20);
			if 		(w< 7)	d = CharacterBodyPartColor.Schwarz;
			else if (w< 9)	d = CharacterBodyPartColor.Blond;
			else if (w<10)	d = CharacterBodyPartColor.Weissblond;
			else if (w<15)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<17)	d = CharacterBodyPartColor.Braun;
			else if (w<20)	d = CharacterBodyPartColor.Rot;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypThorwaler:
	  		LE = 30;
			AE =  0;
			groesse = 164 + Dice.roll(2,20) + Dice.roll(1, 6);
			gewicht = groesse - 100;
			muttersprache = Language.findByID("LANGUAGE_NORBARDISH");
			w = Dice.roll(1, 20);
			if 		(w<10)	g=  1000L;	/* 10 S	*/
			else if (w<14)	g= 10000L;	/* 10 D	*/
			else if (w<20)	g= 20000L;	/* 20 D	*/
			else			g= 50000L;	/* 50 D	*/
			w = Dice.roll(1, 20);
			if 		(w<10)	d = CharacterBodyPartColor.Blond;
			else if (w<16)	d = CharacterBodyPartColor.Rotblond;
			else if (w<18)	d = CharacterBodyPartColor.Weissblond;
			else if (w<19)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<20)	d = CharacterBodyPartColor.Braun;
			else			d = CharacterBodyPartColor.Schwarz;
			break;
		case TypZwerg:
	  		LE = 40;
			AE =  0;
			groesse = 128 + Dice.roll(2,6);
			gewicht = groesse - 80;
			muttersprache = Language.findByID("LANGUAGE_DWARFISH");
			w = Dice.roll(1, 20);
			if 		(w< 8)	g= 10000L;	/* 10 D	*/
			else if (w<17)	g= 20000L;	/* 20 D	*/
			else if (w<20)	g= 30000L;	/* 30 D	*/
			else			g=120000L;	/*120 D	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Blond;
			else if (w<12)	d = CharacterBodyPartColor.Rot;
			else if (w<14)	d = CharacterBodyPartColor.Weiss;
			else if (w<17)	d = CharacterBodyPartColor.Braun;
			else			d = CharacterBodyPartColor.Schwarz;
			break;
		case TypMagier:
	  		LE = 25;
			AE = 30;
			groesse = 160 + Dice.roll(1, 20) + Dice.roll(3,6);
			gewicht = groesse - 110;
			w = Dice.roll(1, 20);
			if 		(w< 2)	g=  100L * Dice.roll(1, 6);			/* W6 S	*/
			else if (w< 4)	g=  100L * Dice.roll(1, 6);			/* W6 S	*/
			else if (w< 5)	g= 1000L * Dice.roll(1, 6);			/* W6 D	*/
			else if (w<12)	g= 1000L * Dice.roll(1, 6);			/* W6 D	*/
			else if (w<14)	g= 1000L * Dice.roll(1, 6);			/* W6 D	*/
			else if (w<17)	g= 1000L * Dice.roll(1, 20);			/* W20D	*/
			else if (w<20)	g= 1000L * Dice.roll(2,20);	/*2W20D	*/
			else			g= 1000L * Dice.roll(5,20);	/*5W20D	*/
			w = Dice.roll(1, 20);
			if 		(w< 8)	d = CharacterBodyPartColor.Schwarz;
			else if (w<11)	d = CharacterBodyPartColor.Braun;
			else if (w<13)	d = CharacterBodyPartColor.Rot;
			else if (w<16)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<18)	d = CharacterBodyPartColor.Mittelblond;
			else if (w<20)	d = CharacterBodyPartColor.Hellblond;
			else			d = CharacterBodyPartColor.Albino;
			break;
		case TypAuelf:
	  		LE = 25;
			AE = 25;
			groesse = 166 + Dice.roll(1, 20) + Dice.roll(3,6);
			gewicht = groesse - 120;
			muttersprache = Language.findByID("LANGUAGE_ELFISH");
			w = Dice.roll(1, 20);
			if 		(w< 5)	g=  100L * Dice.roll(1, 6);			/* W6 S	*/
			else if (w< 7)	g=  100L * Dice.roll(1, 20);			/* W20S	*/
			else if (w<14)	g= 1000L * Dice.roll(1, 6);			/* W6 D	*/
			else if (w<20)	g= 1000L * Dice.roll(1, 20);			/* W20D	*/
			else			g= 1000L * Dice.roll(2,20);	/*2W20D	*/
			w = Dice.roll(1, 20);
			if 		(w< 4)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<10)	d = CharacterBodyPartColor.Mittelblond;
			else if (w<14)	d = CharacterBodyPartColor.Hellblond;
			else if (w<16)	d = CharacterBodyPartColor.Weissblond;
			else if (w<17)	d = CharacterBodyPartColor.Albino;
			else if (w<18)	d = CharacterBodyPartColor.Silber;
			else if (w<20)	d = CharacterBodyPartColor.Schwarz;
			else			d = CharacterBodyPartColor.Blauschwarz;
			break;
		case TypWaldelf:
	  		LE = 25;
			AE = 25;
			groesse = 166 + Dice.roll(1, 20) + Dice.roll(4,6);
			gewicht = groesse - 120;
			muttersprache = Language.findByID("LANGUAGE_ELFISH");
			w = Dice.roll(1, 20);
			if 		(w<15)	g=  100L * Dice.roll(1, 6);			/* W6 S	*/
			else if (w<20)	g=  100L * Dice.roll(1, 20);			/* W20S	*/
			else			g= 1000L * Dice.roll(1, 6);			/* W6 D	*/
			w = Dice.roll(1, 20);
			if 		(w< 5)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w< 8)	d = CharacterBodyPartColor.Mittelblond;
			else if (w<10)	d = CharacterBodyPartColor.Hellblond;
			else if (w<11)	d = CharacterBodyPartColor.Weissblond;
			else if (w<12)	d = CharacterBodyPartColor.Albino;
			else if (w<14)	d = CharacterBodyPartColor.Silber;
			else if (w<18)	d = CharacterBodyPartColor.Schwarz;
			else			d = CharacterBodyPartColor.Blauschwarz;
			break;
		case TypFirnelf:
	  		LE = 30;
			AE = 25;
			groesse = 156 + Dice.roll(1, 20) + Dice.roll(3,6);
			gewicht = groesse - 120;
			muttersprache = Language.findByID("LANGUAGE_ELFISH");
			w = Dice.roll(1, 20);
			if 		(w< 2)	g=  100L * Dice.roll(1, 6);			/* W6 S	*/
			else			g=  100L * Dice.roll(1, 20);			/* W20S	*/
			w = Dice.roll(1, 20);
			if 		(w< 2)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w< 4)	d = CharacterBodyPartColor.Mittelblond;
			else if (w< 7)	d = CharacterBodyPartColor.Hellblond;
			else if (w<12)	d = CharacterBodyPartColor.Weissblond;
			else if (w<16)	d = CharacterBodyPartColor.Schneeweiss;
			else if (w<19)	d = CharacterBodyPartColor.Silberweiss;
			else if (w<20)	d = CharacterBodyPartColor.Schwarz;
			else			d = CharacterBodyPartColor.Blauschwarz;
			break;
		case TypHexe:
	  		LE = 25;
			AE = 25;
			groesse = 150 + Dice.roll(1, 20) + Dice.roll(3,6);
			gewicht = groesse - 110;
			geschlecht = Person.randomSex(90.0F);
			if (Dice.roll(1, 20) == 1)	eihexe=true;		/* Aus dem Ei?	*/
			g = 100L * Dice.roll(1, 20);						/* Geld: W20S	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Pechschwarz;
			else if (w< 8)	d = CharacterBodyPartColor.Blauschwarz;
			else if (w<12)	d = CharacterBodyPartColor.Kastanienrot;
			else if (w<15)	d = CharacterBodyPartColor.Kupferrot;
			else if (w<18)	d = CharacterBodyPartColor.Brandrot;
			else if (w<20)	d = CharacterBodyPartColor.Rotblond;
			else			d = CharacterBodyPartColor.Weissblond;
			break;
		case TypSchelm:
	  		LE = 30;
			AE = 20;
			groesse = 148 + Dice.roll(1, 20) + Dice.roll(1, 6);
			gewicht = groesse - 110;
			g=  1L;								/*  1 K	*/
			w = Dice.roll(1, 20);
			if 		(w< 3)	d = CharacterBodyPartColor.Schwarz;
			else if (w< 5)	d = CharacterBodyPartColor.Nussbraun;
			else if (w< 8)	d = CharacterBodyPartColor.Rostrot;
			else if (w<13)	d = CharacterBodyPartColor.Scharlachrot;
			else if (w<18)	d = CharacterBodyPartColor.Karottenrot;
			else			d = CharacterBodyPartColor.Weizenblond;
			break;
		case TypDruide:
	  		LE = 30;
			AE = 25;
			groesse = 150 + Dice.roll(1, 20) + Dice.roll(3,6);
			gewicht = groesse - 110;
			geschlecht = Person.randomSex(20.0F);
			w = Dice.roll(1, 20);
			if 		(w<15)	g=   10L * Dice.roll(1, 6);			/* W6 H	*/
			else if (w<16)	g=  100L * Dice.roll(1, 6);			/* W6 S	*/
			else if (w<17)	g=  100L * Dice.roll(2,6);	/*2W6 S	*/
			else if (w<19)	g=  100L * Dice.roll(1, 20);			/* W20S	*/
			else if (w<20)	g= 1000L * Dice.roll(1, 6);			/* W6 D	*/
			else			g= 1000L * Dice.roll(1, 20);			/* W20D	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Pechschwarz;
			else if (w< 8)	d = CharacterBodyPartColor.Blauschwarz;
			else if (w<13)	d = CharacterBodyPartColor.Dunkelbraun;
			else if (w<16)	d = CharacterBodyPartColor.Aschblond;
			else if (w<19)	d = CharacterBodyPartColor.Grau;
			else			d = CharacterBodyPartColor.Kahlkoepfig;
			break;
		case TypBuerger:
		default:
	  		LE = 30;
			AE =  0;
			groesse = 145 + Dice.roll(1, 20) + Dice.roll(3, 6);
			gewicht = groesse - 110;
			w = Dice.roll(1, 20);
			if 		(w< 9)	g=   100L;	/*  1 S	*/
			else if (w<15)	g=   500L;	/*  5 S	*/
			else if (w<19)	g=  2000L;	/* 20 S	*/
			else			g= 20000L;	/* 20 D	*/
			w = Dice.roll(1, 20);
			if 		(w< 6)	d = CharacterBodyPartColor.Dunkelblond;
			else if (w<10)	d = CharacterBodyPartColor.Schwarz;
			else if (w<11)	d = CharacterBodyPartColor.Weissblond;
			else if (w<15)	d = CharacterBodyPartColor.Blond;
			else if (w<19)	d = CharacterBodyPartColor.Braun;
			else if (w<20)	d = CharacterBodyPartColor.Rot;
			else			d = CharacterBodyPartColor.Albino;
			break;
		}
		vermoegen = g;
		haarfarbe = d;
		if (null == geschlecht) {
			geschlecht = Person.randomSex(50.0F);
		}
		if (null != muttersprache) {
			sprache[muttersprache.getIDnum()] = muttersprache.getComplexity();
		}
		LEperm = LE;
		AEperm = AE;
	}

	/**
	 * When most important data is known for the hero (type, birthday, magic academy),
	 * talents and spell skills can be initialized,
	 * including add-ons for academy and deity.
	 */
	public void settalente() {
		for (final CharacterTemplate.TalentEntry tp : CharacterTemplate.TalentTabelle) {
			talent[tp.Talent.ordinal()] = tp.Wert[typus.ordinal()];
		}
		if (typus.ordinal() >= CharacterType.TypMagier.ordinal()) {
			for (final CharacterTemplate.ZauberEntry zp : CharacterTemplate.ZauberTabelle) {
				zauber[zp.Zauber.ordinal()] = zp.Wert[typus.ordinal() - CharacterType.TypMagier.ordinal()];
			}
		}
		if (typus == CharacterType.TypMagier) {
			schuleinit();
		}
		switch (geburtsmonat) {
		case 0: //Praios:
			eigenschaft[CharacterAttribute.MU.ordinal()] 			+= 1;
			break;
		case 1: //Rondra:
			talent[CharacterTalents.TA_Schwerter.ordinal()]			+= 1;
		  	break;
		case 2: //Efferd:
			talent[CharacterTalents.TA_Schwimmen.ordinal()]			+= 1;
			talent[CharacterTalents.TA_RudernSegeln.ordinal()]		+= 1;
			break;
		case 3: //Travia:
			talent[CharacterTalents.TA_HeilkundeSeele.ordinal()]	+= 1;
			talent[CharacterTalents.TA_Kochen.ordinal()]			+= 1;
			break;
		case 4: //Boron:
			talent[CharacterTalents.TA_Menschenkenntnis.ordinal()]	+= 1;
			break;
		case 5: // Hesinde:
			talent[CharacterTalents.TA_Alchimie.ordinal()]			+= 1;
			break;
		case 6: // Firun:
			talent[CharacterTalents.TA_Schusswaffen.ordinal()]		+= 1;
			talent[CharacterTalents.TA_Faehrtensuche.ordinal()]		+= 1;
			break;
		case 7: // Tsa:
			eigenschaft[CharacterAttribute.CH.ordinal()]			+= 1;
			break;
		case 8: // Phex:
			talent[CharacterTalents.TA_Schleichen.ordinal()]		+= 1;
			talent[CharacterTalents.TA_Taschendiebstahl.ordinal()]	+= 1;
			break;
		case 9: // Peraine:
			talent[CharacterTalents.TA_HeilkundeKrankh.ordinal()]	+= 1;
			talent[CharacterTalents.TA_HeilkundeWunden.ordinal()]	+= 1;
			break;
		case 10: // Ingerimm:
			talent[CharacterTalents.TA_Waffenschmieden.ordinal()]	+= 1;
			break;
		case 11: // Rahja:
			talent[CharacterTalents.TA_Betoeren.ordinal()]			+= 1;
			talent[CharacterTalents.TA_Tanzen.ordinal()]			+= 1;
			talent[CharacterTalents.TA_Musizieren.ordinal()]		+= 1;
			break;
		default:
			break;
		}
	}

	private int schuleinit() {
		if (akademie < 0) {
			return -1;
		}
		CharacterTemplate.AkademieEntry	ap = CharacterTemplate.Akademie[akademie];
		for (CharacterTemplate.CharacterSpellValue v : ap.Sprueche) {
			zauber[v.spell.ordinal()] += v.value;
		}
		return 0;
	}

	/**
	 * Calculate Ausdauer, Magieresistenz and other depending values.
	 */
	public void heldupdate() {
		int mrz1;
		// Ausdauer
		AU = eigenschaft[CharacterAttribute.KK.ordinal()] + LE;
		// Magieresistenz
		switch (typus) {
		case TypMoha		: mrz1 = -3;	break;
		case TypThorwaler	: mrz1 = -2;	break;
		case TypNovadi	:
		case TypNivese	: mrz1 = -1;	break;
		case TypMagier	:
		case TypHexe		:
		case TypDruide	:
		case TypGaukler	:
		case TypZwerg		: mrz1 =  2;	break;
		case TypSchelm	:
		case TypAuelf		:
		case TypWaldelf	: mrz1 =  3;	break;
		case TypFirnelf	: mrz1 =  4;	break;
		default			: mrz1 =  0;	break;
		}
		int mrz2 = 0;
		if (typus.category == CharacterType.Category.Geweiht) {
			if 		(gottheit == Deity.findByIDnum(0))			mrz2 = 3;
			else if (gottheit == Deity.findByIDnum(5))		mrz2 = 2;
			else if (gottheit != null)	mrz2 = 1;
		}
		if (mrz2 > mrz1) mrz1=mrz2;
		mrz2  = talent[CharacterTalents.TA_HeilkundeSeele.ordinal()] - 10;
		if (mrz2 > 0 && mrz2 > mrz1) mrz1=mrz2;
		MR = (eigenschaft[CharacterAttribute.KL.ordinal()] 
				+ eigenschaft[CharacterAttribute.MU.ordinal()] 
				+ ST) / 3 - 2 * eigenschaft[CharacterAttribute.AG.ordinal()] + mrz1;
	}
};
