package de.ped.dsatool.logic;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.ped.dsatool.dsa.generated.HolidayType;
import de.ped.dsatool.dsa.generated.MonthType;
import de.ped.tools.Assert;

/**
 * Contains information about Holidays in all regions of Aventuria.
 * 
 * @author Copyright (c) 1998 Peter Diefenbach (peter@pdiefenbach.de)
 */
public abstract class Holiday {
	/**
	 * @param holiday
	 *            the holiday entry to check
	 * @param date
	 *            the date to check for holidays
	 * @return <code>true</code> if the holiday is held at the given date.
	 */
	public static boolean isAtDate(final HolidayType holiday, final DSADate date) {
		final MonthType month = (MonthType) holiday.getMonth();
		final DSADateCalendarTwelve datecal = new DSADateCalendarTwelve(date);
		final DSADateCalendarTwelve startcal = new DSADateCalendarTwelve((DSADate) date.clone());
		try {
			startcal.setDayMonthYear(holiday.getFirstDay(), month.getIDnum() + 1, startcal.getYear());
		} catch (final DSAException e) {
			Assert.assertTrue(false);
		}
		if (startcal.getDSADate().isAfter(date) && (startcal.getJDay() < datecal.getJDay())) {
			// Oops, there was a new year's change between startDate and d
			// So correct the start date to the year before
			startcal.getDSADate().addDays(-startcal.getDaysPerYear());
		}
		if (startcal.getDSADate().isAfter(date)) {
			return false;
		}
		// Now look for the end date
		final DSADate enddate = (DSADate) startcal.getDSADate().clone();
		final DSADateCalendarTwelve endcal = new DSADateCalendarTwelve(enddate);
		endcal.getDSADate().addDays(holiday.getDuration());
		if (date.isBefore(endcal.getDSADate())) {
			return true;
		}
		return false;
	}

	/**
	 * @param date
	 *            the date to check for holidays
	 * @return A list of all holidays at the given date. Will be empty for most
	 *         dates.
	 */
	public static List<HolidayType> findHolidaysAt(final DSADate date) {
		final Iterator<HolidayType> it = DSA.instance().getHoliday().iterator();
		final LinkedList<HolidayType> ret = new LinkedList<HolidayType>();
		while (it.hasNext()) {
			final HolidayType holiday = (HolidayType) it.next();
			if (isAtDate(holiday, date)) {
				ret.add(holiday);
			}
		}
		return ret;
	}
}
