package de.ped.dsatool.logic;

import java.util.Iterator;
import java.util.Vector;

import de.ped.dsatool.dsa.generated.MonthType;
import de.ped.tools.TextFormatter;

/**
 * Defines a month in the Aventurian calendar.
 * 
 * @author Copyright (c) 2006 Peter Diefenbach (peter@pdiefenbach.de),
 */
public abstract class Month {
	/**
	 * Convenience overload to output the month in human-readable form.
	 * 
	 * @param month
	 *            the month
	 * @return the human-readable representation of the month
	 */
	public static String toString(final MonthType month) {
		return TextFormatter.format(month.getIDnum() + 1, 2) + ": " + month.getName() + " / " + month.getNameInRealLife();
	}

	private static Vector<String> allNames = null;

	/**
	 * @return A vector containing strings describing each object. Useful e.g. for ComboBoxes.
	 */
	public static Vector<String> getAllStrings() {
		if (null == allNames) {
			// First initialize it all, then assign to static object, to avoid multithreading problems.
			final Vector<String> an = new Vector<String>();
			final Iterator<MonthType> it = DSA.instance().getMonth().iterator();
			while (it.hasNext()) {
				final MonthType month = (MonthType)it.next();
				an.add(toString(month));
			}
			allNames = an;
		}
		return allNames;
	}
}
