package de.ped.dsatool.logic;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.ped.tools.MathUtil;
import de.ped.dsatool.dsa.generated.PubNameCombinationType;
import de.ped.dsatool.dsa.generated.PubNameSubjectType;
import de.ped.dsatool.dsa.generated.PubNameType;

/**
 * A generator for good names for pubs and hotels.
 *
 * <p><b>See:</b> "???", Section "???" (p. ??..??)</p>
 *
 * @author Copyright (c) 1999 Peter Diefenbach (peter@pdiefenbach.de),
 *		based on algorithms by Urs Beeli
 */
public abstract class PubNameGenerator {
	private static final String NULLSTRING = "";

	/**
	 * @return A possible name for a pub or hotel, randomly chosen.
	 */
	public static String generatePubName() {
		final StringBuffer s = new StringBuffer();
		final PubNameType pubName = DSA.instance().getPubName();
		if (null != pubName) {
			// Choose a subject
			final List<PubNameSubjectType> subjects = pubName.getSubject();
			final int subjectIndex = MathUtil.random(subjects.size());
			final PubNameSubjectType subject = (PubNameSubjectType)subjects.get(subjectIndex);
			// Choose an adjective (may be empty)
			String adjective = NULLSTRING;
			final List<String> adjectives = pubName.getAdjective();
			if (!adjectives.isEmpty() && (0 < MathUtil.random(10))) { // Sometimes leave out the adjective
				int adjectiveIndex = MathUtil.random(adjectives.size());
				adjective = DSA.toStringValue(adjectives.get(adjectiveIndex));
			}
			// Choose a combination
			final List<PubNameCombinationType> combinations = pubName.getCombination();
			final List<PubNameCombinationType> allowedCombinations = new LinkedList<PubNameCombinationType>();
			final Iterator<PubNameCombinationType> it = combinations.iterator();
			while (it.hasNext()) {
				final PubNameCombinationType combi = (PubNameCombinationType)it.next();
				if (combi.getGender().equals(subject.getGender())) {
					allowedCombinations.add(combi);
				}
			}
			int combinationIndex = MathUtil.random(allowedCombinations.size());
			final PubNameCombinationType combination = (PubNameCombinationType)allowedCombinations.get(combinationIndex);
			// Now build the string
			final String preArticle = combination.getPreArticle();
			if (null != preArticle) {
				s.append(preArticle);
			}
			if (!NULLSTRING.equals(adjective)) { // If there is an adjective, append it.
				s.append(adjective);
				s.append(combination.getAdjectiveEnding());
				s.append(' ');
			}
			s.append(subject.getName());
			// upperize first letter
			if (s.length() >= 1) {
				s.setCharAt(0, Character.toUpperCase(s.charAt(0)));
			}
		}
		return (s.toString());
	}
}
