package de.ped.dsatool.logic;

import java.util.Iterator;

import de.ped.tools.Assert;
import de.ped.tools.MathUtil;
import de.ped.dsatool.dsa.generated.RandomDayEventTableEntryType;

/**
 * Random day events, things that happen to heros and heroines in the everyday struggle.
 *
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Tagesereignisse" (p. ??..??)</p>
 *
 * @author Copyright (c) 1998 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class RandomDayEvent implements Cloneable {
	/**
	 * Creates a day event randomly.
	 */
	public RandomDayEvent() {
		setDiceEyes(Dice.roll(1, 100));
	}

	/**
	 * Creates the day event to happen with given dice eyes.
	 * 
	 * @param diceEyes
	 *            the eyes of the dices thrown
	 */
	public RandomDayEvent(final int diceEyes) {
		setDiceEyes(diceEyes);
	}

	/**
	 * @param diceEyes
	 *            the eyes of the dices thrown
	 */
	public void setDiceEyes(final int diceEyes) {
		this.diceEyes = MathUtil.modulo(diceEyes, 100, 1);
	}

	/**
	 * @return the eyes of the dices thrown.
	 */
	public int getDiceEyes() {
		return diceEyes;
	}

	/**
	 * @return The message that maps to the dice eyes.
	 */
	public String message() {
		Iterator<RandomDayEventTableEntryType> it = DSA.instance().getRandomDayEventTableEntry().iterator();
		while (it.hasNext()) {
			RandomDayEventTableEntryType entry = (RandomDayEventTableEntryType)it.next();
			if (diceEyes <= entry.getEndValue()) {
				return entry.getEvent();
			}
		}
		return "Kein Ereignis";
	}

	/** What the dice showed. */
	private int diceEyes;

	/**
	 * @param obj
	 *            the object to compare with
	 * @return <code>true</code>if they both are random day events, and if the
	 *         message strings are the same.
	 */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RandomDayEvent other = (RandomDayEvent) obj;
		if (!message().equals(other.message()))
			return false;
		return true;
	}

	@Override
	public int hashCode() {
		return message().hashCode();
	}

	@Override
	public RandomDayEvent clone() {
		RandomDayEvent ret = null;
		try {
			ret = (RandomDayEvent)super.clone();
		} catch (final CloneNotSupportedException e) {
			Assert.assertTrue(false);
		}
		return ret;
	}
}
