package de.ped.dsatool.logic;

import java.util.List;

import de.ped.dsatool.dsa.generated.RegionType;

/**
 * A class representing different regions of Aventuria.
 *
 * <p><b>See:</b> "???", Section "???" (p. ??..??)</p>
 *
 * @author Copyright (c) 1999 Peter Diefenbach (peter@pdiefenbach.de),
 *		based on algorithms by Urs Beeli
 */
public abstract class Region {
	public static List<RegionType> findAll() {
		return DSA.instance().getRegion();
	}

	public static RegionType findByIDnum(final int id) {
		return (RegionType)(DSA.findBOByIDnum(findAll(), id));
	}

	public static RegionType findByName(final String name) {
		return (RegionType)(DSA.findBOByName(findAll(), name));
	}

	public static RegionType findByID(final String id) {
		return (RegionType)(DSA.findBOByID(findAll(), id));
	}

	/** The ID of the default region. */
	public static final String DEFAULT_REGION_ID = "REGION_GA";

	/** 
	 * @return the default region.
	 */
	public static RegionType findDefault() {
		final List<RegionType> regions = DSA.instance().getRegion();
		RegionType region = (RegionType)DSA.findBOByID(regions, DEFAULT_REGION_ID);
		if ((null == region) && (!regions.isEmpty())) {
			region = (RegionType)regions.get(0);
		}
		return region;
	}

	/**
	 * @param id
	 *            The ID to search for.
	 * @return the region with the given <em>id</em>, or if that does not exist,
	 *         the default region.
	 */
	public static RegionType findByIDorDefault(final String id) {
		RegionType ret = findByID(id);
		if (null == ret) {
			ret = findDefault();
		}
		return ret;
	}
}
