package de.ped.dsatool.logic;

import java.util.Iterator;

import de.ped.dsatool.dsa.generated.DeityType;
import de.ped.dsatool.dsa.generated.PersonType;
import de.ped.dsatool.dsa.generated.TempleType;
import de.ped.tools.TextFormatter;

/**
 * The concept of a single temple. In a single temple may several deities be worshipped at once.
 *
 * <p><b>See:</b> "Das Handbuch für den Reisenden", Section "Stadt- und Dorfentwicklung" (p.20..23).</p>
 *
 * @author Copyright (c) 1998 Peter Diefenbach (peter@pdiefenbach.de)
 */
public abstract class Temple {
	public static TempleType createTemple(final TempleType sourceTemple) {
		final TempleType newTemple = DSA.instance().createTempleType();
		newTemple.setChief(sourceTemple.getChief());
		newTemple.setComment(sourceTemple.getComment());
		newTemple.getDeities().addAll(sourceTemple.getDeities());
		newTemple.getInhabitant().addAll(sourceTemple.getInhabitant());
		return newTemple;
	}

	/**
	 * @return The correct title for the person being the chief of this building
	 */
	public static String getChiefTitle() {
		return DSAEnv.instance().messages().getText("Temple.Abbot"); 
	}

	/**
	 * @param temple
	 *            the temple
	 * @param tf
	 *            the text formatter to use
	 * @return A string containing a human-readable list of all the deities
	 *         worshipped in this temple.
	 */
	public static String getDeityListString(final TempleType temple, final TextFormatter tf) {
		final StringBuffer ret = new StringBuffer();
		final Iterator<Object> it = temple.getDeities().iterator();
		boolean isFirst = true;
		while (it.hasNext()) {
			final DeityType deity = (DeityType) it.next();
			if (isFirst) {
				isFirst = false;
			} else {
				ret.append(", ");
			}
			ret.append(DSA.instance().getContentText(deity, tf));
		}
		return ret.toString();
	}

	/**
	 * @param temple
	 *            the temple
	 * @param tf
	 *            the text formatter to use
	 * @return A string describing all important aspects of this temple.
	 */
	public static String getContentText(final TempleType temple, final TextFormatter tf) {
		final StringBuffer ret = new StringBuffer();
		final int indent = 9;
		ret.append("\t");
		ret.append(DSAEnv.instance().messages().getText("DSA.Deities"));
		ret.append(": ");
		ret.append(getDeityListString(temple, tf));
		if (null != temple.getComment()) {
			ret.append(" (");
			ret.append(temple.getComment());
			ret.append(")");
		}
		if (null != temple.getChief()) {
			ret.append(TextFormatter.format(getChiefTitle(), indent, TextFormatter.LEFT, '.'));
			ret.append(": ");
			ret.append(DSA.instance().getContentText((PersonType) temple.getChief(), tf));
		}
		return ret.toString();
	}
}
