package de.ped.dsatool.logic;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import de.ped.dsatool.dsa.generated.TerrainType;

/**
 * The different terrains where you can look for plants.
 *
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public abstract class Terrain {
	public static List<TerrainType> findAll() {
		return DSA.instance().getTerrain();
	}

	public static TerrainType findByIDnum(final int id) {
		return (TerrainType)(DSA.findBOByIDnum(findAll(), id));
	}

	public static TerrainType findByName(final String name) {
		return (TerrainType)(DSA.findBOByName(findAll(), name));
	}

	public static TerrainType findByID(final String id) {
		return (TerrainType)(DSA.findBOByID(findAll(), id));
	}

	/**
	 * Convenience overload to output the object in human-readable form.
	 * 
	 * @param o
	 *            the terrain
	 * @return the name of the terrain
	 */
	public static String toString(final TerrainType o) {
		return o.getName();
	}

	private static Vector<String> allNames = null;

	/**
	 * @return A vector containing strings describing each object. Useful e.g. for ComboBoxes.
	 */
	public static Vector<String> getAllStrings() {
		if (null == allNames) {
			// First initialize it all, then assign to static object, to avoid multithreading problems.
			final Vector<String> an = new Vector<String>();
			final Iterator<TerrainType> it = DSA.instance().getTerrain().iterator();
			while (it.hasNext()) {
				final TerrainType t = (TerrainType) it.next();
				an.add(toString(t));
			}
			allNames = an;
		}
		return allNames;
	}
}
