package de.ped.dsatool.logic;

import java.util.Iterator;

import de.ped.dsatool.dsa.generated.TownType;
import de.ped.tools.Messages;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextDocumentType;
import de.ped.tools.Textable;

/**
 * A model to keep the actual town.
 * 
 * @author Copyright (c) 2006 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TownModel implements Textable {
	private TownType town;
	private TownMap map;
	private int selectedIndexForPredefined = -1;

	public TownType getTown() {
		return town;
	}

	public void setTown(final TownType value) {
		town = value;
		map = new TownMap(town);
		recalcSelectedIndexForPredefined();
	}

	@Override
	public String getContentText(final TextDocumentBuilder doc) {
		final Messages messages = doc.messages();
		final TextDocumentBuilder ldoc = new TextDocumentBuilder(messages);
		ldoc.setTextDocumentType(doc.tf().getDocType());
		ldoc.setEmbedded(true);
		Town.addContentText(ldoc, town);
		final int depth = 2 + 1;
		ldoc.addSectionHeading(depth, "TownMap.Map");
		ldoc.add(ldoc.tf().beginPreformatted());
		ldoc.add(map.getContentText(ldoc.tf()));
		ldoc.add(ldoc.tf().endPreformatted());
		return ldoc.result();
	}

	@Override
	public String getHeadingText(final TextDocumentBuilder doc) {
		return DSA.instance().getHeadingText(town, doc.tf());
	}

	public void changeMap() {
		map = new TownMap(town);
		final Messages messages = DSAEnv.instance().messages();
		final TextDocumentBuilder doc = new TextDocumentBuilder(messages);
		doc.setTextDocumentType(TextDocumentType.FIXEDFONT);
		doc.setEmbedded(true);
		doc.add(map.getContentText(doc.tf()));
		town.setMap(doc.result());
	}

	/**
	 * Recalculates the index of this town in the list of predefined towns.
	 * Should be called each time the town name has changed.
	 */
	public void recalcSelectedIndexForPredefined() {
		selectedIndexForPredefined = findSelectedIndexForPredefined(getTown().getName());
	}

	/**
	 * Finds a town name in the list of predefined towns.
	 * 
	 * @param townName
	 *            the name of the town to search for
	 * @return the index of the predefined town with the same name, or -1 if
	 *         there is no such predefined town
	 */
	public static int findSelectedIndexForPredefined(final String townName) {
		int idx = -1;
		if (null != townName) {
			final Iterator<TownType> it = DSA.instance().getTown().iterator();
			int cnt = 0;
			while (it.hasNext() && (idx < 0)) {
				final TownType town = (TownType) it.next();
				if (townName.equalsIgnoreCase(town.getName())) {
					idx = cnt;
				}
				cnt++;
			}
		}
		return idx;
	}

	/**
	 * Returns the index in the list of predefined towns.
	 * 
	 * @return the index of the predefined town with the same name, or -1 if
	 *         there is no such predefined town
	 */
	public int getSelectedIndexForPredefined() {
		return selectedIndexForPredefined;
	}
}
