package de.ped.dsatool.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.dsatool.logic.DSATestExtremlyLuckyException;
import de.ped.dsatool.logic.DSATestExtremlyUnluckyException;
import de.ped.dsatool.logic.DSATestVeryLuckyException;
import de.ped.dsatool.logic.DSATestVeryUnluckyException;
import de.ped.dsatool.logic.Dice;

/**
 * Tests {@link Dice}.
 * 
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DiceTest {
	/*
	 * Class under test for int roll(int, int)
	 */
	@Test
	public void testRollintint() {
		for (int i = 0; i < 10; i++) {
			int r = Dice.roll(1, 6);
			assertTrue(1 <= r);
			assertTrue(6 >= r);
		}
		for (int i = 0; i < 10; i++) {
			int r = Dice.roll(1, 20);
			assertTrue(1 <= r);
			assertTrue(20 >= r);
		}
	}

	protected void assertRightException(final int expectedResult, final int[] roll) {
		try {
			Dice.checkForExceptions(roll);
			assertEquals(0, expectedResult);
		} catch (final DSATestExtremlyLuckyException ex) {
			assertEquals(+2, expectedResult);
		} catch (final DSATestExtremlyUnluckyException ex) {
			assertEquals(-2, expectedResult);
		} catch (final DSATestVeryLuckyException ex) {
			assertEquals(+1, expectedResult);
		} catch (final DSATestVeryUnluckyException ex) {
			assertEquals(-1, expectedResult);
		}
	}

	@Test
	public void testCheckForExceptions() {
		assertRightException( 0, new int[]{});
		assertRightException( 1, new int[]{01});
		assertRightException( 0, new int[]{02});
		assertRightException( 0, new int[]{19});
		assertRightException(-1, new int[]{20});
		assertRightException( 0, new int[]{01, 01});
		assertRightException( 0, new int[]{20, 20});
		assertRightException( 2, new int[]{01, 01, 01});
		assertRightException( 1, new int[]{02, 01, 01});
		assertRightException( 0, new int[]{02, 01, 02});
		assertRightException(-2, new int[]{20, 20, 20});
		assertRightException(-1, new int[]{19, 20, 20});
		assertRightException( 0, new int[]{19, 20, 19});
	}

	@Test
	public void testTestForProperties() {
		assertEquals(  0, Dice.testForProperties(new int[]{}, new int[]{}, 0, 0));
		assertEquals(  2, Dice.testForProperties(new int[]{}, new int[]{}, 5, 3));
		assertEquals(  0, Dice.testForProperties(new int[]{10}, new int[]{10},  0,  0));
		assertEquals(  1, Dice.testForProperties(new int[]{10}, new int[]{11},  0,  0));
		assertEquals(  2, Dice.testForProperties(new int[]{10}, new int[]{11},  1,  0));
		assertEquals( -1, Dice.testForProperties(new int[]{10}, new int[]{11},  1,  3));
		assertEquals( -1, Dice.testForProperties(new int[]{11}, new int[]{10},  0,  0));
		assertEquals( -2, Dice.testForProperties(new int[]{11}, new int[]{10}, -1,  0));
		assertEquals( +1, Dice.testForProperties(new int[]{11}, new int[]{10}, -1, -3));
		assertEquals(  0, Dice.testForProperties(new int[]{10, 10, 10}, new int[]{10, 10, 10},  0,  0));
		assertEquals( -2, Dice.testForProperties(new int[]{10, 12, 10}, new int[]{10, 10, 10},  0,  0));
		assertEquals(  2, Dice.testForProperties(new int[]{10,  8, 10}, new int[]{10, 10, 10},  0,  0));
		assertEquals(  1, Dice.testForProperties(new int[]{10,  8, 10}, new int[]{10, 10, 10}, -1,  0));
		assertEquals(  4, Dice.testForProperties(new int[]{ 7,  8, 10}, new int[]{10, 10, 10}, -1,  0));
		assertEquals( -2, Dice.testForProperties(new int[]{ 7,  8, 12}, new int[]{10, 10, 10},  0,  0));
		assertEquals( -5, Dice.testForProperties(new int[]{ 7, 11, 12}, new int[]{10, 10, 10}, -1,  1));
	}

	public void checkRoll(final int propsSize) {
		final int[] props = new int[propsSize];
		for (int i = 1; i <= 20; i++) {
			for (int j = 0; j < props.length; j++) {
				props[j] = i;
			}
			try {
				final int res = Dice.rollForProperties(props, 0, 0);
				assertTrue(-60 < res);
				assertTrue(res < 60);
			} catch (final DSATestVeryLuckyException e) {
				// Nothing to do
			} catch (final DSATestVeryUnluckyException e) {
				// Nothing to do
			}
		}
	}

	@Test
	public void testRoll() {
		checkRoll(1);
		checkRoll(3);
	}
}
