package de.ped.dsatool.test;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import java.util.Iterator;
import java.util.List;

import de.ped.dsatool.dsa.generated.LanguageType;
import de.ped.dsatool.dsa.generated.PersonType;
import de.ped.dsatool.dsa.generated.SexType;
import de.ped.dsatool.logic.DSA;
import de.ped.dsatool.logic.Language;
import de.ped.dsatool.logic.Person;

/**
 * Tests {@link Person}.
 * 
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PersonTest {

	@BeforeClass
	public static void setUp() {
		dsa = DSA.instance();
	}

	private static DSA dsa;

	@Test
	public void testCreateDefault() {
		PersonType p = Person.createPerson();
		final SexType male = (SexType) dsa.getSex().get(0);
		final SexType female = (SexType) dsa.getSex().get(1);
		assertTrue((male == p.getSex()) || (female == p.getSex()));
		assertEquals(Language.findDefault(), p.getLanguage());
	}

	@Test
	public void testCreatePerson() {
		PersonType p = Person.createPerson();
		PersonType q = Person.createPerson(p);
		assertEquals(p.getSex(), q.getSex());
		assertEquals(p.getLanguage(), q.getLanguage());
		assertEquals(p.getName(), q.getName());
	}

	@Test
	public void testCreateSexLang() {
		List<LanguageType> list = Language.findAll();
		assertTrue(list.size() > 0);
		Iterator<LanguageType> it = list.iterator();
		final SexType male = (SexType) dsa.getSex().get(0);
		final SexType female = (SexType) dsa.getSex().get(1);
		while (it.hasNext()) {
			LanguageType language = (LanguageType)it.next();
			PersonType malePerson = Person.createPerson(male, language);
			assertEquals(male, malePerson.getSex());
			assertEquals(language, malePerson.getLanguage());
			assertNotNull(malePerson.getName());
			PersonType femalePerson = Person.createPerson(female, language);
			assertEquals(female, femalePerson.getSex());
			assertEquals(language, femalePerson.getLanguage());
			assertNotNull(femalePerson.getName());
		}
	}
}
