package de.ped.dsatool.test;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import java.util.List;

import de.ped.dsatool.dsa.generated.PlantType;
import de.ped.dsatool.dsa.generated.RegionType;
import de.ped.dsatool.dsa.generated.TerrainType;
import de.ped.dsatool.logic.DSA;
import de.ped.dsatool.logic.Dice;
import de.ped.dsatool.logic.Plant;
import de.ped.dsatool.logic.Region;
import de.ped.dsatool.logic.Terrain;

/**
 * Tests {@link Plant}.
 * 
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlantTest {

	@BeforeClass
	public static void setUp() {
		dsa = DSA.instance();
	}

	private static DSA dsa;

	/*
	 * Class under test for String[] listOfObjectNames()
	 */
	@Test
	public void testListOfObjectNames() {
		List<PlantType> plants = dsa.getPlant();
		assertNotNull(plants);
		assertTrue(0 < plants.size());
	}

	@Test
	public void testIsThereWithoutTest() {
		final PlantType plant = Plant.findByID("PLANT_ALRAUNE");
		assertNotNull(plant);
		final RegionType bo = Region.findByID("REGION_BO");
		assertNotNull(bo);
		final RegionType wa = Region.findByID("REGION_WA");
		assertNotNull(wa);
		final TerrainType forest    = Terrain.findByID("TERRAIN_FOREST");
		assertNotNull(forest);
		final TerrainType ice       = Terrain.findByID("TERRAIN_ICE");
		assertNotNull(ice);
		final TerrainType grassland = Terrain.findByID("TERRAIN_GRASSLAND");
		assertNotNull(grassland);
		final int[] arrForest = DSA.toIDnumArray(forest);
		assertEquals(1, arrForest.length);
		assertEquals(forest.getIDnum().intValue(), arrForest[0]);
		final int[] arrIce    = DSA.toIDnumArray(ice);
		assertEquals(1, arrIce.length);
		assertEquals(ice.getIDnum().intValue(), arrIce[0]);
		final int[] arrIceGrass = { ice.getIDnum() , grassland.getIDnum() };
		int[] terrainInfo;
		// Positive test
		terrainInfo = Plant.isThere(plant, bo, arrForest, false);
		assertNotNull(terrainInfo);
		assertEquals(forest.getIDnum().intValue(), terrainInfo[0]);
		assertEquals(Dice.VERY_RARE, terrainInfo[1]);
		// Positive test
		terrainInfo = Plant.isThere(plant, bo, arrIceGrass, false);
		assertNotNull(terrainInfo);
		assertEquals(grassland.getIDnum().intValue(), terrainInfo[0]);
		assertEquals(Dice.VERY_RARE, terrainInfo[1]);
		// Negative test: wrong region
		terrainInfo = Plant.isThere(plant, wa, arrForest, false);
		assertNull(terrainInfo);
		// Negative test: wrong terrain
		terrainInfo = Plant.isThere(plant, bo, arrIce, false);
		assertNull(terrainInfo);
	}

	@Test
	public void testWhatPlantsAreThere() {
		final RegionType region = Region.findByID("REGION_BO");
		final TerrainType forest = Terrain.findByID("TERRAIN_FOREST");
		assertNotNull(forest);
		final int[] arrForest = DSA.toIDnumArray(forest);
		assertEquals(1, arrForest.length);
		assertEquals(forest.getIDnum().intValue(), arrForest[0]);
		int[][] arr = Plant.whatPlantsAreThere(region, arrForest, 15, false, false, null);
		assertNotNull(arr);
		assertEquals(dsa.getPlant().size(), arr.length);
	}
}
