package de.ped.dsatool.test;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import java.util.List;

import de.ped.dsatool.dsa.generated.RegionType;
import de.ped.dsatool.dsa.generated.TempleType;
import de.ped.dsatool.dsa.generated.TownType;
import de.ped.dsatool.logic.DSA;
import de.ped.dsatool.logic.Town;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Tests {@link Town}.
 * 
 * @author Copyright (c) 2004 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TownTest {

	protected void createTowns(String[] townNames) {
		assertTrue(townNames.length > 0);
		for (int i = 0; i < townNames.length; i++) {
			TownType t = Town.findPredefinedTown(townNames[i]);
			assertNotNull(t);
		}
	}

	@BeforeClass
	public static void setUp() {
		Logger.getRootLogger().setLogLevel(Level.WARN);
		DSA.instance();
	}

	protected static final String NAME_ALBENHUS = "Albenhus";

	protected void checkAlbenhus(TownType albenhus) {
		assertNotNull(albenhus);
		assertEquals(NAME_ALBENHUS, albenhus.getName());
		assertEquals(2200, albenhus.getNumberOfInhabitants().intValue());
		RegionType region = (RegionType)albenhus.getRegion();
		assertNotNull(region);
		assertEquals("REGION_ZW", region.getID());
		List<TempleType> temples = albenhus.getTemple();
		assertNotNull(temples);
		assertTrue(5 <= temples.size());
	}

	@Test
	public void testCreateTown() {
		TownType albenhus = Town.findPredefinedTown(NAME_ALBENHUS);
		checkAlbenhus(albenhus);
		TownType clone = Town.create(albenhus);
		checkAlbenhus(clone);
	}
}
