package de.ped.empire.dev;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import de.ped.empire.logic.ZoomLevel;
import de.ped.tools.TopologyMode;
import de.ped.empire.logic.ImageKey;

/**
 * Helper for building the Empire project. Reads in image templates files and
 * image files for all zoom sizes for squares, and outputs them for hexagon
 * maps.
 * 
 * @author Copyright (c) 2020 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ImgSquareTemplatesNew extends ImgTemplates {
	public static void main(final String[] args) {
		ImgTemplates task = new ImgSquareTemplatesNew();
		task.executeMain(args);
	}

	protected ImgSquareTemplatesNew() {
		props.setTopologyMode(TopologyMode.SQUARE8);
	}

	/** Adds a prefix to the file name and changes the filename extension to png. */
	@Override
	public String getDestFilename(final String srcFilename) {
		return changeFilename(srcFilename, "map", "", "png");
	}

	@Override
	public String getName() {
		return "Updating Empire square icon templates";
	}

	@Override
	protected int getBits() {
		return 4;
	}

	@Override
	protected void copyBlock(final BufferedImage destImg, final BufferedImage srcImg, final ZoomLevel zoomLevel,
			final ImageKey imageKey, final int variant) {
		final Point d = getDestIconPos(imageKey, variant);
		final Rectangle s = getSrcIconRect(imageKey, variant);
		convertBlock(destImg, srcImg, zoomLevel, d.x, d.y, s.x, s.y, s.width, s.height);
	}

	@Override
	protected int[][] calcCopyParamsUpDownSides(final ZoomLevel zoomLevel) {
		return appendCopyParams(copyParamsUpDown, copyParamsSides);
	}

	private final int[][] copyParamsUpDown = new int[][] { // dest pos, source pos
		{ 1, 0, 1, 0, ALL, },
		{ 0, 1, 2, 0, ALL, },
		{ 1, 1, 3, 0, ALL, },
	};

	final int[][] copyParamsSides = new int[][] {
		{ 2, 0, 0, 1, ALL, },
		{ 4, 0, 0, 1, ALL, },
	};
}
