package de.ped.empire.gui;

import de.ped.empire.logic.ProcessorSpeedType;
import de.ped.tools.Assert;
import de.ped.tools.MathUtil;

/**
 * The state of the crosshair or unit blinking.
 * If you don't want to blink and just want to display permanently, 
 * just create or call {@link #reset()}, and never call {@link #step(ProcessorSpeedType)}.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class BlinkingState {
	public static final int[] DURATION_IN_STEPS = { 3, 6 };
	private boolean isVisible;
	private int counter;

	public BlinkingState() {
		reset();
	}

	public void reset() {
		isVisible = true;
		counter = 0;
	}

	public boolean isVisible() {
		return isVisible;
	}

	/** 
	 * Executes one step on this object.
	 * @param pst the processor speed type, to adapt number of steps of visualizations to the step duration
	 * @return <code>true</code> if the value of {@link #isVisible()} changed its value
	 */
	public boolean step(final ProcessorSpeedType pst) {
		Assert.assertEquals(ProcessorSpeedType.values().length, DURATION_IN_STEPS.length);
		counter = MathUtil.modulo(counter - 1, DURATION_IN_STEPS[pst.ordinal()]);
		boolean ret = false;
		if (0 == counter) {
			isVisible = !isVisible;
			ret = true;
		}
		return ret;
	}
}