package de.ped.empire.gui;

import java.io.File;
import java.util.Collection;

import de.ped.empire.logic.CircularReferenceException;
import de.ped.empire.logic.Commandable;
import de.ped.empire.logic.CommandableKey;
import de.ped.empire.logic.CommandableTemplate;
import de.ped.empire.logic.GameFacade;
import de.ped.empire.logic.GameObserver;
import de.ped.empire.logic.GameFacade.SoundToPlayData;
import de.ped.empire.logic.GameFacade.UserMapTileContextOptions;
import de.ped.empire.logic.GameMapInfo;
import de.ped.empire.logic.GameProperties;
import de.ped.empire.logic.GameState;
import de.ped.empire.logic.GameViewProperties;
import de.ped.empire.logic.MapViewMode;
import de.ped.empire.logic.MovePath;
import de.ped.empire.logic.Order;
import de.ped.empire.logic.OrderTemplate;
import de.ped.empire.logic.ProcessorSpeedType;
import de.ped.empire.logic.Tileable;
import de.ped.empire.logic.TutorialLevel;
import de.ped.empire.logic.Unit;
import de.ped.tools.ApplicationEnvironment;
import de.ped.tools.DigitalJoystickDirection;
import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.PlayfieldPosition;

/**
 * The proxy for the facade looking at the game state.
 * 
 * @author Copyright (c) 2022 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GameFacadeProxy {
	public void setEnvironment(ApplicationEnvironment applicationEnvironment) {
		gameFacade.setEnvironment(applicationEnvironment);
	}

	/* TODO: Remove when transition is done */
	GameFacade getGameFacade() {
		return gameFacade;
	}

	private GameFacade gameFacade = new GameFacade();

	public PlayfieldPosition getPositionInFocus(final int playerId) {
		return gameFacade.getPositionInFocus(playerId);
	}

	public UserMapTileContextOptions collectMapTileContextOptions(final int playerId, final PlayfieldPosition position) {
		return gameFacade.collectMapTileContextOptions(playerId, position);
	}

	public GameProperties getGameProperties() {
		return gameFacade.getGameProperties();
	}

	public boolean isPerspectiveEnabled(final int playerId, final int perspectiveId) {
		return gameFacade.isPerspectiveEnabled(playerId, perspectiveId);
	}

	public Commandable getVisibleCommandableInFocus(final int playerId) {
		return gameFacade.getVisibleCommandableInFocus(playerId);
	}

	public boolean isOrderActionEnabled(final int playerId, final Order order) {
		return gameFacade.isOrderActionEnabled(playerId, order);
	}

	public boolean isOrderActionTargetModeEnabled(final int playerId, final OrderTemplate template) {
		return gameFacade.isOrderActionTargetModeEnabled(playerId, template);
	}

	public GameState getGameState() {
		return gameFacade.getGameState();
	}

	public int createGame(final GameObserver observer, final int localeIndex, final GameProperties props, final TutorialLevel tutorialLevel) {
		/** Hint: Setter/Action */
		return gameFacade.createGame(observer, localeIndex, props, tutorialLevel);
	}

	public GameViewProperties getGameViewProperties(final int playerId) {
		return gameFacade.getGameViewProperties(playerId);
	}

	public boolean isGameRunning() {
		return (null != gameFacade.getGameState());
	}

	public Order getMoveOrAttackOrder(final Unit unit, final PlayfieldPosition destination) {
		return gameFacade.getMoveOrAttackOrder(unit, destination);
	}

	public boolean step(final int playerId, final ProcessorSpeedType pst) {
		/** Hint: Setter/Action */
		return gameFacade.step(playerId, pst);
	}

	public void stepIfNot(int playerId, final ProcessorSpeedType pst) {
		/** Hint: Setter/Action */
		gameFacade.stepIfNot(playerId, pst);
	}

	public int getCurrentPlayerId() {
		try {
			return gameFacade.getGameState().getGameStateBasics().getPlayerId();
		} catch (final NullPointerException e) {
			return -1;
		}
	}

	public int getTurn() {
		try {
			return gameFacade.getGameState().getGameStateBasics().getTurn();
		} catch (final NullPointerException e) {
			return -1;
		}
	}

	public SoundToPlayData getSoundToPlay(final int playerId) {
		return gameFacade.getSoundToPlay(playerId);
	}

	public boolean isPlayerAlive(final int playerId) {
		return gameFacade.isPlayerAlive(playerId);
	}

	public boolean isPlayerWinner(final int playerId) {
		/** TODO: Merge this in structure with {@link #isPlayerAlive} */
		return gameFacade.isPlayerWinner(playerId);
	}

	public boolean finishTurn(final int playerId) {
		/** Hint: Setter/Action */
		return gameFacade.finishTurn(playerId);
	}

	public void autoSaveGame() {
		/** Hint: Setter/Action */
		gameFacade.autoSaveGame();
	}

	public void nextCommandable(final int playerId) {
		/** Hint: Setter/Action */
		gameFacade.nextCommandable(playerId);
	}

	public void nextCommandable(final int playerId, final Commandable commandable) {
		/** Hint: Setter/Action */
		gameFacade.nextCommandable(playerId, commandable);
	}

	public boolean onActionSelectOrderMakesSense(int playerId, Order order) {
		/** Hint: Setter/Action (why?) */
		return gameFacade.onActionSelectOrderMakesSense(playerId, order);
	}

	public void setOrder(final Commandable commandable, final Order order) {
		/** Hint: Setter/Action */
		gameFacade.setOrder(commandable, order);
	}

	public void setOrder(int playerId, Order order) {
		/** Hint: Setter/Action */
		gameFacade.setOrder(playerId, order);
	}

	public void setPositionInFocus(int playerId, PlayfieldPosition position) {
		/** Hint: Setter/Action */
		gameFacade.setPositionInFocus(playerId, position);
	}

	public void setStatusMessageToField(int playerId, PlayfieldPosition position) {
		/** Hint: Setter/Action */
		gameFacade.setStatusMessageToField(playerId, position);
	}

	public Tileable getOrderOptionsObject(int playerId, PlayfieldPosition position) {
		return gameFacade.getOrderOptionsObject(playerId, position);
	}

	public void setStatusMessageToToolTip(int playerId, Tileable tileable) {
		/** Hint: Setter/Action */
		gameFacade.setStatusMessageToToolTip(playerId, tileable);
	}

	public Order actionTargetSelectionAtPosition(int playerId, OrderTemplate targetSelection, PlayfieldPosition pos) throws CircularReferenceException {
		/** Hint: Setter/Action */
		return gameFacade.actionTargetSelectionAtPosition(playerId, targetSelection, pos);
	}

	public GameMapInfo createGameMapInfo(int playerId) {
		return gameFacade.createGameMapInfo(playerId);
	}

	public MapViewMode getMapViewMode(int playerId) {
		return gameFacade.getMapViewMode(playerId);
	}

	public Collection<MovePath> getMovePaths(int playerId) {
		return gameFacade.getMovePaths(playerId);
	}

	public String getStatusMessage(int playerId) {
		return gameFacade.getStatusMessage(playerId);
	}

	public boolean isFinishTurnEnabled(int playerId) {
		return gameFacade.isFinishTurnEnabled(playerId);
	}

	public boolean isAcceptingUserInput(int playerId) {
		return gameFacade.isAcceptingUserInput(playerId);
	}

	public Commandable findCommandable(int playerId, CommandableKey key) {
		return gameFacade.findCommandable(playerId, key);
	}

	public int getPerspective(int playerId) {
		return gameFacade.getPerspective(playerId);
	}

	public Commandable getUnitProduced(int playerId, Commandable commandable) {
		return gameFacade.getUnitProduced(playerId, commandable);
	}

	public void exitGame(final GameObserver observer) {
		/** Hint: Setter/Action */
		gameFacade.exitGame(observer);
	}

	public void resign(int playerId) {
		/** Hint: Setter/Action */
		gameFacade.resign(playerId);
	}

	public void clearUnitProduced(final int playerId, final Commandable commandable) {
		/** Hint: Setter/Action */
		gameFacade.clearUnitProduced(playerId, commandable);
	}

	public void setCityProduction(final int playerId, final CommandableKey cityKey, final CommandableTemplate unitToProduce, final boolean isChange) {
		/** Hint: Setter/Action */
		gameFacade.setCityProduction(playerId, cityKey, unitToProduce, isChange);
	}

	public void clearGetReady(int playerId) {
		/** Hint: Setter/Action */
		gameFacade.clearGetReady(playerId);
	}

	public boolean setMovePaths(final Commandable commandable, final Object[][] data, final boolean isToCleanup) {
		/** Hint: Setter/Action */
		return gameFacade.setMovePaths(commandable, data, isToCleanup);
	}

	public void popStatusMessage(int playerId) {
		/** Hint: Setter/Action */
		gameFacade.popStatusMessage(playerId);
	}

	public void pushStatusMessage(int playerId, I18NStringWithFillIns statusMessage) {
		/** Hint: Setter/Action */
		gameFacade.pushStatusMessage(playerId, statusMessage);
	}

	public void setGameViewProperties(int playerId, GameViewProperties gamesViewProperties) {
		/** Hint: Setter/Action */
		gameFacade.setGameViewProperties(playerId, gamesViewProperties);
	}

	public void setPerspective(int playerId, int perspectiveId) {
		/** Hint: Setter/Action */
		gameFacade.setPerspective(playerId, perspectiveId);
	}

	public void setMapViewMode(int playerId, MapViewMode mode) {
		/** Hint: Setter/Action */
		gameFacade.setMapViewMode(playerId, mode);
	}

	public boolean isMapViewModeEnabled(int playerId, MapViewMode mode) {
		return gameFacade.isMapViewModeEnabled(playerId, mode);
	}

	public boolean isInMoveOver() {
		return gameFacade.isInMoveOver();
	}

	public void saveGame(final GameObserver observer, final File file) {
		/** Hint: Setter/Action */
		gameFacade.saveGame(observer, file);
	}

	public void loadGame(final GameObserver observer, final File file) {
		/** Hint: Setter/Action */
		gameFacade.loadGame(observer, file);
	}

	public void setZoomLevel(int playerId, int zoomLevel) {
		/** Hint: Setter/Action */
		gameFacade.setZoomLevel(playerId, zoomLevel);
	}

	public void setLocaleIndex(int playerId, int localeIndex) {
		/** Hint: Setter/Action */
		gameFacade.setLocaleIndex(playerId, localeIndex);
	}

	public void setPositionInFocus(int playerId, DigitalJoystickDirection direction) {
		/** Hint: Setter/Action */
		gameFacade.setPositionInFocus(playerId, direction);
	}

	public boolean isDefaultPerspective(int playerId) {
		return gameFacade.isDefaultPerspective(playerId);
	}
}
