package de.ped.empire.gui;

import java.awt.BorderLayout;
import java.awt.Frame;

import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.TableColumnModel;
import de.ped.tools.gui.TableModel;
import de.ped.tools.gui.TableTools;

/**
 * A dialog to show the log book for a player.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LogReportDialog extends PedJDialog {

	private static final long serialVersionUID = 1L;

	/**
	 * Represents for the log book the content of the table to display.
	 * 
	 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class LogbookTableModel extends TableModel {
		private static final long serialVersionUID = 1L;
		
		private static final TableColumnModel[] columns = new TableColumnModel[]{
			new TableColumnModel("Game.Logbook.Heading.Turn",       Integer.class,  5), 
			new TableColumnModel("Game.Logbook.Heading.Player",      String.class, 10), 
			new TableColumnModel("Game.Logbook.Heading.Position",    String.class,  5),
			new TableColumnModel("Game.Logbook.Heading.Commandable", String.class, 10), 
			new TableColumnModel("Game.Logbook.Heading.Entry",       String.class, 30),
		};

		final Object[][] data;
		
		public LogbookTableModel(final Object[][] data, final Messages msg) {
			super(msg);
			this.data = data;
		}

		@Override
		public TableColumnModel[] getColumns() {
			return columns;
		}

		@Override
		public int getRowCount() {
			return data.length;
		}

		@Override
		public Object getValueAt(final int rowIndex, final int columnIndex) {
			Object ret = data[rowIndex][columnIndex];
			if (ret instanceof I18NStringWithFillIns) {
				ret = msg.getString((I18NStringWithFillIns) ret);
			}
			return ret;
		}

		@Override
		public boolean isCellEditable(final int row, final int col) {
			return false;
		}
	}

	public LogReportDialog(final ApplicationMainWindow mainWindow, final Frame parent, final int playerId) {
		super(mainWindow, parent);
		final EmpireMainWindow empWindow = (EmpireMainWindow) mainWindow;
		setTitle(getMessages().getString("Game.Logbook.Title"));
		final Object[][] data = empWindow.getGameFacade().getLogbookData(playerId);
		final LogbookTableModel model = new LogbookTableModel(data, empWindow.getMessages());
		final JTable table = new JTable(model);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setFocusable(false);
		TableTools.adjustColumns(table, 20, empWindow.getApplicationEnvironment().resources());
		ApplicationMainWindow.adjustRowHeightsToRenderers(table);

		final JScrollPane scrollPane = new JScrollPane(table);
		// scrollPane.setMinimumSize(new Dimension(100, 20));

		add(scrollPane, BorderLayout.CENTER);
		setOptionType(JOptionPane.DEFAULT_OPTION);
		finishLayout();
	}

	public void enterDialog(final int playerId) {
		enterDialog();
	}
}
