package de.ped.empire.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.ped.tools.log.Logger;

/**
 * Acts as logger for as many events as possible, for debugging purposes.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LoggingListener
		implements ActionListener, MouseListener, FocusListener, ChangeListener, ComponentListener {

	protected final String name;
	protected final Logger logger;

	public LoggingListener(final String name) {
		this.name = name;
		this.logger = Logger.getLogger(name);
	}

	public void addAsListener(final JButton c) {
		c.addActionListener(this);
		c.addComponentListener(this);
		c.addFocusListener(this);
		c.addMouseListener(this);
	}

	@Override
	public void focusGained(final FocusEvent e) {
		logger.debug("focusGained");
	}

	@Override
	public void focusLost(final FocusEvent e) {
		logger.debug("focusLost");
	}

	@Override
	public void mouseClicked(final MouseEvent e) {
		logger.debug("mouseClicked");
	}

	@Override
	public void mouseEntered(final MouseEvent e) {
		logger.debug("mouseEntered");
	}

	@Override
	public void mouseExited(final MouseEvent e) {
		logger.debug("mouseExited");
	}

	@Override
	public void mousePressed(final MouseEvent e) {
		logger.debug("mousePressed");
	}

	@Override
	public void mouseReleased(final MouseEvent e) {
		logger.debug("mouseReleased");
	}

	@Override
	public void actionPerformed(final ActionEvent e) {
		logger.debug("actionPerformed");
	}

	@Override
	public void stateChanged(final ChangeEvent e) {
		logger.debug("stateChanged");
	}

	@Override
	public void componentHidden(final ComponentEvent e) {
		logger.debug("componentHidden");
	}

	@Override
	public void componentMoved(final ComponentEvent e) {
		logger.debug("componentMoved");
	}

	@Override
	public void componentResized(final ComponentEvent e) {
		logger.debug("componentResized");
	}

	@Override
	public void componentShown(final ComponentEvent e) {
		logger.debug("componentShown");
	}

}
