package de.ped.empire.gui;

import java.awt.BorderLayout;

import javax.swing.JOptionPane;

import de.ped.empire.logic.GameProperties;
import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.PropertyElement;

/**
 * The dialog to start a new game.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionDialog extends PedJDialog {
	private static final long serialVersionUID = 1L;

	private NewGameSelectionPaneller paneller;

	protected NewGameSelectionDialog(
			final ApplicationMainWindow parent, 
			final boolean isEditable) {
		super(parent, parent);
		final Messages messages = getMessages();
		setTitle(messages.getString("Action.ViewGameProperties.Text")); 
		paneller = new NewGameSelectionPaneller(this, isEditable);

		setResizable(false);
		setTitlePanel(messages.getString("GamePropertiesWindow.Heading"));

		add(paneller.mainPanel, BorderLayout.CENTER);
		if (isEditable) {
			setOptionType(JOptionPane.OK_CANCEL_OPTION);
		} else {
			setOptionType(JOptionPane.DEFAULT_OPTION);
		}
		finishLayout();

		updateUIFieldsBasedOnProperties();
		paneller.updateEditable(isEditable);
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		paneller.updateUIFieldsBasedOnProperties();
	}

	@Override
	public void readUIFieldsIntoProperties() {
		paneller.readUIFieldsIntoProperties();
	}

	public GameProperties getModel() {
		return paneller.getModel();
	}

	public PropertyElement[] getTableProperties() {
		return paneller.getTableProperties();
	}

	public PlayersSelectionTableModel getTableModel() {
		return paneller.getTableModel();
	}
}
