package de.ped.empire.gui;

import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.Wizard;
import de.ped.tools.gui.WizardPanelDescriptor;

/**
 * For the new game selection wizard, this is the first panel to be shown. 
 * It allows the user to pick between a tutorial or a conventional game.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionPanelGameType extends WizardPanelDescriptor implements ChangeListener {
	public static final String ID = "GameType";

	public NewGameSelectionPanelGameType(final Wizard wizard) {
		super(wizard);
		final ApplicationMainWindow mainWindow = (ApplicationMainWindow) wizard.getParent();

		final JPanel panel = new JPanel();
		final ButtonGroup group = new ButtonGroup();
		final String[] keys = new String[] { "GameProperties.Tutorial", "GameProperties.FullGame", };
		buttons = new JRadioButton[keys.length];
		for (int i = 0; i < keys.length; i++) {
			final JRadioButton button = mainWindow.createRadioButton(new I18NStringWithFillIns(keys[i], null, null));
			buttons[i] = button;
			group.add(button);
			panel.add(button);
			button.addChangeListener(this);
		}
		setPanelComponent(panel);
		setPanelId(ID);
	}

	@Override
	public void stateChanged(ChangeEvent event) {
		update();
	}

	@Override
	protected void updateNavDescriptors() {
		if (buttons[0].isSelected()) {
			setNextPanelId(FINISH_ID);
		} else if (buttons[1].isSelected()) {
			setNextPanelId(NewGameSelectionPanelGameProps.ID);
		} else {
			setNextPanelId(null);
		}
	}

	public boolean isTutorial() {
		return buttons[0].isSelected();
	}

	private final JRadioButton[] buttons;
}