package de.ped.empire.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;

import de.ped.empire.logic.GameProperties;
import de.ped.empire.logic.PlayerHandicap;
import de.ped.empire.logic.PlayerType;
import de.ped.empire.logic.RulesComplexity;
import de.ped.tools.BorderMode;
import de.ped.tools.Messages;
import de.ped.tools.PlayerColor;
import de.ped.tools.PlayfieldPosition;
import de.ped.tools.ResourceFinder;
import de.ped.tools.TopologyMode;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.JIconComboBox;
import de.ped.tools.gui.JInfoDialog;
import de.ped.tools.gui.JSliderWithTextFieldPanel;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.PlayerColorEditor;
import de.ped.tools.gui.PlayerColorRenderer;
import de.ped.tools.gui.PropertyElement;
import de.ped.tools.gui.TableColumnModel;
import de.ped.tools.gui.TableRowNumberRenderer;
import de.ped.tools.gui.TableStringRenderer;
import de.ped.tools.gui.TableTools;

/**
 * Builds the main panel for all the game options for starting a new game.
 * Needed to be separated to be both part of a dialog and of a wizard.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionPaneller {

	public NewGameSelectionPaneller(final PedJDialog dialog, final boolean isEditable) {
		final Messages messages = dialog.getMessages();
		final EmpireMainWindow mainWindow = (EmpireMainWindow) (dialog.getParent());
		this.dialog = dialog;
		comboRulesComplexity = mainWindow.createComboBox((Enum<?>[]) RulesComplexity.values(), RulesComplexity.KEY,
				null, ResourceFinder.Folder.APP);
		comboTopologyMode = mainWindow.createComboBox((Enum<?>[]) EmpireMainWindow.TOPOLOGY_MODES, TopologyMode.KEY,
				null, ResourceFinder.Folder.COMMON);
		comboBorderMode = mainWindow.createComboBox((Enum<?>[]) BorderMode.values(), BorderMode.KEY, null,
				ResourceFinder.Folder.COMMON);
		sliderXsize         = new JSliderWithTextFieldPanel(dialog, GameProperties.SLIDERDEF_SIZE, "GameProperties.MapSizeX", ResourceFinder.Folder.COMMON);
		sliderYsize         = new JSliderWithTextFieldPanel(dialog, GameProperties.SLIDERDEF_SIZE, "GameProperties.MapSizeY", ResourceFinder.Folder.COMMON);
		sliderCityDensity   = new JSliderWithTextFieldPanel(dialog, GameProperties.SLIDERDEF_CONTENT, "GameProperties.CityDensity", ResourceFinder.Folder.APP);
		sliderLandDensity   = new JSliderWithTextFieldPanel(dialog, GameProperties.SLIDERDEF_CONTENT, "GameProperties.LandDensity", ResourceFinder.Folder.APP);
		sliderContinentSize = new JSliderWithTextFieldPanel(dialog, GameProperties.SLIDERDEF_CONTENT, "GameProperties.ContinentSize", ResourceFinder.Folder.APP);
		textTimestamp.setFocusable(false);
		textTimestamp.setEditable(false);

		dialog.setPropertyElements(new PropertyElement[] {
			new PropertyElement(RulesComplexity.KEY, comboRulesComplexity),
			new PropertyElement("GameProperties.MapSizeX", sliderXsize),
			new PropertyElement("GameProperties.MapSizeY", sliderYsize),
			new PropertyElement(TopologyMode.KEY, comboTopologyMode),
			new PropertyElement(BorderMode.KEY, comboBorderMode),
			new PropertyElement("GameProperties.CityDensity", sliderCityDensity),
			new PropertyElement("GameProperties.LandDensity", sliderLandDensity),
			new PropertyElement("GameProperties.ContinentSize", sliderContinentSize),
			new PropertyElement("GameProperties.Quickstart", checkIsQuickstart),
			new PropertyElement("GameProperties.Name", textName),
			new PropertyElement("GameProperties.Timestamp", textTimestamp),
		} );

		model = (GameProperties) mainWindow.getGameProperties().clone();
		if (isEditable) {
			model.setTimestampEmpty();
			model.setName("");
		}

		final JPanel mapPropertiesPanel = new JPanel();
		{
			final GridBagLayout gridbag = new GridBagLayout();
			final GridBagConstraints c = new GridBagConstraints();
			c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
			mapPropertiesPanel.setLayout(gridbag);
			final Border lineBorder = BorderFactory.createLineBorder(Color.gray);
			mapPropertiesPanel.setBorder(BorderFactory.createTitledBorder(lineBorder, messages.getText("GameProperties.World")));
			dialog.addAllPropertyElementsTo(mapPropertiesPanel, c);
		}
		final JPanel playersPanel = new JPanel();
		{
			playersPanel.setName("Players");
			playersPanel.setLayout(new BorderLayout());
			final Border lineBorder = BorderFactory.createLineBorder(Color.gray);
			playersPanel.setBorder(BorderFactory.createTitledBorder(lineBorder, messages.getText("GameProperties.Players")));
			tableModel = new PlayersSelectionTableModel(model, messages, isEditable);
			table = new JTable(tableModel);
			table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			table.setDefaultRenderer(PlayerColor.class, new PlayerColorRenderer(messages));
			table.setDefaultEditor(PlayerColor.class, new PlayerColorEditor(messages));
			table.setDefaultRenderer(PlayerType.class, new PlayerTypeRenderer(messages));
			table.setDefaultEditor(PlayerType.class, new PlayerTypeEditor(messages));
			table.setDefaultRenderer(PlayerHandicap.class, new PlayerHandicapRenderer(messages));
			table.setDefaultEditor(PlayerHandicap.class, new PlayerHandicapEditor(messages));
			table.setDefaultRenderer(Integer.class, new TableRowNumberRenderer());
			table.setDefaultRenderer(String.class, new TableStringRenderer());
			TableTools.adjustColumns(table, 5, mainWindow.getApplicationEnvironment().resources());
			ApplicationMainWindow.adjustRowHeightsToRenderers(table);
			final JPanel tablePanel = new JPanel();
			tablePanel.setLayout(new BorderLayout());
			tablePanel.add(table, BorderLayout.CENTER);
			tablePanel.add(table.getTableHeader(), BorderLayout.NORTH);
			playersPanel.add(tablePanel, BorderLayout.CENTER);
		}
		mainPanel = dialog.createMainPanel();
		{
			final JPanel summaryPanel = new JPanel();
			summaryPanel.setLayout(new BorderLayout());
			summaryPanel.add(mapPropertiesPanel, BorderLayout.CENTER);
			summaryPanel.add(playersPanel, BorderLayout.SOUTH);
			final BorderLayout mainPanelLayout = new BorderLayout();
			mainPanel.setLayout(mainPanelLayout);
			mainPanel.add(new JScrollPane(summaryPanel), BorderLayout.CENTER);
		}
	}

	public void updateEditable(boolean isEditable) {
		if (!isEditable) {
			final JLabel titleLabel = dialog.getTitleLabel();
			if (null != titleLabel) {
				titleLabel.setEnabled(false);
			}
			for (final JComboBox<?> combo : new JComboBox[] { comboRulesComplexity, comboTopologyMode, comboBorderMode, } ) {
				combo.setEditable(false);
				combo.setEnabled(false);
			}
			for (final JTextField field : new JTextField[] { /* textXsize, textYsize, textCityDensity, textLandDensity, textContinentSize, */ textName, textTimestamp, } ) {
				field.setEditable(false);
				field.setFocusable(false);
			}
			for (final JCheckBox field : new JCheckBox[] { checkIsQuickstart, } ) {
				field.setEnabled(false);
				field.setFocusable(false);
			}
			for (final JSliderWithTextFieldPanel field : new JSliderWithTextFieldPanel[] { sliderXsize, sliderYsize, sliderCityDensity, sliderLandDensity, sliderContinentSize, } ) {
				field.setEnabled(false);
			}
			table.setEnabled(false);
		}
	}

	public void updateUIFieldsBasedOnProperties() {
		comboRulesComplexity.setSelectedIndex(model.getRulesComplexity().ordinal());
		sliderXsize.setValue(model.getSize().getX());
		sliderYsize.setValue(model.getSize().getY());
		sliderCityDensity.setValue(model.getCityDensity());
		sliderLandDensity.setValue(model.getLandRatio());
		sliderContinentSize.setValue(model.getContinentSize());
		final TopologyMode mode = model.getTopologyMode();
		int idx = 0;
		for (int i = 0; i < EmpireMainWindow.TOPOLOGY_MODES.length; i++) {
			if (EmpireMainWindow.TOPOLOGY_MODES[i] == mode) {
				idx = i;
				break;
			}
		}
		comboTopologyMode.setSelectedIndex(idx);
		comboBorderMode.setSelectedIndex(model.getBorderMode().ordinal());
		textName.setText(model.getName());
		textName.setColumns(20);
		final String timestr;
		final long timestamp = model.getTimestamp();
		if (0L == timestamp) {
			final Messages messages = dialog.getMessages();
			timestr = messages.getText("GameProperties.Timestamp.Never.Text");
		} else {
			timestr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(timestamp));
		}
		textTimestamp.setText(timestr);
		textTimestamp.setColumns(20);
		checkIsQuickstart.setSelected(model.isQuickstart());
	}

	public void readUIFieldsIntoProperties() {
		model.setRulesComplexity(RulesComplexity.values()[comboRulesComplexity.getSelectedIndex()]);
		model.setSize(PlayfieldPosition.X, sliderXsize.getValue());
		model.setSize(PlayfieldPosition.Y, sliderYsize.getValue());
		model.setCityDensity(sliderCityDensity.getValue());
		model.setLandRatio(sliderLandDensity.getValue());
		model.setContinentSize(sliderContinentSize.getValue());
		model.setTopologyMode(EmpireMainWindow.TOPOLOGY_MODES[comboTopologyMode.getSelectedIndex()]);
		model.setBorderMode(BorderMode.values()[comboBorderMode.getSelectedIndex()]);
		model.setName(textName.getText());
		if (0L == model.getTimestamp()) {
			model.setTimestampNow();
		}
		model.setQuickstart(checkIsQuickstart.isSelected());
	}

	public JInfoDialog getExceptionDialog() {
		final JInfoDialog ret = new JInfoDialog(dialog, 
				dialog.getMessages().getString("GameProperties.Players.Exception.NoHuman.Title.Text"),
				dialog.getMessages().getString("GameProperties.Players.Exception.NoHuman.Message.Text"),
				JOptionPane.ERROR_MESSAGE);
		return ret;
	}

	public GameProperties getModel() {
		return model;
	}

	public PropertyElement[] getTableProperties() {
		final PropertyElement[] ret = new PropertyElement[tableModel.getColumnCount()];
		for (int i = 0; i < ret.length; i++) {
			final TableColumnModel tcm = tableModel.getColumns()[i];
			Component comp = null;
			final String n1 = tcm.clazz.getName();
			final String n2 = PlayerType.class.getName();
			if (n1.equals(n2)) {
				comp = new PlayerTypeRenderer(dialog.getMessages());
			}
			final PropertyElement pe = new PropertyElement(tcm.i18nKey, comp);
			ret[i] = pe;
		}
		return ret;
	}

	public PlayersSelectionTableModel getTableModel() {
		return tableModel;
	}

	final JPanel mainPanel;
	final PedJDialog dialog;

	private final JIconComboBox<?> comboRulesComplexity;
	private final JIconComboBox<?> comboTopologyMode;
	private final JIconComboBox<?> comboBorderMode;
	private final JSliderWithTextFieldPanel sliderXsize;
	private final JSliderWithTextFieldPanel sliderYsize;
	private final JSliderWithTextFieldPanel sliderCityDensity;
	private final JSliderWithTextFieldPanel sliderLandDensity;
	private final JSliderWithTextFieldPanel sliderContinentSize;
	private final JCheckBox checkIsQuickstart = new JCheckBox();
	private final JFormattedTextField textName = new JFormattedTextField();
	private final JFormattedTextField textTimestamp = new JFormattedTextField();

	private final JTable table;

	private final GameProperties model;
	private final PlayersSelectionTableModel tableModel;
}
