package de.ped.empire.gui;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import de.ped.empire.logic.Commandable;
import de.ped.empire.logic.CommandableTemplate;
import de.ped.empire.logic.FieldCity;
import de.ped.empire.logic.Order;
import de.ped.empire.logic.TileTemplate;
import de.ped.empire.logic.Unit;
import de.ped.empire.logic.ZoomLevel;
import de.ped.tools.Messages;
import de.ped.tools.TopologyMode;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.GUIAlgorithms;
import de.ped.tools.gui.JInfoDialog;

/**
 * Presents a yes/no dialog to the user, asking her/him if an order really should be executed.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class OrderSafetyQuestionDialog extends JInfoDialog {

	private static final long serialVersionUID = 1L;

	public OrderSafetyQuestionDialog(final ApplicationMainWindow parent, final Order order) {
		super(parent);
		setTitle(getMessages().getText(order.getMessageKeyPrefix() + Messages.TEXT_POSTFIX));
		setOptionPaneParams(parent.getMessages().getString(order.getSafetyQuestion()), JOptionPane.QUESTION_MESSAGE);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);

		JPanel middleArea = new JPanel();
		middleArea.setLayout(new BoxLayout(middleArea, BoxLayout.PAGE_AXIS));
		Commandable commandable = order.getCommandable();
		if (null == commandable) {
			EmpireMainWindow empWindow = (EmpireMainWindow) parent;
			commandable = empWindow.getGameFacade().getVisibleCommandableInFocus(empWindow.getCurrentPlayerId());
		}
		final JLabel templateLabel = createLabelForCommandable(parent, commandable);
		final JTextArea textArea = new JTextArea(parent.getMessages().getString(order.getSafetyQuestion()));
		GUIAlgorithms.modifyToLabelLAF(textArea);
		textArea.setAlignmentX(Component.CENTER_ALIGNMENT);
		templateLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		middleArea.add(textArea);
		middleArea.add(templateLabel);
		getContentPane().add(middleArea, BorderLayout.CENTER);
		finishLayout();
		setResizable(false);
		getButtonCancel().requestFocusInWindow();
	}

	/**
	 * Creates a label for a commandable
	 * @param parent the application main window
	 * @param commandable the commandable to display
	 * @return a label containing the name of the commandable and an adequate image
	 */
	static JLabel createLabelForCommandable(final ApplicationMainWindow parent, Commandable commandable) {
		final JLabel templateLabel = new JLabel("Unit type");
		if (null != commandable) {
			final String labelText;
			final TileTemplate image;
			if (commandable instanceof Unit) {
				final Unit unit = (Unit) commandable;
				final CommandableTemplate template = unit.getUnitTemplate();
				image = unit;
				labelText = Integer.toString(unit.getId()) + ". " + parent.getMessages().getString(template.getNameKey());
			} else {
				image = (FieldCity) commandable;
				labelText = "";
			}
			templateLabel.setIcon(new ImageIcon(image.createImage(ZoomLevel.DIALOG_ZOOM_LEVEL, TopologyMode.SQUARE8)));
			templateLabel.setText(labelText);
		}
		return templateLabel;
	}
}
