package de.ped.empire.logic;

import de.ped.empire.logic.GameViewProperties.AnimationDelayType;

/**
 * Any part an animation sequence can contain.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum AnimationPart {
	/** Unit can't execute a planned move. */
	ALERT_IMPEDIMENT(new int[][] { { 12,  8,  5 }, { 24, 16, 10 } }, AnimationDelayType.MESSAGES, SoundTypes.MOVE),
	/** Commandable has seen enemy nearby. */
	ALERT_SIGHTING  (new int[][] { { 12,  8,  5 }, { 24, 16, 10 } }, AnimationDelayType.MESSAGES, SoundTypes.ALARM),
	/** Sequence for showing who is fighting. */
	FIGHT_INTRO     (new int[][] { { 10,  7,  5 }, { 20, 14, 10 } }, AnimationDelayType.BATTLE,   SoundTypes.BATTLE),
	/** Attacking unit receives a hit. */
	FIGHT_HIT_OWN   (new int[][] { {  4,  4,  2 }, {  8,  8,  4 } }, AnimationDelayType.BATTLE,   SoundTypes.BATTLE),
	/** Defending unit receives a hit. Duration must be the same as {@link #FIGHT_HIT_OWN}. */
	FIGHT_HIT_ENEMY (new int[][] { {  4,  4,  2 }, {  8,  8,  4 } }, AnimationDelayType.BATTLE,   SoundTypes.BATTLE),
	/** Dying sequence for attacking unit. Currently unused. */
	DEATH_OWN       (new int[][] { { 20, 15, 10 }, { 40, 30, 20 } }, AnimationDelayType.BATTLE,   SoundTypes.BATTLE),
	/** Dying sequence for defending unit. Currently unused. */
	DEATH_ENEMY     (new int[][] { { 20, 15, 10 }, { 40, 30, 20 } }, AnimationDelayType.BATTLE,   SoundTypes.BATTLE),
	/** Dying because of empty fuel. */
	FUEL_EMPTY      (new int[][] { { 12,  8,  5 }, { 24, 16, 10 } }, AnimationDelayType.MESSAGES, null),
	/** Unit is slowed down by rough land. */
	SLOWED_DOWN     (new int[][] { {  3,  1,  1 }, {  6,  2,  2 } }, AnimationDelayType.MESSAGES, null),
	/** Move of distance 1 for attacking / moving unit. */
	MOVE            (new int[][] { {  4,  2,  2 }, {  8,  4,  4 } }, AnimationDelayType.MOVEMENT, SoundTypes.MOVE),
	/** Docking of a sea unit (a special kind of move). */
	DOCK            (new int[][] { { 12,  8,  5 }, { 24, 16, 10 } }, AnimationDelayType.MESSAGES, null),
	/** Loading of a unit (a special kind of move). */
	LOAD            (new int[][] { { 12,  8,  5 }, { 24, 16, 10 } }, AnimationDelayType.MESSAGES, null),
	/** Unit transfers into airbase. */
	GARRISON        (new int[][] { {  3,  1,  1 }, {  6,  2,  2 } }, AnimationDelayType.MESSAGES, null),
	/** Marker for the end of the animation. */
	END_ANIMATION   (new int[][] { {  1,  1,  1 }, {  1,  1,  1 } }, AnimationDelayType.MESSAGES, null);

	/** The duration in steps for slow, medium and fast animation speed (in that order). */
	public final int[][] duration;
	public final AnimationDelayType animationDelayType;
	public final SoundTypes soundType;

	private AnimationPart(final int[][] duration, final AnimationDelayType animationDelayType, final SoundTypes soundType) {
		this.duration = duration;
		this.animationDelayType = animationDelayType;
		this.soundType = soundType;
	}
}