package de.ped.empire.logic;

import java.util.Collection;

/**
 * A field or anything that behaves like a field.
 * 
 * @author Copyright (c) 2015 Peter Diefenbach (peter@pdiefenbach.de)
 */
public interface FieldAlike extends MappedTileable, Comparable<FieldAlike> {

	/**
	 * @param other the object to compare with
	 * @return <code>true</code> if the position is equal
	 */
	@Override
	public int compareTo(final FieldAlike other);

	/** 
	 * @return if this field is in any way a land or city field and possibly needs a coastal line 
	 */
	public abstract boolean isLand();

	/** 
	 * @return if this field is unexplored
	 */
	public abstract boolean isUnexplored();

	/** 
	 * @return if this field is a city
	 */
	public abstract boolean isCity();

	/** 
	 * Gets the game field behind that proxy. May throw an exception if that field is still unexplored.
	 * @return the game field
	 */
	public abstract Field getField();

	/** 
	 * @param unit the unit that wants to be on this field
	 * @return how many units of the type and player of that unit this field can ideally hold.
	 */
	public abstract int getCapacity(Unit unit);

	/**
	 * @return the template ID
	 */
	public abstract byte getTemplateId();

	public abstract FieldTemplate getTemplate();

	/**
	 * Gets all the units known to be on this field.
	 * @return a collection of all the units known to be on this field (depending on the kind of map this field is in)
	 */
	public abstract Collection<UnitAlike> getUnits();
}
