package de.ped.empire.logic;

import de.ped.tools.Messages;

/**
 * The upwards interface an Empire game needs to tell the user interface what happened.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public interface GameObserver {

	Messages getMessages();

	/**
	 * Called when something happened in the game that might affect the
	 * observer.
	 * <p>
	 * The implementer of this method guarantees that it does not last long and does not block.
	 * 
	 * @param playerId
	 *            the ID of the player affected
	 * @param notif
	 *            the game notification to handle by the observer. The caller
	 *            should make sure this notification is a clone, so that the
	 *            implementer can modify it while processing it.
	 */
	void notifyGameNotification(final int playerId, final GameNotification notif);

	/**
	 * Called when a city has produced a new unit. May result in an info dialog.
	 * <p>
	 * The implementer of this method guarantees that it does not last long and does not block.
	 * 
	 * @param playerId
	 *            the ID of the player that produced the unit
	 * @param city
	 *            the city that produced the unit
	 * @param unit
	 *            the unit produced
	 */
	void notifyUnitProduced(final int playerId, final FieldCity city, final Unit unit);

	/** 
	 * Called regularly to do housekeeping. 
	 * 
	 * @param playerId
	 *            the ID of the player
	 */
	void step(final int playerId);
}
