package de.ped.empire.logic;

import de.ped.tools.log.Logger;
import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/**
 * Info about the current basic game state like current player and turn.
 * 
 * @author Copyright (c) 2015 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GameStateBasics implements Marshallable, Cloneable {
	private int currentPlayerId = Player.ID_NONE;
	private int currentTurn = 0;
	private int winnerPlayerId = Player.ID_NONE;
	private boolean hasGameEnded = false;

	public synchronized void setPlayerAndTurn(final int playerId, final int turn) {
		Logger.getLogger(getClass()).trace("setPlayerAndTurn(" + playerId + "," + turn + ")");
		this.currentPlayerId = playerId;
		this.currentTurn = turn;
	}

	public synchronized void setGameEnding(final boolean isGameEnding, final int playerId) {
		hasGameEnded = isGameEnding;
		winnerPlayerId = playerId;
	}

	public synchronized int getPlayerId() {
		return currentPlayerId;
	}

	public synchronized int getTurn() {
		return currentTurn;
	}

	public synchronized int getWinningPlayerId() {
		return winnerPlayerId;
	}

	public synchronized boolean hasGameEnded() {
		return hasGameEnded;
	}

	@Override
	public void marshal(Marshaller marshaller) {
		marshaller.writeInt(currentTurn);
		marshaller.writeByte((byte) currentPlayerId);
		marshaller.writeByte((byte) winnerPlayerId);
		marshaller.writeBoolean(hasGameEnded);
	}

	@Override
	public Marshallable demarshal(Marshaller marshaller) {
		currentTurn = marshaller.readInt();
		currentPlayerId = marshaller.readByte();
		winnerPlayerId = marshaller.readByte();
		hasGameEnded = marshaller.readBoolean();
		return this;
	}

	@Override
	public GameStateBasics clone() {
		GameStateBasics ret = null;
		try {
			ret = (GameStateBasics) super.clone();
		} catch (CloneNotSupportedException e) {
			Logger.getLogger(getClass()).error("clone() threw exception!", e);
		}
		return ret;
	}
}