package de.ped.empire.logic;

import de.ped.tools.Bitfield32;

/**
 * The different views a player can take on the map, depending on his needs and intended actions.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum MapViewMode {
	//         message key                 units  enable  
	NORMAL    ("Action.ViewNormalMap",     0xffffffff, true),
	SURVEY    ("Action.ViewSurveyMap",     0xffffffff, true),
	PRODUCTION("Action.ViewProductionMap", 0x00000000, true),
	GROUP     ("Action.ViewGroupMap",      0xffffffff, false),
	PATH      ("Action.ViewPathMap",       Bitfield32.bitmask(ImageKey.CMDL_ID_CITY) | Bitfield32.bitmask(ImageKey.CMDL_ID_AIRBASE), true),
	REPLAY    ("Action.ViewReplayMap",     0xffffffff, true),
	REPLAY_OWN("Action.ViewReplayOwnMap",  0xffffffff, true);

	public final String messageKeyPrefix;

	/** A bitmask for all units to be displayed. */
	public final int bitmaskUnitsToShow;
	/** <code>true</code> if this mode is already implemented. */
	public final boolean isEnabled;

	MapViewMode(final String messageKeyPrefix, final int bitmaskUnitsToShow, final boolean isEnabled) {
		this.messageKeyPrefix = messageKeyPrefix;
		this.bitmaskUnitsToShow = bitmaskUnitsToShow;
		this.isEnabled = isEnabled;
	}

	public boolean isReplay() {
		return (this == REPLAY) || (this == REPLAY_OWN);
	}

	/** 
	 * @return <code>true</code> if the user is allowed to pick another view by him/herself
	 */
	public boolean isLeavable() {
		return !isReplay();
	}

	public boolean isShowingCrosshair() {
		return !isShowingAnimation();
	}

	public boolean isShowingAnimation() {
		return ((this == NORMAL) || isReplay());
	}
}