package de.ped.empire.logic;

import de.ped.tools.Marshallable;

/**
 * Anything on the map that has a position and is really part of the world to be
 * described in between moves and to be persisted, like fields or units.
 * 
 * @author Copyright (c) 2015 Peter Diefenbach (peter@pdiefenbach.de)
 */
public interface MappedTileable extends PositionedTileable, Marshallable, Cloneable {

	/**
	 * Defines if this object can take units of the given type.
	 * 
	 * @param unitToLoad
	 *            the unit to load
	 * @param isNow
	 *            if <code>false</code>, it is only checked if this commandable
	 *            could sometimes take this unit; if <code>true</code>, it is
	 *            checked if this commandable can take this unit now, so it
	 *            takes e.g. already loaded units into account
	 * @return if this object can take units of the given type
	 */
	abstract boolean canLoad(final UnitAlike unitToLoad, final boolean isNow);
}
