package de.ped.empire.logic;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;

import de.ped.tools.CollectionUtil;
import de.ped.tools.MathUtil;
import de.ped.tools.ResourceFinder;
import de.ped.tools.log.Logger;

/**
 * Handles name suggestions for nameable commandables.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NameSuggestion {

	private final Logger logger = Logger.getLogger(getClass());

	protected final ResourceFinder resources;
	protected final String fileName;
	protected final String[] fallback;
	protected ArrayList<String> suggestions = new ArrayList<String>();
	protected boolean wasInitialized = false;

	public NameSuggestion(final ResourceFinder resources, final String fileName, final String[] fallback) {
		this.resources = resources;
		this.fileName = fileName;
		this.fallback = fallback;
	}

	public String getNameSuggestion() {
		String suggestion = "";
		init();
		if (!CollectionUtil.isNullOrEmpty(suggestions)) {
			suggestion = suggestions.get(MathUtil.random(suggestions.size()));
		} else {
//			suggestion = Integer.toString(getID()) + ". " + messages.getText(getUnitTemplate().getNameKey());
		}
		return suggestion;
	}

	protected void init() {
		if (!wasInitialized) {
			initWithResources();
			if (CollectionUtil.isNullOrEmpty(suggestions)) {
				initWitDefault();
			}
			wasInitialized = true;
		}
	}

	protected void initWithResources() {
		try {
			if (!CollectionUtil.isNullOrEmpty(fileName)) {
				final URL url = resources.getURLToResource(fileName);
				final InputStreamReader reader = new InputStreamReader(url.openStream());
				final BufferedReader br = new BufferedReader(reader);
				String str;
				while (null != (str = br.readLine())) {
					if (!str.trim().isEmpty()) {
						suggestions.add(str);
					}
				}
			}
		} catch (final IOException e) {
			logger.warn("Failed to load resources " + fileName, e);
		}
	}

	protected void initWitDefault() {
		for (final String str : fallback) {
			suggestions.add(str);
		}
	}
}
