package de.ped.empire.logic;

import de.ped.tools.Marshaller;

/**
 * The order that can be given to a military {@link Unit} to unload all cargo.
 * The order is executed immediatly. It's a shortcut to un-sentry all cargy units.
 * 
 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class OrderBuildAirbase extends Order {

	@SuppressWarnings("unused")
	private static final long serialVersionUID = 1L;

	private static final OrderTemplate TEMPLATE = OrderTemplate.BUILD_AIRBASE;

	public OrderBuildAirbase(final GameState gameState, final Commandable commandable) {
		super(gameState, commandable, TEMPLATE);
	}

	OrderBuildAirbase(final GameState gameState, final Commandable commandable, final Marshaller marshaller) {
		super(gameState, commandable, TEMPLATE, marshaller);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		super.marshal(marshaller);
	}

	@Override
	public MoveResult evaluate(final boolean isNow) {
		final Unit unit = (Unit) getCommandable();
		final GameState gameState = unit.getGameState();
		final FieldAlike field = gameState.getMap().getAt(unit.getPosition());
		gameState.getUnits().remove(unit);
		final Unit airbase = Unit.create(gameState, unit.getPlayerId(), Templates.instance().AIRBASE, field);
		gameState.addUnit(null, airbase);
		final MoveResult ret = new MoveResult(getGameView(), unit, null);
		ret.setResultMask(MoveResult.MOVE_MASK_GARRISON);
		return ret;
	}

	@Override
	public String getSafetyQuestion() {
		return getSafetyQuestionDefault();
	}
}
