package de.ped.empire.logic;

import de.ped.tools.Marshaller;

/**
 * An order given to a {@link FieldCity} to change production.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class OrderChangeProduction extends Order {

	@SuppressWarnings("unused")
	private static final long serialVersionUID = 1L;

	private static final OrderTemplate TEMPLATE = OrderTemplate.CHANGE_PRODUCTION;

	public OrderChangeProduction(final GameState gameState, final Commandable commandable) {
		super(gameState, commandable, TEMPLATE);
	}

	OrderChangeProduction(final GameState gameState, final Commandable commandable, final Marshaller marshaller) {
		super(gameState, commandable, TEMPLATE, marshaller);
		throw new IllegalStateException(ERRORMSG_TRANSIENT);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		throw new IllegalStateException(ERRORMSG_TRANSIENT);
	}

	@Override
	public MoveResult evaluate(final boolean isNow) {
		final MoveResult ret = null;
		final GameView gameView = getGameView();
		final FieldCity city = (FieldCity) getCommandable();
		gameView.setGameNotificationEvent(GameNotification.MASK_CHANGE_PROD_REQ, city.getKey());
		return ret;
	}

	private CommandableTemplate unitToProduce = Templates.instance().UNDEFINED;

	/**
	 * Defines the unit to produce.
	 * 
	 * @param template
	 *            the unit template to produce
	 */
	void setUnitToProduce(CommandableTemplate template) {
		unitToProduce = template;
	}

	/**
	 * @return the unit template to produce. Default value is
	 *         {@link Templates#UNDEFINED} to force the user or the strategy to
	 *         decide.
	 */
	CommandableTemplate getUnitToProduce() {
		return unitToProduce;
	}
}
