package de.ped.empire.logic;

import de.ped.tools.Marshaller;

/**
 * The order that can be given to a military {@link Unit} to disband itself.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class OrderDisband extends Order {

	@SuppressWarnings("unused")
	private static final long serialVersionUID = 1L;

	private static final OrderTemplate TEMPLATE = OrderTemplate.DISBAND;

	public OrderDisband(final GameState gameState, final Commandable commandable) {
		super(gameState, commandable, TEMPLATE);
	}

	OrderDisband(final GameState gameState, final Commandable commandable, final Marshaller marshaller) {
		super(gameState, commandable, TEMPLATE, marshaller);
		throw new IllegalStateException(ERRORMSG_TRANSIENT);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		throw new IllegalStateException(ERRORMSG_TRANSIENT);
	}

	@Override
	public String getSafetyQuestion() {
		return getSafetyQuestionDefault();
	}

	@Override
	public MoveResult evaluate(final boolean isNow) {
		final Unit unit = (Unit) getCommandable();
		unit.consumeAllStepsThisTurn();
		unit.setOrder(null);
		MoveResult ret = new MoveResult(getGameView(), unit, null);
		ret.setResultMask(MoveResult.MOVE_MASK_LOSING_FIGHT);
		return ret;
	}
}
