package de.ped.empire.logic;

import java.util.Collection;

import de.ped.tools.Marshaller;

/**
 * The order that can be given to a military {@link Unit} 
 * to go to the nearest city of the same player.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class OrderHome extends Order {

	private static final OrderTemplate TEMPLATE = OrderTemplate.HOME;

	/**
	 * @param gameView
	 *            the game view
	 * @param commandable
	 *            the commandable
	 * @return <code>true</code> if the unit has already arrived in a city
	 */
	public static boolean isOrderAlreadyFulfilled(final GameView gameView, final Commandable commandable) {
		boolean ret = (gameView.getPlayerMap().getAt(commandable.getPosition()).isCity());
		return ret;
	}

	public OrderHome(final GameState gameState, final Commandable commandable) {
		super(gameState, commandable, TEMPLATE);
	}

	OrderHome(final GameState gameState, final Commandable commandable, final Marshaller marshaller) {
		super(gameState, commandable, TEMPLATE, marshaller);
	}

	protected OrderHome(final GameState gameState, final Commandable commandable, final OrderTemplate orderTemplate) {
		super(gameState, commandable, orderTemplate);
	}

	protected OrderHome(final GameState gameState, final Commandable commandable, final OrderTemplate orderTemplate, final Marshaller marshaller) {
		super(gameState, commandable, orderTemplate, marshaller);
	}

	@Override
	public boolean isFulfilled() {
		return isOrderAlreadyFulfilled(getGameView(), getCommandable());
	}

	/**
	 * Sets the nearest city as target seed for the step evaluation process. 
	 * If there are several cities in the same distance, sets them all.
	 * 
	 * @return <code>true</code> if the order goal was reachable and further
	 *         rating makes sense
	 */
	@Override
	public boolean prepareRating() {
		final Unit unit = (Unit) getCommandable();
		calculateForwardRating(StepRatingPurpose.NORMAL);
		final Collection<FieldAlike> nearest = unit.findNearestOf(unit.findHomeBases()); 
		final boolean ret = unit.setTargetSeeds(nearest);
		return ret;
	}

	@Override
	public MoveResult evaluate(final boolean isNow) {
		MoveResult ret = null;
		final Unit unit = (Unit) getCommandable();
		if (isFulfilled()) {
			unit.setOrder(null);
		} else if (!isNow || unit.canMove()) {
			ret = findNextMove(unit);
		}
		return ret;
	}
}
