package de.ped.empire.logic;

import de.ped.tools.Marshaller;

/**
 * The order that can be given to a military {@link Unit} to unload all cargo.
 * The order is executed immediatly. It's a shortcut to un-sentry all cargy units.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class OrderUnload extends Order {

	@SuppressWarnings("unused")
	private static final long serialVersionUID = 1L;

	private static final OrderTemplate TEMPLATE = OrderTemplate.UNLOAD;

	public OrderUnload(final GameState gameState, final Commandable commandable) {
		super(gameState, commandable, TEMPLATE);
	}

	OrderUnload(final GameState gameState, final Commandable commandable, final Marshaller marshaller) {
		super(gameState, commandable, TEMPLATE, marshaller);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		super.marshal(marshaller);
	}

	@Override
	public MoveResult evaluate(final boolean isNow) {
		final Unit unit = (Unit) getCommandable();
		if (unit.getUnitTemplate().canLoad(unit.getGameView())) {
			unit.unload();
			unit.skipMove();
			unit.setOrder(null);
		}
		return null;
	}
}
