package de.ped.empire.logic;

import de.ped.tools.Idable;

/**
 * Represents one of the possible player types - may they be human or computer strategies.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum PlayerType implements Idable {
	DEFAULT        (0, "GameProperties.Players.Type.Human"         ), // Default player is: human on this device
	COMPUTER_SIMPLE(1, "GameProperties.Players.Type.ComputerSimple"),
	NONE           (2, "GameProperties.Players.Type.None"          ); // No player at all - represents an empty slot in the list of players

	public final int id;
	public final String key;

	private PlayerType(final int id, final String key) {
		this.id = id;
		this.key = key;
	}

	public int getId() {
		return id;
	}

	public boolean isPlayingOnThisDevice() {
		return isPlayer();
	}

	/**
	 * @return <code>true</code> if this is a human player
	 */
	public boolean isHuman() {
		return (this == DEFAULT);
	}

	/** 
	 * @return <code>true</code> if this is a player at all; returns only <code>false</code> for {@link #NONE}
	 */
	public boolean isPlayer() {
		return (this != NONE);
	}
}
