package de.ped.empire.logic;

/**
 * Defines the different rule complexities a game can be played with.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum RulesComplexity {
	BEGINNER(false, false, false, 0.0f,        false),
	STANDARD(true,  false, false, 1.0f / 6.0f, false),
	ADVANCED(true,  true,  true,  1.0f / 6.0f, true);

	public static final String KEY = "GameProperties.RulesComplexity";

	private RulesComplexity(
			final boolean isWithFogOfUncertainty, 
			final boolean isProductivityRandom,
			final boolean isWithTroopSupportFactor,
			final float productionRepetitionBonusFactor,
			final boolean isWithFightMalusWhenDamaged) {
		this.isWithFogOfUncertainty = isWithFogOfUncertainty;
		this.isProductivityRandom = isProductivityRandom;
		this.isWithTroopSupportFactor = isWithTroopSupportFactor;
		this.productionRepetitionBonusFactor = productionRepetitionBonusFactor;
		this.isWithFightMalusWhenDamaged = isWithFightMalusWhenDamaged;
	}

	public final boolean isWithFogOfUncertainty;

	/** If <code>true</code>, city productivity may vary; if <code>false</code>, it is always 100%. */
	public final boolean isProductivityRandom;

	/** If <code>true</code>, cities need to support troops, which may negatively affect global production rate. */
	public final boolean isWithTroopSupportFactor;

	public final float productionRepetitionBonusFactor;

	/** Empire Deluxe Manual page 135, rule l: Advanced game only - attack ratings are x 1/2 when crippled */
	public final boolean isWithFightMalusWhenDamaged;
}