package de.ped.empire.logic;

import de.ped.tools.ResourceFinder;

/**
 * A classification of sounds to be handled together in user settings.
 * 
 * @author Copyright (c) 2015 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum SoundTypes {
	/** A prompt sound when a unit gets the focus and waits for user input. */
	PROMPT ("GameViewProperties.Sounds.Prompt", "Action.NextCommandable", ResourceFinder.Folder.APP),
	/** A command is executed that is not a move. */
	COMMAND("GameViewProperties.Sounds.Command", null, ResourceFinder.Folder.APP),
	/** A move is executed. */
	MOVE   ("GameViewProperties.Sounds.Move", "Action.GoTo", ResourceFinder.Folder.APP),
	/** An attack is executed. */
	BATTLE ("GameViewProperties.Sounds.Battle", "GameViewProperties.AnimationDelay.Battle", ResourceFinder.Folder.APP),
	/** An alarm sound. */
	ALARM  ("GameViewProperties.Sounds.Alarm", null, ResourceFinder.Folder.APP);

	/** The message key for this game view property */
	public final String key;
	/** The image key for this game view property, if different from message key */
	public final String imageKey;
	/** The image folder where to find the image key, or <code>null</code> if no image to show */
	public final ResourceFinder.Folder imageFolder;

	private SoundTypes(final String key, final String imageKey, ResourceFinder.Folder imageFolder) {
		this.key = key;
		this.imageKey = imageKey;
		this.imageFolder = imageFolder;
	}
}