package de.ped.empire.logic;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 * Some game statistics for a player.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class StatusReportPlayerData {

	/**
	 * Some game statistics for a player and unit type.
	 * 
	 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class StatusReportUnitData {

		// Primary key:

		/** The game the data belongs to */
		public final GameView gameView;

		/** The ID of the player the data belongs to */
		public final int playerId;

		/** The unit template the data belongs to */
		public final CommandableTemplate template;

		// Data collected for the key

		/** Number of cities currently producing that kind of unit for the player */
		public final int inProduction;

		/**
		 * Number of turns until the next city will produce that kind of unit for the player, 
		 * or {@link Integer#MAX_VALUE} if there is no city producing that kind of unit.
		 */
		public final int nextReadyIn;

		/** Number of units of that kind in the field for the player */
		public final int cntInField;

		/** Number of units of that kind already lost by the player */
		public final int cntLosses;

		/** Number of units of that kind already killed by the player */
		public final int cntKills;

		public StatusReportUnitData(final GameView view, final int playerId, final CommandableTemplate template) {
			this.gameView = view;
			this.playerId = playerId;
			this.template = template;

			int ip = 0;
			int nri = Integer.MAX_VALUE;
			final Collection<FieldCity> cities = view.getGameState().getMap().findCities(playerId);
			for (final FieldCity city : cities) {
				if (city.getUnitInProduction() == template) {
					ip++;
					final int turnsToProduce = city.getTurnsToProduce();
					if (turnsToProduce < nri) {
						nri = turnsToProduce;
					}
				}
			}
			inProduction = ip;
			nextReadyIn = nri;

			int cif = 0;
			final Collection<UnitAlike> units = view.getUnits();
			for (final UnitAlike unit : units) {
				if ((unit.getKey().getPlayerId() == playerId) 
						&& (unit.getKey().getTemplateId() == template.id))  {
					cif++;
				}
			}
			cntInField = cif;
			final Player player = view.getPlayer(playerId);
			cntLosses = player.getCntUnitsLost(template);
			cntKills = player.getCntUnitsKilled(template);
		}

		public int getPlayerColorId() {
			return gameView.getPlayer(playerId).getPlayerColorId();
		}
	}

	public final List<StatusReportPlayerData.StatusReportUnitData> units = new LinkedList<StatusReportPlayerData.StatusReportUnitData>();
	public final int mapTotal;
	public final int mapKnown;
	public final int citiesTotal;
	public final int citiesPlayer;
	public final int troopsPlayer;
	public final float troopsFactor;

	public StatusReportPlayerData(final GameView gameView, final int playerId, final List<CommandableTemplate> unitTemplates) {
		for (final CommandableTemplate template : unitTemplates) {
			StatusReportPlayerData.StatusReportUnitData pum = new StatusReportUnitData(gameView, playerId, template);
			units.add(pum);
		}
		final GameMap map = gameView.getPlayerMap();
		mapTotal = map.countAllFields();
		mapKnown = map.countKnownFields();
		citiesTotal = map.getAllCities().size();
		citiesPlayer = map.findCities(playerId).size();
		troopsPlayer = gameView.getGameState().findCommandables(playerId).size() - citiesPlayer;
		troopsFactor = gameView.getGameState().getTroopsSupportFactor(playerId);
	}
}