package de.ped.empire.logic;

import de.ped.empire.logic.FieldTemplate.RulesComplexityDependentValues;
import de.ped.tools.Bitfield32;

/**
 * A class containing all template data that is constant between games.
 * <p>
 * Was necessary to organize static initialization order.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Templates {

	public static Templates instance;

	public static synchronized Templates instance() {
		if (null == instance) {
			instance = new Templates();
		}
		return instance;
	}

	private Templates() {
		final String[] emptyNameSuggestionsDefault = new String[]{};
		final int[][] emptyInt2Array = new int[][] { };

		final int bitmaskUnitsNone = 0x00;

		final int bitmaskUnitsTankOnly = Bitfield32.bitmask(ImageKey.CMDL_ID_TANK);

		final int bitmaskUnitsCityOnly = Bitfield32.bitmask(ImageKey.CMDL_ID_CITY);

		/** All units including infrastructure. */
		final int bitmaskUnitsAll = Bitfield32.bitmask(ImageKey.CMDL_ID_TANK)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_ARMY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_FIGHTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BOMBER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_TRANSPORTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_SUBMARINE)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CRUISER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BATTLESHIP)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CARRIER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CITY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_AIRBASE);

		/** All units including infrastructure, but without cities. */
		final int bitmaskUnitsAllButCity = bitmaskUnitsAll & (~bitmaskUnitsCityOnly);

		/** All moving units with the exception of army. */
		final int bitmaskUnitsAllMovingButArmy = Bitfield32.bitmask(ImageKey.CMDL_ID_TANK)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_FIGHTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BOMBER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_TRANSPORTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_SUBMARINE)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CRUISER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BATTLESHIP)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CARRIER);

		/** All units that can see submarines. */
		final int bitmaskUnitsAllButSubmarine = Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_ARMY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_FIGHTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BOMBER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_TRANSPORTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CRUISER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BATTLESHIP)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CARRIER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CITY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_AIRBASE);

		/** All moving units with the exception of army. */
		final int bitmaskUnitsWhichSeeSubarines = Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_SUBMARINE)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CRUISER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CITY);

		/** All land units - in this case all units that can be attacked without move by shore bombardment. */
		final int bitmaskUnitsLand = Bitfield32.bitmask(ImageKey.CMDL_ID_ARMY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_TANK)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_AIRBASE);

		/** All sea units with the exception of submarines. */
		final int bitmaskUnitsShips = Bitfield32.bitmask(ImageKey.CMDL_ID_TRANSPORTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CRUISER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BATTLESHIP)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CARRIER);

		/** All infrastructure "units", and all land and air units. */
		final int bitmaskUnitsInfraLandAir = Bitfield32.bitmask(ImageKey.CMDL_ID_ARMY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_TANK)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_FIGHTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BOMBER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CITY)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_AIRBASE);

		/** All infrastructure "units", and all land and air units. */
		final int bitmaskUnitsAirSea = Bitfield32.bitmask(ImageKey.CMDL_ID_FIGHTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BOMBER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_TRANSPORTER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_DESTROYER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_SUBMARINE)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CRUISER)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_BATTLESHIP)
				| Bitfield32.bitmask(ImageKey.CMDL_ID_CARRIER);

		final CommandableTemplate.RulesComplexityDependentValues emptyCmdlRcdv = new CommandableTemplate.RulesComplexityDependentValues(
				LoadDefinition.NO_LOAD, emptyInt2Array, 0, 
				0, 0,					// Sighting range
				bitmaskUnitsNone,		// Attack without move
				bitmaskUnitsNone,		// Attack including move
				1.0f);

		final LoadDefinition[] loadDefinitionCity = new LoadDefinition[] {
				new LoadDefinition(ImageKey.CMDL_ID_ARMY, 1),
				new LoadDefinition(ImageKey.CMDL_ID_TANK, 1),
				new LoadDefinition(ImageKey.CMDL_ID_FIGHTER, 1),
				new LoadDefinition(ImageKey.CMDL_ID_BOMBER, 1),
				new LoadDefinition(ImageKey.CMDL_ID_TRANSPORTER, 1),
				new LoadDefinition(ImageKey.CMDL_ID_DESTROYER, 1),
				new LoadDefinition(ImageKey.CMDL_ID_SUBMARINE, 1),
				new LoadDefinition(ImageKey.CMDL_ID_CRUISER, 1),
				new LoadDefinition(ImageKey.CMDL_ID_BATTLESHIP, 1),
				new LoadDefinition(ImageKey.CMDL_ID_CARRIER, 1),
		};

		UNDEFINED = new CommandableTemplate(
				ImageKey.CMDL_ID_UNDEFINED, 		// ID
				MilitaryBranch.NO_UNIT, // Unit type
				false,					// is focusable
				"Game.Unit.Nothing",	// messsage key prefix
				Integer.MAX_VALUE,		// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
					emptyCmdlRcdv,
					emptyCmdlRcdv,
					emptyCmdlRcdv,
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				0, 0, 0, 9,				// attack, defense, shot damage
				false,					// Image resource
				null,					// Prompt sound file name (~5 seconds)
				null,					// Alert impediment sound file name (~1 second)
				null,					// Alert sighting sound file name (~1 second)
				null,					// Move sound file name (0.1 seconds)
				null,					// Fight intro sound file name (0.7 seconds)
				null,					// Fight hit sound file name (0.4 seconds)
				null,					// Death sound file name (0.4 seconds)
				"",						// Name suggestions file name
				emptyNameSuggestionsDefault);
		NOTHING = new CommandableTemplate(
				ImageKey.CMDL_ID_UNDEFINED, 		// ID
				MilitaryBranch.NO_UNIT, // Unit type
				false,					// is focusable
				"Game.Unit.Nothing", 	// messsage key prefix
				Integer.MAX_VALUE, 		// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv,
						emptyCmdlRcdv,
						emptyCmdlRcdv,
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				0, 0, 0, 9,				// attack, defense, shot damage
				false,					// Image resource
				null,					// Prompt sound file name (~5 seconds)
				null,					// Alert impediment sound file name (~1 second)
				null,					// Alert sighting sound file name (~1 second)
				null,					// Move sound file name (0.1 seconds)
				null,					// Fight intro sound file name (0.7 seconds)
				null,					// Fight hit sound file name (0.4 seconds)
				null,					// Death sound file name (0.4 seconds)
				"",						// Name suggestions file name
				emptyNameSuggestionsDefault);
		final int[][] hitsConsequencesCity = new int[][] { {0, Integer.MAX_VALUE}, }; // Number of hits, Moves, Cargo hold size
		CMDL_CITY = new CommandableTemplate(
				ImageKey.CMDL_ID_CITY, 			// ID
				MilitaryBranch.LAND_UNIT, // Unit type
				true,					// is focusable
				"Game.Field.City", 		// messsage key prefix
				0, 						// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionCity, 
								hitsConsequencesCity, 
								Integer.MAX_VALUE, 
								1, 1,					// Sighting range
								bitmaskUnitsNone,		// Attack without move
								bitmaskUnitsNone,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionCity, 
								hitsConsequencesCity, 
								Integer.MAX_VALUE, 
								1, 1, 					// User manual, page 135, rule k: Cities have a sighting range of 1 when using standard rules
								bitmaskUnitsNone,		// Attack without move
								bitmaskUnitsNone,		// Attack including move
								1.0f),
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionCity, 
								hitsConsequencesCity, 
								Integer.MAX_VALUE, 
								2, 2,					// Sighting range
								bitmaskUnitsNone,		// Attack without move
								bitmaskUnitsNone,		// Attack including move
								1.0f), 
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				0, 2, 1, 0,				// attack, defense, shot damage
				true,					// Image resource
				null,					// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				null,					// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Army.wav",		// Death sound file name (0.4 seconds)
				"Game.Field.City.txt", 	// Name suggestions file name
				new String[]{
						"Agincourt",
						"Ashdown",
						"Austerlitz",
						"Bautzen",
						"Berlin",
						"Blore Heath",
						"Bosworth",
						"Breed's Hill",
						"Bunker Hill",
						"Cambrai",
						"Cassino",
						"Cawnpore",
						"Chalons",
						"Chancellorsville",
						"Chickamauga Creek",
						"Civitate",
						"Coronel",
						"Cross Lake",
						"Denain",
						"Dien Bien Phu",
						"Dresden",
						"Drepana",
						"Edgehill",
						"Eylau",
						"Fehrbellin",
						"Fleurus",
						"Friedland",
						"Fuentes de Onoro",
						"Fullford",
						"Gaza",
						"Grantham",
						"Golan Heights",
						"Gorlice",
						"Hampton Road",
						"Hastings",
						"Hiroshima",
						"Hohenfriedburg",
						"Ilipa",
						"Issus",
						"Jhelum River",
						"Kablinu",
						"Lech",
						"Little big horn",
						"Lissa",
						"Maiwand",
						"Majuba Hill",
						"Marathon",
						"Minden",
						"Modder River",
						"Nagasaki",
						"Navarino",
						"Novara",
						"Paris",
						"Pavia",
						"Pharsalus",
						"Pydna",
						"Ridaneih",
						"Rivoli",
						"Rocroi",
						"Rome",
						"Saigon",
						"Sedan",
						"Shiloh",
						"Sluys",
						"Somme",
						"Spion Kop",
						"St Gotthard",
						"Stalingrad",
						"Thapsus",
						"Tours",
						"Tunisia",
						"Ulundi",
						"Verdun",
						"Vimeiro",
						"Wandewash",
						"Waterloo",
						"White Mountain",
						"Wounded Knee",
				});
		final int[][] hitsConsequencesArmy = new int[][] { { 1, 0}, }; // Number of hits, Moves, Cargo hold size
		ARMY = new CommandableTemplate(
				ImageKey.CMDL_ID_ARMY, 			// ID
				MilitaryBranch.LAND_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Army", 		// messsage key prefix
				6, 						// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesArmy, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsShips,			// Attack without move
								bitmaskUnitsInfraLandAir,	// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesArmy, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsShips,			// Attack without move
								bitmaskUnitsInfraLandAir,	// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesArmy, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsShips,			// Attack without move
								bitmaskUnitsInfraLandAir,	// Attack including move
								1.0f), 
				},
				bitmaskUnitsAllButSubmarine,	// Visibility
				false,					// can move over
				2, 2, 1, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Army.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Army.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Army.wav",		// Death sound file name (0.4 seconds)
				null,					// Name suggestions file name
				emptyNameSuggestionsDefault);
		TANK = new CommandableTemplate(
				ImageKey.CMDL_ID_TANK, 			// ID
				MilitaryBranch.LAND_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Tank", 		// messsage key prefix
				12, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv, 
						emptyCmdlRcdv, 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								new int[][] { {1, 0}, {2, 0}, }, // Number of hits, Moves, Cargo hold size, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsShips,			// Attack without move
								bitmaskUnitsInfraLandAir,	// Attack including move
								1.0f), 
				},
				bitmaskUnitsAllButSubmarine,	// Visibility
				false,					// can move over
				2, 2, 1, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Army.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Army.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Army.wav",		// Death sound file name (0.4 seconds)
				null,					// Name suggestions file name
				emptyNameSuggestionsDefault);
		FIGHTER = new CommandableTemplate(
				ImageKey.CMDL_ID_FIGHTER, 		// ID
				MilitaryBranch.AIR_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Fighter", 	// messsage key prefix
				12, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv, 
						// Empire Deluxe Manual page 135, rule a: Statistics apply to fighters when using the standard rules
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								new int[][] { {5, 0}, }, 
								20, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAllButCity,		// Attack including move
								1.0f), 
						// Empire Deluxe Manual page 135, rule b: Statistics apply to fighters when using the advanced rules
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								new int[][]{{6, 0}, }, 
								18, 
								2, 2,						// Sighting range
								bitmaskUnitsCityOnly,		// Attack without move: Empire Deluxe Manual page 135, rule m: may bomb cities
								bitmaskUnitsAllButCity,		// Attack including move
								1.0f), 
				},
				bitmaskUnitsAllButSubmarine,	// Visibility
				true,					// can move over
				2, 2, 1, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Airplane.wav",	// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Airplane.wav",	// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				null,					// Death sound file name (0.4 seconds)
				null,					// Name suggestions file name
				emptyNameSuggestionsDefault);
		BOMBER = new CommandableTemplate(
				ImageKey.CMDL_ID_BOMBER, 		// ID
				MilitaryBranch.AIR_UNIT,// Unit type
				true,					// is focusable
				"Game.Unit.Bomber", 	// messsage key prefix
				12, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv, 
						emptyCmdlRcdv, 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								new int[][] { {4, 0}, }, // Number of hits, Moves, Cargo hold size, 
								24, 
								2, 2,						// Sighting range
								bitmaskUnitsCityOnly,		// Attack without move: Empire Deluxe Manual page 135, rule m: may bomb cities
								bitmaskUnitsAllButCity,		// Attack including move
								1.0f), 
				},
				bitmaskUnitsAllButSubmarine,	// Visibility
				true,					// can move over
				2, 2, 2, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Airplane.wav",	// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Airplane.wav",	// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				null,					// Death sound file name (0.4 seconds)
				null,					// Name suggestions file name
				emptyNameSuggestionsDefault);
		final int[][] hitsConsequencesTransporter = new int[][] { {1, 2}, {2, 4}, {2, 6}, }; // Number of hits, Moves, Cargo hold size
		final LoadDefinition[] loadDefinitionTransporter = new LoadDefinition[] { 
				new LoadDefinition(ImageKey.CMDL_ID_ARMY, 1), 
		};
		TRANSPORTER = new CommandableTemplate(
				ImageKey.CMDL_ID_TRANSPORTER, 	// ID
				MilitaryBranch.SEA_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Transporter",// messsage key prefix
				30, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionTransporter, 
								hitsConsequencesTransporter, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionTransporter, 
								hitsConsequencesTransporter, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								new LoadDefinition[] { 
										new LoadDefinition(ImageKey.CMDL_ID_ARMY, 1), 
										new LoadDefinition(ImageKey.CMDL_ID_TANK, 2), 
								}, 
								hitsConsequencesTransporter, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								0.5f), 
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				1, 1, 1, 1,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Ship.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Ship.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Ship.wav",		// Death sound file name (0.4 seconds)
				"Game.Unit.Transporter.txt",	// Name suggestions file name
				new String[]{
							"Aretha Franklyn",
							"Beluga",
							"Blue Whale",
							"Charlotte",
							"Courtney Love",
							"Friendship",
							"Gray Whale",
							"Humpback",
							"Kelly Osbourne",
							"Leviathan",
							"Moby Dick",
							"Montserrat Caball",
							"Narwhal",
							"Orca",
							"Oprah Winfrey",
							"Queen Latifah",
							"Queen Mary",
							"Roseanne",
							"Sperm Whale",
							"Zaafaran",
				});
		final int[][] hitsConsequencesDestroyer = new int[][] { {1, 0}, {2, 0}, {3, 0}, }; // Number of hits, Moves, Cargo hold size
		DESTROYER = new CommandableTemplate(
				ImageKey.CMDL_ID_DESTROYER,  	// ID
				MilitaryBranch.SEA_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Destroyer", 	// messsage key prefix
				24, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesDestroyer, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesDestroyer, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesDestroyer, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								0.5f), 
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				2, 2, 1, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Ship.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Ship.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Ship.wav",		// Death sound file name (0.4 seconds)
				"Game.Unit.Destroyer.txt",	// Name suggestions file name
				new String[]{
							"Killer shark",
				});
		final int[][] hitsConsequencesSubmarine = new int[][] { {1, 0}, {2, 0}, }; // Number of hits, Moves, Cargo hold size
		SUBMARINE = new CommandableTemplate(
				ImageKey.CMDL_ID_SUBMARINE,  	// ID
				MilitaryBranch.SEA_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Submarine", 	// messsage key prefix
				24, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv, 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesSubmarine, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesSubmarine, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								0.5f), 
				},
				bitmaskUnitsWhichSeeSubarines,	// Visibility
				false,					// can move over
				2, 1, 3, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Ship.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Ship.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Ship.wav",		// Death sound file name (0.4 seconds)
				"Game.Unit.Submarine.txt",	// Name suggestions file name
				new String[]{
							"Nautilus",
							"Red October",
				});
		final int[][] hitsConsequencesCruiser = new int[][] { {1, 0}, {1, 0}, {1, 0}, {1, 0}, {2, 0}, {2, 0}, {2, 0}, {2, 0}, }; // Number of hits, Moves, Cargo hold size
		CRUISER = new CommandableTemplate(
				ImageKey.CMDL_ID_CRUISER, 		// ID
				MilitaryBranch.SEA_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Cruiser", 	// messsage key prefix
				36, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesCruiser, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsLand,			// Attack without move: Can coastal attack, see manual page 135 rule f
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesCruiser, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsLand,			// Attack without move: Can coastal attack, see manual page 135 rule f
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesCruiser, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsLand,			// Attack without move: Can coastal attack, see manual page 135 rule f
								bitmaskUnitsAirSea,		// Attack including move
								0.5f), 
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				2, 2, 2, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Ship.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Ship.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Ship.wav",		// Death sound file name (0.4 seconds)
				"Game.Unit.Cruiser.txt",// Name suggestions file name
				new String[]{
							"Nobility",
				});
		final int[][] hitsConsequencesBattleship = new int[][] { {1, 0}, {1, 0}, {1, 0}, {1, 0}, {1, 0}, {1, 0}, {2, 0}, {2, 0}, {2, 0}, {2, 0}, {2, 0}, {2, 0}, }; // Number of hits, Moves, Cargo hold size
		BATTLESHIP = new CommandableTemplate(
				ImageKey.CMDL_ID_BATTLESHIP, 	// ID
				MilitaryBranch.SEA_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Battleship", // messsage key prefix
				60, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesBattleship, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsLand,			// Attack without move: Can coastal attack, see manual page 135 rule f
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesBattleship, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsLand,			// Attack without move: Can coastal attack, see manual page 135 rule f
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								LoadDefinition.NO_LOAD, 
								hitsConsequencesBattleship, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsLand,			// Attack without move: Can coastal attack, see manual page 135 rule f
								bitmaskUnitsAirSea,		// Attack including move
								0.5f), 
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				2, 2, 3, 2,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Ship.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Ship.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Ship.wav",		// Death sound file name (0.4 seconds)
				"Game.Unit.Battleship.txt", // Name suggestions file name
				new String[]{
							"Noblesse oblige",
				});
		final int[][] hitsConsequencesCarrier = new int[][] { {1, 1}, {1, 2}, {1, 3}, {1, 4}, {2, 5}, {2, 6}, {2, 7}, {2, 8}, }; // Number of hits, Moves, Cargo hold size
		final LoadDefinition[] loadDefinitionCarrier = new LoadDefinition[] { 
				new LoadDefinition(ImageKey.CMDL_ID_FIGHTER, 1), 
		};
		CARRIER = new CommandableTemplate(
				ImageKey.CMDL_ID_CARRIER, 		// ID
				MilitaryBranch.SEA_UNIT, // Unit type
				true,					// is focusable
				"Game.Unit.Carrier",	// messsage key prefix
				48, 					// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv, 
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionCarrier, 
								hitsConsequencesCarrier, 
								Integer.MAX_VALUE, 
								1, 1,						// Sighting range
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionCarrier, 
								hitsConsequencesCarrier, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsAirSea,		// Attack including move
								1.0f), 
				},
				bitmaskUnitsAll,		// Visibility
				false,					// can move over
				1, 2, 1, 1,				// attack, defense, shot damage
				true,					// Image resource
				"Prompt_Ship.wav",		// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				"Move_Ship.wav",		// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Ship.wav",		// Death sound file name (0.4 seconds)
				"Game.Unit.Carrier.txt",// Name suggestions file name
				new String[]{
							"Abraham Lincoln",
							"America",
							"Carl Vinson",
							"Constellation",
							"Coral Sea",
							"Dwight D. Eisenhower",
							"Forrestal",
							"Franklin D. Roosevelt",
							"George H. W. Bush",
							"George Washington",
							"Gerald R. Ford",
							"Harry S. Truman",
							"Independence",
							"Iwo Jima",
							"John C. Stennis",
							"John F. Kennedy",
							"Kitty Hawk",
							"Midway",
							"Nimitz",
							"Philippine Sea",
							"Ranger",
							"Ronald Reagan",
							"Saipan",
							"Saratoga",
							"Tarawa",
							"Theodore Roosevelt",
							"United States",
							"Valley Forge",
							"Wright",
				});
		final LoadDefinition[] loadDefinitionAirbase = new LoadDefinition[] {
				new LoadDefinition(ImageKey.CMDL_ID_FIGHTER, 1), 
				new LoadDefinition(ImageKey.CMDL_ID_BOMBER, 1), 
		};
		AIRBASE = new CommandableTemplate(
				ImageKey.CMDL_ID_AIRBASE, 		// ID
				MilitaryBranch.LAND_UNIT, // Unit type
				false,					// is focusable
				"Game.Unit.Airbase", 	// messsage key prefix
				0, 						// turns to produce
				new CommandableTemplate.RulesComplexityDependentValues[] {
						emptyCmdlRcdv, 
						emptyCmdlRcdv, 
						new CommandableTemplate.RulesComplexityDependentValues(
								loadDefinitionAirbase, 
								new int[][] { {0, 8}, }, 
								Integer.MAX_VALUE, 
								1, 2,						// User manual, page 135, rule g: Sighting range of 2 for air units in advanced rules
								bitmaskUnitsNone,			// Attack without move
								bitmaskUnitsNone,			// Attack including move
								1.0f), 
				},
				bitmaskUnitsAllButSubmarine,	// Visibility
				false,					// can move over
				0, 2, 1, 0,				// attack, defense, shot damage
				true,					// Image resource
				null,					// Prompt sound file name (~5 seconds)
				"AlertImp.wav",			// Alert impediment sound file name (~1 second)
				"Alert.wav",			// Alert fighting sound file name (~1 second)
				null,					// Move sound file name (0.1 seconds)
				"Fire.wav",				// Fight intro sound file name (0.4 seconds)
				"Fire.wav",				// Fight hit sound file name (0.4 seconds)
				"Death_Army.wav",		// Death sound file name (0.4 seconds)
				null, 					// Name suggestions file name
				emptyNameSuggestionsDefault);
		allCommandableTemplates = new CommandableTemplate[] {
				CMDL_CITY,
				ARMY,
				TANK,
				FIGHTER,
				BOMBER,
				TRANSPORTER,
				DESTROYER,
				SUBMARINE,
				CRUISER,
				BATTLESHIP,
				CARRIER,
				AIRBASE,
		};

		// -----------------------------------------------------------
		// The game map fields:

		final LoadDefinition[] loadDefinitionLand = new LoadDefinition[] { 
			new LoadDefinition(ImageKey.CMDL_ID_ARMY, 1),
			new LoadDefinition(ImageKey.CMDL_ID_TANK, 1),
			new LoadDefinition(ImageKey.CMDL_ID_BOMBER, 1),
			new LoadDefinition(ImageKey.CMDL_ID_FIGHTER, 1),
		};

		final LoadDefinition[] loadDefinitionMountain = new LoadDefinition[] { 
				new LoadDefinition(ImageKey.CMDL_ID_ARMY, 1),
				new LoadDefinition(ImageKey.CMDL_ID_BOMBER, 1),
				new LoadDefinition(ImageKey.CMDL_ID_FIGHTER, 1),
		};

		final RulesComplexityDependentValues rcdvUnexplored = new RulesComplexityDependentValues(
				loadDefinitionCity,
				bitmaskUnitsNone, // units slowed down
				bitmaskUnitsNone  // land units have defense bonus against
		);

		final RulesComplexityDependentValues rcdvCity = rcdvUnexplored;

		UNEXPLORED = new FieldTemplate(
				ImageKey.FIELD_ID_UNEXPLORED, 
				"Game.Field.Unexplored",
				true,				// isLand
				false,				// isRiverNeighbor
				1, 					// capacity
				new RulesComplexityDependentValues[] {
						rcdvUnexplored,
						rcdvUnexplored,
						rcdvUnexplored,
				} );

		final RulesComplexityDependentValues rcdvBorder = new RulesComplexityDependentValues(
				LoadDefinition.NO_LOAD,
				bitmaskUnitsNone, // units slowed down
				bitmaskUnitsNone  // land units have defense bonus against
		);

		BORDER = new FieldTemplate(
				ImageKey.FIELD_ID_BORDER, 
				"Game.Field.Border",   
				false, 				// isLand
				false,				// isRiverNeighbor
				0, 					// capacity
				new RulesComplexityDependentValues[] {
						rcdvBorder,
						rcdvBorder,
						rcdvBorder,
				} );

		final RulesComplexityDependentValues rcdvSea = new RulesComplexityDependentValues(
				new LoadDefinition[] {
						new LoadDefinition(ImageKey.CMDL_ID_BOMBER, 1),
						new LoadDefinition(ImageKey.CMDL_ID_FIGHTER, 1),
						new LoadDefinition(ImageKey.CMDL_ID_TRANSPORTER, 1),
						new LoadDefinition(ImageKey.CMDL_ID_DESTROYER, 1),
						new LoadDefinition(ImageKey.CMDL_ID_SUBMARINE, 1),
						new LoadDefinition(ImageKey.CMDL_ID_CRUISER, 1),
						new LoadDefinition(ImageKey.CMDL_ID_BATTLESHIP, 1),
						new LoadDefinition(ImageKey.CMDL_ID_CARRIER, 1), 
				},
				bitmaskUnitsNone, // units slowed down
				bitmaskUnitsNone  // land units have defense bonus against
		);

		SEA = new FieldTemplate(
				ImageKey.FIELD_ID_SEA, 
				"Game.Field.Sea",    
				false,				// isLand
				true,				// isRiverNeighbor
				1, 					// capacity
				new RulesComplexityDependentValues[] {
						rcdvSea,
						rcdvSea,
						rcdvSea,
				} );

		final RulesComplexityDependentValues rcdvLand = new RulesComplexityDependentValues(
				loadDefinitionLand,
				bitmaskUnitsNone, // units slowed down
				bitmaskUnitsNone  // land units have defense bonus against
		);

		LAND = new FieldTemplate(
				ImageKey.FIELD_ID_LAND, 
				"Game.Field.Land",     
				true, 				// isLand
				false,				// isRiverNeighbor 
				1, 					// capacity
				new RulesComplexityDependentValues[] {
						rcdvLand,
						rcdvLand,
						rcdvLand,
				} );

		ROUGH = new FieldTemplate(
				ImageKey.FIELD_ID_ROUGH, 
				"Game.Field.Rough",    
				true, 				// isLand
				false,				// isRiverNeighbor 
				1, 					// capacity
//				CommandableTemplate.LAND_LOAD_DEFINITION,
				new RulesComplexityDependentValues[] {
						rcdvLand,
						rcdvLand,
						new RulesComplexityDependentValues(
								loadDefinitionLand,
								bitmaskUnitsTankOnly, // Slowed down
								bitmaskUnitsNone       // defense bonus against
						),
				});

		RIVER = new FieldTemplate(
				ImageKey.FIELD_ID_RIVER, 
				"Game.Field.River",    
				true, 				// isLand
				true,				// isRiverNeighbor 
				1, 					// capacity
				// CommandableTemplate.LAND_LOAD_DEFINITION,
				new RulesComplexityDependentValues[] {
						rcdvLand,
						rcdvLand,
						new RulesComplexityDependentValues(
								loadDefinitionLand,
								bitmaskUnitsTankOnly,   // Slowed down
								bitmaskUnitsAllMovingButArmy  // defense bonus against
						),
				});

		FOREST = new FieldTemplate(
				ImageKey.FIELD_ID_FOREST, 
				"Game.Field.Forest",   
				true, 				// isLand
				false,				// isRiverNeighbor 
				1, 					// capacity
				// CommandableTemplate.LAND_LOAD_DEFINITION,
				new RulesComplexityDependentValues[] {
						rcdvLand,
						rcdvLand,
						new RulesComplexityDependentValues(
								loadDefinitionLand,
								bitmaskUnitsTankOnly,   // Slowed down
								bitmaskUnitsAllMovingButArmy  // defense bonus against
						),
				});

		MOUNTAIN = new FieldTemplate(
				ImageKey.FIELD_ID_MOUNTAIN, 
				"Game.Field.Mountain", 
				true, 				// isLand
				false,				// isRiverNeighbor 
				1, 					// capacity
				// CommandableTemplate.LAND_LOAD_DEFINITION,
				new RulesComplexityDependentValues[] {
						rcdvLand,
						rcdvLand,
						new RulesComplexityDependentValues(
								loadDefinitionMountain,
								bitmaskUnitsTankOnly,   // Slowed down
								bitmaskUnitsAllMovingButArmy  // defense bonus against
						),
				});

		FIELD_CITY = new FieldTemplate(
				ImageKey.FIELD_ID_CITY, 
				CMDL_CITY.messageKeyPrefix, 
				true, 				// isLand
				true,				// isRiverNeighbor 
				Integer.MAX_VALUE, 	// capacity
				new RulesComplexityDependentValues[] {
						rcdvCity,
						rcdvCity,
						rcdvCity,
				} );

		/** All field templates, mapping allows access per field ID. */
		allGameMapFieldTemplates = new FieldTemplate[] {
			UNEXPLORED,
			BORDER,
			SEA,
			LAND,
			FIELD_CITY,
			ROUGH,
			RIVER,
			FOREST,
			MOUNTAIN,
		};
		
	}

	public final CommandableTemplate UNDEFINED;

	public final CommandableTemplate NOTHING;

	public final CommandableTemplate CMDL_CITY;

	public final CommandableTemplate ARMY;

	public final CommandableTemplate TANK;

	public final CommandableTemplate BOMBER;

	public final CommandableTemplate FIGHTER;

	public final CommandableTemplate TRANSPORTER;

	public final CommandableTemplate DESTROYER;

	public final CommandableTemplate SUBMARINE;

	public final CommandableTemplate CRUISER;

	public final CommandableTemplate BATTLESHIP;

	public final CommandableTemplate CARRIER;

	public final CommandableTemplate AIRBASE;

	final CommandableTemplate[] allCommandableTemplates;

	public final FieldTemplate UNEXPLORED;

	public final FieldTemplate BORDER;

	public final FieldTemplate SEA;

	public final FieldTemplate LAND;

	public final FieldTemplate ROUGH;

	public final FieldTemplate RIVER;

	public final FieldTemplate FOREST;

	public final FieldTemplate MOUNTAIN;

	public final FieldTemplate FIELD_CITY;

	public final FieldTemplate[] allGameMapFieldTemplates;

}
