package de.ped.empire.logic;

import java.awt.Graphics;
import java.awt.Point;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.PlayfieldPosition;
import de.ped.tools.TopologyMode;

/**
 * A {@link Tileable} with a position.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TileObjectDelegate implements PositionedTileable {
	TileObjectDelegate(final Tileable tileable, final PlayfieldPosition position) {
		this.tileable = tileable;
		this.position = position;
	}

	TileObjectDelegate(final PositionedTileable tileable) {
		this.tileable = tileable;
		this.position = tileable.getPosition();
	}

	@Override
	public PlayfieldPosition getPosition() {
		return position;
	}

	@Override
	public String toString() {
		return tileable.toString() + " at " + position.toString();
	}

	@Override
	public int getImageKey() {
		return tileable.getImageKey();
	}

	@Override
	public void setImageKey(int imageKey) {
		tileable.setImageKey(imageKey);
	}

	@Override
	public int getPlayerId() {
		return tileable.getPlayerId();
	}

	@Override
	public I18NStringWithFillIns getName() {
		return tileable.getName();
	}

	@Override
	public I18NStringWithFillIns getToolTip(final GameView gameView) {
		return tileable.getToolTip(gameView);
	}

	@Override
	public char getLogRepresentation() {
		return tileable.getLogRepresentation();
	}

	@Override
	public void paint(final Graphics g, final Point offset, final ZoomLevel zoomLevel, final TopologyMode topologyMode) {
		tileable.paint(g, offset, zoomLevel, topologyMode);
	}

	@Override
	public void paint(final Graphics g, final Point offset, final float x, final float y, final int imgKey, final float dyingProgress, final ZoomLevel zoomLevel, final TopologyMode topologyMode) {
		tileable.paint(g, offset, x, y, imgKey, dyingProgress, zoomLevel, topologyMode);
	}

	private final Tileable tileable;
	private final PlayfieldPosition position;
}