package de.ped.empire.logic;

import java.awt.Graphics;
import java.awt.Point;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;
import de.ped.tools.PlayfieldPosition;
import de.ped.tools.TopologyMode;

/**
 * A unit how a player sees it.
 * 
 * @author Copyright (c) 2015 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ViewUnit implements UnitAlike {

	ViewUnit(final Unit unit, final int turn, final boolean isOtherView) {
		this(unit, turn, isOtherView, unit.getPosition());
	}

	ViewUnit(final Unit unit, final int turnOfSighting, final boolean isOtherView, final PlayfieldPosition position) {
		this.stateUnit = unit;
		this.spottedField = unit.getGameState().getMap().getAt(position);
		this.turnOfSighting = turnOfSighting;
		int ikey = unit.getImageKey();
		if (isOtherView) {
			ikey = ImageKey.setBits(ikey, ImageKey.BA_NONEMPTY, 0);
			ikey = ImageKey.setBits(ikey, ImageKey.BA_SENTRIED, 0);
		}
		setImageKey(ikey);
	}

	/**
	 * Marshalling constructor. Should only be called for enemy units in this view.
	 * @param marshaller The marshaller to read from.
	 * @param gameState The game state to read into.
	 * @param key The commandable key of the object to continue reaading.
	 */
	ViewUnit(final Marshaller marshaller, final GameState gameState, final CommandableKey key) {
//		final CommandableKey key = CommandableKey.Factory.create(marshaller);
		this.stateUnit = gameState.findUnit(key);
		this.imageKey = marshaller.readInt();
		this.turnOfSighting = marshaller.readInt();
		final PlayfieldPosition pos = new PlayfieldPosition();
		pos.demarshal(marshaller);
		this.spottedField = gameState.getMap().getAt(pos);
	}

	/**
	 * Writes only the attributes of this subclass. So inherently assumes that the unit key has already been written.
	 * Should only be called for enemy units in this view.
	 */
	@Override
	public void marshal(final Marshaller marshaller) {
//		marshaller.write(stateUnit.getKey());
		marshaller.writeInt(imageKey);
		marshaller.writeInt(turnOfSighting);
		spottedField.getPosition().marshal(marshaller);
	}

	@Override
	public Marshallable demarshal(final Marshaller marshaller) {
		throw Marshaller.createConstantMarshallableDemarshallingException();
	}

	@Override
	public boolean canLoad(final UnitAlike unitToLoad, final boolean isNow) {
		return stateUnit.canLoad(unitToLoad, isNow);
	}

	@Override
	public PlayfieldPosition getPosition() {
		return getField().getPosition();
	}

	@Override
	public int getImageKey() {
		return imageKey;
	}

	@Override
	public void setImageKey(final int imageKey) {
		// TODO: Filtern nach dem, was dieser view sehen darf!
		this.imageKey = imageKey & TileTemplate.getTTD(imageKey).normalizationMask;
	}

	@Override
	public int getPlayerId() {
		return ImageKey.getPlayer(stateUnit.getGameState(), imageKey).getId();
	}

	@Override
	public I18NStringWithFillIns getName() {
		return stateUnit.getName();
	}

	@Override
	public I18NStringWithFillIns getToolTip(final GameView gameView) {
		return stateUnit.getToolTip(gameView);
	}

	@Override
	public char getLogRepresentation() {
		return stateUnit.getLogRepresentation();
	}

	@Override
	public void paint(final Graphics g, final Point offset, final ZoomLevel zoomLevel, final TopologyMode topologyMode) {
		stateUnit.paint(g, offset, zoomLevel, topologyMode);
	}

	@Override
	public void paint(final Graphics g, final Point offset, final float x, final float y, final int imgKey, final float dyingProgress, final ZoomLevel zoomLevel, final TopologyMode topologyMode) {
		stateUnit.paint(g, offset, x, y, imgKey, dyingProgress, zoomLevel, topologyMode);
	}

	@Override
	public Unit getUnit() {
		return stateUnit;
	}

	@Override
	public CommandableKey getKey() {
		return stateUnit.getKey();
	}

	@Override
	public String getNickName() {
		return stateUnit.getNickName();
	}

	@Override
	public void setNickName(final String name) {
		stateUnit.setNickName(name);
	}

	@Override
	public boolean needsNickName() {
		return stateUnit.needsNickName();
	}

	@Override
	public String getNickNameSuggestion() {
		return stateUnit.getNickNameSuggestion();
	}

	@Override
	public int getSightingRange(final MilitaryBranch observed) {
		return stateUnit.getSightingRange(observed);
	}

	@Override
	public boolean hasSeenEnemiesLately() {
		return stateUnit.hasSeenEnemiesLately();
	}

	@Override
	public void setSeenEnemiesLately(final boolean value) {
		stateUnit.setSeenEnemiesLately(value);
	}

	@Override
	public void startTurn() {
		stateUnit.startTurn();
	}

	@Override
	public void finishTurn() {
		stateUnit.finishTurn();
	}

	@Override
	public boolean isNextable() {
		return stateUnit.isNextable();
	}

	@Override
	public boolean wasNexted() {
		return stateUnit.wasNexted();
	}

	@Override
	public void resetNext() {
		stateUnit.resetNext();
	}

	@Override
	public void nextFocusGained() {
		stateUnit.nextFocusGained();
	}

	@Override
	public void nextFocusLost() {
		stateUnit.nextFocusLost();
	}

	@Override
	public AttackMode getAttackModeAgainst(final Commandable opponent) {
		return stateUnit.getAttackModeAgainst(opponent);
	}

	@Override
	public float getFightStrength(final Commandable opponent, final boolean isAttacker) {
		return stateUnit.getFightStrength(opponent, isAttacker);
	}

	@Override
	public int getAttack() {
		return stateUnit.getAttack();
	}

	@Override
	public int getDefense() {
		return stateUnit.getDefense();
	}

	@Override
	public int getShotDamage() {
		return stateUnit.getShotDamage();
	}

	@Override
	public boolean isAlive() {
		return stateUnit.isAlive();
	}

	@Override
	public boolean isDamaged() {
		return stateUnit.isDamaged();
	}

	@Override
	public int getHitsTillDeath() {
		return stateUnit.getHitsTillDeath();
	}

	@Override
	public void receiveHits(final int numberOfHits) {
		stateUnit.receiveHits(numberOfHits);
	}

	@Override
	public boolean hasMovedThisTurn() {
		return stateUnit.hasMovedThisTurn();
	}

	@Override
	public void setSkippingThisTurn(final boolean value) {
		stateUnit.setSkippingThisTurn(value);
	}

	@Override
	public Order getOrder() {
		return stateUnit.getOrder();
	}

	@Override
	public void setOrder(final Order order) {
		stateUnit.setOrder(order);
	}

	@Override
	public MoveResult evaluateOrder(final GameView gameView, final boolean isNow) {
		return stateUnit.evaluateOrder(gameView, isNow);
	}

	public Field getField() {
		return spottedField.getField();
	}

	@Override
	public CommandableKey getParentKey() {
		return stateUnit.getParentKey();
	}

	@Override
	public CommandableTemplate getUnitTemplate() {
		return stateUnit.getUnitTemplate();
	}

	@Override
	public int hashCode() {
		return ((stateUnit.getKey() == null) ? 0 : stateUnit.getKey().hashCode());
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof UnitAlike)) {
			return false;
		}
		UnitAlike other = (UnitAlike) obj;
		if (stateUnit.getKey() == null) {
			if (other.getKey() != null) {
				return false;
			}
		} else if (!stateUnit.getKey().equals(other.getKey())) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(final UnitAlike other) {
		return stateUnit.getKey().compareTo(other.getKey());
	}

	@Override
	public int getTurnOfSighting() {
		return turnOfSighting;
	}

	private final Unit stateUnit;
	private final int turnOfSighting;
	private int imageKey;
	private FieldAlike spottedField;
}
