package de.ped.empire.middleware;

import de.ped.empire.logic.FieldCity;
import de.ped.empire.logic.GameNotification;
import de.ped.empire.logic.GameView;
import de.ped.empire.logic.Unit;
import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;
import de.ped.tools.PlayfieldPosition;

/**
 * The base class for all messages between client and server.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class EmpireMessage implements Marshallable {
	public enum MSGTYPE {
		GAME_NOTIFICATION,
		UNIT_PRODUCED_NOTIFICATION,
		STEP_NOTIFICATION,
		RESERVED1,
	}

	public static EmpireMessage createGameNotification(final int playerId, final GameNotification notif) {
		return new EmpireMessage(MSGTYPE.GAME_NOTIFICATION, notif, null, null);
	}

	public static EmpireMessage createUnitProducedNotification(final int playerId, final FieldCity city, final Unit unit) {
		return new EmpireMessage(MSGTYPE.UNIT_PRODUCED_NOTIFICATION, null, city, unit);
	}

	public static EmpireMessage createStepNotification() {
		return new EmpireMessage(MSGTYPE.STEP_NOTIFICATION, null, null, null);
	}

	public final MSGTYPE msgType;
	public final GameNotification gameNotification;
	public final FieldCity city;
	public final Unit unit;

	public EmpireMessage(final MSGTYPE msgType, final GameNotification gameNotification, final FieldCity city, final Unit unit) {
		this.msgType = msgType;
		this.gameNotification = gameNotification;
		this.city = city;
		this.unit = unit;
	}

	public EmpireMessage(final Marshaller marshaller, final GameView gameView) {
		this.msgType = MSGTYPE.values()[marshaller.readByte()];
		if (marshaller.readNullIndicator()) {
			this.gameNotification = null;
		} else {
			this.gameNotification = new GameNotification(marshaller, gameView);
		}
		if (marshaller.readNullIndicator()) {
			this.city = null;
		} else {
			PlayfieldPosition pos = new PlayfieldPosition();
			pos.demarshal(marshaller);
			this.city = (FieldCity) gameView.getAt(pos).getField();
		}
		if (marshaller.readNullIndicator()) {
			this.unit = null;
		} else {
			this.unit = Unit.create(gameView.getGameState(), marshaller);
		}
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		marshaller.writeByte((byte) msgType.ordinal()); 
		marshaller.writeNullOrMarshallable(gameNotification);
		if (!marshaller.writeNullIndicator(city)) {
			marshaller.writeMarshallable(city.getField().getPosition());
		}
		marshaller.writeNullOrMarshallable(unit);
	}

	@Override
	public Marshallable demarshal(Marshaller marshaller) {
		throw Marshaller.createConstantMarshallableDemarshallingException();
	}

}
