package de.ped.empire.test;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.ped.empire.logic.CommandableTemplate;
import de.ped.empire.logic.FieldCity;
import de.ped.empire.logic.GameInitializationException;
import de.ped.empire.logic.GameMap;
import de.ped.empire.logic.GameNotification;
import de.ped.empire.logic.GameState;
import de.ped.empire.logic.GameView;
import de.ped.empire.logic.Player;
import de.ped.empire.logic.Templates;
import de.ped.empire.logic.Tutorial;
import de.ped.empire.logic.TutorialLevel;
import de.ped.empire.logic.Unit;
import de.ped.empire.middleware.EmpireMessage;
import de.ped.tools.Marshaller;
import de.ped.tools.PlayfieldPosition;

public class EmpireMessageTest {

	@Before
	public void setUp() throws GameInitializationException {
		final Tutorial tutorial = new Tutorial();
		final GameMap map = tutorial.createMap(TutorialLevel.LAND_UNITS);
		observer = new GameObserverStub();
		ownCity = map.findCities(0).get(0);
		enemyCity = map.findCities(1).get(0);
		final GameState gameState = new GameState(observer, map);
		gameState.startGame();
		gameView = gameState.getCurrentGameView();
		gameView.clearGetReady();
	}

	@After
	public void tearDown() {
		if (null != gameView.getMap()) {
		    final Marshaller marshaller = new Marshaller();
		    marshaller.writeMarshallable(gameView.getGameState());
		    final byte[] data = marshaller.pack();
			final Marshaller d = new Marshaller(data);
			final GameState received = new GameState(observer, d);
			GameMapTest.compareMaps(gameView.getGameState().getMap(), received.getMap());
			for (int i = 0; i < Player.MAX_NUMBER_OF_PLAYERS; i++) {
				GameMapTest.compareMaps(gameView.getGameState().getGameView(i).getMap(), received.getGameView(i).getMap());
			}
		}
	}

	@Test
	public void test() {
		EmpireMessage msg1 = EmpireMessage.createStepNotification();
		EmpireMessage msg2 = EmpireMessage.createGameNotification(gameView.getPlayer().getId(), new GameNotification());
		final Unit unit1 = createUnit(gameView, ownCity, Templates.instance().ARMY, null);
		EmpireMessage msg3 = EmpireMessage.createUnitProducedNotification(gameView.getPlayer().getId(), ownCity, unit1);
		Marshaller m = new Marshaller();
		m.writeMarshallable(msg1);
		m.writeMarshallable(msg2);
		m.writeMarshallable(msg3);
		EmpireMessage msg1n = new EmpireMessage(m, gameView);
		EmpireMessage msg2n = new EmpireMessage(m, gameView);
		EmpireMessage msg3n = new EmpireMessage(m, gameView);
		assertEquals(msg2.gameNotification.getAnimationPartStep(), msg2n.gameNotification.getAnimationPartStep());
		assertEquals(msg3.gameNotification, msg3n.gameNotification);
		assertEquals(msg3.city, msg3n.city);
		assertEquals(msg3.unit, msg3n.unit);
	}

	static Unit createUnit(final GameView gameView, final FieldCity city, final CommandableTemplate template, final PlayfieldPosition position) {
		return Tutorial.createUnit(gameView, city, template, position);
	}

	private GameObserverStub observer;
	private GameView gameView;
	private FieldCity ownCity;
	@SuppressWarnings("unused")
	private FieldCity enemyCity;
}
