package de.ped.empire.test;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Test;

import de.ped.empire.logic.Commandable;
import de.ped.empire.logic.FieldAlike;
import de.ped.empire.logic.GameInitializationException;
import de.ped.empire.logic.GameState;
import de.ped.empire.logic.GameView;
import de.ped.empire.logic.GameMap;
import de.ped.empire.logic.FieldCity;
import de.ped.empire.logic.GameProperties;
import de.ped.empire.logic.Order;
import de.ped.empire.logic.PlayerType;
import de.ped.empire.logic.Templates;
import de.ped.empire.logic.Tutorial;
import de.ped.empire.logic.TutorialLevel;
import de.ped.empire.logic.Unit;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Tests for the {@link GameView} class.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GameEndTest {

	/** 
	 * Tests switching units and selecting units. 
	 * @throws GameInitializationException if game initialization failed
	 */
	@Test
	public void testWinByKillingLastCommandble() throws GameInitializationException {
		final Tutorial tutorial = new Tutorial();
		final GameMap map = tutorial.createMap(TutorialLevel.ENEMY_ISLANDS);
		final GameObserverStub observer;
		final GameView gameView;
		final FieldCity ownCity;
		final FieldCity enemyCity;
		observer = new GameObserverStub();
		ownCity = map.findCities(0).get(0);
		enemyCity = map.findCities(1).get(0);
		final GameState gameState = new GameState(observer, map);
		gameState.startGame();
		gameView = gameState.getCurrentGameView();
		gameView.clearGetReady();

		final GameProperties props = gameState.getProperties();
		props.getPlayers()[0].setPlayerTypeId(PlayerType.DEFAULT.id);
		props.getPlayers()[1].setPlayerTypeId(PlayerType.COMPUTER_SIMPLE.id);
		// Enemy city gets surrounded by lands
		List<FieldAlike> neighbors = map.getAllFieldsInAirDistance(enemyCity, 1, false, 2);
		gameView.setCityProduction(ownCity, Templates.instance().NOTHING);
		for (FieldAlike f : neighbors) {
			final Unit unit1 = GameMoveTest.createUnit(gameView, ownCity, Templates.instance().TANK, f.getPosition());
			assertEquals(Templates.instance().TANK.id, unit1.getKey().getTemplateId());
		}
		gameView.clearGetReady();
		while(true) {
			Commandable c = gameView.getVisibleCommandableInFocus();
			if (null == c) {
				 break;
			} else if (c instanceof Unit) {
				Unit unit1 = (Unit) c;
				assertEquals(Templates.instance().TANK.id, unit1.getKey().getTemplateId());
				final Order order = gameView.getMoveOrAttackOrder(unit1, enemyCity.getPosition());
				assertTrue(order.makesSense());
				unit1.setOrder(order);
				assertEquals(2, unit1.getStepsLeftThisTurn());
				final int resultMask = GameMoveTest.clearAnimation(gameView, observer);
				logger.info("Attack of " + unit1.toString() + " resulted in result mask " + Integer.toHexString(resultMask));
			}
			gameView.nextCommandable();
		}
//		gameView.nextCommandable();
//		assertEquals(unit1, gameView.getVisibleCommandableInFocus());
//		gameView.nextCommandable();
//		assertEquals(unit2, gameView.getVisibleCommandableInFocus());
	}

	private Logger logger = Logger.getLogger(getClass());

	public GameEndTest() {
		logger.setLogLevel(Level.INFO);
	}
}
