package de.ped.empire.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.empire.logic.CircularReferenceException;
import de.ped.empire.logic.GameViewProperties;
import de.ped.empire.logic.MilitaryBranch;
import de.ped.empire.logic.MovePath;
import de.ped.empire.logic.MovePaths;
import de.ped.tools.PlayfieldPosition;
import de.ped.tools.test.GenericTest;

/**
 * Unit tests for {@link MovePaths} and its content, without interaction with game state or game map.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class MovePathsTest {

	@Test
	public void testAdd() throws CircularReferenceException {
		final MovePaths paths = new MovePaths();
		final int playerId = 0;
		final PlayfieldPosition start = new PlayfieldPosition( 5,  5);
		final PlayfieldPosition end1  = new PlayfieldPosition(10, 10);
		final PlayfieldPosition end2  = new PlayfieldPosition( 5, 10);
		paths.add(new MovePath(playerId, start, MilitaryBranch.AIR_UNIT.mask | MilitaryBranch.LAND_UNIT.mask, end1));
		paths.add(new MovePath(playerId, start, MilitaryBranch.SEA_UNIT.mask, end2));
		{
			assertEquals(2, paths.findMovePaths(playerId, start).size());
			final MovePath land = paths.findMovePath(playerId, start, MilitaryBranch.LAND_UNIT);
			final MovePath air = paths.findMovePath(playerId, start, MilitaryBranch.AIR_UNIT);
			final MovePath sea = paths.findMovePath(playerId, start, MilitaryBranch.SEA_UNIT);
			assertEquals(end1, land.getEndPos());
			assertEquals(land, air);
			assertEquals(end2, sea.getEndPos());
		}
		paths.add(new MovePath(playerId, start, MilitaryBranch.SEA_UNIT.mask, end1));
		{
			assertEquals(2, paths.findMovePaths(playerId, start).size());
			paths.cleanup();
			assertEquals(1, paths.findMovePaths(playerId, start).size());
			final MovePath land = paths.findMovePath(playerId, start, MilitaryBranch.LAND_UNIT);
			final MovePath air = paths.findMovePath(playerId, start, MilitaryBranch.AIR_UNIT);
			final MovePath sea = paths.findMovePath(playerId, start, MilitaryBranch.SEA_UNIT);
			assertNotNull(land);
			assertEquals(end1, land.getEndPos());
			assertEquals(land, air);
			assertEquals(land, sea);
		}
		paths.add(new MovePath(playerId, start, MilitaryBranch.SEA_UNIT.mask, end2));
		{
			assertEquals(2, paths.findMovePaths(playerId, start).size());
			final MovePath land = paths.findMovePath(playerId, start, MilitaryBranch.LAND_UNIT);
			final MovePath air = paths.findMovePath(playerId, start, MilitaryBranch.AIR_UNIT);
			final MovePath sea = paths.findMovePath(playerId, start, MilitaryBranch.SEA_UNIT);
			assertEquals(end1, land.getEndPos());
			assertEquals(land, air);
			assertEquals(end2, sea.getEndPos());
		}
	}

	@Test(expected=CircularReferenceException.class)
	public void testCircularReferenceException() throws CircularReferenceException {
		final MovePaths paths = new MovePaths();
		final int playerId = 0;
		final PlayfieldPosition start = new PlayfieldPosition( 5, 5);
		final PlayfieldPosition end1 = new PlayfieldPosition( 10, 10);
		paths.add(new MovePath(playerId, start, MilitaryBranch.AIR_UNIT.mask, end1));
		paths.add(new MovePath(playerId, end1, MilitaryBranch.AIR_UNIT.mask, start));
	}

	/**
	 * @see GameViewProperties#equals(Object)
	 * @see GameViewProperties#clone()
	 * @throws CircularReferenceException if some move paths would lead to an endless loop for a unit
	 */
	@Test
	public void testObjectBasics() throws CircularReferenceException {
		final int playerId = 0;
		final PlayfieldPosition start = new PlayfieldPosition( 5,  5);
		final PlayfieldPosition end1  = new PlayfieldPosition(10, 10);
		final PlayfieldPosition end2  = new PlayfieldPosition( 5, 10);
		MovePath path1 = new MovePath(playerId, start, MilitaryBranch.AIR_UNIT.mask | MilitaryBranch.LAND_UNIT.mask, end1);
		MovePath path2 = new MovePath(playerId, start, MilitaryBranch.SEA_UNIT.mask, end2);
		final MovePaths paths = new MovePaths();
		paths.add(path1);
		paths.add(path2);
		final MovePaths other = new MovePaths();
		other.add(path1);
		final Object a = path1;
		final Object b = path2;
		GenericTest.checkObjectBasics(false, a, b);
		GenericTest.checkObjectBasics(false, paths, other);
	}
}
