package de.ped.empire.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.empire.logic.FieldCity;
import de.ped.empire.logic.GameInitializationException;
import de.ped.empire.logic.GameMap;
import de.ped.empire.logic.GameState;
import de.ped.empire.logic.GameView;
import de.ped.empire.logic.StepRatingMap;
import de.ped.empire.logic.Templates;
import de.ped.empire.logic.Tutorial;
import de.ped.empire.logic.TutorialLevel;
import de.ped.empire.logic.Unit;
import de.ped.tools.PlayfieldPosition;

/**
 * Unit tests for the {@link StepRatingMap} class.
 * 
 * @author Copyright (c) 2019 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class StepRatingMapTest {

	@Test
	public void testLand() throws GameInitializationException {
		final StepRatingMap map = init(TutorialLevel.SINGLE_PATH_LAND);
		assertEquals(2, map.get(new PlayfieldPosition(4, 2)).distanceFromStart);
		assertEquals(3, map.get(new PlayfieldPosition(5, 2)).distanceFromStart);
		assertEquals(4, map.get(new PlayfieldPosition(6, 2)).distanceFromStart);
	}

	@Test
	public void testRough() throws GameInitializationException {
		final StepRatingMap map = init(TutorialLevel.SINGLE_PATH_ROUGH);
		assertEquals(3, map.get(new PlayfieldPosition(4, 2)).distanceFromStart);
		assertEquals(4, map.get(new PlayfieldPosition(5, 2)).distanceFromStart);
		assertEquals(5, map.get(new PlayfieldPosition(6, 2)).distanceFromStart);
	}

	private StepRatingMap init(final TutorialLevel tutorialLevel) throws GameInitializationException {
		final Tutorial tutorial = new Tutorial();
		final GameMap map = tutorial.createMap(tutorialLevel);
		final GameObserverStub observer = new GameObserverStub();
		final FieldCity[] playerCities = new FieldCity[2]; 
		playerCities[0] = map.findCities(0).get(0);
		playerCities[1] = map.findCities(1).get(0);
		final GameState gameState = new GameState(observer, map);
		gameState.startGame();
		final GameView player1GameView = gameState.getCurrentGameView();
		player1GameView.clearGetReady();
		final Unit unit1 = Tutorial.createUnit(gameState, playerCities[0], Templates.instance().TANK, playerCities[0].getPosition());
		player1GameView.nextCommandable();
		assertEquals(unit1, player1GameView.getVisibleCommandableInFocus());
		final StepRatingMap srmap = unit1.getStepRatings();
		assertEquals(0, srmap.get(new PlayfieldPosition(2, 2)).distanceFromStart);
		assertEquals(1, srmap.get(new PlayfieldPosition(3, 2)).distanceFromStart);
		return srmap;
	}
}
