package de.ped.empire.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.empire.logic.Commandable;
import de.ped.empire.logic.GameInitializationException;
import de.ped.empire.logic.TileTemplate;
import de.ped.empire.logic.Field;
import de.ped.empire.logic.GameMap;
import de.ped.empire.logic.GameProperties;
import de.ped.empire.logic.GameState;
import de.ped.empire.logic.GameView;
import de.ped.empire.logic.ImageKey;
import de.ped.empire.logic.Player;
import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.Messages;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Tests for the {@link TileTemplate} class and {@link Commandable} interface.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TileTemplateTest {

	@Test
	public void test() throws GameInitializationException {
		final Logger logger = Logger.getLogger(getClass());
		logger.setLogLevel(Level.INFO);
		final GameObserverStub observer = new GameObserverStub();
		final GameProperties props = new GameProperties();
		final GameMap map = new GameMap(props);
		map.addAtRandom(props);
		map.finish();
		final GameState gameState = new GameState(observer, map);
		gameState.initForQuickStart();
		gameState.startGame();
		final GameView gameView = gameState.getCurrentGameView();
		final TileTemplate[] testlings = new TileTemplate[] {
				new Field(0, 0, ImageKey.FIELD_ID_BORDER),
				new Field(1, 1, ImageKey.FIELD_ID_SEA),
				new Field(2, 2, ImageKey.FIELD_ID_LAND),
				gameView.getMap().findCities(0).get(0),
				gameView.getMap().findCities(1).get(0),
				gameView.getMap().findCities(Player.ID_NONE).get(0),
		};
		for (final TileTemplate testling : testlings) {
			log(logger, gameView, testling);
		}
		for (final TileTemplate testling : gameState.getAllUnits()) {
			log(logger, gameView, testling);
		}
	}

	private void log(final Logger logger, final GameView gameView, final TileTemplate d_o) {
		final Messages messages = gameView.getMessages();
		log(logger, messages, "Name", d_o.getName());
		log(logger, messages, "TTip", d_o.getToolTip(gameView));
	}

	private void log(final Logger logger, final Messages messages, final String prefix, final I18NStringWithFillIns i18ns) {
		final String str = messages.getString(i18ns); 
		assertNotNull(str);
		logger.debug(prefix + " " + str + " <-- " + i18ns.toString());
	}
}
