package de.ped.gedicht.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Locale;

import javax.swing.JEditorPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;

import de.ped.gedicht.logic.Poem;
import de.ped.gedicht.logic.PoemFacade;
import de.ped.tools.ApplicationEnvironment;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextDocumentType;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PedAction;

/**
 * The main window of the application.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GedichtMainWindow extends ApplicationMainWindow {
	private static final long serialVersionUID = 1L;

	/** The big text area displaying the logical model. */
	private JEditorPane textBuf;
	private JScrollPane worldScrollPane;

	/**
	 * Creates a poem.
	 * 
	 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	class CreatePoemAction extends PedAction {
		private static final long serialVersionUID = 1L;

		public CreatePoemAction(Poem.Algorithm algo) {
			super(algo.actionKey, GedichtMainWindow.this, null, false);
			this.algo = algo;
		}

		@Override
		public void securedActionPerformed(final ActionEvent e) {
			createPoem(algo);
		}

		private Poem.Algorithm algo;
	}

	private CreatePoemAction[] createPoemActions = new CreatePoemAction[] {
			new CreatePoemAction(Poem.Algorithm.WITHOUT_RHYME),
			new CreatePoemAction(Poem.Algorithm.WITH_RHYME),
	};

	public GedichtMainWindow(final ApplicationEnvironment appEnv) {
		super(appEnv);
		getContentPane().setLayout(new BorderLayout());
		textBuf = new JEditorPane();
		textBuf.setPreferredSize(new Dimension(400, 300));
		worldScrollPane = new JScrollPane(textBuf);
		getContentPane().add(worldScrollPane, BorderLayout.CENTER);
		setJMenuBar(createMenuBar());
		createPoem(Poem.Algorithm.WITHOUT_RHYME);
		pack(); // Resize the window to the preferred size of its components.
		switchLocale(getMessages().getCurrentLocale());
	}

	@Override
	public void switchLocale(final Locale locale) {
		super.switchLocale(locale);
		repaint();
	}

	@Override
	public void buildHelpDocument(final TextDocumentBuilder doc) {
		doc.setTitle(getMessages().getString("HelpWindow.Title"));
		doc.setHeading("Help");
		doc.beginDocument();
		doc.addContentPart(2, "Help.Introduction");
		buildMenuDescriptions(doc, getJMenuBar());
		doc.endDocument();
	}

	private JMenuBar createMenuBar() {
		final JMenuBar menuBar = createJMenuBar();
		final JMenu menuFile = createJMenu("Menu.File");
		for (CreatePoemAction action : createPoemActions) {
			createActionItem(menuFile, action);
		}
		menuFile.addSeparator();
		createActionItem(menuFile, quitAction);
		menuBar.add(menuFile);
		final JMenu menuHelp = createJMenu("Menu.Help");
		createActionItem(menuHelp, helpAction);
		createActionItem(menuHelp, aboutAction);
		menuBar.add(menuHelp);
		try {
			menuBar.setHelpMenu(menuHelp);
		} catch (final Throwable e) {
			// Nothing to do - this feature is a nice to have.
		}
		return menuBar;
	}

	private void createPoem(final Poem.Algorithm algo) {
		final Poem poem = PoemFacade.instance().create(algo);
		textBuf.setText(poem.createPoem(TextDocumentType.TEXT, getMessages().getString(algo.heading)));
	}
}
