package de.ped.gedicht.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.TextDocumentBuilder;

/**
 * Creates a poem based on a vocabulary of less than 100 words.
 * There is no rhyme, but several different sentence strucures.
 * 
 * @author Copyright (c) 1987-2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Poem1 extends Poem {

	/**
	 * A noun needs to be aware of its genus.
	 * 
	 * @author Copyright (c) 1987-2008 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	private static class Noun {
		public Noun(final Genus genus, final String subject) {
			this.genus = genus;
			this.subject = subject;
		}
	
		public String getSubject() {
			return subject;
		}
	
		public Genus getGenus() {
			return genus;
		}
	
		private final Genus genus;
		private final String subject;
	}

	/**
	 * Sends a poem to the console.
	 * 
	 * @param args ignored
	 */
	public static void main(String[] args) {
		Poem.debug(Poem.Algorithm.WITHOUT_RHYME);
	}

	private static final String[][] ARTICLES = new String[][] {
			{"der",   "die",   "das"},
			{"ein",   "eine",  "ein"},
			{"jeder", "jede",  "jedes"},
			{"kein",  "keine", "kein"},
	};

	private static final Noun[] NOUNS = new Noun[] {
		new Noun(Genus.MALE, "Wald"),
		new Noun(Genus.MALE, "Friede"),
		new Noun(Genus.MALE, "Vogel"),
		new Noun(Genus.MALE, "Gott"),
		new Noun(Genus.MALE, "Acker"),
		new Noun(Genus.MALE, "Mensch"),
		new Noun(Genus.MALE, "Engel"),
		new Noun(Genus.MALE, "Himmel"),
		new Noun(Genus.MALE, "Urknall"),
		new Noun(Genus.MALE, "Mond"),
		new Noun(Genus.MALE, "Stern"),
		new Noun(Genus.MALE, "Stein"),
		new Noun(Genus.FEMALE, "Wiese"),
		new Noun(Genus.FEMALE, "Maus"),
		new Noun(Genus.FEMALE, "Wüste"),
		new Noun(Genus.FEMALE, "Höhle"),
		new Noun(Genus.FEMALE, "Blume"),
		new Noun(Genus.FEMALE, "Erde"),
		new Noun(Genus.FEMALE, "Sonne"),
		new Noun(Genus.FEMALE, "Welle"),
		new Noun(Genus.NEUTRUM, "Licht"),
		new Noun(Genus.NEUTRUM, "Gestein"),
		new Noun(Genus.NEUTRUM, "Haus"),
	};

	private static final String[] INTROS = new String[] {
		"Doch",
		"Denn",
		"Schon",
		"Und",
		"Oder",
		"Auch",
	};

	private static final String[] PREDICATES = new String[] {
		"ist", 
		"bleibt",
		"blickt",
		"wäre",
		"duftet",
		"bliebe",
		"sei",
		"schweigt",
		"rauscht",
		"summt",
		"leuchtet",
		"warnt",
		"atmet",
		"erzählt",
		"wartet",
		"ruht",
		"zieht",
		"steigt",
		"jagt",
		"strahlt",
		"plätschert",
	};

	private static final String[] ADVERBS = new String[] {
		"grün", 
		"grau", 
		"schwarz", 
		"hell", 
		"klar", 
		"neblig", 
		"satt", 
		"reich", 
		"arm", 
		"schön", 
		"ruhig", 
		"edel", 
		"majestätisch", 
		"geknickt", 
		"unruhig", 
		"leise", 
		"matt", 
		"löchrig", 
		"pleite", 
		"schweigsam", 
	};

	private static final String[] ADDITIONS1 = new String[] {
		"deutlich",
		"verschleiert",
		"vielleicht",
		"nun",
		"immer",
		"manchmal",
		"nie",
		"nicht",
	};

	private static final String[] ADDITIONS2 = new String[] {
		"seit Anbeginn",
		"auf Ewigkeit",
		"im Morgengrauen",
		"in tiefem Schlaf",
	};

	private static String createIntro(final int prob) {
		return createOneOf(INTROS, prob);
	}

	private static String createSubject() {
		final StringBuffer buf = new StringBuffer();
		final Noun subject = NOUNS[MathUtil.random(NOUNS.length)];
		buf.append(ARTICLES[MathUtil.random(ARTICLES.length)][subject.getGenus().ordinal()]);
		buf.append(" ");
		buf.append(subject.getSubject());
		buf.append(" ");
		return buf.toString();
	}

	private static String createPredicate() {
		return createOneOf(PREDICATES);
	}

	private static String createAdverb() {
		return createOneOf(ADVERBS);
	}

	private static String createAddition1(final int prob) {
		return createOneOf(ADDITIONS1, prob);
	}

	private static String createAddition2(final int prob) {
		return createOneOf(ADDITIONS2, prob);
	}

	private static String createLine() {
		final StringBuffer buf = new StringBuffer();
		buf.append(createIntro(3));
		final String[] interpunction;
		switch(MathUtil.random(4)) {
			case 0: {
				buf.append(createSubject());
				buf.append(createPredicate());
				buf.append(createAddition1(3));
				buf.append(createAdverb());
				buf.append(createAddition2(5));
				interpunction = new String[]{".", "&nbsp;&hellip;", "?", "!"};
			}
			break;
			case 1: {
				buf.append(createAddition1(4));
				buf.append(createPredicate());
				buf.append(createAddition2(8));
				buf.append(createSubject());
				buf.append(createAddition1(2));
				buf.append(createAdverb());
				buf.append(createAddition2(4));
				interpunction = new String[]{"?", "!"};
			}
			break;
			case 2: {
				buf.append(createAddition1(4));
				buf.append(createPredicate());
				String add2 = createAddition2(6);
				buf.append(add2);
				if (add2.isEmpty()) {
					buf.append(createAddition1(4));
				}
				buf.append(createSubject());
				buf.append(createAddition2(4));
				interpunction = new String[]{"?", "&nbsp;&hellip;"};
			}
			break;
			case 3: {
				buf.append(createAdverb());
				buf.append(createPredicate());
				buf.append(createAddition1(3));
				buf.append(createAddition2(10));
				buf.append(createSubject());
				buf.append(createAddition2(5));
				interpunction = new String[]{".", "&nbsp;&hellip;", "!"};
			}
			break;
		default:
			interpunction = new String[] { "." };
		}
		return Poem.upperFirst(buf.toString().trim()) + createOneOf(interpunction);
	}

	Poem1(final Algorithm algo) {
		super(algo);
	}

	@Override
	public void createPoem(final TextDocumentBuilder doc) {
		for (int verse = 0; verse < 3; verse++) {
			beginVerse(doc);
			createVerse(doc);
			endVerse(doc);
		}
	}

	private void createVerse(final TextDocumentBuilder doc) {
		for (int lineNo = 0; lineNo < 4; lineNo++) {
			addLine(doc, lineNo, createLine());
		}
	}
}
