package de.ped.gedicht.logic;

import de.ped.gedicht.logic.Poem.Algorithm;
import de.ped.tools.ApplicationEnvironment;
import de.ped.tools.ApplicationParameter;
import de.ped.tools.Messages;
import de.ped.tools.ResourceFinder;

/**
 * A facade class to encapsulate the creation of poems.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PoemFacade implements ApplicationEnvironment {

	/**
	 * @return the singleton instance of this class
	 */
	public static PoemFacade instance() {
		if (null == instance) {
			instance = new PoemFacade();
		}
		return instance;
	}

	private static PoemFacade instance;

	@Override
	public ApplicationParameter findParameter(String key) {
		return null;
	}

	@Override
	public ResourceFinder resources() {
		return null;
	}

	@Override
	public Messages messages() {
		if (null == messages) {
			messages = new Messages("de.ped.gedicht.logic.messages",
					"Aleatorische Gedichte", 
					"v2.0", 
					"Copyright 1987-2008 Peter Diefenbach (peter@pdiefenbach.de)",
					"",
					"A (German) poem creator");
		}
		return messages;
	}

	@Override
	public boolean isExitable() {
		return false;
	}

	@Override
	public boolean isSilent() {
		return false;
	}

	/**
	 * @param algorithm
	 *            the algorithm to create the object for
	 * @return an object able to create poems.
	 */
	public Poem create(final Algorithm algorithm) {
		final Poem ret;
		switch(algorithm) {
		case WITHOUT_RHYME:
			ret = new Poem1(algorithm);
			break;
		case WITH_RHYME:
			ret = new Poem2(algorithm);
			break;
		default:
			ret = null;
		}
		return ret;
	}

	private Messages messages;
}
