package de.ped.ituneslibconverter;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

/**
 * Not really a test, but a development tool for creating playlists. So far only used for Edith's birthday.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlaylistTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		// Nothing to do
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// Nothing to do
	}

	@Before
	public void setUp() throws Exception {
		// Nothing to do
	}

	@After
	public void tearDown() throws Exception {
		// Nothing to do
	}
	
	public static final String legendLine = "Titelname	Interpret	Komponist	Album	Werk	Genre	Gre	Dauer	CD-Nummer	Zhler	Titelnummer	Zhler	Jahr	Gendert	Hinzugefgt	Datenrate	Abtastrate	Lautstrkeanpassung	Art	Equalizer	Kommentar	Wiedergaben	Zuletzt gespielt	bersprungen	Zuletzt bersprungen	Meine Wertung	Ort";

	public static final String[] legend = new String[] {
			"Titelname",
			"Interpret",
			"Komponist",
			"Album",
			"Werk",
			"Genre",
			"Gre",
			"Dauer",
			"CD-Nummer",
			"Zhler",
			"Titelnummer",
			"Zhler",
			"Jahr",
			"Gendert",
			"Hinzugefgt",
			"Datenrate",
			"Abtastrate",
			"Lautstrkeanpassung",
			"Art",
			"Equalizer",
			"Kommentar",
			"Wiedergaben",
			"Zuletzt gespielt",
			"bersprungen",
			"Zuletzt bersprungen",
			"Meine Wertung",
			"Ort"
	};

	@Test
	public void testParseLine() {
		final Playlist playlist = new Playlist();
		assertNull(playlist.parseLine(null));
		final String[] ret = playlist.parseLine(legendLine);
		assertEquals(legend.length, ret.length);
		for (int i = 0; i < legend.length; i++) {
			assertEquals(legend[i], ret[i]);
		}
	}

	@Test
	public void testComposeLine() {
		final Playlist playlist = new Playlist();
		assertNull(playlist.composeLine(null));
		final String ret = playlist.composeLine(legend);
		assertEquals(legendLine, ret);
	}

	@Test
	public void testLoadEmptyList() {
		final Playlist playlist = new Playlist();
		try {
			playlist.load(new StringReader(legendLine));
		} catch (final IOException e) {
			fail();
		}
		assertEquals(legend.length, playlist.getLegend().length);
		assertEquals("Interpret", playlist.getLegend()[1]);
		for (int i = 0; i < legend.length; i++) {
			assertEquals(legend[i], playlist.getLegend()[i]);
			if (i != 11) { // "Zhler" is double in legend :-(
				assertEquals(i, playlist.getLegendIndex(legend[i]));
			}
		}
		final int locationIndex = playlist.getLegendIndex("Ort");
		assertEquals(26, locationIndex);
		assertEquals(0, playlist.size());
	}

	@Test
	public void testLoadAndWriteListWithSingleLine() {
		final Playlist playlist = new Playlist();
		final String content = 
			legendLine + "\n" + 
			"I Will Talk And Hollywood Will Listen	Williams, Robbie	Guy Chambers/Robbie Williams	Swing When You're Winning	Swing When You're Winning	Retro-Swing	3519218	197	1	1	1	15	2001	09.12.2010 04:52	17.10.2010 09:47	128	44100		MPEG-Audiodatei		 00000A60 00000A0E 00004237 000042EC 000138AE 000138AE 00007DEF 00007395 00011170 00011187	3	11.04.2010 10:56	2	23.01.2011 03:39	60	C:\\Users\\Public\\MP3\\Pop\\Williams, Robbie\\2001-Swing When You're Winning\\01-I Will Talk And Hollywood Will Listen.mp3" + "\n";
		try {
			playlist.load(new StringReader(content));
		} catch (final IOException e) {
			fail();
		}
		assertEquals("Titelname", playlist.getLegend()[0]);
		assertEquals("Interpret", playlist.getLegend()[1]);
		assertEquals(27, playlist.getLegend().length);
		assertEquals( 0, playlist.getLegendIndex("Titelname"));
		assertEquals( 1, playlist.getLegendIndex("Interpret"));
		assertEquals(26, playlist.getLegendIndex("Ort"));
		assertEquals( 1, playlist.size());
		final String title = "I Will Talk And Hollywood Will Listen";
		final String interpret = "Williams, Robbie";
		assertEquals(title, playlist.getEntry(0, playlist.getLegendIndex("Titelname")));
		assertEquals(interpret, playlist.getEntry(0, playlist.getLegendIndex("Interpret")));
		StringWriter again = new StringWriter();
		try {
			playlist.save(again);
		} catch (final IOException e) {
			fail();
		}
		final String result = again.toString();
		assertEquals(content, result);
	}

	@Test
	public void testCreateToNameTest() {
		assertEquals("001-Dummy", Playlist.createToName("Dummy", "", 001));
		assertEquals("D:\\001-Dummy", Playlist.createToName("C:\\Test\\Dummy", "D:\\", 001));
	}
	
	public void execMoveAndCollect(final String sourceFile, final String destFile, final String replaceWhat, final String replaceBy, final String collectionFolder) {
		final Playlist playlist = new Playlist();
		try {
			playlist.loadFile(sourceFile);
		} catch (final IOException e) {
			fail();
		}
		assertEquals("Titelname", playlist.getLegend()[0]);
		assertEquals("Interpret", playlist.getLegend()[1]);
		assertEquals(27, playlist.getLegend().length);
		assertEquals( 0, playlist.getLegendIndex("Titelname"));
		assertEquals( 1, playlist.getLegendIndex("Interpret"));
		assertEquals(26, playlist.getLegendIndex("Ort"));
		final int locationIndex = playlist.getLegendIndex("Ort");
		playlist.replaceForAllLines(locationIndex, replaceWhat, replaceBy);
		try {
			playlist.saveFile(destFile);
		} catch (final IOException e) {
			fail();
		}
		try {
			playlist.collectAll(collectionFolder);
		} catch (final IOException e) {
			fail();
		}
	}

	public void execMoveAndCollect(final String[] basics, final String replaceWhat, final String replaceBy) {
		for (final String basic : basics) {
			execMoveAndCollect(basic + ".txt", basic + "2.txt", replaceWhat, replaceBy, basic + "\\");
		}
	}

	@Ignore
	public void testLoadRealFileAndWriteWithNewFilename() {
		final String[] playlists = new String[]{
				"E:\\Public\\MP3\\Playlists\\Ediths Geburtstag Empfang", 
				"E:\\Public\\MP3\\Playlists\\Ediths Geburtstag Essen", 
				"E:\\Public\\MP3\\Playlists\\Ediths Geburtstag Kaffee", 
				"E:\\Public\\MP3\\Playlists\\Ediths Geburtstag Sonstiges", 
				"E:\\Public\\MP3\\Playlists\\Ediths Geburtstag Tanz", 
		}; 
		execMoveAndCollect(playlists, "C:\\Users\\Public\\MP3", "E:\\Public\\MP3");
	}
}
