package de.ped.kitten.dev;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;

import de.ped.kitten.gui.Kitten;
import de.ped.kitten.logic.HTSLogo;
import de.ped.kitten.logic.KittenLevelSet;
import de.ped.tools.ResourceFinder;
import de.ped.tools.TextFormatter;
import de.ped.tools.log.Logger;

/**
 * Creates a hexdump of all binary data used in Kitten, to use it in JavaScript version of the game.
 * 
 * @author Copyright (c) 2018 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class BinaryFilesConverter {

	public static void main(String[] params) throws IOException {
		final ResourceFinder resources = new ResourceFinder(Kitten.RESOURCES_PATH);
		final Logger logger = Logger.getRootLogger();
//		logger.info("HTS Logo = \n" + extractHTSLogo(resources));
		for (int set : new int[] { 
//				KittenLevelSet.HTS, 
				KittenLevelSet.ANDREAS_KOCH 
				}) {
			final String path = KittenLevelSet.PATHS[set];
			final ArrayList<byte[]> ret = create(resources, path);
//			ret = create(resources, "AndreasKoch");
			for (int i = 0; i < ret.size(); i++) {
				logger.info(path + "_" + KittenLevelSet.FILENAME_PREFIX + (i + 1) + " = \n" + TextFormatter.hexdumpCode(ret.get(i)));
			}
		}
	}

	@SuppressWarnings("unused")
	private static String extractHTSLogo(final ResourceFinder resources) throws IOException {
		final HTSLogo logo = new HTSLogo(resources);
		return TextFormatter.hexdumpCode(logo.data);
	}

	private static ArrayList<byte[]> create(final ResourceFinder resources, final String path) throws IOException {
		final ArrayList<byte[]> ret = new ArrayList<byte[]>();
		int index = 1;
		boolean isDone = false;
		while(!isDone) {
			try {
				final String pathAndFilename = path + "/" + KittenLevelSet.FILENAME_PREFIX + index;
				final InputStream is = resources.getURLToResource(pathAndFilename).openStream();
				final byte[] buf = new byte[0x1000];
				final int length = is.read(buf);
				final byte[] arr = Arrays.copyOf(buf, length);
				ret.add(arr);
			} catch (final FileNotFoundException ex) {
				isDone = true;
			} catch (final NullPointerException ex) {
				isDone = true;
			}
			index++;
		}
		return ret;
	}
}
