package de.ped.kitten.dev;

import java.io.IOException;
import java.util.TreeMap;

import de.ped.kitten.gui.Kitten;
import de.ped.kitten.gui.KittenSounds;
import de.ped.kitten.logic.Game;
import de.ped.kitten.logic.LevelInfo;
import de.ped.tools.ResourceFinder;
import de.ped.tools.TextFormatter;

/**
 * Creates the sound files for JavaScript Kitten.
 * 
 * @author Copyright (c) 2018 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class KittenSoundCreator {
	public static void main(final String[] args) throws IOException {
		// Event sounds
		final TreeMap<KittenSounds.SoundKey, String> map = new TreeMap<KittenSounds.SoundKey, String>();
		map.put(new KittenSounds.SoundKey(Game.Event.COLLISION,  0), "KittenCollision.");
		map.put(new KittenSounds.SoundKey(Game.Event.FLOWER,     0), "KittenEatsFlower.");
		map.put(new KittenSounds.SoundKey(Game.Event.SPRING,     0), "KittenUsesSpring.");
		map.put(new KittenSounds.SoundKey(Game.Event.LEVEL_DONE, 0), "KittenLevelDone.");
		map.put(new KittenSounds.SoundKey(Game.Event.GAME_OVER,  0), "KittenGameOver.");
		map.put(new KittenSounds.SoundKey(Game.Event.BACKGROUND, 0), "KittenBackground.");
		for (int i = 0; i < LevelInfo.SIZE.getHeight(); i++) {
			map.put(new KittenSounds.SoundKey(Game.Event.JUMP, i), "KittenFlies" + TextFormatter.format(i, 2, TextFormatter.RIGHT, '0') + ".");
		}
		final KittenSounds sounds = new KittenSounds(new ResourceFinder(Kitten.RESOURCES_PATH));
		for (KittenSounds.SoundKey soundKey : map.keySet()) {
			sounds.writeSoundFile(soundKey, map.get(soundKey));
		}
	}
}
