package de.ped.kitten.gui;

import java.awt.Frame;
import java.io.IOException;

import de.ped.tools.ApplicationParameter;
import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationOrApplet;

/**
 * Starts the whole game - if as applet or as application.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Kitten extends ApplicationOrApplet {
	public static final String RESOURCES_PATH = "de/ped/kitten/resources";

	private static final long serialVersionUID = 1L;

	/**
	 * The main entry point for the application.
	 * 
	 * @param args
	 *            array of parameters passed to the application via the command
	 *            line
	 */
	public static void main(final String[] args) {
		applicationPreCreate();
		applicationPostCreate(new Kitten(), args);
	}

	public Kitten() {
		addParameter(getParamChristmas());
		addParameter(new ApplicationParameter(PARAM_KEY_GAME, null, "If set, defines a single level game to offer to the player (in format <index of level set>.<index of level>)"));
	}

	@Override
	protected String getPathToResources() {
		return RESOURCES_PATH;
	}

	@Override
	protected Messages createMessages() {
		return new Messages(
				"de.ped.kitten.logic.messages", 
				"Kitten", 
				"v2.0", 
				"Copyright 1986-2024 Peter Diefenbach (peter@pdiefenbach.de)\nHappy Trash Software levels by Alexander Diefenbach\nAndreas Koch levels by Andreas Koch", 
				"A jump (but not run) game",
				"A jump (but not run) game",
				Messages.DEFAULT_LOCALES, 
				findParameter(PARAM_KEY_LANGUAGE).getValue(),
				Messages.DEFAULT_LOCALE_INDEX);
	}

	@Override
	protected Frame createFrame() {
		Frame ret = null;
		try {
			ret = new KittenMainWindow(this);
		} catch (final IOException e) {
			// Nothing to do
		}
		return ret;
	}
}
