package de.ped.kitten.gui;

import java.awt.Dimension;
import java.util.Iterator;
import java.util.LinkedList;

import de.ped.tools.log.Logger;

/**
 * An 8-bit font character.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TrueTypeMonospaceChar implements Cloneable {
	private LinkedList<TrueTypeMonospaceSingleSizeChar> ssc = new LinkedList<TrueTypeMonospaceSingleSizeChar>();
	private Logger logger = Logger.getLogger(getClass());

	public void add(final TrueTypeMonospaceSingleSizeChar chardef) {
		final Iterator<TrueTypeMonospaceSingleSizeChar> it = ssc.iterator();
		while (it.hasNext()) {
			final TrueTypeMonospaceSingleSizeChar c = it.next();
			if (chardef.equals(c)) {
				it.remove();
				break;
			}
		}
		ssc.add(chardef);
	}

	public String[] get(final Dimension size) {
		final TrueTypeMonospaceSingleSizeChar c = findBestFit(size);
		String[] ret = c.getDef();
		if (!c.getSize().equals(size)) {
			logger.trace("Resizing from " + c.getSize() + " to " + size);
			ret = TrueTypeMonospaceSingleSizeChar.stretch(ret, size.width / c.getSize().width, size.height / c.getSize().height);
		}
		return ret;
	}
	
	public String[] get(final int width, final int height) {
		return get(new Dimension(width, height));
	}

	/**
	 * Rates a single dimension of a given size to an expected size.
	 * 
	 * @param expected
	 *            The size in which to draw
	 * @param current
	 *            The size of the template
	 * @return A small number indicates a good, a high number a bad rating.
	 */
	private int rateDimension(final int expected, final int current) {
		int ret = 1000000; // Default is very bad rating
		if (expected == current) {
			ret = 1;
		} else if (expected > current) {
			if (0 == (expected % current)) {
				ret = expected / current;
			}
		}
		return ret;
	}

	private TrueTypeMonospaceSingleSizeChar findBestFit(final Dimension size) {
		TrueTypeMonospaceSingleSizeChar best = null;
		int bestRating = Integer.MAX_VALUE;
		for (final TrueTypeMonospaceSingleSizeChar c : ssc) {
			final int rating = rateDimension(size.width, c.getSize().width) + rateDimension(size.height, c.getSize().height);
			if (rating < bestRating) {
				best = c;
				bestRating = rating;
			}
		}
		return best;
	}

	/** 
	 * @return a deep copy
	 */
	@Override
	public TrueTypeMonospaceChar clone() {
		final TrueTypeMonospaceChar target = new TrueTypeMonospaceChar();
		for (final TrueTypeMonospaceSingleSizeChar c : ssc) {
			target.ssc.add(c.clone());
		}
		return target;
	}
}
