package de.ped.kitten.logic;

/**
 * Calculates for durations in seconds, how many steps to wait.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Duration {

	/** The delay in milliseconds between two game steps */
	public static final long GAME_STEP_DELAY = 33L; // 17L;

	public static final Duration FOREVER = new Duration(Long.MAX_VALUE);
	public static final Duration DYING = new Duration(2.0);
	public static final Duration SUCCEEDING = new Duration(3.5);
	public static final Duration TILL_DIRECTION_LOSS = new Duration(5.0);
	public static final Duration FALLING_PART = new Duration(132L);
	public static final Duration JUMP_PART = new Duration(165L);
	public static final Duration FLOWER = new Duration(99L);
	public static final Duration MICROFLICRO_CHEW = new Duration(132L);
	public static final Duration BACKGROUND_NOTE = new Duration(0.11);
	public static final Duration ANIMATION = new Duration(0.25);

	private static final long fromSeconds(final double timeInSecs) {
		return fromMillis(Math.round(1000L * timeInSecs));
	}

	private static final double toSeconds(final long steps) {
		return toMillis(steps) / 1000.0;
	}

	private static final long fromMillis(final long timeInMillis) {
		return timeInMillis / GAME_STEP_DELAY;
	}

	private static final long toMillis(final long steps) {
		return steps * GAME_STEP_DELAY;
	}

	public final long steps;

	public Duration(final double timeInSecs) {
		steps = fromSeconds(timeInSecs);
	}

	public Duration(final long timeInMillis) {
		steps = fromMillis(timeInMillis);
	}

	public long getDurationInMillis() {
		return toMillis(steps);
	}

	public double getDurationInSeconds() {
		return toSeconds(steps);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (steps ^ (steps >>> 32));
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Duration other = (Duration) obj;
		if (steps != other.steps)
			return false;
		return true;
	}
}