package de.ped.kitten.logic;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import de.ped.tools.ResourceFinder;
import de.ped.tools.log.Logger;

/**
 * The Happy Trash Software logo data.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class HTSLogo {

	private Logger logger = Logger.getLogger(getClass());

	public static final int DOTS_PER_BYTE = 8;
	public static final int XBYTES = 16;
	public static final int XSIZE = XBYTES * DOTS_PER_BYTE;
	public static final int YSIZE = 65;

	public final byte[] data = new byte[YSIZE * XBYTES];
	public final int bytesRead;

	public HTSLogo(final ResourceFinder resources) throws IOException {
		final URL url = resources.getURLToResource("TITEL.DAT");
		InputStream is = null;
		int br = 0;
		try {
			is = url.openStream();
			br = is.read(data, 40, 985);
		} catch (final Throwable ex) {
			logger.error("Unable to load HTS logo!", ex);
		} finally {
			if (null != is) {
				is.close();
			}
			bytesRead = br;
		}
	}

	private static final int[] PATTERNS = new int[]{ 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 }; 

	public boolean get(final int x, final int y) {
		final int idx = y * (XBYTES) + (x / DOTS_PER_BYTE);
		final int pattern = PATTERNS[x % DOTS_PER_BYTE];
		return (0 != (data[idx] & pattern));
	}
}
