package de.ped.kitten.logic;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

import de.ped.tools.ResourceFinder;
import de.ped.tools.log.Logger;

/**
 * An array of levels that should be played in that order.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class KittenLevelSet {

	private static Logger logger = Logger.getRootLogger();

	static {
		logger.setLogLevel(de.ped.tools.log.Level.INFO);
	}

	public static final int MINIMUM = 0;
	public static final int HTS = 1;
	public static final int ANDREAS_KOCH = 2;
	public static final String[] PATHS = new String[] { null, "HappyTrashSoftware", "AndreasKoch" };
	public static final String FILENAME_PREFIX = "KITTENSC.";

	public static Vector<LevelInfo> create(final ResourceFinder resources, final int levelSetIndex) throws IOException {
		Vector<LevelInfo> ret;
		switch(levelSetIndex) {
		case MINIMUM:
			ret = createMinimum();
			break;
		case ANDREAS_KOCH:
		case HTS:
			ret = create(resources, PATHS[levelSetIndex]);
			break;
		default:
			ret = create(resources, PATHS[HTS]);
		}
		return ret;
	}

	public static boolean isAvailable(final ResourceFinder resources, final int levelSetIndex) {
		boolean ret = true;
		try{
			create(resources, levelSetIndex);
		} catch (final Throwable ex) {
			ret = false;
			logger.error("Unable to create level set " + levelSetIndex, ex);
		}
		return ret;
	}

	private static Vector<LevelInfo> createMinimum() {
		final LevelInfo level1 = new LevelInfo(2, 4, 30, "......M........M..", "GOOD START", new String[]{
				"..................",
				"..................",
				"..................",
				"..................",
				".....7..........5.",
				"....1111111111.222",
				"...1............22",
				"........1........2",
				"..1....1...3......",
				".1..11...........5",
				"...1..1.1.....2222",
				".1.......1.3...222",
				"................22",
				"..5.4.4..........2",
				".111111111131.....",
				"...............5..",
				"..................",
				"...6....3......4..",
		});
		final LevelInfo level2 = new LevelInfo(2, 6, 20, "...M...M..........", "FLOWER POWER", new String[]{
				"..7...............",
				".11...............",
				"...1...4.4.4......",
				"....1111111111111.",
				"5....5.....5...5..",
				"..5....5.5..55....",
				"..............3.3.",
				"..................",
				"...6.......5....5.",
				".5115.5115.111..1.",
				".1..1.1..1..1...1.",
				".1....1..1..1.5.1.",
				".1....15.1..1...1.",
				"51.5..1111.51...15",
				".1....1.51..1..51.",
				".1..1.1..1..1.....",
				"..11..1..1..1...1.",
				".555555.555555.5.5",
		});
		final Vector<LevelInfo> ret = new Vector<LevelInfo>();
		ret.add(level1);
		ret.add(level2);
		return ret;
	}

	private static Vector<LevelInfo> create(final ResourceFinder resources, final String path) throws IOException {
		final Vector<LevelInfo> ret = new Vector<LevelInfo>();
		int index = 1;
		boolean isDone = false;
		while(!isDone) {
			try {
				final String pathAndFilename = path + "/" + FILENAME_PREFIX + index;
				logger .debug("Loading " + pathAndFilename);
				final InputStream is = resources.getURLToResource(pathAndFilename).openStream();
				ret.add(new LevelInfo(is));
			} catch (final FileNotFoundException ex) {
				isDone = true;
			} catch (final NullPointerException ex) {
				isDone = true;
			}
			index++;
		}
		return ret;
	}
}
