package de.ped.kitten.logic;

/**
 * The state the tomcat can be in.
 * Realized as data of a state machine that handles all the automated transitions on its own.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public enum TomcatState {
	/** The final state for a level. Indicates that either the game is over, the level should be restarted, or the next level loaded. */
	FINISHED           (TomcatOverallState.FINISHED, Characters.CAT_IDLE,        0,  0,  0, null,                Duration.FOREVER),
	DYING              (TomcatOverallState.DYING,    Characters.CAT_MEAOW,       0,  0,  0, FINISHED,            Duration.DYING),
	SUCCEEDED          (TomcatOverallState.SUCCEEDED,Characters.CAT_LOOKS_RIGHT, 0,  0,  0, FINISHED,            Duration.SUCCEEDING),
	IDLE_UNDIRECTED    (TomcatOverallState.IDLE,     Characters.CAT_IDLE,        0,  0,  0, null,                Duration.FOREVER), 
	IDLE_LEFT          (TomcatOverallState.IDLE,     Characters.CAT_LOOKS_LEFT, -1,  0,  0, IDLE_UNDIRECTED,     Duration.TILL_DIRECTION_LOSS), 
	IDLE_RIGHT         (TomcatOverallState.IDLE,     Characters.CAT_LOOKS_RIGHT, 1,  0,  0, IDLE_UNDIRECTED,     Duration.TILL_DIRECTION_LOSS), 
	FALLING_LEFT       (TomcatOverallState.FALLING,  Characters.CAT_DOWN_LEFT , -1,  0,  1, null,                Duration.FALLING_PART),
	FALLING_RIGHT      (TomcatOverallState.FALLING,  Characters.CAT_DOWN_RIGHT,  1,  0,  1, null,                Duration.FALLING_PART),
	SPRING_LEFT4       (TomcatOverallState.SPRING,   Characters.CAT_UP_LEFT,    -1,  0, -1, FALLING_LEFT,        Duration.JUMP_PART), 
	SPRING_LEFT3       (TomcatOverallState.SPRING,   Characters.CAT_UP_LEFT,    -1,  0, -1, SPRING_LEFT4,        Duration.JUMP_PART), 
	SPRING_LEFT2       (TomcatOverallState.SPRING,   Characters.CAT_UP_LEFT,    -1,  0, -1, SPRING_LEFT3,        Duration.JUMP_PART), 
	SPRING_LEFT1       (TomcatOverallState.SPRING,   Characters.CAT_UP_LEFT,    -1,  0, -1, SPRING_LEFT2,        Duration.JUMP_PART), 
	SPRING_LEFT0       (TomcatOverallState.SPRING,   Characters.CAT_UP_LEFT,    -1,  0, -1, SPRING_LEFT1,        Duration.JUMP_PART), 
	SPRING_RIGHT4      (TomcatOverallState.SPRING,   Characters.CAT_UP_RIGHT,    1,  0, -1, FALLING_RIGHT,       Duration.JUMP_PART), 
	SPRING_RIGHT3      (TomcatOverallState.SPRING,   Characters.CAT_UP_RIGHT,    1,  0, -1, SPRING_RIGHT4,       Duration.JUMP_PART), 
	SPRING_RIGHT2      (TomcatOverallState.SPRING,   Characters.CAT_UP_RIGHT,    1,  0, -1, SPRING_RIGHT3,       Duration.JUMP_PART), 
	SPRING_RIGHT1      (TomcatOverallState.SPRING,   Characters.CAT_UP_RIGHT,    1,  0, -1, SPRING_RIGHT2,       Duration.JUMP_PART), 
	SPRING_RIGHT0      (TomcatOverallState.SPRING,   Characters.CAT_UP_RIGHT,    1,  0, -1, SPRING_RIGHT1,       Duration.JUMP_PART), 
	JUMPING_LEFT_HIGH4 (TomcatOverallState.JUMPING,  Characters.CAT_DOWN_LEFT,  -1, -1,  1, FALLING_LEFT,        Duration.JUMP_PART), 
	JUMPING_LEFT_HIGH3 (TomcatOverallState.JUMPING,  Characters.CAT_DOWN_LEFT,  -1, -1,  1, JUMPING_LEFT_HIGH4,  Duration.JUMP_PART), 
	JUMPING_LEFT_HIGH2 (TomcatOverallState.JUMPING,  Characters.CAT_UP_LEFT,    -1, -1,  0, JUMPING_LEFT_HIGH3,  Duration.JUMP_PART), 
	JUMPING_LEFT_HIGH1 (TomcatOverallState.JUMPING,  Characters.CAT_UP_LEFT,    -1, -1, -1, JUMPING_LEFT_HIGH2,  Duration.JUMP_PART), 
	JUMPING_LEFT_HIGH0 (TomcatOverallState.JUMPING,  Characters.CAT_UP_LEFT,    -1, -1, -1, JUMPING_LEFT_HIGH1,  Duration.JUMP_PART), 
	JUMPING_RIGHT_HIGH4(TomcatOverallState.JUMPING,  Characters.CAT_DOWN_RIGHT,  1,  1,  1, FALLING_RIGHT,       Duration.JUMP_PART), 
	JUMPING_RIGHT_HIGH3(TomcatOverallState.JUMPING,  Characters.CAT_DOWN_RIGHT,  1,  1,  1, JUMPING_RIGHT_HIGH4, Duration.JUMP_PART), 
	JUMPING_RIGHT_HIGH2(TomcatOverallState.JUMPING,  Characters.CAT_UP_RIGHT,    1,  1,  0, JUMPING_RIGHT_HIGH3, Duration.JUMP_PART), 
	JUMPING_RIGHT_HIGH1(TomcatOverallState.JUMPING,  Characters.CAT_UP_RIGHT,    1,  1, -1, JUMPING_RIGHT_HIGH2, Duration.JUMP_PART), 
	JUMPING_RIGHT_HIGH0(TomcatOverallState.JUMPING,  Characters.CAT_UP_RIGHT,    1,  1, -1, JUMPING_RIGHT_HIGH1, Duration.JUMP_PART), 
	JUMPING_LEFT_LOW2  (TomcatOverallState.JUMPING,  Characters.CAT_DOWN_LEFT,  -1, -1,  1, FALLING_LEFT,        Duration.JUMP_PART), 
	JUMPING_LEFT_LOW1  (TomcatOverallState.JUMPING,  Characters.CAT_UP_LEFT,    -1, -1,  0, JUMPING_LEFT_LOW2,   Duration.JUMP_PART), 
	JUMPING_LEFT_LOW0  (TomcatOverallState.JUMPING,  Characters.CAT_UP_LEFT,    -1, -1, -1, JUMPING_LEFT_LOW1,   Duration.JUMP_PART), 
	JUMPING_RIGHT_LOW2 (TomcatOverallState.JUMPING,  Characters.CAT_DOWN_RIGHT,  1,  1,  1, FALLING_RIGHT,       Duration.JUMP_PART),
	JUMPING_RIGHT_LOW1 (TomcatOverallState.JUMPING,  Characters.CAT_UP_RIGHT,    1,  1,  0, JUMPING_RIGHT_LOW2,  Duration.JUMP_PART),
	JUMPING_RIGHT_LOW0 (TomcatOverallState.JUMPING,  Characters.CAT_UP_RIGHT,    1,  1, -1, JUMPING_RIGHT_LOW1,  Duration.JUMP_PART);

	/** Describes to which overall state this state is a substate of. */
	public final TomcatOverallState overallState;
	/** Describes which character should be drawn for a tomcat in that state. */
	public final char character;
	/** Describes in which direction the tomcat should look. -1 means left, 0 means front, 1 means right. */
	public final int faceDirectionX;
	/** Describes in which horizontal direction the tomcat should move. -1 means left, 0 means no horizontal movement, 1 means right. */
	public final int directionX;
	/** Describes in which vertical direction the tomcat should move. -1 means up, 0 means no vertical movement, 1 means down. */
	public final int directionY;
	/** Describes what is the next tomcat state when nothing else happens. */
	public final TomcatState naturalNextState;
	/** Describes after how many steps the {@link #naturalNextState} will be reached if nothing else happens. */
	public final Duration timeTillNaturalNextState;

	TomcatState(
			final TomcatOverallState overallState, 
			final char character, 
			final int faceDirectionX, 
			final int directionX, 
			final int directionY, 
			final TomcatState naturalNextState, 
			final Duration timeTillNaturalNextState) {
		this.overallState = overallState;
		this.character = character;
		this.faceDirectionX = faceDirectionX;
		this.directionX = directionX;
		this.directionY = directionY;
		this.naturalNextState = naturalNextState;
		this.timeTillNaturalNextState = timeTillNaturalNextState;
	}

	/** 
	 * @return which idle state to choose when tomcat should switch to an idle state
	 */
	public TomcatState getIdleState() {
		TomcatState ret;
		if (0 < faceDirectionX) {
			ret = IDLE_RIGHT;
		} else if (0 > faceDirectionX) {
			ret = IDLE_LEFT;
		} else {
			ret = IDLE_UNDIRECTED;
		}
		return ret;
	}

	/** 
	 * @return which falling state to choose when tomcat should switch to a falling state
	 */
	public TomcatState getFallingState() {
		TomcatState ret;
		if (0 < faceDirectionX) {
			ret = FALLING_RIGHT;
		} else if (0 > faceDirectionX) {
			ret = FALLING_LEFT;
		} else {
			ret = FALLING_RIGHT;
		}
		return ret;
	}

	/** 
	 * @return which spring state to choose when tomcat should switch to a spring state
	 */
	public TomcatState getSpringState() {
		TomcatState ret;
		if (0 < faceDirectionX) {
			ret = SPRING_RIGHT0;
		} else if (0 > faceDirectionX) {
			ret = SPRING_LEFT0;
		} else {
			ret = SPRING_RIGHT0;
		}
		return ret;
	}

	public long getSmoothSteps() {
		long ret = timeTillNaturalNextState.steps;
		if ((this == SUCCEEDED) || (this == DYING)) {
			ret = Duration.JUMP_PART.steps;
		}
		return ret;
	}
}