package de.ped.kitten.test;


import static org.junit.Assert.*;

import java.util.LinkedList;

import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.kitten.gui.Kitten;
import de.ped.kitten.gui.KittenSounds;
import de.ped.kitten.gui.KittenSounds.SoundKey;
import de.ped.kitten.logic.Game;
import de.ped.kitten.logic.LevelInfo;
import de.ped.tools.ResourceFinder;
import de.ped.tools.sound.SoundClip;
import de.ped.tools.test.GenericTest;

/**
 * Tests {@link KittenSounds}.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class KittenSoundTest {

	private static boolean IS_NOISY = false;

	@BeforeClass
	public static void beforeClass() {
		final ResourceFinder resources = new ResourceFinder(Kitten.RESOURCES_PATH);
		sounds = new KittenSounds(resources);
	}

	private static KittenSounds sounds;

	@Test
	public void testObjectBasics() {
		LinkedList<SoundKey> keys = new LinkedList<SoundKey>();
		for (int i = 0; i < 100; i++) {
			SoundKey key = sounds.beatToSoundKey(i);
			if (null != key) {
				keys.add(key);
				break;
			}
		}
		for (Game.Event event : new Game.Event[] {
				Game.Event.MEOW,
				Game.Event.GAME_OVER,
				Game.Event.LEVEL_DONE,
				// Game.Event.DIED, // Same as MEOW
		} ) {
			keys.add(new KittenSounds.SoundKey(event, 0));
		}
		for (int i = 0; i < LevelInfo.SIZE.getHeight(); i++) {
			keys.add(new KittenSounds.SoundKey(Game.Event.JUMP, i));
		}
		for (Game.Event event : new Game.Event[] {
				Game.Event.SPRING,
				Game.Event.COLLISION,
				Game.Event.FLOWER,
		} ) {
			keys.add(new KittenSounds.SoundKey(event, 0));
		}
		GenericTest.checkObjectBasics(true, keys.toArray(new Object[] { } ));
	}

	@Test
	public void testBackground() throws InterruptedException {
		for (int i = 0; i < 100; i++) {
			final SoundClip clip = sounds.findSound(sounds.beatToSoundKey(i));
			playIfNoisy(clip, 100L);
		}
	}

	@Test
	public void testFallSounds() throws InterruptedException {
		for (int i = 0; i < LevelInfo.SIZE.getHeight(); i++) {
			final KittenSounds.SoundKey key1 = new KittenSounds.SoundKey(Game.Event.JUMP, i);
			final KittenSounds.SoundKey key2 = new KittenSounds.SoundKey(Game.Event.FALL, i);
			assertEquals(key1, key2);
			final SoundClip clip = sounds.findSound(key1);
			assertNotNull(clip);
			playIfNoisy(clip, 300L);
		}
	}

	@Test
	public void testEvents() {
		final Game.Event[] events = new Game.Event[] {
				Game.Event.MEOW,
				Game.Event.COLLISION,
				Game.Event.FLOWER,
				Game.Event.SPRING,
				Game.Event.LEVEL_DONE,
				Game.Event.DIED,
				Game.Event.GAME_OVER,
		};
		for (Game.Event event : events) {
			final KittenSounds.SoundKey key = new KittenSounds.SoundKey(event, 0);
			final SoundClip clip = sounds.findSound(key);
			assertNotNull(clip);
			if (IS_NOISY) {
				clip.play();
				// TODO: How to play synchronously?
			}
		}
	}

	private void playIfNoisy(final SoundClip clip, final long duration) throws InterruptedException {
		if (IS_NOISY) {
			if (null != clip) {
				clip.play();
			}
			Thread.sleep(duration);
		}
	}
}
