package de.ped.kitten.test;


import static org.junit.Assert.*;

import java.io.IOException;
import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.kitten.gui.Kitten;
import de.ped.kitten.logic.KittenLevelSet;
import de.ped.kitten.logic.Level.Symbol;
import de.ped.kitten.logic.LevelInfo;
import de.ped.tools.ResourceFinder;

/**
 * Tests {@link LevelInfo}.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LevelInfoTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		// Nothing to do
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// Nothing to do
	}

	@Before
	public void setUp() throws Exception {
		// Nothing to do
	}

	@After
	public void tearDown() throws Exception {
		// Nothing to do
	}

	@Test
	public void testLevelInfoInitByByteArray() throws IOException {
		final ResourceFinder resources = new ResourceFinder(Kitten.RESOURCES_PATH);
		final Vector<LevelInfo> set = KittenLevelSet.create(resources, KittenLevelSet.HTS); 
		assertEquals(20, set.size());
		final LevelInfo info = set.get(0);
		assertEquals(LevelInfo.SIZE.getWidth(), info.screen.length);
		for (int x = 0; x < info.screen.length; x++) {
			assertEquals("Column " + x + " ", LevelInfo.SIZE.getHeight(), info.screen[x].length);
			for (int y = 0; y < LevelInfo.SIZE.getHeight(); y++) {
				assertNotNull(info.screen[x][y]);
			}
		}
		final LevelInfo expected = KittenLevelSet.create(resources, KittenLevelSet.MINIMUM).get(0);
		assertEquals(expected.flowers, info.flowers);
		assertEquals(expected.time,    info.time);
		assertEquals(expected.microflicroSpeed, info.microflicroSpeed);
		assertArrayEquals(expected.screen, info.screen);
//		assertEquals(expected.destination, info.destination);
		assertEquals(expected.start, info.start);
		assertArrayEquals(info.microflicros, expected.microflicros);
		assertArrayEquals(expected.title, info.title);
	}

	public void assertArrayEquals(final Symbol[][] expected, final Symbol[][] result) {
		assertEquals(expected.length, result.length);
		for (int x = 0; x < expected.length; x++) {
			assertEquals(expected[x].length, result[x].length);
			for (int y = 0; y < expected[x].length; y++) {
				assertEquals(expected[x][y], result[x][y]);
			}
		}
	}

	public void assertArrayEquals(final boolean[] expected, final boolean[] result) {
		assertEquals(expected.length, result.length);
		for (int i = 0; i < expected.length; i++) {
			assertEquals(expected[i], result[i]);
		}
	}

	public byte norm(byte value) {
		if (0 == (value & 0x3f)) {
			value = 0;
		}
		return value;
	}

	public void assertArrayEquals(final byte[] expected, final byte[] result) {
		assertEquals(expected.length, result.length);
		for (int i = 0; i < expected.length; i++) {
			assertTrue("Index " + i + " expected: <0x" + Integer.toHexString(expected[i]) + "> but was: <0x" + Integer.toHexString(result[i]) + ">", (norm(expected[i]) == norm(result[i])));
		}
	}
}
