package de.ped.mp3sync;

import java.io.File;
import java.util.Collections;
import java.util.LinkedList;

import de.ped.tools.log.Logger;

/**
 * Compares two directories (without recursion) for synchronization, 
 * without changing anything within the file structure.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DirectoryComparator {
	private LinkedList<FileComparator> files = null;
	public final File destination;
	public final File source;
	public final Logger logger;
	public final FileActionFilter[] filters;

	public DirectoryComparator(final File destination, final File source, final FileActionFilter[] filters, Logger logger) {
		this.destination = destination;
		this.source = source;
		this.logger = logger;
		this.filters = filters;
	}

	private void init() {
		files = new LinkedList<FileComparator>();
		if ((null != source) && (null != destination)) {
			final File[] dests = destination.listFiles();
			final File[] srcs = source.listFiles();
			filter(srcs, FileActionFilter.COPY);
			// Find all matches starting from source
			if (null != srcs) {
				for (int i = 0; i < srcs.length; i++) {
					if (null != srcs[i]) { // Sourcefile not filtered away?
						boolean found = false; // Find matching destination file
						if (null != dests) {
							for (int j = 0; j < dests.length; j++) {
								if ((null != dests[j]) && (srcs[i].getName().equals(dests[j].getName()))) {
									final FileComparator fc = new FileComparator(destination, dests[j], srcs[i], logger);
									files.add(fc);
									dests[j] = null;
									found = true;
									break;
								}
							}
						}
						if (!found) {
							files.add(new FileComparator(destination, null, srcs[i], logger));
						}
					}
				}
			}
			// Add all destination files not used so far
			if (null != dests) {
				filter(dests, FileActionFilter.DELETE);
				for (int j = 0; j < dests.length; j++) {
					if (null != dests[j]) {
						files.add(new FileComparator(destination, dests[j], null, logger));
					}
				}
			}
		}
	}

	/**
	 * Sets all files not fitting to filter criteria to <code>null</code>.
	 * 
	 * @param files
	 *            the list of files that gets filtered
	 * @param filterMode
	 *            the filter mode to apply
	 */
	private void filter(final File[] files, final int filterMode) {
		if (null != files) {
			for (int i = 0; i < files.length; i++) {
				if (filters(files[i], filterMode)) {
					files[i] = null;
				}
			}
		}
	}

	private boolean filters(final File file, final int filterMode) {
		boolean ret = false;
		if (null != file) {
			for (final FileActionFilter filter : filters) {
				if (filter.filtersForMode(filterMode)  && file.getName().matches(filter.regex)) {
					ret = true;
					break;
				}
			}
		}
		return ret;
	}

	public LinkedList<FileComparator> result() {
		if (null == files) {
			init();
			Collections.sort(files);
		}
		return files;
	}
}