package de.ped.pacman.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import de.ped.pacman.logic.GameProperties;
import de.ped.pacman.logic.PacmanLevelSet;
import de.ped.tools.MathUtil;
import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.JIconComboBox;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.PropertyElement;

/**
 * The dialog to show all the game options.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NewGameSelectionDialog extends PedJDialog {
	private static final long serialVersionUID = 1L;

	private static final String[] LEVEL_KEYS = new String[]{
					"GameProperties.Level.0.Text", 
					"GameProperties.Level.1.Text", 
					"GameProperties.Level.2.Text", 
					"GameProperties.Level.3.Text", 
					"GameProperties.Level.4.Text"};

	public NewGameSelectionDialog(final ApplicationMainWindow parent) {
		super(parent, parent);
		final Messages messages = getMessages();
		setTitle(messages.getString("GamePropertiesWindow.Title"));
		choiceLevelSet = getMainWindow().createComboBox(new String[] { "", }, false);
		choiceLevelSet.setEnabled(false);
		choiceLevel = getMainWindow().createComboBox(LEVEL_KEYS, false);
		setPropertyElements(new PropertyElement[] {
				new PropertyElement("GameProperties.LevelSet", choiceLevelSet),
				new PropertyElement("GameProperties.Level", choiceLevel),
		});
		setResizable(false);
		setTitlePanel(messages.getString("GamePropertiesWindow.Heading"));
		final JPanel mainPanel = createMainPanel();
		final GridBagLayout gridbag = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainPanel.setLayout(gridbag);
		addAllPropertyElementsTo(mainPanel, c);

		add(mainPanel, BorderLayout.CENTER);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);
		addHelpButton("Help.Properties");
		finishLayout();
	}

	protected int levelToIndex(final int level) {
		return MathUtil.bound(level, 0, LEVEL_KEYS.length);
	}

	protected int indexToLevel(final int index) {
		return index;
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		choiceLevel.setSelectedIndex(levelToIndex(gameProperties.getLevel()));
	}

	@Override
	public void readUIFieldsIntoProperties() {
		gameProperties.setLevel(indexToLevel(choiceLevel.getSelectedIndex()));
	}

	public GameProperties getProperties() {
		return gameProperties;
	}

	public void enterDialog(final GameProperties props) {
		gameProperties = (GameProperties) props.clone();
		final PacmanLevelSet ls = gameProperties.getLevelSet();
		final String levelSetName;
		switch (ls) {
		case STANDARD:
			levelSetName = getMessages().getString("GameProperties.LevelSet.Default");
			break;
		default:
			levelSetName = ls.name;
		}
		choiceLevelSet.removeAllItems();
		choiceLevelSet.addItem(levelSetName);
		enterDialog();
	}

	private JIconComboBox<?> choiceLevel;
	private JIconComboBox<Object> choiceLevelSet;

	private GameProperties gameProperties;
}
