package de.ped.pacman.logic;

import java.util.ArrayList;

import de.ped.tools.PlayfieldDimension;
import de.ped.tools.PlayfieldPosition;

/**
 * Knows the playboard.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Labyrinth {

	public static final char EMPTY  = ' ';
	public static final char DOT    = '.';
	public static final char PILL   = 'o';
	public static final char DEN    = ':';
	public static final char DOOR   = '=';
	public static final char WALL   = '#';
	public static final String EOL  = "\n";

	public static final String LEGAL_CHARS = "" + EMPTY + DOT + PILL + DEN + DOOR + WALL;

	public static final String LABYRINTH_TITLE = 
	"    # . #  #####  ######    \n" +
	"  . #   # .  #    #  .      \n" +
	"  o #####    #.   ###### o  \n" +
	"    #   #  . #         #    \n" +
	"    #.  #    #  . ######    \n" +
	"  .       .          .   .  \n" +
	"   ########    .            \n" +
	" . ########       .         \n" +
	"   ##    ## ###### .#####   \n" +
	"   ## .  ## ######  ##### . \n" +
	"   ######## ##. ##  ##      \n" +
	" . ######## ##  ##  ##  .   \n" +
	"   ## .     ####### #####   \n" +
	".  ##       ####### #####  .\n" +
	"         .             .    \n" +
	" ## .     ##      .         \n" +
	" ###     ### ######. ###### \n" +
	" ####   #### ######  ######.\n" +
	" ########### ## .##  ## .## \n" +
	".## ##### ## ##  ##  ##  ## \n" +
	" ##  ###  ## ####### ##. ## \n" +
	" ##    .  ## ####### ##  ## \n" +
	"         .             .    \n" +
	"############################\n" +
	"#............o............ #\n" +
	"#####==#####################\n" +
	"  #::::::#                  \n" +
	"  #::::::#                  \n" +
	"  #::::::#                  \n" +
	"  ########                  \n" +
	"                            \n";

	public static final Being[] BEINGS_TITLE = new Being[] {
			new Pacman(26F, 24F, Direction.LEFT, 20),
			new Fruit(15F, 24F, Direction.RIGHT, 40),
			new Opponent(5.5F, 27F, Direction.LEFT, Being.BLINKY, 28),
			new Opponent(3.5F, 28F, Direction.RIGHT, Being.PINKY, 26),
			new Opponent(5.5F, 17F, Direction.LEFT, Being.INKY, 24),
			new Opponent(15F, 6F, Direction.RIGHT, Being.CLYDE, 22),
		};

	public static final String LABYRINTH_PACMAN = 
	"############################\n" +
	"#............##............#\n" +
	"#.####.#####.##.#####.####.#\n" +
	"#o#  #.#   #.##.#   #.#  #o#\n" +
	"#.####.#####.##.#####.####.#\n" +
	"#..........................#\n" +
	"#.####.##.########.##.####.#\n" +
	"#.####.##.########.##.####.#\n" +
	"#......##....##....##......#\n" +
	"######.##### ## #####.######\n" +
	"     #.##### ## #####.#     \n" +
	"     #.##          ##.#     \n" +
	"     #.## ###==### ##.#     \n" +
	"######.## #::::::# ##.######\n" +
	"      .   #::::::#   .      \n" +
	"######.## #::::::# ##.######\n" +
	"     #.## ######## ##.#     \n" +
	"     #.##          ##.#     \n" +
	"     #.## ######## ##.#     \n" +
	"######.## ######## ##.######\n" +
	"#............##............#\n" +
	"#.####.#####.##.#####.####.#\n" +
	"#o####.#####.##.#####.####o#\n" +
	"#...##.......  .......##...#\n" +
	"###.##.##.########.##.##.###\n" +
	"###.##.##.########.##.##.###\n" +
	"#......##....##....##......#\n" +
	"#.##########.##.##########.#\n" +
	"#.##########.##.##########.#\n" +
	"#..........................#\n" +
	"############################\n";

	public static final Being[] BEINGS = new Being[] {
		new Pacman(13.5F, 23F, Direction.LEFT, 0),
		new Fruit(13.5F, 17F, Direction.LEFT, 8 * Opponent.STEPS_IN_DEN),
		new Opponent(13.5F, 13F, Direction.LEFT, Being.BLINKY, 0),
		new Opponent(11.5F, 14F, Direction.RIGHT, Being.PINKY, 0),
		new Opponent(13.5F, 14F, Direction.LEFT, Being.INKY, 0),
		new Opponent(15.5F, 14F, Direction.RIGHT, Being.CLYDE, 0),
	};

	public static final String LABYRINTH_PACMAN2 = 
		"############################\n" +
		"#............##............#\n" +
		"#.####.#####.##.#####.####.#\n" +
		"#o#  #.#   #.##.#   #.#  #o#\n" +
		"#.####.#####.##.#####.####.#\n" +
		"#..........................#\n" +
		"#.####.##.########.##.####.#\n" +
		"#.####.##.########.##.####.#\n" +
		"#......##....##....##......#\n" +
		"######.##### ## #####.######\n" +
		"     #.##### ## #####.#     \n" +
		"     #.##          ##.#     \n" +
		"     #.## ###==### ##.#     \n" +
		"######.## #::::::# ##.######\n" +
		"      .   #::::::#   .      \n" +
		"######.## #::::::# ##.######\n" +
		"     #.## ######## ##.#     \n" +
		"     #.##          ##.#     \n" +
		"     #.## ######## ##.#     \n" +
		"######.## ######## ##.######\n" +
		"#............##............#\n" +
		"#.####.#####.##.#####.####.#\n" +
		"#...##.#####.##.#####.##...#\n" +
		"###.##.......  .......##.###\n" +
		"###.##..##..####..##..##.###\n" +
		"#.......##..####..##.......#\n" +
		"#.########...##...########.#\n" +
		"#.########...##...########.#\n" +
		"#.##....##...##...##....##.#\n" +
		"#o...##..............##...o#\n" +
		"############################\n";

	public static final String LABYRINTH_PACMAN3 = 
			"          ###               \n" +
			"         ##.##              \n" +
			"         #.o.#         ###  \n" +
			"         #.#.#        ##.## \n" +
			"         #.#.#        #.o.# \n" +
			"         #.#.#        #.#.# \n" +
			"  ###    #.#.#        #.#.# \n" +
			" ##.##   #.#.#    ### #.#.# \n" +
			"##.o.#   #.#.######.###.#.# \n" +
			"##.#.#####.#.#...#.o.##.#.# \n" +
			"##.#.#...#.#.#.#.#.#.##.#.# \n" +
			"##.#.#.#.........#.#.##.#.# \n" +
			"##.#.#...####==###.#.##.#.# \n" +
			"##.#.#.#..#::::::#.#.##...# \n" +
			"##.#...##.#::::::#.#.#..####\n" +
			"##.#.#.#..#::::::#.#.#.#...#\n" +
			" ..#.#...#########.#.##..#. \n" +
			"##...#.#....#....#.#....#..#\n" +
			"#..##..#.##...##.#...###..##\n" +
			"##....#..##.#..#...##.....# \n" +
			"##..#...#...##...#....##..# \n" +
			"###..######....########..## \n" +
			"  ##.......#####........##  \n" +
			"   #######.......########   \n" +
			"         #########          \n" +
			"                            \n" +
			" ##  ###  #  # #### #  # ###\n" +
			"#### # ## #  # #    ## #  # \n" +
			"#  # #  # #  # ###  ####  # \n" +
			"#### # ## #### #    # ##  # \n" +
			"#  # ###   ##  #### #  #  # \n";

	private char[][] lab;
	private int remainingDots;
	private ArrayList<Being> beings = new ArrayList<Being>();
	private Being[] beingTemplates;
	private boolean wasFruitOutOnThisLevel = false;
	private ArrayList<PlayfieldPosition> doorPositions = new ArrayList<PlayfieldPosition>();

	public synchronized void initLabyrinth(final String labyrinthDesc) {
		final String[] lines = labyrinthDesc.split(EOL);
		int xsize = 0;
		for (final String line : lines) {
			xsize = Math.max(xsize, line.length());
		}
		remainingDots = 0;
		doorPositions.clear();
		lab = new char[xsize][lines.length];
		for (int y = 0; y < lines.length; y++) {
			for (int x = 0; x < xsize; x++) {
				lab[x][y] = EMPTY;
			}
			final String line = lines[y];
			for (int x = 0; x < line.length(); x++) {
				set(x, y, line.charAt(x));
			}
		}
		wasFruitOutOnThisLevel = false;
	}

	public synchronized void initBeings(final Being[] beings) {
		this.beingTemplates = beings;
		this.beings.clear();
		if (null != beings) {
			for (Being being : beings) {
				if (0 == being.stepsBeforeInStage) {
					addBeing(being.getId());
				}
			}
		}
	}

	public void init(final String labyrinthDesc, final Being[] beings) {
		initLabyrinth(labyrinthDesc);
		initBeings(beings);
	}

	public void assertNotNull(final String failure, final Being being) {
		if (null != being) {
			throw new IllegalArgumentException(failure);
		}
	}

	public synchronized PlayfieldDimension getSize() {
		return new PlayfieldDimension(lab.length, lab[0].length);
	}

	public synchronized boolean bounds(final int x, final int y) {
		return ((0 <= x) && (x < lab.length) &&
				(0 <= y) && (y < lab[0].length));
	}

	public synchronized char get(final int x, final int y) {
		return lab[x][y];
	}

	public synchronized void set(final int x, final int y, final char value) {
		final char oldValue = lab[x][y];
		if (value != oldValue) {
			if (DOT == oldValue) {
				remainingDots--;
			}
			if (DOT == value) {
				remainingDots++;
			}
			if (DOOR == oldValue) {
				doorPositions.remove(new PlayfieldPosition(x, y));
			}
			if (DOOR == value) {
				doorPositions.add(new PlayfieldPosition(x, y));
			}
		}
		lab[x][y] = value;
	}

	public synchronized int getRemainingDots() {
		return remainingDots;
	}

	public double calcDistance(final PlayfieldPosition pos, final double x, final double y) {
		return Math.abs(x - pos.getX()) + Math.abs(y - pos.getY());
	}

	/**
	 * Returns the position of the nearest door to the given coordinates. Never
	 * change the content of the object you receive here, it's only a reference
	 * to crucial data!
	 * 
	 * @param x
	 *            the x coordinate of the position
	 * @param y
	 *            the y coordinate of the position
	 * @return the position of the nearest door to the given coordinates
	 */
	public PlayfieldPosition getNearestDoor(final double x, final double y) {
		PlayfieldPosition ret = doorPositions.get(0);
		for (final PlayfieldPosition pos : doorPositions) {
			if (calcDistance(pos, x, y) < calcDistance(ret, x, y)) {
				ret = pos;
			}
		}
		return ret;
	}

	public synchronized ArrayList<Being> getBeings() {
		return beings;
	}
	
	Being[] getBeingTemplates() {
		return beingTemplates;
	}

	public synchronized Being getBeing(final String beingID) {
		for (final Being being : beings) {
			if (being.getId().equals(beingID)) {
				return being;
			}
		}
		return null;
	}

	/**
	 * Adds the being to this labyrinth.
	 * 
	 * @param beingID
	 *            the ID of the being to add
	 * @return <code>true</code> if the being was not there before
	 */
	public synchronized boolean addBeing(final String beingID) {
		final Being old = getBeing(beingID);
		if (null == old) {
			boolean isToAdd = true;
			if (beingID == Being.FRUIT) {
				if (wasFruitOutOnThisLevel) {
					isToAdd = false;
				} else {
					wasFruitOutOnThisLevel = true;
				}
			}
			if (isToAdd) {
				for (final Being being : beingTemplates) {
					if (being.getId().equals(beingID)) {
						beings.add(being.clone());
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Removes the being from this labyrinth.
	 * 
	 * @param being
	 *            the being to remove
	 * @return <code>true</code> if the being was there before
	 */
	public synchronized boolean removeBeing(final Being being) {
		boolean ret = beings.contains(being);
		beings.remove(being);
		return ret;
	}
}
