package de.ped.pacman.logic;

/**
 * All that the game needs to know about the move result.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class MoveResultInformation {
	public final float xDestPos;
	public final float yDestPos;
	public final char valAtCurentPos;
	public final char valAtDestinationPos;

	public MoveResultInformation(final float xDestPos, final float yDestPos) {
		this.xDestPos = xDestPos;
		this.yDestPos = yDestPos;
		this.valAtCurentPos = 0;
		this.valAtDestinationPos = 0;
	}

	public MoveResultInformation(
			final float xDestPos, 
			final float yDestPos, 
			final char valueAtCurentPos, 
			final char valueAtDestinationPos) {
		this.xDestPos = xDestPos;
		this.yDestPos = yDestPos;
		this.valAtCurentPos = valueAtCurentPos;
		this.valAtDestinationPos = valueAtDestinationPos;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + valAtCurentPos;
		result = prime * result + valAtDestinationPos;
		result = prime * result + Float.floatToIntBits(xDestPos);
		result = prime * result + Float.floatToIntBits(yDestPos);
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final MoveResultInformation other = (MoveResultInformation) obj;
		if (valAtCurentPos != other.valAtCurentPos)
			return false;
		if (valAtDestinationPos != other.valAtDestinationPos)
			return false;
		if (Float.floatToIntBits(xDestPos) != Float.floatToIntBits(other.xDestPos))
			return false;
		if (Float.floatToIntBits(yDestPos) != Float.floatToIntBits(other.yDestPos))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "MoveResultInformation [xDestPos=" + xDestPos
				+ ", yDestPos=" + yDestPos + ", valAtCurentPos="
				+ valAtCurentPos + ", valAtDestinationPos="
				+ valAtDestinationPos + "]";
	}
}