package de.ped.pacman.logic;

import java.util.LinkedList;

import de.ped.tools.PlayfieldPosition;

/**
 * The PacMan being.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Pacman extends Being {

	protected int waitSteps = 0;

	public Pacman(final float x, final float y, final int direction, int stepsBeforeInStage) {
		super(Being.PACMAN, x, y, direction, stepsBeforeInStage, 0);
	}

	@Override
	protected void evalAfterMove(final Game game, final LinkedList<Game.Event> events) {
		final PlayfieldPosition pos = this.field();
		if (null != pos) {
			final char c = game.getBoard().get(pos.getX(), pos.getY());
			if (c == Labyrinth.PILL) {
				game.getBoard().set(pos.getX(), pos.getY(), Labyrinth.EMPTY);
				game.addPoints(10, this);
				game.eatPill();
				setWaitSteps(1);
				events.add(Game.Event.EATING_PILL);
			}
			if (c == Labyrinth.DOT) {
				game.getBoard().set(pos.getX(), pos.getY(), Labyrinth.EMPTY);
				game.addPoints(1, null);
				setWaitSteps(1);
				events.add(Game.Event.EATING_DOT);
			}
		}
//		logger.debug("Being " + id + " moved " + Direction.DIRECTIONS[direction].name + " to position (" + vector[0] + ", " + vector[1] + ").");
	}

	/**
	 * When eating a dot or a pill, PacMan is slowed down for one step.
	 * 
	 * @param game
	 *            the game the being is in
	 * @return <code>false</code>, meaning that the being is never waiting and
	 *         always running at full speed
	 */
	@Override
	public int numberOfMovesForThisStep(final Game game) {
		int ret = 1;
		if (0 < waitSteps) {
//			logger.debug("waitSteps = " + waitSteps);
			waitSteps--;
			ret = 0;
		}
		return ret;
	}

	/**
	 * When eating a dot or a pill, PacMan is slowed down for one step.
	 * 
	 * @param waitSteps
	 *            the number of steps to be slowed down
	 */
	protected void setWaitSteps(final int waitSteps) {
		this.waitSteps = waitSteps;
	}

	@Override
	public int planNextDirection(final Game game) {
		int ret = Direction.UNDEFINED;
		if (null != game) {
			ret = game.getSuggestedUserGameAction();
		}
		return ret;
	}
}
