package de.ped.pacman.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.pacman.logic.Labyrinth;
import de.ped.tools.PlayfieldPosition;

/**
 * Tests {@link Labyrinth}.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LabyrinthTest {

	public static final String LABYRINTH_1x1 = 
		"###\n" +
		"#P#\n" +
		"###\n";

	public static final String LABYRINTH_3x2 = 
		"#######\n" +
		"#     #\n" +
		"# ### #\n" +
		"#  P  #\n" +
		"#######\n";

	public static final String LABYRINTH_2x3 = 
		"#####\n" +
		"#   #\n" +
		"# # #\n" +
		"# # #\n" +
		"# # #\n" +
		"#   #\n" +
		"#####\n";

	@Test(expected=NullPointerException.class)
	public void testInitNull() {
		Labyrinth labyrinth = new Labyrinth();
		labyrinth.init(null, null);
	}

	@Test
	public void testInitPacMan() {
		final Labyrinth labyrinth = new Labyrinth();
		labyrinth.init(Labyrinth.LABYRINTH_PACMAN, null);
		assertEquals(28, labyrinth.getSize().getWidth());
		assertEquals(31, labyrinth.getSize().getHeight());
		assertEquals(Labyrinth.PILL, labyrinth.get(1, 3));
		labyrinth.set(1, 3, Labyrinth.EMPTY);
		assertEquals(Labyrinth.EMPTY, labyrinth.get(1, 3));
		assertEquals(240, labyrinth.getRemainingDots());
		labyrinth.set(1, 1, Labyrinth.DOT);
		assertEquals(240, labyrinth.getRemainingDots());
		labyrinth.set(1, 1, Labyrinth.EMPTY);
		assertEquals(239, labyrinth.getRemainingDots());
		labyrinth.set(1, 1, Labyrinth.DOT);
		assertEquals(240, labyrinth.getRemainingDots());

		assertEquals(Labyrinth.DOOR, labyrinth.get(13, 12)); // Precondition for the following tests
		assertEquals(Labyrinth.DOOR, labyrinth.get(14, 12)); // Precondition for the following tests
		assertEquals(new PlayfieldPosition(13, 12), labyrinth.getNearestDoor(1, 1));
		assertEquals(new PlayfieldPosition(13, 12), labyrinth.getNearestDoor(13, 11));
		assertEquals(new PlayfieldPosition(13, 12), labyrinth.getNearestDoor(13, 12));
		assertEquals(new PlayfieldPosition(13, 12), labyrinth.getNearestDoor(13, 13));
		assertEquals(new PlayfieldPosition(14, 12), labyrinth.getNearestDoor(15, 1));
		assertEquals(new PlayfieldPosition(14, 12), labyrinth.getNearestDoor(14, 11));
		assertEquals(new PlayfieldPosition(14, 12), labyrinth.getNearestDoor(14, 12));
		assertEquals(new PlayfieldPosition(14, 12), labyrinth.getNearestDoor(14, 13));
	}
}
