package de.ped.pacman.test;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Collection;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import de.ped.pacman.logic.WallMaskFilter;

/**
 * Tests {@link WallMaskFilter}.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
@RunWith(value=Parameterized.class)
public class WallMaskFilterTest {

	private static WallMaskFilter wallMaskFilter;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		wallMaskFilter = new WallMaskFilter();
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		wallMaskFilter = null;
	}

	@Before
	public void setUp() throws Exception {
		// Nothing to do
	}

	@After
	public void tearDown() throws Exception {
		// Nothing to do
	}

	@Parameters
	public static Collection<Integer[]> data() {
		return Arrays.asList( new Integer[][] {
			{ 
				0x00, 
				0x00, 
			},
			{ 
				WallMaskFilter.MASK_DOWN, 
				WallMaskFilter.MASK_DOWN, 
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP, 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_LEFT, 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_LEFT
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP | WallMaskFilter.MASK_LEFT , 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP | WallMaskFilter.MASK_LEFT
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_LEFT |	WallMaskFilter.MASK_DOWN_LEFT, 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_LEFT
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP | WallMaskFilter.MASK_LEFT | 
				WallMaskFilter.MASK_DOWN_LEFT | WallMaskFilter.MASK_UP_LEFT, 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP | WallMaskFilter.MASK_LEFT | 
				WallMaskFilter.MASK_DOWN_LEFT | WallMaskFilter.MASK_UP_LEFT | WallMaskFilter.MASK_DOWN_RIGHT, 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP
			},
			{ 
				WallMaskFilter.MASK_DOWN | WallMaskFilter.MASK_UP | WallMaskFilter.MASK_LEFT | WallMaskFilter.MASK_RIGHT |  
				WallMaskFilter.MASK_DOWN_LEFT | WallMaskFilter.MASK_UP_LEFT | WallMaskFilter.MASK_DOWN_RIGHT, 
				WallMaskFilter.MASK_UP | WallMaskFilter.MASK_RIGHT
			},
		});
	}

	private int input;
	private int expected;

	public WallMaskFilterTest(final int input, final int expected) {
		this.input = input;
		this.expected = expected;
	}

	@Override
	public String toString() {
		return "Input=" + Integer.toHexString(input);
	}

	@Test
	public void testFilterMaskWalls() {
		assertEquals(expected, wallMaskFilter.filterMaskWalls(input));
	}
}
