package de.ped.tools;

/**
 * Provides all the environment a typical application needs.
 *
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public interface ApplicationEnvironment {

	public static final String PARAM_KEY_CHRISTMAS = "christmas";
	public static final String PARAM_KEY_GAME      = "level";
	public static final String PARAM_KEY_LANGUAGE  = "language";
	public static final String PARAM_KEY_DEBUG     = "debug";

	/**
	 * Finds an application parameter by key in the list of parameters.
	 * 
	 * @param key
	 *            the parameter key to search for
	 * @return the parameter object, if found; <code>null</code> otherwise.
	 */
	public ApplicationParameter findParameter(final String key);

	/**
	 * Creates if necessary and returns the resource finder object.
	 * 
	 * @return the resource finder
	 */
	public ResourceFinder resources();

	/**
	 * Creates if necessary and returns the messages object for
	 * internationalization.
	 * 
	 * @return the messages object
	 */
	public Messages messages();

	/** 
	 * @return <code>true</code> if the user can actively exit the application 
	 */
	public boolean isExitable();

	/**
	 * @return <code>true</code> if the application shall run silently (without
	 *         sound and frames opening), e.g., in special test scenarios
	 */
	public boolean isSilent();
}
