package de.ped.tools;

/**
 * A bit area, consisting of a starting bit and the number of bits.
 * Used in the Bitfield classes.
 * 
 * @author Copyright (c) 2015 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class BitArea {
	/**
	 * Constructor.
	 * 
	 * @param firstBit
	 *            the number of the least significant bit of this bit area
	 * @param numberOfBits
	 *            the number bits of this bit area
	 */
	public BitArea(final int firstBit, final int numberOfBits) {
		this.firstBit = firstBit;
		this.numberOfBits = numberOfBits;
		this.bits32mask = Bitfield32.bitmask(firstBit, numberOfBits);
		this.bits64mask = Bitfield64.bitmask(firstBit, numberOfBits);
	}

	/** The number of the least significant bit of this bit area */
	public final int firstBit;
	/** The number bits of this bit area */
	public final int numberOfBits;
	/** The bitmask representing this bit area in an int */
	public final int bits32mask;
	/** The bitmask representing this bit area in a long */
	public final long bits64mask;
}