package de.ped.tools;

/**
 * An {@link I18NStringWithFillIns} object with a messages object, so it can handle long descriptions for online help.
 * @author Copyright (c) 2019 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LocalizedString {
	public LocalizedString(final Messages messages, final String key, final String imageFilename) {
		this.messages = messages;
		this.key = key;
		this.imageFilename = imageFilename;
	}

	public LocalizedString(final Messages messages, final String key) {
		this(messages, key, null);
	}

	public String getText() {
		return messages.getText(key);
	}

	public String getImageFilename() {
		return (null == imageFilename ? key : imageFilename);
	}

	@Override
	public String toString() {
		return getText();
	}

	public String getToolTip() {
		return messages.getShortDescription(key);
	}

	public String getLongDescription() {
		return messages.getLongDescription(key);
	}

	private Messages messages;
	public final String key;
	private final String imageFilename;
}