package de.ped.tools;

import java.util.HashSet;
import java.util.Observer;

/**
 * My own implementation of the Observer pattern, as the implementation in
 * {@link java.util.Observable} seems faulty. 
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PedObservable extends java.util.Observable {
	@Override
	public synchronized void addObserver(final Observer observer) {
		observers.add(observer);
	}

	@Override
	public synchronized void deleteObserver(final Observer observer) {
		observers.remove(observer);
	}

	@Override
	public synchronized void deleteObservers() {
		observers.clear();
	}

	@Override
	public synchronized int countObservers() {
		return observers.size();
	}

	@Override
	public void notifyObservers() {
		notifyObservers(null);
	}

	@Override
	public void notifyObservers(final Object what) {
		final HashSet<Observer> set = myclone();
		for (final Observer observer : set) {
			observer.update(this, what);
		}
	}

	protected synchronized HashSet<Observer> myclone() {
		final HashSet<Observer> ret = new HashSet<Observer>(observers);
		return ret;
	}

	private HashSet<Observer> observers = new HashSet<Observer>();
}
