package de.ped.tools;

import java.awt.Color;

/**
 * Represents one of the possible colors a player's units will be displayed with.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlayerColor implements Idable {
	/**
	 * Initializes this class with the list of allowed player colors. Must be the first call to this class or any object of that class.
	 * @param insts the list of possible player colors
	 */
	public static void init(final PlayerColor[] insts) {
		instances = insts;
	}

	public static int size() {
		final int len = instances.length - 1;
		return instances.length - (instances[len].id == len ? 0 : 1); // workaround for Player.ID_NONE in Empire
	}

	public static PlayerColor get(final int colorId) {
		PlayerColor ret;
		final PlayerColor[] list = instances;
		if (colorId == list[list.length - 1].id) { // workaround for Player.ID_NONE in Empire
			ret = list[list.length - 1];
		} else {
			ret = list[colorId];
		}
		return ret;
	}

	private static PlayerColor[] instances;

	public PlayerColor(final int id, final String key, final String foregroundRgb, final String backgroundRgb) {
		this.id = id;
		this.key = key;
		this.foreground = Color.getColor(null, Integer.parseInt(foregroundRgb, 16));
		this.background = Color.getColor(null, Integer.parseInt(backgroundRgb, 16));
	}

	@Override
	public int getId() {
		return id;
	}

	public String getKey() {
		return key;
	}

	public Color getColor() {
		return foreground;
	}

	public Color getBackground() {
		return background;
	}

	private final int id;
	private final String key;
	private final Color foreground;
	private final Color background;
}
