package de.ped.tools;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * A tree structure containing nodes of text titles and descriptions, taking use of the {@link Messages} and {@link I18NStringWithFillIns} possibilities. 
 * 
 * @author Copyright (c) 2019 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TextTree {
	public TextTree() {
		// Nothing to do
	}

	public TextTree(final TextTreeNodeType type, 
			final String key, 
			final String text, 
			final String longDescription, 
			final String htmlAttribs, 
			final String imageFilename, 
			final TextTree[] children) {
		this.type = type;
		this.node = new I18NStringWithFillIns(key, text, null);
		this.htmlAttribs = htmlAttribs;
		this.imageFilename = imageFilename;
		if (null != children) {
			this.children = new ArrayList<TextTree>(Arrays.asList(children)); 
		}
	}

	public TextTreeNodeType getType() {
		return type;
	}

	public void setType(TextTreeNodeType type) {
		this.type = type;
	}

	public I18NStringWithFillIns getNode() {
		return node;
	}

	public void setNode(final I18NStringWithFillIns node) {
		this.node = node;
	}

	public void setKey(final String key) {
		if (null == node) {
			node= new I18NStringWithFillIns(key, null, null);
		} else {
			node.setKey(key);
		}
	}

	public void setText(final String text) {
		if (null == node) {
			node= new I18NStringWithFillIns(null, text, null);
		} else {
			node.setAlternative(text);
		}
	}

	public String getLongDescription() {
		return longDecription;
	}

	public void setLongDescription(final String longDecription) {
		this.longDecription = longDecription;
	}

	public String getHtmlAttribs() {
		return htmlAttribs;
	}

	public void setHtmlAttribs(final String htmlAttribs) {
		this.htmlAttribs = htmlAttribs;
	}

	public String getImageFilename() {
		return imageFilename;
	}

	public void setImageFilename(final String imageFilename) {
		this.imageFilename = imageFilename;
	}

	public ArrayList<TextTree> getChildren() {
		return children;
	}

	public void setChildren(final List<TextTree> children) {
		this.children = new ArrayList<TextTree>(children);
	}

	public void setChildren(final TextTree[] children) {
		this.children = new ArrayList<TextTree>(Arrays.asList(children));
	}

	public void add(final TextTree child) {
		if (null != child) {
			if (null == children) {
				children = new ArrayList<TextTree>(1);
			}
			children.add(child);
		}
	}

	@Override
	public String toString() {
		return "TextTree(" + type + "," + (null == node ? "<null node>" : node.toString()) + ")";
	}

	public String dumpRecursive(final int numIndent) {
		final String indentStr = "  ";
		StringBuffer buf = new StringBuffer();
		buf.append(TextFormatter.indent(indentStr, numIndent, toString()));
		buf.append("\n");
		if (null != children) {
			for (TextTree child : children) {
				buf.append(child.dumpRecursive(numIndent + 1));
			}
		}
		return buf.toString();
	}

	protected TextTreeNodeType type;
	protected I18NStringWithFillIns node;
	protected String longDecription;
	protected String htmlAttribs;
	protected String imageFilename;
	protected ArrayList<TextTree> children;
}