package de.ped.tools.dev;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.TreeSet;

import de.ped.tools.Messages;
import de.ped.tools.log.Logger;

/**
 * Converts the I18N'ed key/value string properties through a resource bundle to JavaScript plain objects.
 * 
 * @author Copyright (c) 2020 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class MessagesConverter {

	public static void main(final String[] args) throws IOException {
		final MessagesConverter converter = new MessagesConverter();
		converter.convertBundles(Messages.DEFAULT_RESOURCE_BUNDLE_NAME, "src/html/js/tools/toolsMessages.js");
		converter.convertBundles("de.ped.tools.test.messages",   "src/html/js/spec/testMessages.js");
		converter.convertBundles("de.ped.kitten.logic.messages", "src/html/js/kitten/kittenMessages.js");
		converter.convertBundles("de.ped.pacman.logic.messages", "src/html/js/pacman/pacmanMessages.js");
		converter.convertBundles("de.ped.troff.client.logic.messages", "src/html/js/troff/troffMessages.js");
	}

	public void convertBundles(final String bundleName, final String path) throws IOException {
		final FileWriter fw = new FileWriter(path);
		for (Locale locale : Messages.DEFAULT_LOCALES) {
			fw.write(convertBundle(bundleName, path, locale));
		}
		fw.close();
	}

	public String convertBundle(final String bundleName, final String path, final Locale locale) throws IOException {
		final String varname = bundleName.replaceAll("\\.", "_") + "_" + locale.toString();
		final StringBuffer buf = new StringBuffer();
		final ResourceBundle bundle = ResourceBundle.getBundle(bundleName, locale);
		final Enumeration<String> keys = bundle.getKeys();
		// Sort the keys, to make the resulting code more readable for the human eye
		final TreeSet<String> keysTree = new TreeSet<String>();
		while(keys.hasMoreElements()) {
			keysTree.add(keys.nextElement());
		}
		// Write the resource bundle as JavaScript var
		boolean isFirst = true;
		for (String key : keysTree) {
			if (isFirst) {
				buf.append("var " + varname + " = {");
				isFirst = false;
			} else {
				buf.append(",");
			}
			buf.append("\n");
			final String value = bundle.getString(key).replaceAll("\"", "\\\\\\\"").replaceAll("\n", "\\\\\\\n\\\t\\\t");
			buf.append("\t" + key.replaceAll("\\.", "_") + ": \"" + value + "\"");
		}
		buf.append("\n");
		buf.append("};\n");
		logger.info(buf.toString());
		return buf.toString();
	}

	final Logger logger = Logger.getLogger(getClass());
}
