package de.ped.tools.gui;

import java.awt.event.ActionEvent;

import javax.swing.JCheckBoxMenuItem;

import de.ped.tools.ResourceFinder;
import de.ped.tools.log.Logger;

/**
 * An action keeping a boolean state for a check box menu item.
 * If the boolean state should be kept somewhere else for decoupling reasons,
 * you can override {@link #isOn()} and {@link #setOn(boolean)}.
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class CheckBoxMenuItemAction extends PedAction {
	private static final long serialVersionUID = 1L;

	private boolean isOn = true;

	private JCheckBoxMenuItem checkBoxMenuItem;

	public CheckBoxMenuItemAction() {
		// Nothing to do
	}

	public CheckBoxMenuItemAction(final boolean isOn) {
		setOn(isOn);
	}

	public CheckBoxMenuItemAction(final boolean isOn, final String messageKey, final Logger logger) {
		setOn(isOn);
		setMessageKeyPrefix(messageKey);
		setLogger(logger);
	}

	public CheckBoxMenuItemAction(final boolean isOn, final String messageKey, final ApplicationMainWindow appMainWindow, final ResourceFinder.Folder folder) {
		super(messageKey, appMainWindow, folder, false);
		setOn(isOn);
	}

	@Override
	public void securedActionPerformed(final ActionEvent e) {
		setOn(!isOn);
	}

	/**
	 * Returns if the checkbox should be displayed as checked.
	 * <p>
	 * Override if you want to keep the state somewhere else.
	 * 
	 * @return <code>true</code> if the checkbox should be displayed as checked
	 */
	public boolean isOn() {
		return isOn;
	}

	/**
	 * Changes the underlying data value of the checkbox.
	 * <p>
	 * Override if you want to keep the state somewhere else.
	 * 
	 * @param on <code>true</code> if the checkbox should be displayed as checked
	 */
	public void setOn(final boolean on) {
		isOn = on;
		if (null != checkBoxMenuItem) {
			checkBoxMenuItem.setSelected(isOn);
		}
	}

	public JCheckBoxMenuItem createCheckBoxMenuItem() {
		if (null == checkBoxMenuItem) {
			checkBoxMenuItem = new JCheckBoxMenuItem(this);
			checkBoxMenuItem.setSelected(isOn);
		}
		return checkBoxMenuItem;
	}
}
