package de.ped.tools.gui;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 * A tree renderer to support JTree with a checkbox for each node.
 * 
 * @author Santhosh Kumar T - santhosh@in.fiorano.com, from a <a href=
 *         "http://www.jroller.com/page/santhosh/Weblog/jtree_with_checkboxes?catname=">thread
 *         at jroller.com</a>.
 * @author Peter Diefenbach (peter@pdiefenbach.de) did modifications to use
 *         directly the TreeModel, not the TreeSelectionModel.
 */
public class CheckTreeCellRenderer extends JPanel implements TreeCellRenderer {
	private static final long serialVersionUID = 1L;

	private CheckTreeSelectionModel selectionModel;
	private TreeModel treeModel;
	private TreeCellRenderer delegate;
	private TristateCheckBox checkBox = new TristateCheckBox();

	/**
	 * Constructor for use with selection model - Kumar's original
	 * implementation.
	 * 
	 * @param delegate
	 *            the renderer
	 * @param selectionModel
	 *            the selection model
	 */
	public CheckTreeCellRenderer(final TreeCellRenderer delegate, final CheckTreeSelectionModel selectionModel) {
		this.delegate = delegate;
		this.selectionModel = selectionModel;
		setLayout(new BorderLayout());
		setOpaque(false);
		checkBox.setOpaque(false);
	}

	/**
	 * Constructor for use with tree model containing selection state - Peter's
	 * implementation.
	 * 
	 * @param delegate
	 *            the renderer
	 * @param model
	 *            the tree model
	 */
	public CheckTreeCellRenderer(final TreeCellRenderer delegate, final TreeModel model) {
		this.delegate = delegate;
		this.treeModel = model;
		setLayout(new BorderLayout());
		setOpaque(false);
		checkBox.setOpaque(false);
	}

	@Override
	public Component getTreeCellRendererComponent(
			final JTree tree, 
			final Object value, 
			final boolean selected, 
			final boolean expanded, 
			final boolean leaf, 
			final int row, 
			final boolean hasFocus) {
		final Component renderer = delegate.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
		final TreePath path = tree.getPathForRow(row); 
		if (null != path) {
			// We have a selection model, so we use that - following Kumar's
			// original implementation.
			if (null != selectionModel) {
				if (selectionModel.isPathSelected(path, true)) {
					checkBox.setState(TristateCheckBox.SELECTED);
				} else {
					checkBox.setState(selectionModel.isPartiallySelected(path) ? TristateCheckBox.DONT_CARE : TristateCheckBox.NOT_SELECTED);
				}
			} else if (null != treeModel) {
				final TreeNode node = (TreeNode) path.getLastPathComponent();
				if (node instanceof CheckTreeNode) {
					final CheckTreeNode anode = (CheckTreeNode) node;
					checkBox.setState(anode.getSelectionState());
				}
			}
		}
		removeAll();
		add(checkBox, BorderLayout.WEST);
		add(renderer, BorderLayout.CENTER);
		return this;
	}
}
