package de.ped.tools.gui;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 * A configurable file filter for JFileChooser, filtering for file name
 * extensions.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ConfigurableFileFilter extends FileFilter  {
	public static char EXTENSION_DELIMITER = '.';

	/**
	 * @param file
	 *            the file
	 * @return the file name extension of a file.
	 */
	public static String getExtension(final File file) {
		String ext = null;
		String s = file.getName();
		int i = s.lastIndexOf(EXTENSION_DELIMITER);
		if ((0 < i) && (i < s.length() - 1)) {
			ext = s.substring(i + 1).toLowerCase();
		}
		return ext;
	}

	private String[] allowedExtensions;
	private String description;

	/**
	 * @param file
	 *            the file
	 * @return <code>true</code> if the file is a directory, or the file has one
	 *         of the allowed file name extensions
	 * @see #setAllowedExtensions
	 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
	 */
	@Override
	public boolean accept(final File file) {
		if (file.isDirectory()) {
			return true;
		}
		final String extension = getExtension(file);
		if ((null != extension) && (null != allowedExtensions)) {
			for (int i = 0; i < allowedExtensions.length; i++) {
				if (extension.equalsIgnoreCase(allowedExtensions[i])) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public String getDescription() {
		return description;
	}

	/**
	 * @return the allowed file name extensions
	 */
	public String[] getAllowedExtensions() {
		return allowedExtensions;
	}

	/**
	 * @param allowedExtensions The allowed file name extensions
	 */
	public void setAllowedExtensions(final String[] allowedExtensions) {
		this.allowedExtensions = allowedExtensions;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(final String description) {
		this.description = description;
	}

	public File correctExtensionOfSelectedFile(final File file) {
		File ret = file;
		if ((null != allowedExtensions) && (0 < allowedExtensions.length)) {
			boolean isFitting = false;
			for (final String extension : allowedExtensions) {
				if (file.getName().endsWith(EXTENSION_DELIMITER + extension)) {
					isFitting = true;
					break;
				}
			}
			if (!isFitting) {
				ret = new File(file.getPath() + EXTENSION_DELIMITER + allowedExtensions[0]);
			}
		}
		return ret;
	}
}
