package de.ped.tools.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

/**
 * The dialog that shows some help info about the application.
 * 
 * @author Copyright (c) 1999 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class HtmlPaneDialog extends PedJDialog implements KeyListener {
	private static final long serialVersionUID = 1L;

	/**
	 * Creates a dialog containing a read-only editor pane containing HTML text,
	 * and an OK button. Can be used as any info box like about or help.
	 * 
	 * @param parent
	 *            the parent frame
	 * @param title
	 *            the title to show for the window
	 * @param content
	 *            the HTML content to show in the editor pane
	 */
	public HtmlPaneDialog(final Frame parent, final String title, final String content) {
		super((ApplicationMainWindow)parent, parent);
		setTitle(title);
		setResizable(true);
		text = new JHtmlPane(content);
		text.addKeyListener(this);

		JScrollPane textScrollPane = new JScrollPane(text);
		getContentPane().add(textScrollPane, BorderLayout.CENTER);
		final Dimension size = parent.getPreferredSize();
		size.width = Math.max(600, size.width);
		size.height = Math.max(450, size.height);
		textScrollPane.setPreferredSize(size);
		setOptionType(JOptionPane.DEFAULT_OPTION);
		finishLayout();
	}

	@Override
	public void keyPressed(final KeyEvent e) {
		// Nothing to do
	}

	@Override
	public void keyReleased(final KeyEvent e) {
		// Nothing to do
	}

	/**
	 * Invoked when a key has been typed, especially in the text field. This
	 * event occurs when a key press is followed by a key release.
	 * <p>
	 * Here we just listen to the RETURN / {@link KeyEvent#VK_ENTER}  button to close the dialog box.
	 * 
	 * @param e
	 *            the key event to react to
	 */
	@Override
	public void keyTyped(final KeyEvent e) {
		if (KeyEvent.VK_ENTER == e.getKeyChar()) {
			onOk();
		}
	}

	public void scrollToInternalAnchor(final String anchor) {
		text.scrollToInternalAnchor(anchor, true);
	}

	private final JHtmlPane text;
}
