package de.ped.tools.gui;

import java.io.IOException;
import java.util.Enumeration;

import javax.swing.JEditorPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.ElementIterator;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.html.HTML.Attribute;
import javax.swing.text.html.HTML.Tag;

import de.ped.tools.TextFormatter;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * A pane showing HTML content, and tries to follow hyperlinks within the text.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class JHtmlPane extends JEditorPane implements HyperlinkListener {
	private static final long serialVersionUID = 1L;

	private Logger logger = Logger.getLogger(this.getClass());

	public JHtmlPane(final String content) {
		super(TextFormatter.CONTENT_TYPE_HTML, content);
		// See that the focus is at the beginning of the document.
		setSelectionStart(0);
		setSelectionEnd(0);
		setEditable(false);
		addHyperlinkListener(this);
	}
	
	@Override
	public void hyperlinkUpdate(final HyperlinkEvent event) {
		if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			if (null == event.getURL()) {
				scrollToInternalAnchor(event.getDescription(), false);
			} else {
				logger.debug(event.getURL().toString());
				try {
					setPage(event.getURL());
				} catch (final IOException iox) {
					logger.error(iox.toString());
					iox.printStackTrace();
				}
			}
		}
	}

	protected void scrollToInternalAnchor(final String description, final boolean isWithOffset) {
		final String hrefName = description.substring(1);
		final ElementIterator it = new ElementIterator(getDocument());
		Element e = it.first();
		Element anchorElement = null;
		while ((null != e) && (null == anchorElement)) {
			final AttributeSet set = e.getAttributes();
			final int level = Level.TRACE;
			if (Logger.isFor(level, logger.getLogLevel())) {
				final Enumeration<?> attribNames = set.getAttributeNames();
				while (attribNames.hasMoreElements()) {
					final Object key = attribNames.nextElement();
					if (key.equals(Tag.A)) {
						logger.log(level, "Element: " + e.getName());
						logger.log(level, " attrib " + key + " values " + set.getAttribute(key));
						final Object val = set.getAttribute(key);
						logger.log(level, "  val = " + val.getClass().getName());
						if (val instanceof SimpleAttributeSet) {
							final SimpleAttributeSet sas = (SimpleAttributeSet) val;
							final Enumeration<?> ats = sas.getAttributeNames();
							while (ats.hasMoreElements()) {
								final Object ikey = ats.nextElement();
								logger.log(level, "   key = " + ikey.getClass().getName());
								if (ikey.equals(Attribute.NAME)) {
									final Object ival = sas.getAttribute(ikey);
									logger.log(level, "   val = " + ival.getClass().getName());
								}
							}
						}
					}
				}
			}
			final Object aval = set.getAttribute(Tag.A);
			if (aval instanceof SimpleAttributeSet) {
				final SimpleAttributeSet sas = (SimpleAttributeSet) aval;
				final Object nval = sas.getAttribute(Attribute.NAME);
				if (hrefName.equals(nval)) {
					anchorElement = e;
					logger.log(level, "Bingo!");
				}
			}
			e = it.next();
		}
		if (null != anchorElement) {
			final int docLength = getDocument().getLength();
			final int off = (isWithOffset ? 200 : 0);
			final int pos = Math.min(docLength, anchorElement.getStartOffset() + off);
			setSelectionStart(pos);
			setSelectionEnd(pos);
			logger.debug("URL is null - description is " + description + ", textpos=" + pos);
		} else {
			logger.debug("URL is null - description is " + description + ", no fitting element found.");
		}
	}
}
