package de.ped.tools.gui;

import java.awt.Frame;

import javax.swing.JButton;
import javax.swing.JOptionPane;

/**
 * A typical information dialog with only a message text, a title, an optional icon, an ok button and optionally a cancel button.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class JInfoDialog extends PedJDialog {
	private static final long serialVersionUID = 1L;

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 */
	protected JInfoDialog(final Frame parentFrame) {
		super((ApplicationMainWindow) parentFrame, parentFrame);
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 * @param title
	 *            the title of the dialog
	 * @param message
	 *            the text to be displayed, may contain newlines
	 */
	public JInfoDialog(final Frame parentFrame, final String title, final String message) {
		this(parentFrame, title, message, JOptionPane.PLAIN_MESSAGE);
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 * @param title
	 *            the title of the dialog
	 * @param message
	 *            the text to be displayed, may contain newlines
	 * @param messageType
	 *            One of {@link JOptionPane#ERROR_MESSAGE},
	 *            {@link JOptionPane#WARNING_MESSAGE},
	 *            {@link JOptionPane#INFORMATION_MESSAGE},
	 *            {@link JOptionPane#QUESTION_MESSAGE}
	 */
	public JInfoDialog(final Frame parentFrame, final String title, final String message, final int messageType) {
		this(parentFrame);
		setTitle(title);
		setOptionPaneParams(message, messageType);
		finishLayout();
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 * @param title
	 *            the title of the dialog
	 * @param message
	 *            the text to be displayed, may contain newlines
	 * @param messageType
	 *            one of {@link JOptionPane#ERROR_MESSAGE},
	 *            {@link JOptionPane#WARNING_MESSAGE},
	 *            {@link JOptionPane#INFORMATION_MESSAGE},
	 *            {@link JOptionPane#QUESTION_MESSAGE}
	 * @param optionType
	 *            one of {@link JOptionPane#DEFAULT_OPTION},
	 *            {@link JOptionPane#OK_CANCEL_OPTION}
	 */
	public JInfoDialog(final Frame parentFrame, final String title, final String message, final int messageType, final int optionType) {
		this(parentFrame);
		setTitle(title);
		setOptionPaneParams(message, messageType);
		setOptionType(optionType);
		finishLayout();
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 */
	protected JInfoDialog(final PedJDialog parentFrame) {
		super(parentFrame.getMainWindow(), parentFrame);
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 * @param title
	 *            the title of the dialog. Non-I18N'ed.
	 * @param message
	 *            the text to be displayed, may contain newlines
	 */
	public JInfoDialog(final PedJDialog parentFrame, final String title, final String message) {
		this(parentFrame, title, message, JOptionPane.PLAIN_MESSAGE);
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 * @param title
	 *            the title of the dialog. Non-I18N'ed.
	 * @param message
	 *            the text to be displayed, may contain newlines
	 * @param messageType
	 *            one of {@link JOptionPane#ERROR_MESSAGE},
	 *            {@link JOptionPane#WARNING_MESSAGE},
	 *            {@link JOptionPane#INFORMATION_MESSAGE},
	 *            {@link JOptionPane#QUESTION_MESSAGE}
	 */
	public JInfoDialog(final PedJDialog parentFrame, final String title, final String message, final int messageType) {
		this(parentFrame);
		setTitle(title);
		setOptionPaneParams(message, messageType);
		finishLayout();
	}

	/**
	 * Constructor for modal info dialogs.
	 * 
	 * @param parentFrame
	 *            the frame responsible for that dialog (May be
	 *            <code>null</code>)
	 * @param title
	 *            the title of the dialog. Non-I18N'ed.
	 * @param message
	 *            the text to be displayed, may contain newlines
	 * @param messageType
	 *            one of {@link JOptionPane#ERROR_MESSAGE},
	 *            {@link JOptionPane#WARNING_MESSAGE},
	 *            {@link JOptionPane#INFORMATION_MESSAGE},
	 *            {@link JOptionPane#QUESTION_MESSAGE}
	 * @param optionType
	 *            one of {@link JOptionPane#DEFAULT_OPTION},
	 *            {@link JOptionPane#OK_CANCEL_OPTION}
	 */
	public JInfoDialog(final PedJDialog parentFrame, final String title, final String message, final int messageType, final int optionType) {
		this(parentFrame);
		setTitle(title);
		setOptionPaneParams(message, messageType);
		setOptionType(optionType);
		finishLayout();
	}

	@Override
	public void setVisible(final boolean isVisible) {
		if (isVisible) {
			final JButton button = getDefaultButton();
			if (null != button) {
				button.requestFocusInWindow(); 
			}
		}
		super.setVisible(isVisible);
	}
}
