package de.ped.tools.gui;

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.AbstractCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

import de.ped.tools.Idable;
import de.ped.tools.Messages;
import de.ped.tools.PlayerColor;
import de.ped.tools.log.Logger;

/**
 * Allows the user to edit (choose) the player color.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlayerColorEditor extends AbstractCellEditor implements TableCellEditor, ItemListener {
	private static final long serialVersionUID = 1L;

	private final Logger logger = Logger.getLogger(getClass());
	private final JComboBox<Integer> comboBox = new JComboBox<Integer>();
	
	public PlayerColorEditor(final Messages msg) {
		comboBox.setRenderer(new PlayerColorRenderer(msg));
		for (int i = 0; i < PlayerColor.size(); i++) {
			comboBox.addItem(new Integer(i));
		}
//		comboBox.addItemListener(this);
	}

	@Override
	public Object getCellEditorValue() {
		Object ret = null;
		if (null != comboBox) {
			ret = Integer.valueOf(comboBox.getSelectedIndex());
		}
		return ret;
	}

	@Override
	public Component getTableCellEditorComponent(
			final JTable table, 
			final Object value,
			final boolean isSelected, 
			final int row, 
			final int column) {
		final Idable val = (Idable) value;
		comboBox.setSelectedIndex(val.getId());
		comboBox.addItemListener(this);
		return comboBox;
	}

	@Override
	public void itemStateChanged(final ItemEvent event) {
		if (event.getStateChange() == ItemEvent.SELECTED) {
			logger.debug("item selected: " + event);
			stopCellEditing();
		}
	}
}
