package de.ped.tools.gui;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;

import de.ped.tools.Messages;
import de.ped.tools.PlayerColor;

/**
 * Knows how to show a player color to the user inside a list (e.g., combobox) or a table.
 * 
 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlayerColorRenderer extends JLabel implements TableCellRenderer, ListCellRenderer<Integer> {
	private static final long serialVersionUID = 1L;

//	private static final int BORDER_HORIZONTAL = 0;
//	private static final int BORDER_VERTICAL = 0;
//
//	private Border unselectedBorder = null;
//	private Border selectedBorder = null;
//	private final boolean isBordered = true;

	private final Messages messages;

	public PlayerColorRenderer(final Messages msg) {
		this.messages = msg;
		setOpaque(true); // MUST do this for background to show up.
	}

	@Override
	public Component getTableCellRendererComponent(
			final JTable table, 
			final Object value,
			final boolean isSelected, 
			final boolean hasFocus,
			final int row, 
			final int column) {
		return getRendererComponent(value);
	}

	@Override
	public Component getListCellRendererComponent(
			final JList<? extends Integer> list, 
			final Integer value,
			final int index, 
			final boolean isSelected, 
			final boolean cellHasFocus) {
		return getRendererComponent(value);
	}

	private Component getRendererComponent(Object value) {
		if (value instanceof Integer) {
			value = PlayerColor.get((((Integer) value).intValue()));
		}
		final PlayerColor playerColor = (PlayerColor) value;
//		if (isBordered) {
//			if (isSelected) {
//				if (selectedBorder == null) {
//					selectedBorder = BorderFactory.createMatteBorder(BORDER_VERTICAL, BORDER_HORIZONTAL,
//							BORDER_VERTICAL, BORDER_HORIZONTAL, table.getSelectionBackground());
//				}
//				setBorder(selectedBorder);
//			} else {
//				if (unselectedBorder == null) {
//					unselectedBorder = BorderFactory.createMatteBorder(BORDER_VERTICAL, BORDER_HORIZONTAL,
//							BORDER_VERTICAL, BORDER_HORIZONTAL, table.getBackground());
//				}
//				setBorder(unselectedBorder);
//			}
//		}
		setBackground(playerColor.getColor());
		setForeground(playerColor.getBackground());
		setHorizontalAlignment(CENTER);
		setVerticalAlignment(CENTER);
		setText(messages.getText(playerColor.getKey()));
//		setToolTipText("RGB value: " + playerColor.getColor().getRed() + ", " + playerColor.getColor().getGreen() + ", "
//				+ playerColor.getColor().getBlue());
		return this;
	}
}
