package de.ped.tools.gui;

import java.awt.Component;

import de.ped.tools.ResourceFinder;

/**
 * Helps to uniformly set up property dialogs containing label/component pairs.
 * Implements the composite pattern to represent dialogs with nested structures like panels.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PropertyElement {

	/**
	 * Creates a property element.
	 * 
	 * @param key
	 *            the I18N key for the label for the property
	 * @param component
	 *            the component to edit the property
	 */
	public PropertyElement(final String key, final Component component) {
		this.key = key;
		this.component = component;
	}

	/**
	 * Creates a property element.
	 * 
	 * @param key
	 *            the I18N key for the label for the property
	 * @param component
	 *            the component to edit the property
	 * @param childs
	 *            the children of that property, or <code>null</code> if there are none
	 */
	public PropertyElement(final String key, final Component component, final PropertyElement[] childs) {
		this.key = key;
		this.component = component;
		this.childs = childs;
	}

	/**
	 * Creates a property element.
	 * 
	 * @param key
	 *            the I18N key for the label for the property
	 * @param component
	 *            the component to edit the property
	 * @param childs
	 *            the children of that property, or <code>null</code> if there are none
	 * @param imageKey the image file name (without file type postfix), or <code>null</code> to simply use the key attribute
	 * @param folder the resource folder where to find the image icon file, or <code>null</code> for no image
	 */
	public PropertyElement(final String key, final Component component, final PropertyElement[] childs, final String imageKey, final ResourceFinder.Folder folder) {
		this.key = key;
		this.component = component;
		this.childs = childs;
		this.imageKey = imageKey;
		this.folder = folder;
	}

	/** 
	 * @return the I18N key for the label for the property
	 */
	public String getKey() {
		return key;
	}

	/** 
	 * @return the component to edit the property 
	 */
	public Component getComponent() {
		return component;
	}

	public ResourceFinder.Folder getResourceFolder() {
		return folder;
	}

	public String getImageKey() {
		return (null == imageKey ? key : imageKey);
	}

	public PropertyElement[] getChilds() {
		return childs;
	}

	public void setChilds(final PropertyElement[] childs) {
		this.childs = childs;
	}

	/** The I18N key for the label for the property. */
	private String key;

	/** The component to edit the property */
	private Component component;

	/** Building a tree structure */
	private PropertyElement[] childs;

	private ResourceFinder.Folder folder;
	private String imageKey;
}
