package de.ped.tools.gui;

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.AbstractCellEditor;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

import de.ped.tools.Idable;
import de.ped.tools.Messages;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Knows how to edit a combo box inside a table.
 * 
 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
 */
public abstract class TableCellComboBoxEditor extends AbstractCellEditor implements TableCellEditor, ItemListener {
	private static final long serialVersionUID = 1L;

	public TableCellComboBoxEditor(final Messages messages) {
		this.messages = messages;
		logger.setLogLevel(Level.INFO);
	}

	@Override
	public Object getCellEditorValue() {
		Object ret = null;
		if (null != comboBox) {
			ret = comboBox.getSelectedItem();
		}
		return ret;
	}

	@Override
	public Component getTableCellEditorComponent(
			final JTable table, 
			final Object value,
			final boolean isSelected, 
			final int row, 
			final int column) {
		final Idable item = (Idable) value;
		comboBox = createRenderer();
		comboBox.setSelectedIndex(item.getId());
		comboBox.addItemListener(this);
		return comboBox;
	}

	/** 
	 * Overload to enable the standard behavior of {@link #getTableCellEditorComponent} of just using the renderer as editor.
	 * @return the renderer object fitting to this editor
	 */
	protected TableCellComboBoxRenderer createRenderer() {
		return null;
	}

	@Override
	public void itemStateChanged(final ItemEvent event) {
		if (event.getStateChange() == ItemEvent.SELECTED) {
			logger.debug("item selected: " + event);
			stopCellEditing();
		}
	}

	protected final Messages messages;
	protected TableCellComboBoxRenderer comboBox = null;
	private final Logger logger = Logger.getLogger(getClass());
}
