package de.ped.tools.gui;

import java.awt.Component;

import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;

import de.ped.tools.Idable;
import de.ped.tools.LocalizedString;

/**
 * A renderer for a combo box within a table cell.
 * 
 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TableCellComboBoxRenderer extends JIconComboBox<Object> implements TableCellRenderer, ListCellRenderer<Object> {
	private static final long serialVersionUID = 1L;

	public static int getIdxOf(final Object value) {
		int id = 0;
		if (value instanceof Idable) {
			id = ((Idable) value).getId();
		} else {
			id = ((Integer) value).intValue();
		}
		return id;
	}

	/**
	 * One item in a table cell combo box. Capable of keeping an index, and showing an i18n'ed string.
	 * 
	 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class TableCellComboBoxItem implements Idable {
		public final Object str;
		public final Idable obj;

		/**
		 * Constructor
		 * @param str The object that defines the text shown in the combo box. 
		 * 		If of type {@link LocalizedString}, additional functionality in help text creation is given.
		 * @param obj The object identifying the underlying data that is selected.
		 */
		public TableCellComboBoxItem(final Object str, final Idable obj) {
			this.str = str;
			this.obj = obj;
		}

		@Override
		public String toString() {
			final String ret;
			if (null == str) {
				if (null == obj) {
					ret = null;
				} else {
					ret = obj.toString();
				}
			} else {
				ret = str.toString();
			}
			return ret;
		}

		public int getId() {
			return obj.getId();
		}
	}

	@Override
	public Component getTableCellRendererComponent(
			final JTable table, 
			final Object value,
			final boolean isSelected, 
			final boolean hasFocus,
			final int row, 
			final int column) {
		final Component ret = getRendererComponent(value);
		ret.setEnabled(table.getModel().isCellEditable(row, column)); 
		return ret;
	}

	@Override
	public Component getListCellRendererComponent(
			final JList<?> list, 
			final Object value,
			final int index, 
			final boolean isSelected, 
			final boolean cellHasFocus) {
		return getRendererComponent(value);
	}

	protected Component getRendererComponent(Object value) {
		setSelectedIndex(getIdxOf(value));
		return this;
	}
}
