package de.ped.tools.gui;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import de.ped.tools.Assert;
import de.ped.tools.ResourceFinder;

/**
 * A self-defined table header renderer. Necessary to allow tool tips.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TableHeaderRenderer implements TableCellRenderer {
	final DefaultTableCellRenderer renderer;
	final TableModel model;
	final ResourceFinder finder;
	final int col;
	final JLabel iconLabel;

	public TableHeaderRenderer(final JTable table, final TableModel model, final ResourceFinder finder, final int col) {
		this.model = model;
		this.renderer = (DefaultTableCellRenderer) table.getTableHeader().getDefaultRenderer();
		this.finder = finder;
		this.col = col;
		final TableColumnModel tcm = model.getColumns()[col];
		if ((null != finder) && (null != tcm.folder)) {
			iconLabel = new JLabel();
			final java.net.URL url = finder.getURLToResource(tcm.imageFilename + ".png", tcm.folder);
			iconLabel.setIcon(new ImageIcon(url));
			iconLabel.setBorder(renderer.getBorder());
			iconLabel.setHorizontalAlignment(JLabel.CENTER);
		} else {
			iconLabel = null;
		}
	}

	@Override
	public Component getTableCellRendererComponent(
			final JTable table, 
			final Object value, 
			final boolean isSelected,
			final boolean hasFocus, 
			final int row, 
			final int col) {
		Assert.assertTrue(this.col == col);
		renderer.setHorizontalAlignment(JLabel.CENTER);
		renderer.setToolTipText(model.getColumnToolTip(col));
		final Component center = renderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
		final Component ret;
		if (null == iconLabel) {
			ret = center;
		} else {
			final JPanel panel = new JPanel();
			panel.setLayout(new BorderLayout());
			panel.add(center, BorderLayout.CENTER);
			panel.add(iconLabel, BorderLayout.NORTH);
			ret = panel;
		}
		return ret;
	}
}