package de.ped.tools.gui;

import java.awt.Component;
import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;

/**
 * Knows how to show a row number to the user inside a table. It ignores any selection and always displays more it like a table heading.
 * 
 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TableRowNumberRenderer extends JLabel implements TableCellRenderer, ListCellRenderer<Object> {
	private static final long serialVersionUID = 1L;

	@Override
	public Component getTableCellRendererComponent(
			final JTable table, 
			final Object value,
			final boolean isSelected, 
			final boolean hasFocus,
			final int row, 
			final int column) {
		return getRendererComponent(value, row);
	}

	@Override
	public Component getListCellRendererComponent(
			final JList<?> list, 
			final Object value,
			final int index, 
			final boolean isSelected, 
			final boolean cellHasFocus) {
		return getRendererComponent(value, index);
	}

	private Component getRendererComponent(final Object value, final int row) {
		setOpaque(true); // MUST do this for background to show up.
		setFont(getFont().deriveFont(Font.PLAIN));
		setHorizontalAlignment(CENTER);
		setVerticalAlignment(CENTER);
		final int val;
		if (value instanceof Integer) {
			val =  (Integer) value;
		} else {
			val = row;
		}
		setText(Integer.toString(val));
		return this;
	}
}
