package de.ped.tools.gui;

import java.awt.Component;
import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;

/**
 * Knows how to show a string inside a table. It ignores any selection, but
 * takes care of disabling if the cell is not editable according to table model.
 * 
 * @author Copyright (c) 2021 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TableStringRenderer extends JLabel implements TableCellRenderer, ListCellRenderer<String> {
	private static final long serialVersionUID = 1L;

	@Override
	public Component getTableCellRendererComponent(
			final JTable table, 
			final Object value,
			final boolean isSelected, 
			final boolean hasFocus,
			final int row, 
			final int column) {
		final Component ret = getRendererComponent(value);
		ret.setEnabled(table.getModel().isCellEditable(row, column)); 
		return ret;
	}

	@Override
	public Component getListCellRendererComponent(
			final JList<? extends String> list, 
			final String value,
			final int index, 
			final boolean isSelected, 
			final boolean cellHasFocus) {
		return getRendererComponent(value);
	}

	private Component getRendererComponent(final Object value) {
		setFont(getFont().deriveFont(Font.PLAIN));
		setText((String) value);
		return this;
	}
}
