package de.ped.tools.gui;

import javax.swing.JTable;
import javax.swing.table.TableColumn;

import de.ped.tools.ResourceFinder;

/**
 * Tools for handling tables with I18N support.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TableTools {

	/**
	 * Adjusts for the table all columns by changing the header renderer to support
	 * tool tips, and adjusts the column width according to the column model.
	 * 
	 * @param table   the table to adjust
	 * @param stretch the stretch factor all columns are stretched with
	 * @param finder  the resource finder to lead to image resources, or
	 *                <em>null</em> to not look for images
	 */
	public static void adjustColumns(final JTable table, final int stretch, final ResourceFinder finder) {
		if (table.getModel() instanceof TableModel) {
			TableModel tableModel = (TableModel) table.getModel();
			for (int i = 0; i < tableModel.getColumnCount(); i++) {
				final TableColumnModel col = tableModel.getColumns()[i];
				final TableColumn c = table.getColumnModel().getColumn(i);
				c.setHeaderRenderer(new TableHeaderRenderer(table, tableModel, finder, i));
				c.setPreferredWidth(col.preferredWidth * stretch);
				c.setWidth(c.getPreferredWidth());
			}
		}
	}
}
