package de.ped.tools.gui;

import javax.swing.KeyStroke;

import de.ped.tools.Messages;
import de.ped.tools.log.Logger;

/**
 * Defines a user keystroke input that can be interpreted as a command for the pilots. 
 * Please use only for simple keyboard-only inputs; in most cases defining a {@link PedAction} is more mighty.
 * 
 * @author Copyright (c) 2001 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class UserGameActionInput {
	private static Logger logger = Logger.getRootLogger();

	/**
	 * Finds the user game action input object for a specified player and
	 * action.
	 * 
	 * @param userGameActionInputs
	 *            the array to search
	 * @param playerIdx
	 *            the player that is controlled by that action
	 * @param playerAction
	 *            the ID of the action of the user
	 * @return the user game action input object, or <code>null</code> if none
	 *         found
	 */
	public static UserGameActionInput find(
			final UserGameActionInput[] userGameActionInputs, 
			final int playerIdx, 
			final int playerAction) {
		for (final UserGameActionInput ugai : userGameActionInputs) {
			if ((playerIdx == ugai.playerIdx) && (playerAction == ugai.playerAction)) {
				return ugai;
			}
		}
		return null;
	}

	public final int keyCode;
	public final int playerIdx;
	public final int playerAction;
	public final String keyCodeString;

	/**
	 * Creates a user game action input using I18N options.
	 * 
	 * @param messages
	 *            the messages object for i18n
	 * @param keyCodeKey
	 *            the message key that defines the code that will start the
	 *            action
	 * @param defaultKeyCode
	 *            the key code to use if no message key (or an invalid) found
	 * @param playerIdx
	 *            the player that is controlled by that action
	 * @param playerAction
	 *            the ID of the action of the user
	 */
	public UserGameActionInput(
			final Messages messages, 
			final String keyCodeKey, 
			final int defaultKeyCode, 
			final int playerIdx, 
			final int playerAction) {
		keyCodeString = messages.getString(keyCodeKey);
		final KeyStroke keyStroke = KeyStroke.getKeyStroke(keyCodeString);
		if (null == keyStroke) {
			keyCode = defaultKeyCode;
			logger.info("Using default key code instead of " + keyCodeString + " for " + keyCodeKey);
		} else {
			keyCode = keyStroke.getKeyCode();
		}
		this.playerIdx = playerIdx;
		this.playerAction = playerAction;
	}
}
