package de.ped.tools.gui;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * The class defining a {@link Wizard} model.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de), 
 * 		based on an <a href="http://www.oracle.com/technetwork/articles/javase/wizard-136789.html">article</a> 
 * 		by Robert Eckstein, February 10, 2005 
 */
public class WizardModel {
	public void registerPanel(final WizardPanelDescriptor panel) {
		panels.put(panel.getPanelId(), panel);
		ids.add(panel.getPanelId());
	}

	/**
	 * @param id the panel ID to look for
	 * @return a specific panel descriptor for the panel ID
	 */
	public WizardPanelDescriptor getPanelDescriptor(final Object id) {
		return panels.get(id);
	}

	/**
	 * @return a specific order of panel IDs in order of registration. Useful e.g. for help text creation.
	 */
	public ArrayList<Object> getPanelDescriptorIds() {
		return ids;
	}

	public void setCurrentPanelId(final Object id) {
		if (panels.containsKey(id)) {
			currentId = id;
		}
	}

	public WizardPanelDescriptor getCurrentPanelDescriptor() {
		return panels.get(currentId);
	}

	private HashMap<Object, WizardPanelDescriptor> panels = new HashMap<Object, WizardPanelDescriptor>();
	private ArrayList<Object> ids = new ArrayList<Object>();
	private Object currentId;
}
