package de.ped.tools.log;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Defines a new logging drain for any print writer.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PrintWriterAppender extends AbstractAppender {

	public PrintWriterAppender(final PrintWriter writer) {
		this(writer, null);
	}

	public PrintWriterAppender(final PrintWriter writer, final String name) {
		super(name);
		this.writer = writer;
	}

	/**
	 * Creates an appender that writes to a file.
	 * @param file the file to write into
	 * @throws IOException any problem in creating or opening the file to write into
	 */
	public PrintWriterAppender(final File file) throws IOException {
		this(new PrintWriter(new FileWriter(file)), file.getName());
	}

	@Override
	public synchronized void println(final String message) {
		writer.println(message);
	}

	@Override
	public synchronized void printStackTrace(final Throwable ex) {
		ex.printStackTrace(writer);
	}

	@Override
	public synchronized void flush() {
		writer.flush();
	}

	private final PrintWriter writer;
}
