package de.ped.tools.log;

import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * Defines a new logging drain for writing into a string.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class StringWriterAppender extends AbstractAppender {

	public StringWriterAppender() {
		this(null);
	}

	public StringWriterAppender(final String name) {
		super(name);
		clearContent();
	}

	public synchronized void clearContent() {
		content = new StringWriter();
		writer = new PrintWriter(content);
		appender = new PrintWriterAppender(writer);
	}

	public synchronized String getContent() {
		return content.toString();
	}

	@Override
	public void println(final String message) {
		appender.println(message);
	}

	@Override
	public void printStackTrace(final Throwable ex) {
		appender.printStackTrace(ex);
	}

	@Override
	public void flush() {
		appender.flush();
	}

	private StringWriter content;
	private PrintWriter writer;
	private PrintWriterAppender appender;
}
