package de.ped.tools.sound;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;

/**
 * Concats (sequencially) some wave objects.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ConcatWave extends AbstractWave {

	/**
	 * A wave to concat to the concatenated wave.
	 * 
	 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class Part {
		public Part(final MonoWave wave, final double duration) {
			this.wave = wave;
			this.duration = duration;
		}

		@Override
		public String toString() {
			final StringBuffer ret = new StringBuffer();
			ret.append("(dur=");
			ret.append(duration);
			ret.append(",");
			ret.append(wave);
			ret.append(")");
			return ret.toString();
		}

		public final MonoWave wave;
		public final double duration;
	}

	public ConcatWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final MonoWave mw1, 
			final double duration1, 
			final MonoWave mw2, 
			final double duration2) {
		this(smwc, new Part[]{ new Part(mw1, duration1), new Part(mw2, duration2) } );
	}

	public ConcatWave(final DynamicSoundMonoWaveCreator smwc, final Part[] parts) {
		this(new AbstractWave.Params(smwc), Arrays.asList(parts));
	}

	public ConcatWave(final DynamicSoundMonoWaveCreator smwc, final Collection<Part> parts) {
		this(new AbstractWave.Params(smwc), parts);
	}

	public ConcatWave(final AbstractWave.Params params, final Collection<Part> parts) {
		super(params);
		this.parts.addAll(parts);
	}

	@Override
	protected double calcCurrentValue() {
		double ret = 0.0;
		final int timeslice = getTimeslice();
		int partStartTimeslice = 0;
		int partLength = 0;
		for (final Part part : parts) {
			final double duration = part.duration;
			partLength = getSoundSteroWaveCreator().durationToSize(duration);
			if (partStartTimeslice + partLength >= timeslice) {
				ret = part.wave.get();
//				logger.debug("Envelope part " + i + ", timeslice " + timeslice + " gets " + ret);
				break;
			} else {
				partStartTimeslice += partLength;
			}
		}
		return ret;
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("concat(");
		for (final Part part : parts) {
			ret.append(part);
		}
		ret.append(")");
		return ret.toString();
	}

	LinkedList<Part> parts = new LinkedList<Part>();
}
