package de.ped.tools.sound;

/**
 * The a mono wave that always returns the same constant value. In itself it's a
 * silent signal, but useful as building block for modulations.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ConstantWave implements MonoWave {
	/** 
	 * Creates a mono wave with constant value 0.0. 
	 */
	public ConstantWave() {
		// Nothing to do
	}

	/**
	 * Creates a mono wave with given constant value.
	 * 
	 * @param value
	 *            the constant value
	 */
	public ConstantWave(final double value) {
		this.value = value;
	}

	@Override
	public double next() { 
		return get();
	}

	@Override
	public double get() {
		return value;
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("const(");
		ret.append(value);
		ret.append(")");
		return ret.toString();
	}

	private double value = 0.0;
}