package de.ped.tools.sound;

import de.ped.tools.MathUtil;

/**
 * Modulate the amplitude of a main signal by cutting any value that is louder
 * than the hull signal to the hull signal.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class CutAmplitudeWave extends AbstractWave  {
	/**
	 * Creates a mono wave object that cuts values louder than the hull signal
	 * from the main signal.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param mainSignal
	 *            the main signal that will be amplitude modulated
	 * @param hullSignal
	 *            the modulator that is applied to the main signal
	 */
	public CutAmplitudeWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final MonoWave mainSignal, 
			final MonoWave hullSignal) {
		super(smwc, 0.0);
		this.mainSignal = mainSignal;
		this.hullSignal = hullSignal;
	}

	/**
	 * Creates a mono wave object that cuts values louder than the hull signal
	 * from the main signal.
	 * 
	 * @param params
	 *            the wave parameters
	 * @param mainSignal
	 *            the main signal that will be amplitude modulated
	 * @param hullSignal
	 *            the modulator that is applied to the main signal
	 */
	public CutAmplitudeWave(
			final AbstractWave.Params params, 
			final MonoWave mainSignal, 
			final MonoWave hullSignal) {
		super(params);
		this.mainSignal = mainSignal;
		this.hullSignal = hullSignal;
	}

	@Override
	protected double calcCurrentValue() {
		final double maxValue = Math.abs(hullSignal.get());
		return MathUtil.bound(mainSignal.get(), -maxValue, maxValue);
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("cut(");
		ret.append(mainSignal);
		ret.append(",");
		ret.append(hullSignal);
		ret.append(")");
		return ret.toString();
	}

	private MonoWave mainSignal;
	private MonoWave hullSignal;
}
