package de.ped.tools.sound;

/**
 * A mono wave is defined by a wave value for each point in time. The mono wave
 * only needs to know its current state and have a way to get to the next
 * state, so in this sense you can think of it like an iterator through the
 * wave.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public interface MonoWave {
	/**
	 * Switches to the next time slice and returns that value.
	 * 
	 * @return the value for the next time slice
	 */
	public double next();

	/**
	 * Returns the wave value of the current time slice.
	 * 
	 * @return the wave value of the current time slice.
	 */
	public double get();
}