package de.ped.tools.sound;

import java.util.ArrayList;
import java.util.List;

/**
 * The a mono wave that results from the multiplication of each value of other
 * mono waves.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class MultipliedWave extends AbstractWave {
	/**
	 * Creates a mono wave that multiplies the given multiplicands.
	 * 
	 * @param params
	 *            the abstract wave parameters
	 * @param multiplicands
	 *            the mono waves to multiply
	 */
	public MultipliedWave(final AbstractWave.Params params, final List<MonoWave> multiplicands) {
		super(params);
		this.multiplicands = multiplicands;
	}

	/**
	 * Creates a mono wave that multiplies the given multiplicands.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param multiplicands
	 *            the mono waves to multiply
	 */
	public MultipliedWave(final DynamicSoundMonoWaveCreator smwc, final List<MonoWave> multiplicands) {
		this(new AbstractWave.Params(smwc), multiplicands);
	}

	/**
	 * Creates a mono wave that multiplies the given two multiplicands.
	 * 
	 * @param params
	 *            the abstract wave parameters
	 * @param m1
	 *            the first mono wave
	 * @param m2
	 *            the second mono wave
	 */
	public MultipliedWave(final AbstractWave.Params params, final MonoWave m1, final MonoWave m2) {
		super(params);
		multiplicands = new ArrayList<MonoWave>(2);
		multiplicands.add(m1);
		multiplicands.add(m2);
	}

	/**
	 * Creates a mono wave that multiplies the given two multiplicands.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param m1
	 *            the first mono wave
	 * @param m2
	 *            the second mono wave
	 */
	public MultipliedWave(final DynamicSoundMonoWaveCreator smwc, final MonoWave m1, final MonoWave m2) {
		this(new AbstractWave.Params(smwc), m1, m2);
	}

	@Override
	protected double calcCurrentValue() {
		double ret = 1.0;
		if (null != multiplicands) {
			for (final MonoWave wave : multiplicands) {
				ret *= wave.get();
			}
		}
		return ret;
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("mult(");
		for (MonoWave wave : multiplicands) {
			ret.append(wave);
			ret.append(",");
		}
		ret.append(")");
		return ret.toString();
	}

	List<MonoWave> multiplicands; 
}
