package de.ped.tools.sound;

import de.ped.tools.MathUtil;

/**
 * Creates a random noise sample of amplitude 1.0.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PinkNoiseWave extends AbstractWave {
	/**
	 * Creates a random mono wave (with saw blades leading down) without
	 * frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param fmWave
	 *            the modulator that is applied to the main signal. A value of
	 *            1.0 is nearly white noise, smaller values cut off higher
	 *            frequencies and make the sound deeper.
	 */
	public PinkNoiseWave(final DynamicSoundMonoWaveCreator smwc, final MonoWave fmWave) {
		super(smwc, 0.0);
		this.fmWave = fmWave;
	}

	@Override
	protected double calcCurrentValue() {
		final double maxIncrease = fmWave.get();
		return MathUtil.modulo(get() - maxIncrease + 2.0 * maxIncrease * Math.random(), 2.0, -1.0);
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("pink(");
		ret.append(fmWave);
		ret.append(")");
		return ret.toString();
	}

	protected MonoWave fmWave;
}
