package de.ped.tools.sound;

/**
 * Defines a saw wave (with saw blades leading down).
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class SawDownWave extends RepeatingWave {
	/**
	 * Creates a saw mono wave (with saw blades leading down) of amplitude 1
	 * without frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 */
	public SawDownWave(final DynamicSoundMonoWaveCreator smwc, final double normalFrequency) {
		super(smwc, normalFrequency);
	}

	/**
	 * Creates a saw mono wave (with saw blades leading down) of amplitude 1
	 * with frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 * @param frequencyModulator
	 *            the wave the frequency is modulated with
	 */
	public SawDownWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final double normalFrequency, 
			final MonoWave frequencyModulator) {
		super(smwc, normalFrequency, frequencyModulator);
	}

	public SawDownWave(final RepeatingWave.Params params) {
		super(params);
	}

	@Override
	protected double calcCurrentValue() {
		return (1.0 - 2.0 * getPhase()) * getParams().amplitudeModulator.get();
	}

	@Override
	public String toString() {
		StringBuffer ret = new StringBuffer();
		ret.append("sawDn(f=");
		ret.append(getNormalFrequency());
		ret.append(",fm=");
		ret.append(getFrequencyModulator());
		ret.append(")");
		return ret.toString();
	}
}