package de.ped.tools.sound;

/**
 * Defines a sinus wave.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class SinusWave extends RepeatingWave {
	/**
	 * Creates a sinus wave of amplitude 1 without frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 */
	public SinusWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final double normalFrequency) {
		super(smwc, normalFrequency);
	}

	/**
	 * Creates a sinus wave of amplitude 1 with frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 * @param frequencyModulator
	 *            the wave the frequency is modulated with
	 */
	public SinusWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final double normalFrequency, 
			final MonoWave frequencyModulator) {
		super(smwc, normalFrequency, frequencyModulator);
	}

	public SinusWave(final RepeatingWave.Params params) {
		super(params);
	}

	@Override
	protected double calcCurrentValue() {
		return (Math.sin(getPhase() * 2.0 * Math.PI)) * getParams().amplitudeModulator.get();
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("sin(f=");
		ret.append(getNormalFrequency());
		ret.append(",fm=");
		ret.append(getFrequencyModulator());
		ret.append(")");
		return ret.toString();
	}
}