package de.ped.tools.sound;

import de.ped.tools.MathUtil;

/**
 * Defines a square wave.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class SquareWave extends RepeatingWave {
	/**
	 * Creates a square wave of amplitude 1 without frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 */
	public SquareWave(final DynamicSoundMonoWaveCreator smwc, final double normalFrequency) {
		super(smwc, normalFrequency);
	}

	/**
	 * Creates a square wave of amplitude 1 with frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 * @param frequencyModulator
	 *            the wave the frequency is modulated with
	 */
	public SquareWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final double normalFrequency, 
			final MonoWave frequencyModulator) {
		super(smwc, normalFrequency, frequencyModulator);
	}

	public SquareWave(final RepeatingWave.Params params) {
		super(params);
	}
	
	@Override
	protected double calcCurrentValue() {
		final double normedPhase = MathUtil.modulo(getPhase(), 1.0, 0.0);
		return (normedPhase < 0.5 ? -1.0 : 1.0) * getParams().amplitudeModulator.get();
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("sq(f=");
		ret.append(getNormalFrequency());
		ret.append(",fm=");
		ret.append(getFrequencyModulator());
		ret.append(")");
		return ret.toString();
	}
}
