package de.ped.tools.sound;

import de.ped.tools.MathUtil;

/**
 * Defines a triangle wave.
 * 
 * @author Copyright (c) 2009 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TriangleWave extends RepeatingWave {
	/**
	 * Creates a triangle wave of amplitude 1 without frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 */
	public TriangleWave(final DynamicSoundMonoWaveCreator smwc, final double normalFrequency) {
		super(smwc, normalFrequency);
	}

	/**
	 * Creates a triangle wave of amplitude 1 with frequency modulation.
	 * 
	 * @param smwc
	 *            the SoundStereoWaveCreator object this wave belongs to
	 * @param normalFrequency
	 *            the normal, unmodulated frequency of the signal created
	 * @param frequencyModulator
	 *            the wave the frequency is modulated with
	 */
	public TriangleWave(
			final DynamicSoundMonoWaveCreator smwc, 
			final double normalFrequency, 
			final MonoWave frequencyModulator) {
		super(smwc, normalFrequency, frequencyModulator);
	}

	public TriangleWave(final RepeatingWave.Params params) {
		super(params);
	}

	@Override
	protected double calcCurrentValue() {
		final double normedPhase = MathUtil.modulo(getPhase(), 1.0, 0.0);
		double ret = 0.0;
		if (normedPhase < 0.5) {
			ret = -1.0 + 4.0 * normedPhase;
		} else {
			ret =  3.0 - 4.0 * normedPhase; // 0.5 --> 1; 1 --> -1
		}
		return ret * getParams().amplitudeModulator.get();
	}

	@Override
	public String toString() {
		final StringBuffer ret = new StringBuffer();
		ret.append("tri(f=");
		ret.append(getNormalFrequency());
		ret.append(",fm=");
		ret.append(getFrequencyModulator());
		ret.append(")");
		return ret.toString();
	}
}