package de.ped.tools.test;

import static org.junit.Assert.*;

import java.io.IOException;
import java.util.TreeSet;

import org.junit.Test;

import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextDocumentType;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.log.Logger;

/**
 * Unit tests for the {@link ApplicationMainWindow} class.
 * 
 * @author Copyright (c) 2010 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ApplicationMainWindowTest {
	public static final String DEST_PATH = "bin";

	public static void testAllDocTypes(final ApplicationMainWindow window, final String fileNameInfix, TextDocumentType[] types) throws IOException {
		for (TextDocumentType type : types) {
			testDocType(window, type, DEST_PATH, fileNameInfix);
		}
		testConsistency(window);
	}

	public static void testDocType(final ApplicationMainWindow window, final TextDocumentType type, final String folder, final String fileNameInfix) throws IOException {
		window.buildHtmlHelp(type, false, folder, fileNameInfix);
	}

	public static void testConsistency(final ApplicationMainWindow window) {
		final TextDocumentBuilder doc = new TextDocumentBuilder(window.getMessages());
		doc.setTextDocumentType(TextDocumentType.HTML5);
		doc.setEmbedded(false);
		window.buildHelpDocument(doc);
		assertEquals("UTF-8", doc.getCharset().name());
		final String result = doc.result(null);
		TreeSet<String> ids = new TreeSet<String>();
		final String IDP = "id=\"";
		int spos = -1;
		do {
			spos = result.indexOf(IDP, spos + 1);
			if (0 <= spos) {
				int epos = result.indexOf("\"", spos + IDP.length());
				final String id = result.substring(spos + IDP.length(), epos);
				assertFalse("Non-unique ID " + id, ids.contains(id));
				ids.add(id);
			}
		} while (0 <= spos);
		Logger logger = Logger.getRootLogger();
		for (String s : ids) {
			logger.trace(s);
		}
		final String HREFP = "href=\"#";
		spos = -1;
		do {
			spos = result.indexOf(HREFP, spos + 1);
			if (0 <= spos) {
				int epos = result.indexOf("\"", spos + HREFP.length());
				final String id = result.substring(spos + HREFP.length(), epos);
				assertTrue("Local dangling href " + id, ids.contains(id));
			}
		} while (0 <= spos);
	}

	@Test
	public void test() {
		// Does nothing so far
		assertTrue(true);
	}
}
