package de.ped.tools.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationOrApplet;

/**
 * Unit tests for the {@link ApplicationOrApplet} class.
 * 
 * @author Copyright (c) 2010 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ApplicationOrAppletTest {

	/**
	 * Stub concrete class for the {@link ApplicationOrApplet} class.
	 * 
	 * @author Copyright (c) 2010 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class ApplicationOrAppletStub extends ApplicationOrApplet {
		private static final long serialVersionUID = 1L;

		@Override
		protected Messages createMessages() {
			return null;
		}
	}

	/**
	 * @see ApplicationOrApplet#getParameterInfo()
	 * @see ApplicationOrApplet#init()
	 * @see ApplicationOrApplet#start()
	 * @see ApplicationOrApplet#stop()
	 * @see ApplicationOrApplet#destroy()
	 */
	@Test
	public void testAppletFunctionality() {
		final ApplicationOrAppletStub stub = new ApplicationOrAppletStub();
		assertArrayEquals(new String[][] { 
			{"label", "String", "Label string to be displayed"}, 
			{"background", "String", "Label background color, format \"rrggbb\" in hex notation"}, 
			{"foreground", "String", "Label foreground color, format \"rrggbb\" in hex notation"}, 
			{"language", "String", "Currently most of my apps support \"de_DE\" or \"en_US\""}, 
			{"silent", "String", "If not \"no\", will start without opening a visible window - used for testing purposes"}, 
			{"htmlhelp", "String", "If not empty, will write help web page to this path (use \".\" for current directory)"}, 
			{"resources", "String", "If not \"no\", will create some resource files"},  }, 
				stub.getParameterInfo());
		stub.init();
		stub.start();
		stub.stop();
		stub.destroy();
	}
}
