package de.ped.tools.test;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import de.ped.tools.Assert;
import de.ped.tools.AssertionException;

/**
 * Unit tests for the {@link Assert} and {@link AssertionException} classes.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class AssertTest {

	@Before
	public void setup() {
		Assert.NDEBUG = true;
	}

	/** 
	 * @see AssertionException#AssertionException()
	 * @see AssertionException#AssertionException(String)
	 */
	@Test
	public void testAssertionException() {
		GenericTest.checkObjectBasics(false, new AssertionException());
		final String cause = "Test";
		assertEquals(cause, new AssertionException(cause).getMessage());
	}

	/**
	 * @see Assert#assertTrue(boolean)
	 */
	@Test
	public void testAssertTrue_ok() {
		assertTrue(Assert.assertTrue(true));
	}

	/**
	 * @see Assert#assertTrue(boolean)
	 */
	@Test
	public void testAssertTrue_NDEBUGfalse_ok() {
		Assert.NDEBUG = false;
		try {
			assertTrue(Assert.assertTrue(false));
		} catch (AssertionError ex) {
			// The java assert hit us before we could test further
		}
	}

	/**
	 * @see Assert#assertTrue(boolean)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertTrue_fail() {
		Assert.assertTrue(false);
	}

	/**
	 * @see Assert#assertFalse(boolean)
	 */
	@Test
	public void testAssertFalse_ok() {
		assertTrue(Assert.assertFalse(false));
	}

	/**
	 * @see Assert#assertFalse(boolean)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertFalse_fail() {
		Assert.assertFalse(true);
	}

	/**
	 * @see Assert#assertNull(Object)
	 */
	@Test
	public void testAssertNull_ok() {
		assertTrue(Assert.assertNull(null));
	}

	/**
	 * @see Assert#assertNull(Object)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertNull_fail() {
		Assert.assertNull(this);
	}

	/**
	 * @see Assert#assertNotNull(Object)
	 */
	@Test
	public void testAssertNotNull_ok() {
		assertTrue(Assert.assertNotNull(this));
	}

	/**
	 * @see Assert#assertNotNull(Object)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertNotNull_fail() {
		Assert.assertNotNull(null);
	}

	/**
	 * @see Assert#assertEquals(boolean, boolean)
	 */
	@Test
	public void testAssertEqualsBool_ok() {
		assertTrue(Assert.assertEquals(true, true));
	}

	/**
	 * @see Assert#assertEquals(boolean, boolean)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertEqualsBool_fail() {
		Assert.assertEquals(true, false);
	}

	/**
	 * @see Assert#assertNotEquals(boolean, boolean)
	 */
	@Test
	public void testAssertNotEqualsBool_ok() {
		assertTrue(Assert.assertNotEquals(true, false));
	}

	/**
	 * @see Assert#assertNotEquals(boolean, boolean)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertNotEqualsBool_fail() {
		Assert.assertNotEquals(true, true);
	}

	/**
	 * @see Assert#assertEquals(long, long)
	 */
	@Test
	public void testAssertEqualsLong_ok() {
		assertTrue(Assert.assertEquals(0L, 0L));
	}

	/**
	 * @see Assert#assertEquals(long, long)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertEqualsLong_fail() {
		Assert.assertEquals(0L, 1L);
	}

	/**
	 * @see Assert#assertNotEquals(long, long)
	 */
	@Test
	public void testAssertNotEqualsLong_ok() {
		assertTrue(Assert.assertNotEquals(0L, 1L));
	}

	/**
	 * @see Assert#assertNotEquals(long, long)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertNotEqualsLong_fail() {
		Assert.assertNotEquals(0L, 0L);
	}

	/**
	 * @see Assert#assertEquals(Object, Object)
	 */
	@Test
	public void testAssertEqualsObject_ok() {
		assertTrue(Assert.assertEquals(null, null));
		assertTrue(Assert.assertEquals(Integer.valueOf(0), Integer.valueOf(0)));
	}

	/**
	 * @see Assert#assertEquals(Object, Object)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertEqualsObject_fail() {
		Assert.assertEquals(Integer.valueOf(0), Integer.valueOf(1));
	}

	/**
	 * @see Assert#assertNotEquals(Object, Object)
	 */
	@Test
	public void testAssertNotEqualsObject_ok() {
		assertTrue(Assert.assertNotEquals(Integer.valueOf(0), null));
		assertTrue(Assert.assertNotEquals(null, Integer.valueOf(0)));
		assertTrue(Assert.assertNotEquals(Integer.valueOf(0), Integer.valueOf(1)));
	}

	/**
	 * @see Assert#assertNotEquals(Object, Object)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertNotEqualsObject_null_fail() {
		Assert.assertNotEquals(null, null);
	}

	/**
	 * @see Assert#assertNotEquals(Object, Object)
	 */
	@Test(expected=AssertionError.class)
	public void testAssertNotEqualsObject_fail() {
		Assert.assertNotEquals(Integer.valueOf(0), Integer.valueOf(0));
	}
}
