package de.ped.tools.test;

import static org.junit.Assert.*;

import java.util.Collection;
import java.util.LinkedList;

import org.junit.Test;

import de.ped.tools.CollectionUtil;

/**
 * Unit tests for the {@link CollectionUtil} class.
 * 
 * @author Copyright (c) 2010 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class CollectionUtilTest {

	private void checkNullOrEmpty_String(final boolean isNullOrEmpty, final String s) {
		assertEquals(isNullOrEmpty, CollectionUtil.isNullOrEmpty(s));
	}

	private void checkNullOrEmpty_Collection(final boolean isNullOrEmpty, final Collection<String> s) {
		assertEquals(isNullOrEmpty, CollectionUtil.isNullOrEmpty(s));
	}

	/**
	 * @see CollectionUtil#isNullOrEmpty(String)
	 */
	@Test
	public void testNullOrEmpty_String() {
		checkNullOrEmpty_String(true, null);
		checkNullOrEmpty_String(true, "");
		checkNullOrEmpty_String(false, " ");
	}

	/**
	 * @see CollectionUtil#isNullOrEmpty(Collection)
	 */
	@Test
	public void testNullOrEmpty_Collection() {
		checkNullOrEmpty_Collection(true, null);
		LinkedList<String> c = null;
		checkNullOrEmpty_Collection(true, c);
		c = new LinkedList<String>();
		checkNullOrEmpty_Collection(true, c);
		c.add("");
		checkNullOrEmpty_Collection(false, c);
	}

	public void checkContains(final String[] arr, final String val, final boolean contains, final boolean containsIgnoreCase) {
		assertEquals(contains, CollectionUtil.contains(arr, val));
		assertEquals(containsIgnoreCase, CollectionUtil.containsIgnoreCase(arr, val));
	}

	/**
	 * @see CollectionUtil#contains(Object[], Object)
	 * @see CollectionUtil#containsIgnoreCase(String[], String)
	 */
	@Test
	public void testContains_String() {
		checkContains(null, null, false, false);
		checkContains(null, "", false, false);
		checkContains(new String[]{}, null, false, false);
		checkContains(new String[]{"test"}, null, false, false);
		checkContains(new String[]{"test"}, "egal", false, false);
		checkContains(new String[]{"test"}, "test", true, true);
		checkContains(new String[]{"test"}, "TeSt", false, true);
	}

	/**
	 * @see CollectionUtil#contains(boolean[], boolean)
	 */
	@Test
	public void testContains_boolean() {
		final boolean val1 = false;
		final boolean val2 = true;
		assertEquals(false, CollectionUtil.contains((boolean[]) null, val1));
		assertEquals(false, CollectionUtil.contains(new boolean[]{}, val1));
		assertEquals(false, CollectionUtil.contains(new boolean[]{val2}, val1));
		assertEquals(true, CollectionUtil.contains(new boolean[]{val1}, val1));
	}

	/**
	 * @see CollectionUtil#contains(byte[], byte)
	 */
	@Test
	public void testContains_byte() {
		final byte val1 = (byte) 0;
		final byte val2 = (byte) 1;
		assertEquals(false, CollectionUtil.contains((byte[]) null, val1));
		assertEquals(false, CollectionUtil.contains(new byte[]{}, val1));
		assertEquals(false, CollectionUtil.contains(new byte[]{val2}, val1));
		assertEquals(true, CollectionUtil.contains(new byte[]{val1}, val1));
	}

	/**
	 * @see CollectionUtil#contains(char[], char)
	 */
	@Test
	public void testContains_char() {
		final char val1 = '0';
		final char val2 = '1';
		assertEquals(false, CollectionUtil.contains((char []) null, val1));
		assertEquals(false, CollectionUtil.contains(new char []{}, val1));
		assertEquals(false, CollectionUtil.contains(new char []{val2}, val1));
		assertEquals(true, CollectionUtil.contains(new char []{val1}, val1));
	}

	/**
	 * @see CollectionUtil#contains(int[], int)
	 */
	@Test
	public void testContains_int() {
		final int val1 = 0;
		final int val2 = 1;
		assertEquals(false, CollectionUtil.contains((int[]) null, val1));
		assertEquals(false, CollectionUtil.contains(new int[]{}, val1));
		assertEquals(false, CollectionUtil.contains(new int[]{val2}, val1));
		assertEquals(true, CollectionUtil.contains(new int[]{val1}, val1));
	}

	/**
	 * @see CollectionUtil#contains(long[], long)
	 */
	@Test
	public void testContains_long() {
		final long val1 = 0;
		final long val2 = 1;
		assertEquals(false, CollectionUtil.contains((long[]) null, val1));
		assertEquals(false, CollectionUtil.contains(new long[]{}, val1));
		assertEquals(false, CollectionUtil.contains(new long[]{val2}, val1));
		assertEquals(true, CollectionUtil.contains(new long[]{val1}, val1));
	}

	/**
	 * @see CollectionUtil#indexOfPart(byte[], byte[])
	 */
	@Test
	public void testIndexOfPart_byte() {
		final byte[] pattern1 = new byte[]{(byte) 0xbd, (byte) 0xbd, (byte) 0xbd};
		final byte[] pattern2 = new byte[]{(byte) 0xaf, (byte) 0xfe, (byte) 0xaf};
		final byte[] pattern3 = new byte[]{(byte) 0xaf, (byte) 0xfe, (byte) 0xfe};
		final byte[] input1 = new byte[]{(byte) 0xaf, (byte) 0xfe, (byte) 0xbd, (byte) 0xbd, (byte) 0xbd};
		final byte[] input2 = new byte[]{(byte) 0xaf, (byte) 0xfe, (byte) 0xbd, (byte) 0xaf, (byte) 0xfe, (byte) 0xaf};
		assertEquals(-1, CollectionUtil.indexOfPart(input1, null));
		assertEquals(-1, CollectionUtil.indexOfPart((byte[])null, pattern1));
		assertEquals( 2, CollectionUtil.indexOfPart(input1, pattern1));
		assertEquals(-1, CollectionUtil.indexOfPart(input1, pattern2));
		assertEquals(-1, CollectionUtil.indexOfPart(input1, pattern3));
		assertEquals(-1, CollectionUtil.indexOfPart(input2, pattern1));
		assertEquals( 3, CollectionUtil.indexOfPart(input2, pattern2));
		assertEquals(-1, CollectionUtil.indexOfPart(input2, pattern3));
	}

	/**
	 * @see CollectionUtil#indexOfFirstPart(byte[], byte[][])
	 */
	@Test
	public void testIndexOfFirstPart_byte() {
		final byte[] pattern1 = new byte[]{(byte) 0xbd, (byte) 0xbd, (byte) 0xbd};
		final byte[] pattern2 = new byte[]{(byte) 0xed, (byte) 0xed, (byte) 0xed};
		final byte[] pattern3 = new byte[]{(byte) 0xaa, (byte) 0xaa, (byte) 0xaa};
		final byte[] pattern4 = new byte[]{(byte) 0xff, (byte) 0xff, (byte) 0xff};
		final byte[][] patterns = new byte[][]{ pattern1, pattern2, pattern3, pattern4 };
		final byte[] input1 = new byte[]{(byte) 0xaf, (byte) 0xfe, (byte) 0xbd, (byte) 0xbd, (byte) 0xbd};
		final byte[] input2 = new byte[]{(byte) 0xbd, (byte) 0xbd, (byte) 0xaa, (byte) 0xaa, (byte) 0xaa, (byte) 0xaa};
		assertEquals(-1, CollectionUtil.indexOfFirstPart((byte[])null, patterns));
		assertEquals(-1, CollectionUtil.indexOfFirstPart(input1, null));
		assertEquals(-1, CollectionUtil.indexOfFirstPart(input1, new byte[][]{ null }));
		assertEquals(-1, CollectionUtil.indexOfFirstPart(input1, new byte[][]{ { } }));
		assertEquals( 2, CollectionUtil.indexOfFirstPart(input1, patterns));
		assertEquals( 2, CollectionUtil.indexOfFirstPart(input2, patterns));
	}

	/**
	 * @see CollectionUtil#revert(Object[])
	 */
	@Test
	public void testRevert_Empty() {
		Integer[] arr = new Integer[] {};
		CollectionUtil.revert(arr);
		assertArrayEquals(new Integer[] {},  arr);
	}

	/**
	 * @see CollectionUtil#revert(Object[])
	 */
	@Test
	public void testRevert_Size1() {
		Integer[] arr = new Integer[] { 5 };
		CollectionUtil.revert(arr);
		assertArrayEquals(new Integer[] { 5 },  arr);
	}

	/**
	 * @see CollectionUtil#revert(Object[])
	 */
	@Test
	public void testRevert_Size2() {
		Integer[] arr = new Integer[] { 5, 3 };
		CollectionUtil.revert(arr);
		assertArrayEquals(new Integer[] { 3, 5 },  arr);
	}

	/**
	 * @see CollectionUtil#revert(Object[])
	 */
	@Test
	public void testRevert_Size3() {
		Integer[] arr = new Integer[] { 5, 3, 7 };
		CollectionUtil.revert(arr);
		assertArrayEquals(new Integer[] { 7, 3, 5 },  arr);
	}

	/**
	 * @see CollectionUtil#replaceLast(String, String, String)
	 */
	@Test
	public void testReplaceLast() {
		assertEquals("", CollectionUtil.replaceLast("", ".", "?"));
		assertEquals("Das.Ist.Das.Ende?", CollectionUtil.replaceLast("Das.Ist.Das.Ende.", ".", "?"));
		assertEquals("Das.Ist.Das.Ende", CollectionUtil.replaceLast("Das.Ist.Das.Ende.", ".", ""));
		assertEquals("das.ist.das.ente.", CollectionUtil.replaceLast("das.ist.das.ende.", "d", "t"));
		assertEquals("?", CollectionUtil.replaceLast(".", ".", "?"));
	}
}
