package de.ped.tools.test;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Test;

import de.ped.tools.DigitalJoystickDirection;

/**
 * Unit tests for the {@link DigitalJoystickDirection} class.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DigitalJoystickDirectionTest {

	/**
	 * @see DigitalJoystickDirection#getDirection(int)
	 * @see DigitalJoystickDirection#getDirection(int[])
	 * @see DigitalJoystickDirection#getDirection(int, int)
	 */
	@Test
	public void testBasics() {
		assertEquals(DigitalJoystickDirection.getDirection(DigitalJoystickDirection.MASK_UP), 
				DigitalJoystickDirection.getDirection(new int[] { 0, -1 } ));
		assertEquals(DigitalJoystickDirection.getDirection(DigitalJoystickDirection.MASK_DOWN), 
				DigitalJoystickDirection.getDirection(0, 1));
		assertEquals(DigitalJoystickDirection.getDirection(DigitalJoystickDirection.MASK_LEFT), 
				DigitalJoystickDirection.getDirection(-1, 0));
		assertEquals(DigitalJoystickDirection.getDirection(DigitalJoystickDirection.MASK_RIGHT), 
				DigitalJoystickDirection.getDirection(1, 0));
	}

	/**
	 * @see DigitalJoystickDirection#is(int, int)
	 */
	@Test
	public void testIs() {
		assertTrue(DigitalJoystickDirection.is(DigitalJoystickDirection.MASK_UP_LEFT, DigitalJoystickDirection.MASK_UP));
		assertTrue(DigitalJoystickDirection.is(DigitalJoystickDirection.MASK_UP_LEFT, DigitalJoystickDirection.MASK_LEFT));
		assertFalse(DigitalJoystickDirection.is(DigitalJoystickDirection.MASK_UP_LEFT, DigitalJoystickDirection.MASK_DOWN));
	}

	/**
	 * @see DigitalJoystickDirection#getDirections8()
	 * @see DigitalJoystickDirection#turnLeft8(int)
	 * @see DigitalJoystickDirection#turnRight8(int)
	 * @see DigitalJoystickDirection#getOppositeDirection()
	 * @see DigitalJoystickDirection#getId()
	 */
	@Test
	public void testTurns() {
		final ArrayList<DigitalJoystickDirection> dirs = DigitalJoystickDirection.getDirections8();
		for (int i = 0; i < dirs.size(); i++) {
			final DigitalJoystickDirection dir = dirs.get(i);
			assertEquals(dir, dir.turnLeft8(0));
			assertEquals(dir, dir.turnRight8(0));
			assertEquals(dir, dir.turnLeft8(1).turnRight8(1));
			assertEquals(dir, dir.turnLeft8(4).turnLeft8(4));
			assertEquals(dir, dir.turnRight8(4).turnRight8(4));
			assertEquals(dir, dir.getOppositeDirection().getOppositeDirection());
			assertEquals((0 == (i % 2)), dir.is4Direction());
			assertEquals(dirs.get((i + 4) % 8).getId(), dir.getOppositeDirection().getId());
		}
	}
}
