package de.ped.tools.test;

import static org.junit.Assert.*;

import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;

import org.junit.Before;
import org.junit.Test;

import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Unit tests for the behavior of the {@link GregorianCalendar} class.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GregorianCalendarTest {

	private Logger logger = Logger.getLogger(this.getClass());

	@Before
	public void setup() {
		logger.setLogLevel(Level.INFO);
	}

	public void checkDate(final int year, final int month, final int day) {
		final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		// Month is zero-based
		final GregorianCalendar cal = new GregorianCalendar(year, month - 1, day); 
		final String expected = String.format("%04d-%02d-%02d", year, month, day);
		assertEquals(expected, dateFormat.format(cal.getTime()));
	}

	@Test
	public void testSilvester() {
		checkDate(1582, 10, 25);
	}

	@Test
	public void test15820501() {
		checkDate(1582, 11, 01);
	}

	@Test
	public void testFromEarthDate() {
		checkDate(1982, 05, 01);
		checkDate(1582, 01, 01);
	}

	@Test
	public void testJavaVersion() {
		logger.debug(System.getProperty("java.version"));
	}
}
