package de.ped.tools.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.tools.Marshaller;

/**
 * Unit tests for the {@link I18NStringWithFillIns} class.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class I18NStringWithFIllinsTest {

	/**
	 * @see I18NStringWithFillIns#equals(Object)
	 * @see I18NStringWithFillIns#clone()
	 * @see I18NStringWithFillIns#EMPTY
	 */
	@Test
	public void testObjectBasics() {
		GenericTest.checkObjectBasics(true, 
				new I18NStringWithFillIns(), 
				I18NStringWithFillIns.EMPTY,
				new I18NStringWithFillIns("key", "alt", new String[] { "fill" } )
				);
	}

	/**
	 * @see I18NStringWithFillIns#getFillIns()
	 * @see I18NStringWithFillIns#getAlternative()
	 * @see I18NStringWithFillIns#getKey()
	 * @see I18NStringWithFillIns#setFillIns(Object[])
	 * @see I18NStringWithFillIns#setAlternative(String)
	 * @see I18NStringWithFillIns#setKey(String)
	 */
	@Test
	public void testGetSet() {
		final I18NStringWithFillIns is = new I18NStringWithFillIns();
		is.setKey("key");
		is.setAlternative("alt");
		is.setFillIns(new String[] { "fill" } );
		assertEquals("key", is.getKey());
		assertEquals("alt", is.getAlternative());
		assertEquals("fill", is.getFillIns()[0]);
	}

	@Test
	public void testMarshallDemarshall() {
		final I18NStringWithFillIns o = new I18NStringWithFillIns("key", "alt", new Object[] {
				new I18NStringWithFillIns("inner", "inner", null),
				Boolean.valueOf(true),
				Character.valueOf('C'),
				Integer.valueOf(4711),
				Long.valueOf(1234567890),
				Float.valueOf(3.14f),
				Double.valueOf(Math.PI),
				"Test",
		} );
		Marshaller m = new Marshaller();
		m.writeMarshallable(o);
		I18NStringWithFillIns r;
		r = (I18NStringWithFillIns) m.readMarshallable(o);
		final Object[] f = r.getFillIns();
		assertEquals(true, (Boolean) f[1]);
		assertEquals((char) 'C', ((Character) f[2]).charValue());
		assertEquals(4711, ((Integer) f[3]).intValue());
		assertEquals(1234567890, ((Long) f[4]).longValue());
		assertEquals(3.14f, ((Float) f[5]).floatValue(), 1e-5);
		assertEquals(Math.PI, ((Double) f[6]).doubleValue(), 1e-5);
		assertEquals("Test", (String) f[7]);
	}
}
