package de.ped.tools.test;

import static org.junit.Assert.*;

import org.junit.After;
//import org.junit.AfterClass;
import org.junit.Before;
//import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * Unit tests for the behavior of the {@link Logger} package.
 * 
 * @author Copyright (c) 2006 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class LoggerTest {

//	@BeforeClass
//	public static void setUpBeforeClass() throws Exception {
//	}
//
//	@AfterClass
//	public static void tearDownAfterClass() throws Exception {
//	}

	@Before
	public void setUp() throws Exception {
		Logger.resetErrorLevelCounters();
		Logger.getRootLogger().addAppender(Logger.DEFAULT_STRING_WRITER_APPENDER);
	}

	@After
	public void tearDown() throws Exception {
		Logger.resetErrorLevelCounters();
		Logger.getRootLogger().addAppender(Logger.SYSTEM_OUT_APPENDER);
	}

	/**
	 * @see Logger#getRootLogger()
	 * @see Logger#removeAppender(Appender)
	 * @see Logger#setLogLevel(int)
	 * @see Logger#error(String)
	 * @see Logger#getErrorLevelCounter(int)
	 * @see Logger#getErrorLevelSum(int)
	 * @see Logger#getWholeLog()
	 */
	@Test
	public void testLoggingWithNullMessage() {
		final Logger logger = Logger.getRootLogger();
		logger.removeAppender(Logger.SYSTEM_OUT_APPENDER);
		logger.setLogLevel(Level.ALL);
		logger.error(null);
		assertEquals(1, Logger.getErrorLevelCounter(Level.ERROR));
		assertEquals(1, Logger.getErrorLevelSum(Level.ERROR));
		final String log = Logger.getWholeLog();
		assertTrue(log.length() > 0);
		assertTrue(log.contains(Logger.MESSAGE_LEVEL_STRINGS[Level.ERROR]));
	}

	/**
	 * @see Logger#getRootLogger()
	 * @see Logger#setLogLevel(int)
	 * @see Logger#log(int, String)
	 * @see Logger#getErrorLevelSum(int)
	 * @see Logger#getWholeLog()
	 */
	@Test
	public void testLoggingWithLevelOff() {
		final Logger logger = Logger.getRootLogger();
		logger.setLogLevel(Level.ALL);
		final String text = "OffTst";
		logger.log(Level.OFF, text);
		assertEquals(0, Logger.getErrorLevelSum(Level.ERROR));
		final String log = Logger.getWholeLog();
		assertFalse(log.contains(text));
	}

	/**
	 * @see Logger#getRootLogger()
	 * @see Logger#removeAppender(Appender)
	 * @see Logger#setLogLevel(int)
	 * @see Logger#isEnabledFor(int)
	 * @see Logger#error(String)
	 * @see Logger#warn(String)
	 * @see Logger#getErrorLevelCounter(int)
	 * @see Logger#getErrorLevelSum(int)
	 * @see Logger#getWholeLog()
	 */
	@Test
	public void testLoggingWithErrorAndWarning() {
		final String errorText = "Test";
		final String warnText1 = "Nochwas";
		final String warnText2 = "Nochmehr";
		// Precondition für den Test
		final Logger logger = Logger.getRootLogger();
		logger.removeAppender(Logger.SYSTEM_OUT_APPENDER);
		logger.setLogLevel(Level.ALL);
		assertTrue(logger.isEnabledFor(Level.WARN));
//		System.out.println(logger.toString());
		logger.error(errorText);
		logger.warn(warnText1);
		logger.warn(warnText2);
		assertEquals(1, Logger.getErrorLevelCounter(Level.ERROR));
		assertEquals(1, Logger.getErrorLevelSum(Level.ERROR));
		assertEquals(2, Logger.getErrorLevelCounter(Level.WARN));
		assertEquals(3, Logger.getErrorLevelSum(Level.WARN));
		assertEquals(3, Logger.getErrorLevelSum(Level.ERROR, Level.WARN));
		assertEquals(3, Logger.getErrorLevelSum(Level.WARN, Level.ERROR));
		final String log = Logger.getWholeLog();
		assertTrue(log.length() > 0);
		assertTrue(log.contains(errorText));
		assertTrue(log.contains(warnText1));
		assertTrue(log.contains(warnText2));
	}
}
