package de.ped.tools.test;


import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.tools.Messages;

/**
 * Unit tests for the {@link Messages} class.
 * 
 * @author Copyright (c) 2014 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class MessagesTest {

	public static Messages messagesI18N;

	public static Messages messages;

	/**
	 * @see Messages#Messages(String, String, String, String, String, String, Locale[], String, int)
	 * @see Messages#Messages(String, String, String, String, String, String)
	 * @throws Exception if objects could not be created
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		messagesI18N = new Messages(
				"de.ped.tools.test.messages", 
				"MessagesTest", 
				"1.0", 
				"Copyright 1991-2012 Peter Diefenbach (peter@pdiefenbach.de)", 
				"Testing",
				"Testing",
				Messages.DEFAULT_LOCALES, 
				null,
				Messages.DEFAULT_LOCALE_INDEX);
		messages = new Messages(
				"de.ped.tools.test.messages", 
				"MessagesTest", 
				"1.0", 
				"Copyright 1991-2012 Peter Diefenbach (peter@pdiefenbach.de)",
				"Testing",
				"Testing");
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// Nothing to do
	}

	@Before
	public void setUp() throws Exception {
		// Nothing to do
	}

	@After
	public void tearDown() throws Exception {
		// Nothing to do
	}

	/**
	 * @see Messages#getString(String)
	 * @see Messages#getText(String)
	 * @see Messages#getMnemonic(String)
	 */
	@Test
	public void testDefault() {
		assertEquals("...", messagesI18N.getString("Action.DialogIndicator"));
		assertEquals("...", messages.getString("Action.DialogIndicator"));
		final String button = "Dialog.Cancel";
		assertEquals("Cancel", messagesI18N.getText(button));
		assertEquals("C", messagesI18N.getMnemonic(button));
	}

	/**
	 * @see Messages#getText(String)
	 * @see Messages#getMnemonic(String)
	 * @see Messages#getKeyStroke(String)
	 * @see Messages#getShortDescription(String)
	 * @see Messages#getLongDescription(String)
	 */
	@Test
	public void testConcrete() {
		final String menu = "Menu.Run";
		assertEquals("Run", messagesI18N.getText(menu));
		assertEquals("R", messagesI18N.getMnemonic(menu));
		final String action = "Action.SoundOnOff";
		assertEquals("Sound on/off", messagesI18N.getText(action));
		assertEquals("alt S", messagesI18N.getKeyStroke(action));
		assertEquals("Turns sound effects on or off.", messagesI18N.getShortDescription(action));
		assertEquals("<p>Turns sound effects on or off.</p>", messagesI18N.getLongDescription(action));
	}

	/**
	 * @see Messages#getString(String)
	 */
	@Test
	public void testOverriding() {
		assertEquals("MessageTest help", messagesI18N.getString("HelpWindow.Title"));
	}

	/**
	 * @see Messages#parseString(String, Object[])
	 */
	@Test
	public void testParseStringParameters() {
		assertNull(messagesI18N.parseString(null, new String[]{"egal"}));
		assertEquals("Hallo Welt", messagesI18N.parseString("Hallo Welt", null));
		assertEquals("Hallo % Welt", messagesI18N.parseString("Hallo %% Welt", null));
		assertEquals("Hallo geile Welt", messagesI18N.parseString("Hallo %0 Welt", new String[]{"geile"}));
		assertEquals("Hallo %0 Welt", messagesI18N.parseString("Hallo %0 Welt", new String[]{ null }));
	}

	/**
	 * @see Messages#parseString(String, Object[])
	 */
	@Test
	public void testParseStringRedirection() {
		assertEquals("Slowest", messagesI18N.parseString("%GameProperties.SpeedInfoSlowest", null));
		assertEquals("Slowest Very slow", messagesI18N.parseString("%GameProperties.SpeedInfoSlowest %GameProperties.SpeedInfoVerySlow", null));
	}

	/**
	 * @see Messages#findLocaleIndex(Locale[], String, int)
	 */
	@Test
	public void testFindLocaleIndex() {
		assertEquals(Messages.DEFAULT_LOCALE_INDEX, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, null, Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(Messages.DEFAULT_LOCALE_INDEX, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "", Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(1, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "en", Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(1, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "en_US", Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(1, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "en_UK", Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(0, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "de", Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(0, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "de_DE", Messages.DEFAULT_LOCALE_INDEX));
		assertEquals(0, Messages.findLocaleIndex(Messages.DEFAULT_LOCALES, "de_AT", Messages.DEFAULT_LOCALE_INDEX));
	}

	/**
	 * @see Messages#createLocale(String)
	 */
	@Test
	public void testCreateLocale() {
		assertNull(Messages.createLocale(null));
		assertEquals("", Messages.createLocale("").getLanguage());
	}

	/**
	 * @see Messages#equals(Object)
	 * @see Messages#clone()
	 */
	@Test
	public void testObjectBasics() {
		GenericTest.checkObjectBasics(false, messages, messagesI18N);
	}
}
