package de.ped.tools.test;

import static org.junit.Assert.*;

import org.junit.Test;

import de.ped.tools.sound.Note;
import de.ped.tools.sound.TuningSystem;

/**
 * Unit tests for the {@link Note} and {@link TuningSystem} class.
 * 
 * @author Copyright (c) 2013 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NoteTest {

	/**
	 * @see Note#get(int, int)
	 * @see Note#get(String)
	 */
	@Test
	public void testNoteIDString() {
		assertEquals(Note.get(Note.A,  1), Note.get("A1"));
		assertEquals(Note.get(Note.B,  1), Note.get("B1"));
		assertEquals(Note.get(Note.C,  2), Note.get("B#1"));
		assertEquals(Note.get(Note.C, -1), Note.get("C-1"));
		assertEquals(Note.get(Note.C, -1), Note.get("Cb#-1"));
		assertEquals(Note.get(Note.C, -1), Note.get("Cb#-1"));
	}

	public static final double EPSILON = 0.0000001;

	/**
	 * @see TuningSystem#getNumberOfHalfTones()
	 * @see TuningSystem#getFrequencyFactor(int)
	 * @see TuningSystem#getReverseFrequencyFactor(int)
	 */
	@Test
	public void testTuningSystems() {
		TuningSystem system = TuningSystem.PYTHAGOREAN;
		assertEquals(12, system.getNumberOfHalfTones());
		assertEquals(1.0, system.getFrequencyFactor(0), EPSILON);
		assertEquals(2.0, system.getFrequencyFactor(12), EPSILON);
		assertEquals(0.5, system.getFrequencyFactor(-12), EPSILON);
		assertEquals(system.getFrequencyFactor(1), 1.0 / system.getReverseFrequencyFactor(1), EPSILON);
	}
}
