package de.ped.tools.test;

import static org.junit.Assert.*;

import java.awt.Dimension;

import org.junit.Test;

import de.ped.tools.Marshaller;
import de.ped.tools.PlayfieldDimension;
import de.ped.tools.PlayfieldPosition;


/**
 * Unit tests for the {@link PlayfieldPosition} class.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class PlayfieldPositionTest {

	public void testPlayfieldPosition(final int x, final int y, final PlayfieldPosition pp) {
		assertEquals(x, pp.getX());
		assertEquals(y, pp.getY());
	}

	/**
	 * @see PlayfieldPosition#PlayfieldPosition()
	 * @see PlayfieldPosition#PlayfieldPosition(int, int)
	 * @see PlayfieldPosition#PlayfieldPosition(PlayfieldPosition)
	 * @see PlayfieldPosition#getX()
	 * @see PlayfieldPosition#getY()
	 * @see PlayfieldPosition#setX(int)
	 * @see PlayfieldPosition#setY(int)
	 * @see PlayfieldPosition#setLocation(int, int)
	 * @see PlayfieldPosition#equals(Object)
	 */
	@Test
	public void testPlayfieldPosition() {
		final PlayfieldPosition base = new PlayfieldPosition();
		testPlayfieldPosition(0, 0, base);
		final PlayfieldPosition zeroOne = new PlayfieldPosition(0, 1);
		testPlayfieldPosition(0, 1, zeroOne);
		assertFalse(base.equals(zeroOne));
		assertFalse(base.equals(null));
		assertFalse(base.equals(this));
		final PlayfieldPosition other = new PlayfieldPosition();
		other.setLocation(0, 1);
		assertTrue(zeroOne.equals(other));
		final PlayfieldPosition clone = new PlayfieldPosition(zeroOne);
		assertTrue(clone.equals(zeroOne));
		clone.setX(5);
		clone.setY(7);
		testPlayfieldPosition(5, 7, clone);
	}

	@Test
	public void testMarshalWithNull() {
		final PlayfieldPosition pos = new PlayfieldPosition(6, 12);
		final Marshaller m = new Marshaller();
		PlayfieldPosition.marshalWithNull(m, pos);
		PlayfieldPosition.marshalWithNull(m, null);
		final Marshaller dm = new Marshaller(m.pack());
		final PlayfieldPosition readPos1 = PlayfieldPosition.demarshalWithNull(dm);
		final PlayfieldPosition readPos2 = PlayfieldPosition.demarshalWithNull(dm);
		assertEquals(pos, readPos1);
		assertNull(readPos2);
	}

	@Test
	public void testCreateOriginPosition() {
		final PlayfieldPosition pos = new PlayfieldPosition();
		final PlayfieldPosition cmp = new PlayfieldPosition(0, 0);
		assertEquals(0, pos.get(0));
		assertEquals(0, pos.get(1));
		assertEquals(pos, cmp);
		assertEquals(0, pos.compareTo(cmp));
		assertTrue(pos.isWithin(PlayfieldPosition.BOUNDS));
	}

	@Test
	public void testCreatePositionFromDimension() {
		final PlayfieldDimension pos = new PlayfieldDimension(new Dimension(3, 255));
		final PlayfieldPosition cmp = new PlayfieldPosition(pos);
		final PlayfieldPosition trd = new PlayfieldPosition();
		trd.setLocation(pos);
		assertEquals(3, pos.get(0));
		assertEquals(255, pos.get(1));
		assertArrayEquals(pos.toArray(null), cmp.toArray(null));
		assertArrayEquals(pos.toArray(null), trd.toArray(null));
		assertTrue(pos.equals(cmp));
		assertEquals(0, pos.compareTo(cmp));
		assertTrue(pos.isWithin(PlayfieldPosition.BOUNDS));
	}

	@Test(expected=Throwable.class)
	public void testOutOfBoundsNeg() {
		checkMarshallingOutOfBounds(-1, 0);
	}

	@Test(expected=Throwable.class)
	public void testOutOfBoundsPos() {
		checkMarshallingOutOfBounds(255, 256);
	}

	protected void checkMarshallingOutOfBounds(final int x, final int y) {
		final PlayfieldPosition pos = new PlayfieldPosition(x, y);
		assertFalse(pos.isWithin(PlayfieldPosition.BOUNDS));
		final Marshaller m = new Marshaller();
		m.writeMarshallable(pos);
		final Marshaller dm = new Marshaller(m.pack());
		final PlayfieldPosition cmp = new PlayfieldPosition();
		dm.readMarshallable(cmp);
	}
}
