package de.ped.tools.test;

import static org.junit.Assert.*;

import java.util.TreeMap;

import org.junit.Test;

import de.ped.tools.PlayfieldPosition;
import de.ped.tools.PlayfieldTiling;
import de.ped.tools.TopologyMode;

public class PlayfieldTilingTest {

	@Test
	public void testCalcAirDistanceScalar() {
		// Start pos x + y, distance vector x + y, resulting distance.
		final TreeMap<TopologyMode, int[][]> testData = new TreeMap<TopologyMode, int[][]>();
		testData.put(TopologyMode.SQUARE8,
				new int[][] {
					{ 2, 3,  0,  0, 0 },
					{ 2, 3,  1,  0, 1 },
					{ 2, 3,  0,  1, 1 },
					{ 2, 3, -1,  0, 1 },
					{ 2, 3,  0, -1, 1 },
					{ 2, 3,  1,  1, 1 },
					{ 2, 3,  1, -1, 1 },
					{ 2, 3, -1,  1, 1 },
					{ 2, 3, -1, -1, 1 },
				}
		);
		testData.put(TopologyMode.SQUARE4,
				new int[][] {
					{ 2, 3,  0,  0, 0 },
					{ 2, 3,  1,  0, 1 },
					{ 2, 3,  0,  1, 1 },
					{ 2, 3, -1,  0, 1 },
					{ 2, 3,  0, -1, 1 },
					{ 2, 3,  1,  1, 2 },
					{ 2, 3,  1, -1, 2 },
					{ 2, 3, -1,  1, 2 },
					{ 2, 3, -1, -1, 2 },
				}
		);
		testData.put(TopologyMode.TRIANGLE,
				new int[][] {
					// /\
					{ 1, 2,  0,  0, 0 },
					{ 1, 2, -1,  0, 1 },
					{ 1, 2,  1,  0, 1 },
					{ 1, 2,  0,  1, 1 },
					{ 1, 2,  0, -1, 3 },
					// \/
					{ 1, 1,  0,  0, 0 },
					{ 1, 1, -1,  0, 1 },
					{ 1, 1,  1,  0, 1 },
					{ 1, 1,  0, -1, 1 },
					{ 1, 1,  0,  1, 3 },
				}
		);
		testData.put(TopologyMode.HEXAGON,
				new int[][] {
					// odd
					{ 2, 3,  0,  0, 0 },
					{ 2, 3, -1,  0, 1 },
					{ 2, 3,  1,  0, 1 },
					{ 2, 3,  0, -1, 1 },
					{ 2, 3,  0,  1, 1 },
					{ 2, 3, -1, -1, 2 },
					{ 2, 3,  1, -1, 2 },
					{ 2, 3, -1,  1, 1 },
					{ 2, 3,  1,  1, 1 },
					// even
					{ 3, 2,  0,  0, 0 },
					{ 3, 2, -1,  0, 1 },
					{ 3, 2,  1,  0, 1 },
					{ 3, 2,  0, -1, 1 },
					{ 3, 2,  0,  1, 1 },
					{ 3, 2, -1, -1, 1 },
					{ 3, 2,  1, -1, 1 },
					{ 3, 2, -1,  1, 2 },
					{ 3, 2,  1,  1, 2 },
				}
		);
		testData.put(TopologyMode.HEXAGON_H,
				new int[][] {
					// odd
					{ 2, 3,  0,  0, 0 },
					{ 2, 3, -1,  0, 1 },
					{ 2, 3,  1,  0, 1 },
					{ 2, 3,  0, -1, 1 },
					{ 2, 3,  0,  1, 1 },
					{ 2, 3, -1, -1, 1 },
					{ 2, 3,  1, -1, 2 },
					{ 2, 3, -1,  1, 1 },
					{ 2, 3,  1,  1, 2 },
					// even
					{ 3, 2,  0,  0, 0 },
					{ 3, 2, -1,  0, 1 },
					{ 3, 2,  1,  0, 1 },
					{ 3, 2,  0, -1, 1 },
					{ 3, 2,  0,  1, 1 },
					{ 3, 2, -1, -1, 2 },
					{ 3, 2,  1, -1, 1 },
					{ 3, 2, -1,  1, 2 },
					{ 3, 2,  1,  1, 1 },
				}
		);
		for (TopologyMode mode : testData.keySet()) {
			final int[][] arr = testData.get(mode);
			for (int i = 0; i < arr.length; i++) {
				final int[] ar = arr[i];
				final PlayfieldPosition start = new PlayfieldPosition(ar[0], ar[1]);
				final PlayfieldPosition dist = new PlayfieldPosition(ar[2], ar[3]);
				final String str = mode.name() + "/" + i + ":(" + ar[0] + "," + ar[1] + ")+(" + ar[2] + "," + ar[3] + ")->" + ar[4];
				assertEquals(str, ar[4], PlayfieldTiling.calcAirDistanceScalar(mode, start, dist));
			}
		}
	}
}
