package de.ped.tools.test;

//import static org.junit.Assert.*;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Ignore;
import org.junit.Test;

import de.ped.tools.sound.*;
import de.ped.tools.sound.DynamicSoundMonoWaveCreator.Sound;

/**
 * Unit tests for the {@link de.ped.tools.sound} package.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class SoundTest {
	public static void play(final DynamicSoundMonoWaveCreator smwc, final MonoWave wave, final double duration) {
		final byte[] sample;
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.execute(new Sound(wave, duration)));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
	}

	public static void play(final DynamicSoundMonoWaveCreator smwc, final MonoWave wave) {
		play(smwc, wave, 0.5);
	}

	public static void play(final DynamicSoundMonoWaveCreator smwc, final DynamicSoundMonoWaveCreator.Sound sound) {
		if (null != sound) {
			byte[] sample;
			sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.execute(sound));
			SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		}
	}

	private static void assertArrayEquals(final double[] exp, final double[] ret) {
		assertEquals(exp.length, ret.length);
		for (int i = 0; i < exp.length; i++) {
			assertEquals(exp[i], ret[i], 0.001);
		}
	}

	@Ignore
	public void testSoundMonoWaveCreator() {
		final SoundMonoWaveCreator smwc = new SoundMonoWaveCreator();
		final double[] hull = smwc.sawDownWave(0.5, 1.0 / 2.0 / 0.5);
		byte[] sample;
		// Basic waveforms:
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.multiply(smwc.squareWave(0.5, 220), hull));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.multiply(smwc.sawUpWave(0.5, 220), hull));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.multiply(smwc.triangleWave(0.5, 220), hull));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.multiply(smwc.sinusWave(0.5, 220), hull));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		// Schwebung:
		final double[] sin = smwc.add(smwc.triangleWave(2.0, 220.0), smwc.triangleWave(2.0, 221.5));
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.multiply(sin, 0.5));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		// Noise:
		sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.whiteNoise(0.2));
		SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		for (double i = 1.0; i >= 0.0; i -= 0.01) {
			sample = DynamicSoundMonoWaveCreator.toStereoSample(smwc.pinkNoise(0.1, i));
			SoundMonoWaveCreator.playActive(smwc.getSampleRate(), sample);
		}
	}

	/**
	 * @see DynamicSoundMonoWaveCreator#DynamicSoundMonoWaveCreator()
	 * @see ConstantWave#ConstantWave(double)
	 * @see SinusWave#SinusWave(DynamicSoundMonoWaveCreator, double)
	 * @see MultipliedWave#MultipliedWave(DynamicSoundMonoWaveCreator, MonoWave, MonoWave)
	 * @see SawDownWave#SawDownWave(DynamicSoundMonoWaveCreator, double)
	 * @see SinusWave#SinusWave(DynamicSoundMonoWaveCreator, double, MonoWave)
	 * @see SquareWave#SquareWave(DynamicSoundMonoWaveCreator, double, MonoWave)
	 * @see SawUpWave#SawUpWave(DynamicSoundMonoWaveCreator, double, MonoWave)
	 * @see TriangleWave#TriangleWave(DynamicSoundMonoWaveCreator, double, MonoWave)
	 * @see WhiteNoiseWave#WhiteNoiseWave(DynamicSoundMonoWaveCreator)
	 * @see PinkNoiseWave#PinkNoiseWave(DynamicSoundMonoWaveCreator, MonoWave)
	 * @see PinkNoiseSawWave#PinkNoiseSawWave(DynamicSoundMonoWaveCreator, MonoWave)
	 * @see AmplitudeModulationWave#AmplitudeModulationWave(DynamicSoundMonoWaveCreator, MonoWave, double, MonoWave)
	 * @see EnvelopeWave#EnvelopeWave(DynamicSoundMonoWaveCreator, double, double, double)
	 * @see EnvelopeWave#EnvelopeWave(DynamicSoundMonoWaveCreator, double, double, double, double, double, double, double)
	 * @see CutAmplitudeWave#CutAmplitudeWave(DynamicSoundMonoWaveCreator, MonoWave, MonoWave)
	 * @see AddedWave#AddedWave(DynamicSoundMonoWaveCreator, MonoWave, double, MonoWave, double)
	 */
	@SuppressWarnings("unused")
	@Test
	public void testSoundStereoWaveCreator() {
		final DynamicSoundMonoWaveCreator smwc = new DynamicSoundMonoWaveCreator();
		final AbstractWave.Params snareParams = new AbstractWave.Params(smwc, 0.0);
		final MonoWave c1Wave       = new ConstantWave(1.0);
		final MonoWave vibAmplWave  = new ConstantWave(0.10);
		final MonoWave vibBaseWave  = new SinusWave(smwc, 10.0); 
		final MonoWave vibratoWave  = new MultipliedWave(smwc, vibBaseWave, vibAmplWave);
		final MonoWave hullWave     = new SawDownWave(smwc, 1.0); 
		final MonoWave sinusWave    = new SinusWave(smwc, 220.0, vibratoWave); 
		final MonoWave squareWave   = new SquareWave(smwc, 220.0, vibratoWave); 
		final MonoWave sawUpWave    = new SawUpWave(smwc, 220.0, vibratoWave); 
		final MonoWave triangleWave = new TriangleWave(smwc, 220.0, vibratoWave); 
		final MonoWave whiteNoiseWave = new WhiteNoiseWave(smwc); 
		final MonoWave pinkNoiseWave  = new PinkNoiseWave(smwc, hullWave); 
		final MonoWave pinkNoiseSWave = new PinkNoiseSawWave(smwc, hullWave); 
		final MonoWave hSinusWave   = new AmplitudeModulationWave(smwc, sinusWave, 1.0, hullWave);
		final MonoWave beatADSRWave = new EnvelopeWave(smwc, 0.05, 1.0, 0.0);
		final MonoWave snareWave    = new MultipliedWave(snareParams, whiteNoiseWave, beatADSRWave);
		final MonoWave distADSRWave = new EnvelopeWave(smwc, 0.25, 0.25, 0.25, 0.25, 0.0, 20.0, 1.0);
		final MonoWave tMultWave    = new MultipliedWave(smwc, triangleWave, distADSRWave);
		final MonoWave tCutWave     = new CutAmplitudeWave(smwc, tMultWave, c1Wave);
		final MonoWave resultWave   = new AddedWave(smwc, snareWave, 1.0, tCutWave, 1.0);
		MonoWave sink;
		sink = squareWave;
		sink = sawUpWave;
		sink = pinkNoiseWave;
		sink = pinkNoiseSWave;
		sink = hSinusWave;
//		play(smwc, sinusWave);
//		play(smwc, squareWave);
//		play(smwc, sawUpWave);
//		play(smwc, triangleWave);
//		play(smwc, hSinusWave);
//		play(smwc, whiteNoiseWave);
//		play(smwc, pinkNoiseWave);
//		play(smwc, pinkNoiseSWave);
//		play(smwc, resultWave, 1.0);
	}

	/**
	 * @see Tuning#defaultTuning()
	 * @see Tuning#getFrequency(Note)
	 * @see DynamicSoundMonoWaveCreator#DynamicSoundMonoWaveCreator()
	 * @see ConstantWave#ConstantWave(double)
	 * @see SinusWave#SinusWave(RepeatingWave.Params)
	 * @see MultipliedWave#MultipliedWave(AbstractWave.Params, MonoWave, MonoWave)
	 * @see SawDownWave#SawDownWave(RepeatingWave.Params)
	 * @see SinusWave#SinusWave(RepeatingWave.Params)
	 * @see SquareWave#SquareWave(RepeatingWave.Params)
	 * @see SawUpWave#SawUpWave(RepeatingWave.Params)
	 * @see TriangleWave#TriangleWave(RepeatingWave.Params)
	 * @see WhiteNoiseWave#WhiteNoiseWave(DynamicSoundMonoWaveCreator)
	 * @see PinkNoiseWave#PinkNoiseWave(DynamicSoundMonoWaveCreator, MonoWave)
	 * @see PinkNoiseSawWave#PinkNoiseSawWave(DynamicSoundMonoWaveCreator, MonoWave)
	 * @see AmplitudeModulationWave#AmplitudeModulationWave(DynamicSoundMonoWaveCreator, MonoWave, double, MonoWave)
	 * @see EnvelopeWave#EnvelopeWave(EnvelopeWave.Parameters)
	 * @see CutAmplitudeWave#CutAmplitudeWave(AbstractWave.Params, MonoWave, MonoWave)
	 * @see AddedWave#AddedWave(AbstractWave.Params, MonoWave, double, MonoWave, double)
	 * @see AddedWave#AddedWave(DynamicSoundMonoWaveCreator, List)
	 */
	@SuppressWarnings("unused")
	@Test
	public void testSoundStereoWaveCreator_WithOffset() {
		final DynamicSoundMonoWaveCreator smwc = new DynamicSoundMonoWaveCreator();
		ArrayList<AddedWave.Part> list = new ArrayList<AddedWave.Part>();
		for (int i = 0; i < 5; i++) {
			final AbstractWave.Params params = new AbstractWave.Params(smwc, 0.2 * i);
			final double freq = Tuning.defaultTuning().getFrequency(Note.get(i, 1));
			final MonoWave c1Wave       = new ConstantWave(1.0);
			final MonoWave vibAmplWave  = new ConstantWave(0.10);
			final MonoWave vibBaseWave  = new SinusWave(new RepeatingWave.Params(params, 10.0)); 
			final MonoWave vibratoWave  = new MultipliedWave(params, vibBaseWave, vibAmplWave);
			final MonoWave hullWave     = new SawDownWave(new RepeatingWave.Params(params, 1.0)); 
			final MonoWave sinusWave    = new SinusWave(new RepeatingWave.Params(params, freq, vibratoWave)); 
			final MonoWave squareWave   = new SquareWave(new RepeatingWave.Params(params, freq, vibratoWave)); 
			final MonoWave sawUpWave    = new SawUpWave(new RepeatingWave.Params(params, freq, vibratoWave)); 
			final MonoWave triangleWave = new TriangleWave(new RepeatingWave.Params(params, freq, vibratoWave)); 
			final MonoWave whiteNoiseWave = new WhiteNoiseWave(smwc); 
			final MonoWave pinkNoiseWave  = new PinkNoiseWave(smwc, hullWave); 
			final MonoWave pinkNoiseSWave = new PinkNoiseSawWave(smwc, hullWave); 
			final MonoWave hSinusWave   = new AmplitudeModulationWave(smwc, sinusWave, 1.0, hullWave);
			final MonoWave beatADSRWave = new EnvelopeWave(new EnvelopeWave.Parameters(params, 0.05, 1.0, 0.0));
			final MonoWave snareWave    = new MultipliedWave(params, whiteNoiseWave, beatADSRWave);
			final MonoWave distADSRWave = new EnvelopeWave(new EnvelopeWave.Parameters(params, 0.25, 0.25, 0.25, 0.25, 0.0, 20.0, 1.0));
			final MonoWave tMultWave    = new MultipliedWave(params, triangleWave, distADSRWave);
			final MonoWave tCutWave     = new CutAmplitudeWave(params, tMultWave, c1Wave);
			final MonoWave tuneWave     = new AddedWave(params, snareWave, 1.0, tCutWave, 1.0);
			AddedWave.Part part = new AddedWave.Part(tuneWave, 0.3);
			list.add(part);
		}
		final MonoWave resultWave = new AddedWave(smwc, list);
//		play(smwc, resultWave, 3.0);
	}

	@Test
	public void testConcat_SmoothWidth0() {
		final double[] arr1 = new double[1000];
		final double[] arr2 = new double[500];
		final double[] exp  = new double[1500];
		for (int i = 0; i < arr1.length; i++) {
			arr1[i] = -0.5;
			exp[i] = arr1[i];
		}
		for (int i = 0; i < arr2.length; i++) {
			arr2[i] = 0.5;
			exp[i + arr1.length] = arr2[i];
		}
		final SoundMonoWaveCreator creator = new SoundMonoWaveCreator();
		final double[] ret = creator.concat(arr1, arr2, 0);
		assertArrayEquals(exp, ret);
	}

	// test for feature that is currently not implemented
	@Ignore
	public void testConcat_SmoothWidth10() {
		final int smoothWidth = 10;
		final double[] arr1 = new double[1000];
		final double[] arr2 = new double[500];
		final double[] exp  = new double[1500];
		for (int i = 0; i < arr1.length; i++) {
			arr1[i] = -0.5;
			exp[i] = arr1[i];
		}
		for (int i = 0; i < arr2.length; i++) {
			arr2[i] = 0.5;
			exp[i + arr1.length] = arr2[i];
		}
		for (int i = 0; i < smoothWidth; i++) {
			exp[arr1.length + i] = i * 0.05;
			exp[arr1.length - 1 - i] = -i * 0.05;
		}
		final SoundMonoWaveCreator creator = new SoundMonoWaveCreator();
		final double[] ret = creator.concat(arr1, arr2, smoothWidth);
		assertArrayEquals(exp, ret);
	}
}
