package de.ped.tools.test;

import static org.junit.Assert.*;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.tools.Messages;
import de.ped.tools.TextDocumentBuilder;
import de.ped.tools.TextDocumentType;
import de.ped.tools.TextTree;
import de.ped.tools.TextTreeNodeType;

/**
 * Unit tests for the {@link TextDocumentBuilder} class.
 * 
 * @author Copyright (c) 2019 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TextDocumentBuilderTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		MessagesTest.setUpBeforeClass();
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		MessagesTest.tearDownAfterClass();
	}

	/**
	 * @see TextTree#TextTree(TextTreeNodeType, String, String, String, String, String, TextTree[])
	 * @see TextDocumentBuilder#TextDocumentBuilder(Messages)
	 * @see TextDocumentBuilder#setTextDocumentType(TextDocumentType)
	 * @see TextDocumentBuilder#setEmbedded(boolean)
	 * @see TextDocumentBuilder#add(TextTree, int, int, boolean)
	 * @see TextDocumentBuilder#result()
	 */
	@Test
	public void testTextTree() {
		checkTextTree(new TextTree(null, null, null, null, null, null, null), "");
		checkTextTree(new TextTree(TextTreeNodeType.MENU, "Menu.File", null, null, null, null, null), 
				"\n*** Spiel ***\n\nAktionen, die eine ganze Datei oder das ganze Programm betreffen.\n");
		checkTextTree(new TextTree(TextTreeNodeType.MENU, "Menu.File", null, null, null, null, new TextTree[] { } ), 
				"\n*** Spiel ***\n\nAktionen, die eine ganze Datei oder das ganze Programm betreffen.\n");
		checkTextTree(new TextTree(TextTreeNodeType.MENU, "Menu.File", null, null, null, null, new TextTree[] { new TextTree(TextTreeNodeType.ACTION, "Action.New", null, null, null, null, null), } ), 
				"\n*** Spiel ***\n\nAktionen, die eine ganze Datei oder das ganze Programm betreffen.\n\n=== Neu ===\n\nBeendet das laufende Spiel und startet ein neues.\n");
		checkTextTree(new TextTree(TextTreeNodeType.MENU, "Menu.File", null, null, null, null, new TextTree[] { 
				new TextTree(TextTreeNodeType.MENU, "Menu.TextFormat", null, null, null, null, new TextTree[] {
						new TextTree(TextTreeNodeType.ACTION, "Action.ViewFormatFixedFont", null, null, null, null, null), } ), } ), 
				"\n*** Spiel ***\n\nAktionen, die eine ganze Datei oder das ganze Programm betreffen.\n\n=== Textdarstellung ===\n\nTextdarstellungsoptionen.\nZeichen fester Breite\nNormaler Text, tabellenartige Formatierung möglich\n");
	}

	public void checkTextTree(final TextTree tree, final String result) {
		Messages messages = MessagesTest.messages;
		TextDocumentBuilder doc = new TextDocumentBuilder(messages);
		doc.setTextDocumentType(TextDocumentType.TEXT);
		doc.setEmbedded(true);
		doc.add(tree, 2, 3, true);
		assertEquals(result, doc.result());
	}

	@Test
	public void testRegex() {
//		final String regex = ".*<[a-zA-Z].*"; //".*?<[a-zA-Z].*?";
//		assertTrue("<p>".matches(regex));
//		assertFalse("</p>".matches(regex));
//		assertTrue(" </p> \n <p> </dl> ".matches(regex));
//		assertFalse(" </p> \n </dl> ".matches(regex));
	}

}
