package de.ped.tools.web;

import de.ped.tools.CollectionUtil;
import de.ped.tools.HtmlHelper;
import de.ped.tools.TextDocumentType;

/**
 * The data necessary to describe a link to another page.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach
 */
public class PageRef {

	/**
	 * The HTML flavor the page is written in.
	 * 
	 * @author Copyright (c) 2018 Peter Diefenbach
	 */
	public enum HTMLFlavor {
		/** XHTML 1.0 Transitional */
		XHTML10T(TextDocumentType.XHTML10_TRANSITIONAL),
		/** XHTML 1.0 Strict */
		XHTML10S(TextDocumentType.XHTML10_STRICT),
		/** HTML5 */
		HTML5(TextDocumentType.HTML5);

		public final TextDocumentType doctype;
		HTMLFlavor(final TextDocumentType doctype) {
			this.doctype = doctype;
		}
	}

	public static final PageRef PROFESSION				= createPage(Subdomain.PROFESSION, "profession", "profession", "index.html", "Beruflich", "index_en_US.html", "Profession", "/resources/Sitemap.Profession.png", null);
	public static final PageRef PROFESSION_CV			= createRes(Subdomain.PROFESSION, "Diefenbach Peter Lebenslauf.pdf", "Lebenslauf", "Diefenbach Peter CV.pdf", "Curriculum Vitae");
	public static final PageRef PROFESSION_PROFILE		= createRes(Subdomain.PROFESSION, "Diefenbach Peter Profil.pdf", "Profil", "Diefenbach Peter Profile.pdf", "Profile");
	public static final PageRef PROFESSION_REFERENCES	= createRes(Subdomain.PROFESSION, "Zeugnisse.zip", "Zertifikate und Zeugnisse", "Certificates and references");
	public static final PageRef VIRREL 					= createWithFlavor(PROFESSION, null, "virrel", "Virrel", null);

	public static final PageRef ADVENT			= createPage(Subdomain.ADVENT,  "advent", "advent", "advent.jsp", "Advent", null, "Advent", null, null);

	public static final PageRef MOTIVATION		= createPage(Subdomain.MOTIVATION, "motivation", "motivation", "motivation.jsp", "Motivation", null, "Motivation", null, null);

	public static final PageRef HOME 			= createPage(Subdomain.PRIVATE, "private", "private", "index.html", "Privat", "index_en_US.html", "Private life", "/resources/Sitemap.Private.png", null);
	public static final PageRef BOOKMARK 		= createPage(Subdomain.PRIVATE, "bookmark", "bookmark", "bookmark.html", "Bookmarks", "bookmark_en_US.html", "Bookmarks", "/resources/bookmark32.png", null);
	public static final PageRef BOOKMARK_LYRICS = createWithFlavor(BOOKMARK, null, "bookmarkLyrics");
	public static final PageRef BOOKMARK_RPG	= createWithFlavor(BOOKMARK, null, "bookmarkRPG");
	public static final PageRef GEBURTST1995	= createPage(Subdomain.PRIVATE, "geb1995", "geb1995", "geburtst1995.html", "Geburtstag 1995", null, "Birthday 1995", null, null);
	public static final PageRef PATH_PICTURES_GEB1995 = createRes(Subdomain.PRIVATE, "geb1995/", "", "");
	public static final PageRef IMPRINT 		= createPage(Subdomain.PRIVATE, "imprint", "imprint", "imprint.html", "Impressum", "imprint_en_US.html", "Imprint", "/resources/Sitemap.Imprint.png", null);
	public static final PageRef MUSIC 			= createPage(Subdomain.PRIVATE, "music", "music", "bitsquits.html", "Musik", "bitsquits_en_US.html", "Music", "/resources/Sitemap.Music.png", null);
	public static final PageRef PICTURES 		= createPage(Subdomain.PRIVATE, "pictures", "pictures", "pictures.html", "Bilder", "pictures_en_US.html", "Pictures", "/resources/Sitemap.Pictures.png", null);
	public static final PageRef POEMS 			= createPage(Subdomain.PRIVATE, "poems", "poems", "gedichte.jsp", "Gedichte", "gedichte_en_US.jsp", "Poems", "/resources/Sitemap.Poems.png", null);
	public static final PageRef POEMS_BITTEGLAUBMIRNICHT= createWithFlavor(POEMS, "?item=bitteglaubmirnicht", 		null, "Bitte glaub mir nicht", null);
	public static final PageRef POEMS_GESTATTENBAERTIGER= createWithFlavor(POEMS, "?item=gestattenbaertiger", 		null, "Gestatten, BärTiger", null);
	public static final PageRef POEMS_ICHBINEINTRAUM 	= createWithFlavor(POEMS, "?item=ichbineintraum", 			null, "Ich bin ein Traum", null);
	public static final PageRef POEMS_ICHWEISS			= createWithFlavor(POEMS, "?item=ichweiss", 				null, "Zeit (Ich weiß nicht)", null);
	public static final PageRef POEMS_HEYGETOUTOFMYWAY	= createWithFlavor(POEMS, "?item=heygetoutofmyway", 		null, null, "Hey (Get Out Of My Way)");
	public static final PageRef POEMS_KARLSRUHE 		= createWithFlavor(POEMS, "?item=karlsruheraugustgewitter", null, "Karlsruher Augustgewitter", null);
	public static final PageRef POEMS_LJBF				= createWithFlavor(POEMS, "?item=ljbf", 					null, null, "Let's Just Be Friends");
	public static final PageRef POEMS_UNDICHWARALLEIN 	= createWithFlavor(POEMS, "?item=undichwarallein", 			null, "Und ich war allein", null);
	public static final PageRef POEMSFRIED 		= createPage(Subdomain.PRIVATE, "poemsfried", "poemsfried", "gedichte_fried.jsp", "", null, "", "/resources/Sitemap.Poems.png", null);
	public static final PageRef POEMSALEAT 		= createPage(Subdomain.PRIVATE, "poemsaleat", "poemsaleat", "gedichte_aleatorisch.html", "aleatorische", null, "aleatoric", "/resources/Sitemap.Aleatoric.png", null);
	public static final PageRef QUOTES 			= createPage(Subdomain.PRIVATE, "quotes", "quotes", "quotes.html", "Zitate", "quotes_en_US.html", "Quotations", "/resources/Sitemap.Quotation.png", null);
	public static final PageRef QUOTCOLL 		= createPage(Subdomain.PRIVATE, "quotcoll", "quotcoll", "quotcoll.html", "Zitatesammlung", "quotcoll_en_US.html", "Quotes compilation", "/resources/book32.png", null);
	public static final PageRef KINOATLANTIK 	= createPage(Subdomain.PRIVATE, "kinoatlantik", "kinoatlantik", "atlantik.html", "", null, "", null, null);
	public static final PageRef RECIPES 		= createPage(Subdomain.PRIVATE, "recipes", "recipes", "rezepte.jsp", "Rezepte", null, "Recipes", "/resources/cookcap32.png", null);
	public static final PageRef VIERSCHWINGENSAGA = new PageRef(Subdomain.PRIVATE, null, "vierschwingensaga", "vierschwingensaga.htm", "Vierschwingensaga", null, "Vierschwingensaga", HTMLFlavor.HTML5, TextDocumentType.CHARSET_ISO_8859_1, null, null, null);

	public static final PageRef PROGRAMS_HOME 	= createPage(Subdomain.PRIVATE, "programs", "programs", "programs.html", "Programme", "programs_en_US.html", "Programs", "/resources/Sitemap.Programs.png", null);
	public static final PageRef ATARI8BIT 		= createPage(Subdomain.PRIVATE, "atari8bit", "atari8bit", "atari8bit.html", "Atari8bit", "atari8bit_en_US.html", "Atari8bit", "/resources/Sitemap.Atari.png", null);
	public static final PageRef ATARI8BIT_KITTEN= PageRef.createWithFlavor(ATARI8BIT, null, "kitten");
	public static final PageRef ATARI8BIT_LIFE  = PageRef.createWithFlavor(ATARI8BIT, null, "life");
	public static final PageRef CONQUEST_APP 	= createPage(Subdomain.PRIVATE, "conquestapp", "conquest app", "conquest.html", "Conquest App", "conquest_en_US.html", "Conquest app", "/resources/Sitemap.Conquest.png", null);
	public static final PageRef DEINBAC_APP 	= createPage(Subdomain.PRIVATE, "deinbacapp", "deinbac app", "deinbac.html", "Deinbac App", "deinbac_en_US.html", "Deinbac app", "/resources/Sitemap.Deinbac.png", null);
	public static final PageRef EMPIRE_APP 		= createPage(Subdomain.PRIVATE, "empireapp", "empire app", "empire.html", "Empire App", "empire_en_US.html", "Empire app", "/resources/Sitemap.Empire.png", null);
	public static final PageRef KITTEN_APP 		= createPage(Subdomain.PRIVATE, "kittenapp", "kitten app", "kitten.html", "Kitten App", "kitten_en_US.html", "Kitten app", "/resources/Sitemap.Kitten.App.png", null);
	public static final PageRef KITTEN_WEB 		= createPage(Subdomain.PRIVATE, "kittenweb", "kitten web", "kittenjs.html", "Kitten Web", "kittenjs_en_US.html", "Kitten web", "/resources/Sitemap.Kitten.Web.png", null);
	public static final PageRef PACMAN_APP 		= createPage(Subdomain.PRIVATE, "pacmanapp", "pacman app", "pacman.html", "Pacman App", "pacman_en_US.html", "Pacman app", "/resources/Sitemap.Pacman.App.png", null);
	public static final PageRef PACMAN_WEB		= createPage(Subdomain.PRIVATE, "pacmanweb", "pacman web", "pacmanjs.html", "Pacman Web", "pacmanjs_en_US.html", "Pacman web", "/resources/Sitemap.Pacman.Web.png", null);
	public static final PageRef TROFF_APP 		= createPage(Subdomain.PRIVATE, "troffapp", "troff app", "troff.html", "Troff App", "troff_en_US.html", "Troff app", "/resources/Sitemap.Troff.png", null);
	public static final PageRef TROFF_WEB		= createPage(Subdomain.PRIVATE, "troffweb", "troff web", "troffjs.html", "Troff Web", "troffjs_en_US.html", "Troff web", "/resources/Sitemap.Troff.png", null);

	public static final PageRef DOWNLOAD_HTS	= createRes(Subdomain.PRIVATE, "download/HappyTrashSoftwareAtari8bit.zip", "Download von Happy Trash Software Atari 8bit Software", "Download of Happy Trash Software Atari 8bit Software");
	public static final PageRef DOWNLOAD_JAVA	= createRes(Subdomain.PRIVATE, "download/java.zip", "java.zip");
	public static final PageRef CONQUEST_JAR	= createRes(Subdomain.PRIVATE, "download/conquest.jar", "Conquest");
	public static final PageRef DEINBAC_JAR		= createRes(Subdomain.PRIVATE, "download/deinbac.jar", "Deinbac");
	public static final PageRef DSATOOL_JAR		= createRes(Subdomain.PRIVATE, "download/dsatool.jar", "DSATool");
	public static final PageRef EMPIRE_JAR		= createRes(Subdomain.PRIVATE, "download/empire.jar", "Empire");
	public static final PageRef KITTEN_JAR		= createRes(Subdomain.PRIVATE, "download/kitten.jar", "Kitten");
	public static final PageRef PACMAN_JAR		= createRes(Subdomain.PRIVATE, "download/pacman.jar", "Pacman");
	public static final PageRef TROFF_JAR		= createRes(Subdomain.PRIVATE, "download/troff.jar", "Troff");

	public static final PageRef MP3_INTRO			= createRes(Subdomain.PRIVATE, "music/01-Intro.mp3", 				"Intro");
	public static final PageRef MP3_GETINVOLVED		= createRes(Subdomain.PRIVATE, "music/02-Get Involved.mp3", 		"Get Involved");
	public static final PageRef MP3_MAGICFOREST		= createRes(Subdomain.PRIVATE, "music/03-Old Magic Forest.mp3", "Old Magic Forest");
	public static final PageRef MP3_MAGICSWAMP		= createRes(Subdomain.PRIVATE, "music/04-Magic Swamp.mp3", 		"Magic Swamp");
	public static final PageRef MP3_ROADAMERICAINE	= createRes(Subdomain.PRIVATE, "music/05-Road Americaine.mp3", 	"Road Americaine");
	public static final PageRef MP3_TECHNOENTE		= createRes(Subdomain.PRIVATE, "music/06-Techno-Ente.mp3", 		"Techno-Ente");
	public static final PageRef MP3_LASTDIVE		= createRes(Subdomain.PRIVATE, "music/07-Last Dive.mp3", 		"Last Dive");
	public static final PageRef MP3_PIANOROMANCE	= createRes(Subdomain.PRIVATE, "music/08-Piano Romance.mp3", 	"Piano Romance");
	public static final PageRef MP3_GAMEINTRO		= createRes(Subdomain.PRIVATE, "music/09-Game Intro.mp3", 		"Game Intro");
	public static final PageRef MP3_GETTHERHYTHM	= createRes(Subdomain.PRIVATE, "music/10-Get The Rhythm.mp3", 	"Get The Rhythm");
	public static final PageRef MP3_TIGERHOEHLE		= createRes(Subdomain.PRIVATE, "music/11-Mogi in der Tigerhoehle.mp3", "Mogi in der Tigerhöhle");
	public static final PageRef MP3_CYBERLOVELOOP	= createRes(Subdomain.PRIVATE, "music/12-Cyber Love Loop.mp3", "Cyber Love Loop");
	public static final PageRef MP3_TECHNOTRANCE	= createRes(Subdomain.PRIVATE, "music/13-Technotrance.mp3", 		"Technotrance");
	public static final PageRef MP3_SUMMERNIGHT		= createRes(Subdomain.PRIVATE, "music/14-Summernight's Dance.mp3", "Summernight's Dance");
	public static final PageRef MP3_BRUNCHIFEQUAL	= createRes(Subdomain.PRIVATE, "music/15-Brunch If Equal.mp3", "Brunch If Equal");
	public static final PageRef MP3_REMEMBER		= createRes(Subdomain.PRIVATE, "music/16-Remember The Days.mp3", "Remember The Days");
	public static final PageRef MP3_OPUSLIKINAUDA	= createRes(Subdomain.PRIVATE, "music/OpusLikiNauda.mp3", "OpusLikiNauda.mp3");
	public static final PageRef MID_OPUSLIKINAUDA	= createRes(Subdomain.PRIVATE, "music/OpusLikiNauda.mid", "OpusLikiNauda.mid");
	public static final PageRef MP3_SUMMERCAVESUNRISE	= createRes(Subdomain.PRIVATE, "music/Summer Cave Sunrise.mp3", "Summer Cave Sunrise");
	public static final PageRef MP3_UNJARREDEGERMANIUM	= createRes(Subdomain.PRIVATE, "music/Un Jarre de Germanium (2024-03-12).mp3", "Un Jarre de Germanium");
	public static final PageRef MP3_BADELIEDCHEN	= createRes(Subdomain.PRIVATE, "music/Hans Diefenbach - Badeliedchen (instrumental).mp3", 		"Badeliedchen");
	public static final PageRef MP3_SCHLAFLIEDCHEN	= createRes(Subdomain.PRIVATE, "music/Hans Diefenbach - Schlafliedchen (instrumental).mp3", 	"Schlafliedchen");

	public static final PageRef COMPSONGS		= createRes(Subdomain.PRIVATE, "quotes/compsngs.txt", "Computer related songs");

	public static final PageRef POEMALEAT_DYNAMIC = createPage(Subdomain.GEDICHT, "poemsaldyn", "poemsaldyn", "index.jsp", "Gedichte Web", null, "German Poems web", "/resources/Sitemap.Aleatoric.png", null);

	public static final PageRef DSA_HOME 		= createPage(Subdomain.RPG, "rpgindex",   "rpg",				"index.html", 	"Übersicht", null, "Overview", "/resources/Sitemap.RPG.png", null);
	public static final PageRef DSA_CHAR 		= createPage(Subdomain.RPG, "dsachar",    "rpg",				"dsachar.html", "DSA Charaktere", null, "Dark Eye character types", null, null);
	public static final PageRef DSA_TOOL_APP 	= createPage(Subdomain.RPG, "dsatoolapp", "rpg dsatool app",	"dsatool.html", "DSA-Tool App", null, "The Dark Eye tool app", "/resources/Sitemap.DSATool.App.png", null);
	public static final PageRef DSA_URSUL 		= createPage(Subdomain.RPG, "ursulshort", "rpg", 				"ursulshort/ursulshort.html", "Ursuls Hort", null, "Ursul's hoard", "/resources/Sitemap.Adventure.UrsulsHort.png", null);
	public static final PageRef DSA_DRACHENTAL	= createPage(Subdomain.RPG, "drachental", "rpg dsaDrachental","drachental/drachental.html", "Das Drachental", null, "Dragons' valley", "/resources/Sitemap.Adventure.Drachental.png", null);
	public static final PageRef DSA_OPUSLIKINAUDA = createWithFlavor(PageRef.DSA_DRACHENTAL, null, "LOpusLikiNauda", "Opus Liki Nauda", null);
	public static final PageRef DSA_WIZVALLEY	= createPage(Subdomain.RPG, "talderzauberer", "rpg", 				"talderzauberer.html", "Tal der Zauberer", null, "Wizards' valley", "/resources/Sitemap.Adventure.WizValley.png", null);
	public static final PageRef DSA_RUMRAGH 	= createPage(Subdomain.RPG, "rumragh",    "rpg", 				"rumragh.html", "Held Rumragh", null, "Hero Rumragh", "/resources/Sitemap.Character.Dwarf.png", null);
	public static final PageRef DSA_HESINOR 	= createPage(Subdomain.RPG, "hesinor",    "rpg", 				"hesinor.html", "Held Hesinor", null, "Hero Hesinor", "/resources/Sitemap.Character.Human.png", null);

	public static final PageRef DSA_TOOL_WEB 	= createPage(Subdomain.DSATOOL, "dsatoolweb",  "rpg dsatool web", "index.jsp", "DSA-Tool Web", null, "The Dark Eye tool web", "/resources/Sitemap.DSATool.Web.png", null);
	public static final PageRef DSA_TOOL_HELP 	= createPage(Subdomain.DSATOOL, "dsatoolhelp", "rpg dsatool web help", "help.jsp", "Hilfe", null, "Help", "/de/ped/tools/resources/Action.Help.png", null);

	public static final PageRef JAVA_COM		= createExt("http://www.java.com", "www.java.com");
	public static final PageRef ALTEN			= createExt("https://de.alten.com", "ALTEN");
	public static final PageRef ICT				= createExt("http://www.ict.eu", "ICT");
	public static final PageRef MCC				= createExt("https://loewensteinmedical.de/", "MCC");
	public static final PageRef ROCKWELL		= createExt("https://www.rockwellautomation.com/rockwellsoftware/overview.page", "Rockwell Automation");
	public static final PageRef IRC				= createExt("http://en.wikipedia.org/wiki/Internet_Relay_Chat", "<span lang=\"en\">internet relay chat</span>");
	public static final PageRef MUD				= createExt("http://www.mud.de/", "MUD");
	public static final PageRef HOERWERK		= createExt("https://www.backstagepro.de/hoerwerk", "Hörwerk");
	public static final PageRef HOERWERK_YT		= createExt("https://www.youtube.com/@horwerkcoverband7342", "Hörwerk Youtube Channel");

	public static final PageRef JANTIFF			= createExt("http://www.jantiff.de/", "Jantiff");
	public static final PageRef NIKI			= createExt("http://www.wnop.de", "Nic");
	public static final PageRef NIKI_GEDICHTE	= createExt("http://www.wnop.de/lit.html", "Gedankengießerei");
	public static final PageRef NIKI_RPG		= createExt("http://www.wnop.de/rpg.php", "Chronik einer Odyssee&nbsp;&ndash; oder: Die Leiden des jungen GM");
	public static final PageRef ALEX			= createExt("http://www.alexanderdiefenbach.de", "Alexander");
	public static final PageRef EASTFRIESLAND	= createExt("http://www.ostfriesland.de", "Ostfriesland", "Eastfriesland");
	public static final PageRef SPEKTRUM 		= createExt("https://www.spektrum.de/", "Spektrum der Wissenschaft", "https://www.scientificamerican.com/", "Scientific American");

	/* creates a page ref for an internal web page. */
	public static PageRef createPage(final Subdomain subdomain, final String id, final String classes,
			final String link, final String title, final String link_en_US, final String title_en_US,
			final String titleImg, final String anchor) {
		return new PageRef(subdomain, id, classes, link, title, link_en_US, title_en_US, HTMLFlavor.HTML5,
				TextDocumentType.CHARSET_UTF_8, titleImg, null, anchor);
	}

	/* creates a page ref for resources available in one version for all languages. */
	public static PageRef createRes(final Subdomain subdomain, final String link, final String title) {
		return new PageRef(subdomain, null, null, HtmlHelper.urlify(link), title, null, null, null, null, null, null, null);
	}

	/* creates a page ref for resources available in one version for all languages. */
	public static PageRef createRes(final Subdomain subdomain, final String link, final String title, final String title_en_US) {
		return new PageRef(subdomain, null, null, HtmlHelper.urlify(link), title, null, title_en_US, null, null, null, null, null);
	}

	/* creates a page ref for resources available in both languages. */
	public static PageRef createRes(final Subdomain subdomain, final String link, final String title, final String link_en_US, final String title_en_US) {
		return new PageRef(subdomain, null, null, HtmlHelper.urlify(link), title, HtmlHelper.urlify(link_en_US), title_en_US, null, null, null, null, null);
	}

	/* creates a page ref to a web page with parameters and/or some anchor within the web page. */
	protected static PageRef createWithFlavor(final PageRef page, final String params, final String anchor, final String title, final String title_en_US) {
		final boolean isWithoutTitle = ((null == title) && (null == title_en_US));
		return new PageRef(page.subdomain, page.id, page.bodyClasses, page.link, 
				(isWithoutTitle ? page.title : title), 
				page.link_en_US, 
				(isWithoutTitle ? page.title_en_US : title_en_US), 
				page.htmlFlavor, page.charset, page.titleImg, params, anchor);
	}

	/* creates a page ref to a web page with parameters and/or some anchor within the web page. */
	protected static PageRef createWithFlavor(final PageRef page, final String params, final String anchor) {
		return new PageRef(page.subdomain, page.id, page.bodyClasses, page.link, page.title, page.link_en_US, page.title_en_US, page.htmlFlavor, page.charset, page.titleImg, params, anchor);
	}

	/* creates a page ref to an external link that exists only in one language. */
	public static PageRef createExt(final String link, final String title) {
		return createExt(link, title, null, null);
	}

	/* creates a page ref to an external link that exists only once for all languages, but with different names in the link. */
	public static PageRef createExt(final String link, final String title, final String title_en_US) {
		return createExt(link, title, null, title_en_US);
	}

	/* creates a page ref to an external link that might differ for the available languages. */
	public static PageRef createExt(final String link, final String title, final String link_en_US, final String title_en_US) {
		final String teu = (CollectionUtil.isNullOrEmpty(title_en_US) ? title : title_en_US);
		return new PageRef(Subdomain.EXTERNAL, null, null, link, title, link_en_US, teu, null, null, null, null, null);
	}

	/**
	 * Constructor.
	 * 
	 * @param subdomain   the subdomain the destination page belongs to
	 * @param id          the unique id to refer to this page
	 * @param classes     the CSS classes the body tag should get when a page
	 *                    fitting for this PageRef will be created
	 * @param link        the filename relative to the root of the subdomain
	 * @param title       the title to display in the navigation tree for the reader
	 *                    of the page in German
	 * @param link_en_US  the filename relative to the root of the subdomain
	 * @param title_en_US the title to display in the navigation tree for the reader
	 *                    of the page in English
	 * @param htmlFlavor  the type of HTML, or <code>null</code> if the referred
	 *                    page is no HTML at all.
	 * @param titleImg    the title image for this page, or <code>null</code> if
	 *                    none
	 * @param params      the parameters to provide to the page, or
	 *                    <code>null</code> if none
	 * @param anchor      the anchor to directly navigate to on the target page, or
	 *                    <code>null</code> if none
	 * @param charset     the charset the page is in
	 */
	protected PageRef(final Subdomain subdomain, final String id, final String classes, final String link,
			final String title, final String link_en_US, final String title_en_US, final HTMLFlavor htmlFlavor,
			final String charset, final String titleImg, final String params, final String anchor) {
		this.subdomain = subdomain;
		this.id = id;
		this.bodyClasses = classes;
		this.link = link;
		this.title = title;
		this.link_en_US = link_en_US;
		this.title_en_US = title_en_US;
		this.htmlFlavor = htmlFlavor;
		this.titleImg = titleImg;
		this.params = params;
		this.anchor = anchor;
		this.charset = charset;
	}

	public Subdomain getSubdomain() {
		return subdomain;
	}

	public boolean isMultiLingual() {
		return ((null != link) && (null != link_en_US));
	}

	public String getLink(final RootBean.Language language) {
		String ret;
		if (((RootBean.Language.ENGLISH == language) && (null != link_en_US)) || (null == link)) {
			ret = link_en_US;
		} else {
			ret = link;
		}
		if (!CollectionUtil.isNullOrEmpty(params)) {
			ret += params;
		}
		if (!CollectionUtil.isNullOrEmpty(anchor)) {
			ret += "#" + anchor;
		}
		return ret;
	}

	public String getTitle(final RootBean.Language language) {
		final String ret;
		if (((RootBean.Language.ENGLISH == language) && (null != title_en_US)) || (null == title)) {
			ret = title_en_US;
		} else {
			ret = title;
		}
		return ret;
	}

	/** The subdomain the destination page belongs to. */
	private final Subdomain subdomain;
	/** The unique id (unique with respect to a page, not param or anchor) to refer to this page. Currently unused. */
	public final String id;
	/** The classes the body tag will get when {@link RootBean#bodyStart()} will be called. */
	public final String bodyClasses;
	/** The filename relative to the root of the subdomain for the German page. */
	public final String link;
	/** The title to display in the navigation tree for the reader of the page in German. */
	private final String title;
	/** The filename relative to the root of the subdomain for the English page. */
	public final String link_en_US;
	/** The title to display in the navigation tree for the reader of the page in English. */
	private final String title_en_US;
	/** the type of HTML, or <code>null</code> if the referred page is no HTML at all. */
	public final HTMLFlavor htmlFlavor;
	/* The title image for this page, or <code>null</code> if none. */
	public final String titleImg;
	/* The parameters of the page, or <code>null</code> if none. */
	protected final String params;
	/* The anchor to directly navigate to on the target page, or <code>null</code> if none. */
	protected final String anchor;
	/* The charset of this page. */
	public final String charset;
}
