package de.ped.tools.web;

/**
 * The different subdomains my web site consists of.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach
 */
public enum Subdomain {
	PRIVATE   (Subdomain.ICON_URL_PRIVATE, Subdomain.CSS_PATH_PRIVATE, "/",            ".",          true,  true),
	PROFESSION(Subdomain.ICON_URL_PRIVATE, Subdomain.CSS_PATH_PRIVATE, "/profession/", "profession", true,  true),
	RPG       (Subdomain.ICON_URL_RPG,     Subdomain.CSS_PATH_PRIVATE, "/rpg/",        "rpg",        true,  true),
	DSATOOL   (Subdomain.ICON_URL_RPG,     Subdomain.CSS_PATH_PRIVATE, "/dsatool/",    "rpg",        true,  false),
	GEDICHT   (Subdomain.ICON_URL_PRIVATE, Subdomain.CSS_PATH_PRIVATE, "/gedicht/",     ".",         true,  false),
	ADVENT    (Subdomain.ICON_URL_PRIVATE, Subdomain.CSS_PATH_PRIVATE, "/advent/",      ".",         false, false),
	MOTIVATION(Subdomain.ICON_URL_PRIVATE, Subdomain.CSS_PATH_PRIVATE, "/motivation/",  ".",         false, false),

	/**
	 * Fake subdomain to describe that a link leads to the outside world. 
	 * Hint to build a complete path and display it differently.
	 */
    EXTERNAL(null,                       null,                       "",            null,      false, false);

	private static final String ICON_URL_PRIVATE = "/resources/Sitemap.png";
	private static final String ICON_URL_RPG     = "/resources/Sitemap.RPG.png";

	private static final String CSS_PATH_PRIVATE = "/defines.css";

	private Subdomain(
			final String iconUrl, 
			final String cssPath, 
			final String prefix, 
			final String sourceJspPath, 
			final boolean isWithNav,
			final boolean isSearchableForRobots) {
		this.iconUrl = iconUrl;
		this.prefix = prefix;
		this.sourceJspPath = sourceJspPath; /** Where to find the JSPs the pages from this domain are created from */
		this.cssPath = cssPath;
		this.isWithNav = isWithNav;
		this.isSearchableForRobots = isSearchableForRobots;
	}

	public final String iconUrl;
	public final String cssPath;
	public final String prefix;
	public final String sourceJspPath;
	public final boolean isWithNav;
	public final boolean isSearchableForRobots;

	/**
	 * Returns the prefix to the URL to refer to an item residing in the destination subdomain.
	 * @param destinationSubdomain the subdomain the URL to reach resides in
	 * @return the prefix to the URL to refer to an item residing in the destination subdomain.
	 */
	public String getPrefixTo(final Subdomain destinationSubdomain) {
		if (this.equals(destinationSubdomain)) {
			return "";
		} else {
			return destinationSubdomain.prefix;
		}
	}
}
