package de.ped.troff.client.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import de.ped.tools.CollectionUtil;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.PropertyElement;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;

/**
 * The dialog selecting which group on the server to connect to.
 * <p>
 * Title: Server (Server name)
 * <table>
 *   <caption>Groups (click to join)</caption>
 *   <tr><th>Name</th><th># of players</th></tr>
 *   <tr><td>... </td><td>...         </td></tr>
 * </table>
 * New Group
 * <br>
 * Text Field
 * <br>
 * Button Refresh
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GroupSelectionDialog extends PedJDialog implements ActionListener, ItemListener, TableModelListener {
	private static final long serialVersionUID = 1L;

	protected final Logger logger = Logger.getRootLogger();

	protected GroupSelectionModel model;
	protected String selectedGroup;

	protected JTextField newGroupTextField = new JTextField(20);
	protected JButton refreshButton;
	protected JTable table;

	protected GroupSelectionDialog(
			final ApplicationMainWindow mainWindow, 
			final GroupSelectionModel model) {
		super(mainWindow, mainWindow);
		setTitle(getMessages().getString("GroupSelectionWindow.Title"));
		this.logger.setLogLevel(Level.INFO);
		this.model = model;
		this.selectedGroup = model.getPreselectedGroupName();
		final GridBagLayout mainLayout = new GridBagLayout();
		final JPanel mainPanel = createMainPanel();
		mainPanel.setLayout(mainLayout);
		final GridBagConstraints mainConstraints = new GridBagConstraints();
		mainConstraints.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainConstraints.gridwidth = GridBagConstraints.REMAINDER; // end row
		mainConstraints.fill = GridBagConstraints.HORIZONTAL;
		mainWindow.addLabelAndComponentTo(mainPanel, mainConstraints, 
				new PropertyElement("GroupSelectionWindow.GroupsList", null));
		table = new JTable(model);
		table.getColumnModel().getColumn(GroupSelectionModel.COLUMN_INDEX_GROUPNAME).setPreferredWidth(10);
		table.getColumnModel().getColumn(GroupSelectionModel.COLUMN_INDEX_NUM_OF_PLAYERS).setPreferredWidth(5);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		model.addTableModelListener(this);
		final JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setMinimumSize(new Dimension(100, 50));
		// table.setFillsViewportHeight(true);
		mainConstraints.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
		mainConstraints.fill = GridBagConstraints.HORIZONTAL;
		mainLayout.setConstraints(scrollPane, mainConstraints);
		mainPanel.add(scrollPane);
		mainConstraints.gridheight = 1;
		mainWindow.addLabelAndComponentTo(mainPanel, mainConstraints, 
				new PropertyElement("GroupSelectionWindow.NewGroup", newGroupTextField));
		mainPanel.add(newGroupTextField);
		newGroupTextField.addActionListener(this);

		refreshButton = mainWindow.createButton("GroupSelectionWindow.Refresh");
		addButton(refreshButton);

		getContentPane().add(mainPanel, BorderLayout.CENTER);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);
		finishLayout();
		updateUIFieldsBasedOnProperties();
	}

	public String getSelectedGroup() {
		return selectedGroup;
	}

	/** 
	 * Invoked when an action occurs. 
	 * @param e the event to handle
	 */
	@Override
	public void actionPerformed(final ActionEvent e) {
		logger.debug(e.paramString());
		if (e.getSource() == newGroupTextField) {
			// Enforce to use the new group, even if an existing group is
			// selected in table.
			model.setNewGroupName(newGroupTextField.getText());
			table.clearSelection();
			// Pressing return in a text field should be handled as pressing the
			// OK button
			readUIFieldsIntoProperties();
			onOk();
		} else if (e.getSource() == getButtonOk()) {
			model.setNewGroupName(newGroupTextField.getText());
			readUIFieldsIntoProperties();
			onOk();
		} else if (e.getSource() == getButtonCancel()) {
			logger.debug("[Cancel]");
			onCancel();
		} else if (e.getSource() == refreshButton) {
			logger.debug("[Refresh]");
		}
	}

	/** 
	 * Handles the item state changes of the checkable menu items. 
	 * @param e the event to handle
	 */
	@Override
	public void itemStateChanged(final ItemEvent e) {
		logger.debug(this.getClass().getSimpleName() + " itemStateChanged " + e.paramString());
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		updateView();
	}

	@Override
	public void readUIFieldsIntoProperties() {
		logger.debug("[OK]: Selected row = " + table.getSelectedRow());
		model.setSelectedRow(table.getSelectedRow());
		selectedGroup = model.getSelectedGroupName();
	}

	public void updateView() {
		final String newGroupText = newGroupTextField.getText();
		if (!CollectionUtil.isNullOrEmpty(newGroupText)) {
			model.setNewGroupName(newGroupText);
		}
	}

	@Override
	public void tableChanged(final TableModelEvent e) {
		final int row = e.getFirstRow();
		final int column = e.getColumn();
		final TableModel model = (TableModel) e.getSource();
		// String columnName = model.getColumnName(column);
		final Object data = model.getValueAt(row, column);
		logger.debug("TableChanged: " + data);
	}
}
