package de.ped.troff.client.gui;

import javax.swing.table.AbstractTableModel;

import de.ped.tools.log.Logger;
import de.ped.troff.middleware.GroupInfo;
import de.ped.troff.middleware.ServerInfo;

/**
 * Represents the table content for the table of groups.
 * After all the rows containing the existing groups,
 * one more row is added where the group name is editable to create a new group.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GroupSelectionModel extends AbstractTableModel {
	private static final long serialVersionUID = 1L;

	protected static final boolean IS_TO_ADD_EMPTY_ROW_FOR_NEW_GROUP = true; 

	protected static final int COLUMN_INDEX_GROUPNAME = 0;
	protected static final int COLUMN_INDEX_NUM_OF_PLAYERS = 1;

	protected final Logger logger = Logger.getRootLogger();

	protected final String[] headings;

	protected ServerInfo data;

	protected String newGroupName;

	protected int selectedRow;

	public GroupSelectionModel(final ServerInfo data, final String nameHeading, final String numberHeading) {
		this.data = data;
		headings = new String[] { nameHeading, numberHeading };
	}

	public String getNewGroupName() {
		return newGroupName;
	}

	public void setNewGroupName(final String newGroupName) {
		this.newGroupName = newGroupName;
	}

	public int getSelectedRow() {
		return selectedRow;
	}

	public void setSelectedRow(final int row) {
		selectedRow = row;
	}

	@Override
	public int getColumnCount() {
		return 2;
	}

	@Override
	public int getRowCount() {
		int ret = data.getGroups().size();
		if (IS_TO_ADD_EMPTY_ROW_FOR_NEW_GROUP) {
			ret += 1;
		}
		return ret;
	}

	@Override
	public String getColumnName(final int col) {
		return headings[col];
	}

	@Override
	public Class<?> getColumnClass(final int col) {
		switch(col) {
		case COLUMN_INDEX_GROUPNAME: 
			return String.class;
		case COLUMN_INDEX_NUM_OF_PLAYERS: 
			return Integer.class;
		}
		return null;
	}

	@Override
	public Object getValueAt(final int row, final int col) {
		if (row < data.getGroups().size()) {
			final GroupInfo groupInfo = data.getGroups().get(row);
			switch(col) {
			case COLUMN_INDEX_GROUPNAME: 
				return (getPreselectedRowIndex() == col ? ">" : "_") + groupInfo.getName(); 
			case COLUMN_INDEX_NUM_OF_PLAYERS: 
				return Integer.valueOf(groupInfo.getNumberOfPlayers());
			}
		} else if ((IS_TO_ADD_EMPTY_ROW_FOR_NEW_GROUP) && (row == data.getGroups().size())) {
			switch(col) {
			case COLUMN_INDEX_GROUPNAME: 
				return newGroupName; 
			case COLUMN_INDEX_NUM_OF_PLAYERS: 
				return null;
			}
		}
		return null;
	}

	@Override
	public void setValueAt(final Object value, final int row, final int col) {
		if ((row >= data.getGroups().size()) && (col == COLUMN_INDEX_GROUPNAME)) {
			setNewGroupName(value.toString()); 
		}
	}

	@Override
	public boolean isCellEditable(final int row, final int col) {
		if ((row >= data.getGroups().size()) && (col == COLUMN_INDEX_GROUPNAME)) {
			return true;
		}
		return false;
	}

	protected int getPreselectedRowIndex() {
		int ret = -1;
		final String groupOfClient = data.getGroupOfClient(); 
		if (groupOfClient != null) {
			for (int i = 0; i < data.getGroups().size(); i++) {
				if (groupOfClient.equals(data.getGroups().get(i).getName())) {
					ret = i;
					break;
				}
			}
		}
		return ret;
	}

	public String getPreselectedGroupName() {
		return data.getGroupOfClient(); 
	}

	public String getSelectedGroupName() {
		String selectedGroup;
		if (selectedRow < 0) {
			selectedGroup = getNewGroupName();
			if (null == selectedGroup) {
				selectedGroup = getPreselectedGroupName();
			}
		} else {
			selectedGroup = (String) getValueAt(selectedRow, COLUMN_INDEX_GROUPNAME);
		}
		logger.debug("selectedGroupName=" + selectedGroup);
		return selectedGroup;
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + "(" + getSelectedGroupName() + ")";
	}
}
