package de.ped.troff.client.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.gui.PropertyElement;
import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;
import de.ped.troff.middleware.ServerSelectionModel;

/**
 * The dialog selecting which server to connect to.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ServerSelectionDialog extends PedJDialog implements ActionListener, ItemListener {
	private static final long serialVersionUID = 1L;

	protected final Logger logger = Logger.getRootLogger();

	private ServerSelectionModel model = new ServerSelectionModel();

	private JCheckBox isServerCheckbox = new JCheckBox();
	private JTextField serverPortNo = new JTextField(5);
	private JTextField serverName   = new JTextField(25);

	private static class HostTypeRadioButton extends JRadioButton {
		private static final long serialVersionUID = 1L;
		private final ServerSelectionModel.HostType hostType;
		private HostTypeRadioButton(final ServerSelectionModel.HostType hostType) {
			this.hostType = hostType;
		}
	}
	
	private HostTypeRadioButton[] hostTypeRadioButtons = new HostTypeRadioButton[ServerSelectionModel.HostType.values().length];
	private JTextField ownHostName = new JTextField(30);
	private JTextField ownHostPortNo = new JTextField(5);
	private JTextField clientName = new JTextField(25);
	private JTextField numberOfPlayers = new JTextField(1);

	private PropertyElement[] serverProps;
	private PropertyElement[] clientProps;

	protected ServerSelectionDialog(
			final ApplicationMainWindow mainWindow, 
			final ServerSelectionModel model) {
		super(mainWindow, mainWindow);
		final Messages messages = getMessages();
		this.logger.setLogLevel(Level.INFO);
		this.model = (ServerSelectionModel) model.clone();
		setTitle(messages.getString("ServerSelectionWindow.Title"));
		final GridBagLayout mainLayout = new GridBagLayout();
		final JPanel mainPanel = createMainPanel();
		mainPanel.setLayout(mainLayout);
		final GridBagConstraints mainConstraints = new GridBagConstraints();
		mainConstraints.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainConstraints.gridwidth = GridBagConstraints.REMAINDER; // end row
		mainConstraints.fill = GridBagConstraints.HORIZONTAL;
		{
			serverProps = new PropertyElement[] {
					new PropertyElement("ServerSelectionWindow.ServerPanel.IsServerLabel", isServerCheckbox),
					new PropertyElement("ServerSelectionWindow.ServerPanel.PortNumberLabel", serverPortNo),
					new PropertyElement("ServerSelectionWindow.ServerPanel.ServerNameLabel", serverName), };
			isServerCheckbox.addItemListener(this);

			final GridBagLayout borderLayout = new GridBagLayout();
			final JPanel borderPanel = new JPanel(borderLayout);
			final GridBagConstraints borderConstraints = new GridBagConstraints();
			borderConstraints.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
			borderConstraints.gridwidth = GridBagConstraints.REMAINDER; // end row
			borderConstraints.fill = GridBagConstraints.HORIZONTAL;

			// GridBagLayout positioningLayout = new GridBagLayout();
			final JPanel positioningPanel = new JPanel();
			positioningPanel.setLayout(new GridBagLayout());
			borderLayout.setConstraints(positioningPanel, borderConstraints);
			borderPanel.add(positioningPanel);
			final GridBagConstraints positioningConstraints = new GridBagConstraints();
			positioningConstraints.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
			positioningConstraints.gridwidth = GridBagConstraints.REMAINDER; // end row
			positioningConstraints.fill = GridBagConstraints.HORIZONTAL;

			// borderPanel.add(isServerCheckbox);
			for (int i = 0; i < serverProps.length; i++) {
				final PropertyElement pe = serverProps[i];
				getMainWindow().addPropertyElementTo(positioningPanel, pe, positioningConstraints);
			}

			final String title = messages.getText("ServerSelectionWindow.ServerPanel");
			borderPanel.setBorder(BorderFactory.createTitledBorder(title));
			mainLayout.setConstraints(borderPanel, mainConstraints);
			mainPanel.add(borderPanel);
		}
		{
			final ButtonGroup localOrExternalServerButtonGroup = new ButtonGroup();
			for (ServerSelectionModel.HostType hostType : ServerSelectionModel.HostType.values()) {
				final HostTypeRadioButton btn = new HostTypeRadioButton(hostType);
				hostTypeRadioButtons[hostType.ordinal()] = btn;
				localOrExternalServerButtonGroup.add(btn);
				btn.addItemListener(this);
			}
			clientProps = new PropertyElement[] {
					new PropertyElement(ServerSelectionModel.HostType.KEY + "." + hostTypeRadioButtons[0].hostType, hostTypeRadioButtons[0]),
					new PropertyElement(ServerSelectionModel.HostType.KEY + "." + hostTypeRadioButtons[1].hostType, hostTypeRadioButtons[1]),
					new PropertyElement(ServerSelectionModel.HostType.KEY + "." + hostTypeRadioButtons[2].hostType, hostTypeRadioButtons[2]),
					new PropertyElement(ServerSelectionModel.HostType.KEY + "." + hostTypeRadioButtons[3].hostType, hostTypeRadioButtons[3]),
					new PropertyElement(ServerSelectionModel.HostType.KEY + "." + hostTypeRadioButtons[4].hostType, hostTypeRadioButtons[4]),
					new PropertyElement("ServerSelectionWindow.ClientPanel.HostNameLabel", ownHostName),
					new PropertyElement("ServerSelectionWindow.ClientPanel.PortNumberLabel", ownHostPortNo),
					new PropertyElement("ServerSelectionWindow.ClientPanel.ClientNameLabel", clientName),
					new PropertyElement("ServerSelectionWindow.ClientPanel.NumberOfPlayers", numberOfPlayers),
			};

			final JPanel borderPanel = new JPanel(new GridBagLayout());
			final GridBagConstraints c = new GridBagConstraints();
			c.anchor = GridBagConstraints.WEST; // All elements are left bound
												// to their space
			c.gridwidth = GridBagConstraints.REMAINDER; // end row
			for (int i = 0; i < clientProps.length; i++) {
				final PropertyElement pe = clientProps[i];
				getMainWindow().addPropertyElementTo(borderPanel, pe, c);
			}

			final String title = messages.getText("ServerSelectionWindow.ClientPanel");
			borderPanel.setBorder(BorderFactory.createTitledBorder(title));
			mainLayout.setConstraints(borderPanel, mainConstraints);
			mainPanel.add(borderPanel);
		}

		getContentPane().add(mainPanel, BorderLayout.CENTER);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);
		finishLayout();
		updateUIFieldsBasedOnProperties();
	}

	/** 
	 * Invoked when an action occurs. 
	 * @param e the event to handle
	 */
	@Override
	public void actionPerformed(final ActionEvent e) {
		logger.debug(e.paramString());
		if (e.getSource() == getButtonOk()) {
			// Pressing return in a text field should be handled as pressing the OK button
			readUIFieldsIntoProperties();
			onOk();
		} else if (e.getSource() == getButtonCancel()) {
			onCancel();
		}
	}

	/** 
	 * Handles the item state changes of the checkable menu items. 
	 * @param e the event to handle
	 */
	@Override
	public void itemStateChanged(final ItemEvent e) {
		logger.debug(this.getClass().getSimpleName() + " itemStateChanged " + e.paramString());
		boolean isToUpdateView = false;
		if ((e.getSource() == isServerCheckbox)) {
			isToUpdateView = true;
		} else {
			for (HostTypeRadioButton btn : hostTypeRadioButtons) {
				if (e.getSource() == btn) {
					isToUpdateView = true;
					break;
				}
			}
		}
		if (isToUpdateView) {
			updateView();
		}
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		isServerCheckbox.setSelected(model.isServer());
		serverPortNo.setText(Integer.toString(model.getServerPortNo()));
		serverName.setText(model.getServerName());
		final ServerSelectionModel.HostType hostType = model.getHostType();
		for (HostTypeRadioButton btn : hostTypeRadioButtons) {
			btn.setSelected(btn.hostType == hostType);
		}
		ownHostName.setText(model.getOwnHostName());
		ownHostPortNo.setText(Integer.toString(model.getOwnHostPortNo()));
		clientName.setText(model.getClientName());
		numberOfPlayers.setText(Integer.toString(model.getNumberOfPlayers()));
		updateView();
	}

	public ServerSelectionModel.HostType getLocalServerFromRadioButtons() {
		ServerSelectionModel.HostType ret = ServerSelectionModel.HostType.PORT_LOCALHOST;
		for (HostTypeRadioButton btn : hostTypeRadioButtons) {
			if (btn.isSelected()) {
				ret = btn.hostType;
				break;
			}
		}
		return ret;
	}

	@Override
	public void readUIFieldsIntoProperties() {
		model.setServer(isServerCheckbox.isSelected());
		model.setServerPortNo(Integer.parseInt(serverPortNo.getText()));
		model.setServerName(serverName.getText());
		model.setHostType(getLocalServerFromRadioButtons());
		model.setOwnHostName(ownHostName.getText());
		model.setOwnHostPortNo(Integer.parseInt(ownHostPortNo.getText()));
		model.setClientName(clientName.getText());
		model.setNumberOfPlayers(Integer.parseInt(numberOfPlayers.getText()));
	}

	public void updateView() {
		final boolean isServer = isServerCheckbox.isSelected();
		// If we are no server, setting server settings makes no sense
		serverPortNo.setEnabled(isServer);
		serverName.setEnabled(isServer);
		// If we are no server, the client part cannot connect to it
		if (!isServer && hostTypeRadioButtons[ServerSelectionModel.HostType.PORT_LOCALHOST.ordinal()].isSelected()) {
			hostTypeRadioButtons[ServerSelectionModel.HostType.PORT_LOCALHOST.ordinal()].setSelected(false);
			hostTypeRadioButtons[ServerSelectionModel.HostType.LOCALLY.ordinal()].setSelected(true);
		}
		hostTypeRadioButtons[ServerSelectionModel.HostType.PORT_LOCALHOST.ordinal()].setEnabled(isServer);

		final boolean isExternalOwn = hostTypeRadioButtons[ServerSelectionModel.HostType.PORT_EXTERNAL_OWN.ordinal()].isSelected();
		ownHostName.setEnabled(isExternalOwn);
		ownHostPortNo.setEnabled(isExternalOwn);

		final boolean isLocally = hostTypeRadioButtons[ServerSelectionModel.HostType.LOCALLY.ordinal()].isSelected();
		clientName.setEnabled(!isLocally);
		numberOfPlayers.setEnabled(!isLocally);
	}

	public ServerSelectionModel getModel() {
		return model;
	}
}
