package de.ped.troff.client.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;

import de.ped.troff.server.logic.TroffGameProperties;

/**
 * The area where all the player views are painted.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
class TroffMainCanvas extends JPanel {
	private static final long serialVersionUID = 1L;

	private final TroffMainWindow parent;

	private final int[][] layoutSizes = new int[][] {
			{1, 1}, {1, 2}, {1, 3}, {2, 2}, {2, 3}, {2, 3}, {2, 4}, {2, 4},  
	};

	private JTextPane statusLabel = new JTextPane();

	public TroffMainCanvas(final TroffMainWindow parent) {
		this.parent = parent;
		initGame();
	}

	public void initGame() {
		if ((null != parent) && (null != parent.getClientThread())) {
			final TroffGameProperties props = parent.getClientThread().getGameProperties();
			final BorderLayout frameLayout = new BorderLayout();
			setLayout(frameLayout);
			statusLabel.setText("Hallo");
			statusLabel.setEditable(false);
			statusLabel.setPreferredSize(new Dimension(100, 50));
			statusLabel.addKeyListener(parent);
			final JScrollPane textScrollPane = new JScrollPane(statusLabel);
			add(textScrollPane, BorderLayout.SOUTH);
			final JPanel playfield = new JPanel();
			add(playfield, BorderLayout.CENTER);
			if (props.isAbsoluteView()) {
				final GridLayout gridLayout = new GridLayout(1, 1, 4, 4);
				playfield.setLayout(gridLayout);
				playfield.add(new TroffPlayerViewCanvas(parent, (byte) 0));
			} else {
				final int numOfPlayers = Math.min(layoutSizes.length, props.getNumberOfPlayers());
				final int[] layoutSize = layoutSizes[numOfPlayers - 1];
				final GridLayout gridLayout = new GridLayout(layoutSize[0], layoutSize[1], 4, 4);
				playfield.setLayout(gridLayout);
				for (byte i = 0; i < numOfPlayers; i++) {
					playfield.add(new TroffPlayerViewCanvas(parent, i));
				}
			}
		}
	}

	public void setStatusText(final String text) {
		statusLabel.setText(text);
		statusLabel.repaint();
	}
}
