package de.ped.troff.client.gui;

import de.ped.tools.Messages;
import de.ped.tools.PlayerColor;
import de.ped.tools.gui.TableCellComboBoxRenderer;
import de.ped.tools.gui.TableColumnModel;
import de.ped.tools.gui.TableModel;
import de.ped.troff.server.logic.TroffGameProperties;
import de.ped.troff.server.logic.TroffPlayerProperties;

public class TroffPlayersSelectionTableModel extends TableModel {

	private static final long serialVersionUID = 1L;

	private static final TableColumnModel[] COLUMNS = new TableColumnModel[]{
		new TableColumnModel("GameProperties.Players.Number.Heading", Integer.class,           5), 
		new TableColumnModel("GameProperties.PlayerType",             TroffPlayerType.class,  25), 
		new TableColumnModel("GameProperties.PlayerMode",             TroffPlayerMode.class,  25), 
		new TableColumnModel("GameProperties.PlayerView",             TroffPlayerView.class,  35), 
		new TableColumnModel("GameProperties.Players.Color.Heading",  PlayerColor.class,      15),
		new TableColumnModel("GameProperties.PlayerName",             String.class,           25), 
	};

	static final int COL_IDX_NUMBER = 0;
	static final int COL_IDX_TYPE = 1;
	static final int COL_IDX_MODE = 2;
	static final int COL_IDX_VIEW = 3;
	static final int COL_IDX_COLOR = 4;
	static final int COL_IDX_NAME = 5;

	public TroffPlayersSelectionTableModel(
			final TroffGamePropertiesDialog parent,
			final TroffGameProperties properties, 
			final Messages msg, 
			final boolean isEditable) {
		super(msg);
		this.parent = parent;
		this.players = properties.getPlayers(); // The containing class NewGameSelectionDialog does the cloning already, so we work just on the same model.
		this.isEditable = isEditable;
	}

	@Override
	public TableColumnModel[] getColumns() {
		return COLUMNS;
	}

	@Override
	public int getRowCount() {
		return TroffGameProperties.MAX_NUMBER_OF_PLAYERS;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		logger.trace("getValueAt(" + rowIndex + "," + columnIndex + ")");
		final TroffPlayerProperties entry = players[rowIndex];
		final Object ret;
		switch(columnIndex) {
		case COL_IDX_NUMBER:
			ret = new Integer(rowIndex + 1);
			break;
		case COL_IDX_NAME:
			ret = entry.getName();
			break;
		case COL_IDX_TYPE:
			ret = TroffPlayerType.values()[entry.getTypeId()];
			break;
		case COL_IDX_COLOR:
			ret = PlayerColor.get(entry.getPlayerColorId());
			break;
		case COL_IDX_VIEW:
			ret = TroffPlayerView.instances()[entry.getViewId()];
			break;
		case COL_IDX_MODE:
			ret = TroffPlayerMode.instances()[entry.getModeId()];
			break;
		default:
			ret = "--";
		}
		return ret;
	}

	@Override
	public void setValueAt(final Object value, final int rowIndex, final int columnIndex) {
		logger.trace("setValueAt(" + rowIndex + "," + columnIndex + ")");
		final TroffPlayerProperties entry = players[rowIndex];
		switch(columnIndex) {
		case COL_IDX_NUMBER:
		case COL_IDX_COLOR:
			setPlayerColorId(rowIndex, TableCellComboBoxRenderer.getIdxOf(value));
			break;
		case COL_IDX_NAME:
			entry.setName((String) value);
			break;
		case COL_IDX_TYPE:
			entry.setTypeId(TableCellComboBoxRenderer.getIdxOf(value));
			break;
		case COL_IDX_VIEW:
			entry.setViewId(TableCellComboBoxRenderer.getIdxOf(value));
			break;
		case COL_IDX_MODE:
			entry.setModeId(TableCellComboBoxRenderer.getIdxOf(value));
			break;
		default:
			// Nothing to do
		}
	}

	@Override
	public boolean isCellEditable(final int rowIndex, final int colIndex) {
		boolean ret = false;
		if (isEditable) {
			switch(colIndex) {
			case COL_IDX_TYPE:
			case COL_IDX_MODE:
			case COL_IDX_VIEW:
			case COL_IDX_COLOR:
			case COL_IDX_NAME:
				if (((colIndex != COL_IDX_TYPE) || (rowIndex < TroffGamePropertiesDialog.MAX_NUMBER_OF_HUMAN_PLAYERS)) 
						&& ((colIndex != COL_IDX_VIEW) || (0 != parent.choiceAbsoluteView.getSelectedIndex())) 
						&& (rowIndex < parent.choiceNumberOfPlayers.getSelectedIndex() + 1)) {
					ret = true;
				}
			}
		}
		return ret;
	}

	/**
	 * Sets the colorId for a player. 
	 * But as all colors are unique among players, this swaps color with the other player previously using that color.
	 * @param playerId the player to get the new colorId
	 * @param colorId the new colorId
	 */
	protected void setPlayerColorId(final int playerId, final int colorId) {
		logger.trace("setPlayerColorId(" + playerId + "," + colorId + ")");
		final TroffPlayerProperties entry = players[playerId];
		int otherPlayerIdx = -1;
		for (int i = 0; i < players.length; i++) {
			if (players[i].getPlayerColorId() == colorId) {
				otherPlayerIdx = i;
				break;
			}
		}
		players[otherPlayerIdx].setPlayerColorId(entry.getPlayerColorId());
		entry.setPlayerColorId(colorId);
		fireTableCellUpdated(otherPlayerIdx, COL_IDX_COLOR);
	}

	private TroffGamePropertiesDialog parent;
	private final TroffPlayerProperties[] players;
	private final boolean isEditable;
}
