package de.ped.troff.client.logic;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;

import de.ped.tools.MathVector;
import de.ped.troff.server.logic.BoardInfo;

/**
 * A real 3D view implementation of the pilot's view
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Board3D {
//	public static final int DIMENSIONS = 3;

	private ArrayList<Cube3D> cubes;

	public static final Color[] PLAYER_COLORS = new Color[] {
			Color.red, Color.magenta, Color.green, Color.yellow, 
			Color.cyan, Color.blue, Color.pink, Color.orange,
	};

	public static final Color[] WALL_COLORS = new Color[] {
			Color.black, Color.black, Color.gray, Color.white,
	};

	public static Color fieldColor(final int fieldValue, final BoardInfo.FieldInfo fieldInfo) {
		Color c;
		if (0 <= fieldValue) {
			final int idx = fieldValue;
			c = PLAYER_COLORS[idx];
			if ((null != fieldInfo) && fieldInfo.isHead()) {
				// c = PLAYER_COLORS[idx].brighter().brighter();
				c = Color.orange;
			}
		} else {
			c = WALL_COLORS[-fieldValue];
		}
		return c;
	}

	public Board3D(final BoardInfo boardInfo) {
		cubes = new ArrayList<Cube3D>();
		for (int xboard = -1; xboard <= 1; xboard++) {
			final int xboardOffset = xboard * boardInfo.getSize().getWidth();
			for (int yboard = -1; yboard <= 1; yboard++) {
				final int yboardOffset = yboard * boardInfo.getSize().getHeight();
				for (int x = 0; x < boardInfo.getSize().getWidth(); x++) {
					for (int y = 0; y < boardInfo.getSize().getHeight(); y++) {
						final BoardInfo.FieldInfo fieldInfo = boardInfo.calcFieldInfoAt(x, y);
						final int value = fieldInfo.getField();
						final Cube3D cube = new Cube3D(
								xboardOffset + x + 0.5, 
								yboardOffset + y + 0.5, 0.5, 0.9);
						cube.setFieldInfo(fieldInfo);
						cube.setColor((value == BoardInfo.FREE ? Color.darkGray : fieldColor(value, null)));
						cubes.add(cube);
					}
				}
			}
		}
	}

	public ArrayList<Cube3D> getCubes() {
		return cubes;
	}

	public double calcZ(final Cube3D.Rectangle3D rect, final HashMap<MathVector, MathVector> trans) {
		double ret = 0.0;
		for (final MathVector vertex : rect.vertexes) {
			MathVector transformedVertex = trans.get(vertex);
			ret += transformedVertex.get(MathVector.Z);
		}
		ret /= 4.0;
		return ret;
	}

	public void sortZ(final ArrayList<Cube3D.Rectangle3D> rects, final HashMap<MathVector, MathVector> trans) {
		for (int i = 1; i < rects.size() - 1; i++) {
			for (int j = 0; j < i; j++) {
				final double zi = calcZ(rects.get(i), trans);
				final double zj = calcZ(rects.get(j), trans);
				if (zi < zj) {
					final Cube3D.Rectangle3D buf = rects.get(i);
					rects.set(i, rects.get(j));
					rects.set(j, buf);
				}
			}
		}
	}
}
