package de.ped.troff.client.logic;

/**
 * Approximates a circle of given size to integer coordinates.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Circle {

	public static final int UNDEFINED = -1;

	public static final double SQRT2 = Math.sqrt(2.0);

	public static int[] halfCircle(final double size) {
		final int[] ret = new int[(int) Math.ceil(size)];
		for (int i = 0; i < ret.length; i++) {
			// x*x + y*y = s*s
			// x*x = s*s - y*y
			final double xsize = Math.sqrt(size * size - i * i);
			ret[i] = (int) Math.ceil(xsize);
		}
		return ret;
	}

	public static double getDistance(final int xdistance, final int ydistance) {
		if (xdistance > ydistance) {
			return getDistance(ydistance, xdistance);
		}
		if (xdistance == 0) {
			return ydistance;
		}
		if (xdistance == ydistance) {
			return SQRT2 * Math.abs(xdistance);
		}
		return Math.sqrt(xdistance * xdistance + ydistance * ydistance);
	}

	private final double size;
	private final int[] xposes;

	public Circle(final double size) {
		if (size < 0.0) {
			throw new IllegalArgumentException("Negative circle size");
		}
		this.size = size;
		xposes = halfCircle(size);
	}

	public double getSize() {
		return size;
	}

	public int size() {
		return xposes.length;
	}

	public int getX(final int y) {
		final int halfCircleIndex = Math.abs(y);
		if (halfCircleIndex < xposes.length) {
			return xposes[halfCircleIndex];
		} else {
			return UNDEFINED;
		}
	}
}
