package de.ped.troff.client.logic;

import java.awt.Color;

/**
 * Some useful color mixing routines.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ColorUtil {

	/**
	 * Calculates a value between two others.
	 * @param val1 the first value to mix
	 * @param val2 the second value to mix
	 * @param val2percentage the percentage of the second value. Should be between 0 and 100
	 * @return the value of the mix
	 */
	public static int mix(final int val1, final int val2, final int val2percentage) {
		return (val1 * (100 - val2percentage) + val2 * val2percentage) / 100;
	}

	/**
	 * Mixes two color values.
	 * @param rgb1 the first RGB color value
	 * @param rgb2 the second RGB color value
	 * @param val2percentage the percentage of the second value. Should be between 0 and 100
	 * @return the mixed RGB color value
	 */
	public static int mixRGB(final int rgb1, final int rgb2, final int val2percentage) {
		final int r1 = ((rgb1 >> 16) & 0xff);
		final int g1 = ((rgb1 >> 8) & 0xff);
		final int b1 = ((rgb1 >> 0) & 0xff);
		final int r2 = ((rgb2 >> 16) & 0xff);
		final int g2 = ((rgb2 >> 8) & 0xff);
		final int b2 = ((rgb2 >> 0) & 0xff);
		final int r = mix(r1, r2, val2percentage);
		final int g = mix(g1, g2, val2percentage);
		final int b = mix(b1, b2, val2percentage);
		final int ret = (rgb1 & 0xff000000) | (r << 16) | (g << 8) | b;
		return ret;
	}

	public static Color rgbToColor(final int rgb) {
		final int val = rgb & 0xffffff;
		final String str = "0x" + Integer.toHexString(val);
		final Color ret = Color.decode(str);
		return ret;
	}

	public static final int FADE_RGB = 0x202020;

	/**
	 * Fades a color value.
	 * @param argb the R, G or B color value
	 * @param distance the distance of the color - the more, the more fading
	 * @return the faded RGB color value
	 */
	public static int fade(final int argb, final int distance) {
		return mixRGB(argb, FADE_RGB, 15 * distance);
	}

	/**
	 * Fades a color value.
	 * @param c the color to fade
	 * @param distance the distance of the color - the more, the more fading
	 * @return the faded color
	 */
	public static Color fade(final Color c, final int distance) {
		return rgbToColor(fade(c.getRGB(), distance));
	}
}
