package de.ped.troff.client.logic;

import java.io.IOException;
import java.net.UnknownHostException;

import de.ped.tools.log.Logger;
import de.ped.troff.client.gui.TroffMainWindow;
import de.ped.troff.middleware.ServerSelectionModel;
import de.ped.troff.server.TroffServerThread;

/**
 * Bundles server thread and client thread, so they can be stopped, replaced and started based on the settings.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ThreadBundle {
	protected final Logger logger;

	/** Defines the parameters the threads run in, or if they run at all. */
	protected ServerSelectionModel serverSelectionModel;

	/** Client thread */
	protected ClientThread clientThread;

	/** Server thread */
	protected TroffServerThread serverThread;

	/** Root object, client thread needs to know it to update graphics etc */
	protected TroffMainWindow parent;

	public ThreadBundle(final TroffMainWindow parent) {
		this.parent = parent;
		logger = parent.logger;
	}

	public ServerSelectionModel getServerSelectionModel() {
		return serverSelectionModel;
	}

	public void setServerSelectionModel(final ServerSelectionModel serverSelectionModel)
			throws UnknownHostException, IOException {
		if (this.serverSelectionModel != null) {
			terminate();
			clientThread = null;
			serverThread = null;
		}
		try {
			Thread.sleep(100L);
		} catch (final InterruptedException e) {
			// Nothing to do
		}
		parent.getEnvironment().resetNullSocketInstance();
		this.serverSelectionModel = serverSelectionModel;
		// if (serverSelectionModel.isToStartServer()) {
		serverThread = new TroffServerThread(parent.getEnvironment(), serverSelectionModel);
		// }
		clientThread = new ClientThread(parent, parent.getEnvironment(), serverSelectionModel);
	}

	public ClientThread getClientThread() {
		return clientThread;
	}

	public void start() {
		if (null != serverSelectionModel) {
//			if (null != serverThread) {
			serverThread.start();
//			}
			clientThread.start();
		}
	}

	public boolean isStarted() {
		return (serverThread.isStarted() && clientThread.isStarted());
	}

	public void terminate() {
		if (null != clientThread) {
			clientThread.terminate();
		}
		if (null != serverThread) {
			serverThread.terminate();
		}
	}

}
