package de.ped.troff.middleware;

import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/**
 * The data representation of a client, 
 * usable as data object known on both clients and server. 
 * <p>
 * The natural order is by name.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ClientInfo extends InfoComposite implements Comparable<ClientInfo> {
	private static final long serialVersionUID = 1L;

	public ClientInfo(final String name, final String ip, final byte numberOfPlayers) {
		super(name);
		this.ip = ip;
		this.numberOfPlayers = numberOfPlayers;
	}

	public ClientInfo(final Marshaller marshaller) {
		super(marshaller);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		marshaller.writeString(getName());
		marshaller.writeString(ip);
		marshaller.writeByte(numberOfPlayers);
	}

	@Override
	public Marshallable demarshal(final Marshaller marshaller) {
		setName(marshaller.readString());
		ip   = marshaller.readString();
		numberOfPlayers = marshaller.readByte();
		return this;
	}

	@Override
	public int compareTo(final ClientInfo other) {
		return compareByName(other);
	}

	@Override
	public String toString() {
		final StringBuffer s = new StringBuffer();
		s.append(getName());
		s.append(" (");
		s.append(getNumberOfPlayers());
		s.append("): ");
		s.append(getIP());
		return s.toString();
	}

	public String getIP() {
		return ip;
	}

	@Override
	public int getNumberOfPlayers() {
		return numberOfPlayers;
	}

	public String ip;
	public byte numberOfPlayers;
}