package de.ped.troff.middleware;

import java.io.IOException;

/**
 * Contains all the things that exist once per running instance. 
 * For a simple implementation, those could all be static, 
 * however for unit testing and for object orientation purposes they are bundled together herein.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Environment {

	/** There is maximum of one null socket per Troff instance. */
	protected volatile NullSocket nullSocket;

	/**
	 * Gets (or creates, if not existing) the single instance of a null socket.
	 * 
	 * @return the single null socket instance
	 * @throws IOException
	 *             if something went wrong in null socket creation
	 */
	public NullSocket getOrCreateNullSocketInstance() throws IOException {
		if (null == nullSocket) {
			nullSocket = new NullSocket();
		}
		return nullSocket;
	}

	/** 
	 * Removes the single instance of a null socket. Only needed for testing purposes. 
	 */
	public void resetNullSocketInstance() {
		nullSocket = null;
	}

}
