package de.ped.troff.middleware;

import java.util.ArrayList;

import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/**
 * The data representation of a client group, 
 * usable as data object known on both clients and server. 
 * <p>
 * The natural order is by name.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class GroupInfo extends InfoComposite implements Comparable<GroupInfo> {
	private static final long serialVersionUID = 1L;

	public static final int MAX_NUMBER_OF_PLAYERS = 8;

	public GroupInfo(final String name, final ArrayList<ClientInfo> clients) {
		super(name);
		this.children = clients;
		normChildren();
	}

	public GroupInfo(final Marshaller marshaller) {
		super(marshaller);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		marshaller.writeString(getName());
		normChildren();
		synchronized(children) {
			marshaller.writeInt(children.size());
			for (final ClientInfo child : children) {
				marshaller.writeMarshallable(child);
			}
		}
	}

	@Override
	public synchronized Marshallable demarshal(final Marshaller marshaller) {
		setName(marshaller.readString());
		normChildren();
		synchronized(children) {
			final int size = marshaller.readInt();
			children.clear();
			children.ensureCapacity(size);
			for (int c = 0; c < size; c++) {
				children.add(new ClientInfo(marshaller));
			}
		}
		return this;
	}

	@Override
	public int compareTo(GroupInfo other) {
		return compareByName(other);
	}

	@Override
	public String toString() {
		final StringBuffer s = new StringBuffer();
		s.append(getName());
		s.append(" (");
		s.append(getNumberOfPlayers());
		s.append(")");
		return s.toString();
	}

	public ArrayList<ClientInfo> getClients() {
		return children;
	}

	@Override
	public synchronized int getNumberOfPlayers() {
		int sum = 0;
		if (null != children) {
			for (final ClientInfo ci : children) {
				sum += ci.getNumberOfPlayers();
			}
		}
		return sum;
	}

	protected void normChildren() {
		if (null == children) {
			children = new ArrayList<ClientInfo>();
		}
	}

	protected ArrayList<ClientInfo> children;
}