package de.ped.troff.middleware;

import java.io.Serializable;

import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/**
 * Abstract class for nodes in the object tree spanned by 
 * {@link ClientInfo}, {@link GroupInfo} and {@link ServerInfo}. 
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public abstract class InfoComposite implements Serializable, Marshallable {
	private static final long serialVersionUID = 1L;

	public InfoComposite(final String name) {
		this.name = name;
	}

	public InfoComposite(final Marshaller marshaller) {
		demarshal(marshaller);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final InfoComposite other = (InfoComposite) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	public abstract int getNumberOfPlayers();

	public String getName() {
		return name;
	}

	protected void setName(final String name) {
		this.name = name;
	}

	protected String normalize(final String name) {
		return (name == null ? "" : name);
	}

	protected int compareByName(final InfoComposite other) {
		return normalize(name).compareTo(normalize(other.name));
	}

	private String name;
}
