package de.ped.troff.middleware;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;

/**
 * Represents the pseudo-socket to send data back and forth from local client to local server.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class NullSocket {

	/**
	 * Represents a pipe to send data from a data source to a data sink.
	 * 
	 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
	 */
	public static class NullSocketPipe {
		public static final int SIZE = 0x800;
		final PipedInputStream pis;
		final PipedOutputStream pos;

		public NullSocketPipe() throws IOException {
			pis = new PipedInputStream(SIZE);
			pos = new PipedOutputStream(pis);
		}

		public void close() throws IOException {
			pis.close();
			pos.close();
		}
	}

	public static final String NAME = "local";

	protected NullSocket.NullSocketPipe clientToServer;
	protected NullSocket.NullSocketPipe serverToClient;

	public NullSocket() throws IOException {
		clientToServer = new NullSocketPipe();
		serverToClient = new NullSocketPipe();
	}

	public InputStream getInputStream(final boolean isServer) {
		return (isServer ? clientToServer.pis : serverToClient.pis);
	}

	public OutputStream getOutputStream(final boolean isServer) {
		return (isServer ? serverToClient.pos : clientToServer.pos);
	}

	public void close() throws IOException {
		clientToServer.close();
		serverToClient.close();
	}
}