package de.ped.troff.middleware;

import java.util.ArrayList;

import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/**
 * The data representation of a server, 
 * usable as data object known on both clients and server. 
 * 
 * The natural order is by name.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ServerInfo extends InfoComposite implements Comparable<ServerInfo> {
	private static final long serialVersionUID = 1L;

	public ServerInfo(final String name, final ArrayList<GroupInfo> groups, final String groupOfClient) {
		super(name);
		this.children = groups;
		this.groupOfClient = groupOfClient;
		normChildren();
	}

	public ServerInfo(final Marshaller marshaller) {
		super(marshaller);
	}

	@Override
	public String toString() {
		final StringBuffer s = new StringBuffer();
		s.append(getName());
		s.append(" (");
		s.append(getNumberOfPlayers());
		s.append(")");
		s.append("\n");
		if (null != children) {
			for (final GroupInfo group : children) {
				if (group.getName().equals(groupOfClient)) {
					s.append("  ");
				} else {
					s.append("  ");
				}
				s.append(group);
				s.append("\n");
//				if (null != group.clients) {
					for (ClientInfo client : group.children) {
						s.append("    ");
						s.append(client);
						s.append("\n");
					}
//				}
			}
		}
		return s.toString();
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		marshaller.writeString(getName());
		normChildren();
		synchronized(children) {
			marshaller.writeInt(children.size());
			for (final GroupInfo child : children) {
				marshaller.writeMarshallable(child);
			}
		}
		marshaller.writeString(groupOfClient);
	}

	@Override
	public Marshallable demarshal(final Marshaller marshaller) {
		setName(marshaller.readString());
		normChildren();
		synchronized(children) {
			final int size = marshaller.readInt();
			children.clear();
			children.ensureCapacity(size);
			for (int c = 0; c < size; c++) {
				children.add(new GroupInfo(marshaller));
			}
		}
		groupOfClient = marshaller.readString();
		return this;
	}

	@Override
	public int compareTo(final ServerInfo other) {
		return compareByName(other);
	}

	@Override
	public int getNumberOfPlayers() {
		int sum = 0;
		for (final GroupInfo gi : children) {
			sum += gi.getNumberOfPlayers();
		}
		return sum;
	}

	public ArrayList<GroupInfo> getGroups() {
		return children;
	}

	public String getGroupOfClient() {
		return groupOfClient;
	}

	protected void normChildren() {
		if (null == children) {
			children = new ArrayList<GroupInfo>();
		}
	}

	protected ArrayList<GroupInfo> children;
	protected String groupOfClient;
}