package de.ped.troff.server;

import java.io.IOException;
import java.net.Socket;

import de.ped.tools.FinishableThread;
import de.ped.troff.middleware.Connector;
import de.ped.troff.middleware.Message;

/**
 * The server side of a single connection, 
 * running in its own thread waiting for requests to respond to.
 * However, the server is capable of sending responses without a request to inform the client on special events.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TroffServerConnectionThread extends FinishableThread {
	private final Connector connector;
	private final TroffServerThread parent;
	private final TroffRoundsManagerImpl manager;

	public TroffServerConnectionThread(
			final TroffServerThread parent, 
			final TroffRoundsManagerImpl manager, 
			final Socket socket) {
		super("TroffServerConnection" + parent.getInstanceID() + "_");
		setLogger(TroffServerThread.logger);
		this.parent = parent; 
		this.manager = manager;
		connector = new Connector(parent.getEnvironment(), socket, null, Connector.SERVER_SIDE, TroffServerThread.logger);
	}

	@Override
	public void terminate() {
		connector.disconnect();
	}

	@Override
	public void run() {
		try {
			logger.debug("Started.");
			connector.connect();

			while(!connector.isTerminating()) {
				try {
					Message req = connector.receiveWaiting(100L);
					if (null != req) {
						Message res = manager.handleMessage(connector, req);
						if (null != res) {
							connector.send(res);
						} else {
							logger.debug("No response for request " + req);
						}
					}
				} catch (final IOException e) {
					// Nothing to do, normally a stream closed.
				}
			}
			terminate();
			parent.remove(this);
			logger.debug("Stopped.");
		} catch (final IOException e) {
			e.printStackTrace();
		}
	}
}
