package de.ped.troff.server.logic;


/**
 * Creates all the boards.
 * 
 * @author Copyright (c) 2011 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class BoardFactory {

	private static final int B0SIZE = 20; // Should divide without remainder by 4, and be minimum 8.
	private static final int B1SIZE = 40; // Should divide without remainder by 4, and be minimum 8.

	private static BoardInfo[] boardInfos = null;

	protected static void init() {
		boardInfos = new BoardInfo[] {
				new BoardInfo("BoardInfo.Bordered20x20", B0SIZE + 1, B0SIZE + 1, new PlayerPosInfo[] {
						new PlayerPosInfo(         B0SIZE / 4    , B0SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(B0SIZE - B0SIZE / 4 - 1, B0SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(         B0SIZE / 4    , B0SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B0SIZE - B0SIZE / 4 - 1, B0SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B0SIZE / 2 - 1,          B0SIZE / 4    , TroffGame.WEST),
						new PlayerPosInfo(B0SIZE / 2 - 1, B0SIZE - B0SIZE / 4 - 1, TroffGame.WEST),
						new PlayerPosInfo(B0SIZE / 2    ,          B0SIZE / 4    , TroffGame.EAST),
						new PlayerPosInfo(B0SIZE / 2    , B0SIZE - B0SIZE / 4 - 1, TroffGame.EAST),
				}),
				new BoardInfo("BoardInfo.Bordered40x40", B1SIZE + 1, B1SIZE + 1, new PlayerPosInfo[] {
						new PlayerPosInfo(         B1SIZE / 4    , B1SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(B1SIZE - B1SIZE / 4 - 1, B1SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(         B1SIZE / 4    , B1SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B1SIZE - B1SIZE / 4 - 1, B1SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B1SIZE / 2 - 1,          B1SIZE / 4    , TroffGame.WEST),
						new PlayerPosInfo(B1SIZE / 2 - 1, B1SIZE - B1SIZE / 4 - 1, TroffGame.WEST),
						new PlayerPosInfo(B1SIZE / 2    ,          B1SIZE / 4    , TroffGame.EAST),
						new PlayerPosInfo(B1SIZE / 2    , B1SIZE - B1SIZE / 4 - 1, TroffGame.EAST),
				}),
				new BoardInfo("BoardInfo.Torus40x40", B1SIZE, B1SIZE, new PlayerPosInfo[] {
						new PlayerPosInfo(         B1SIZE / 4    , B1SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(B1SIZE - B1SIZE / 4 - 1, B1SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(         B1SIZE / 4    , B1SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B1SIZE - B1SIZE / 4 - 1, B1SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B1SIZE / 2 - 1,          B1SIZE / 4    , TroffGame.WEST),
						new PlayerPosInfo(B1SIZE / 2 - 1, B1SIZE - B1SIZE / 4 - 1, TroffGame.WEST),
						new PlayerPosInfo(B1SIZE / 2    ,          B1SIZE / 4    , TroffGame.EAST),
						new PlayerPosInfo(B1SIZE / 2    , B1SIZE - B1SIZE / 4 - 1, TroffGame.EAST),
				}),
				new BoardInfo("BoardInfo.Test3D", B0SIZE, B0SIZE, new PlayerPosInfo[] {
						new PlayerPosInfo(         B0SIZE / 4    , B0SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(B0SIZE - B0SIZE / 4 - 1, B0SIZE / 2 - 1, TroffGame.NORTH),
						new PlayerPosInfo(         B0SIZE / 4    , B0SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B0SIZE - B0SIZE / 4 - 1, B0SIZE / 2    , TroffGame.SOUTH),
						new PlayerPosInfo(B0SIZE / 2 - 1,          B0SIZE / 4    , TroffGame.WEST),
						new PlayerPosInfo(B0SIZE / 2 - 1, B0SIZE - B0SIZE / 4 - 1, TroffGame.WEST),
						new PlayerPosInfo(B0SIZE / 2    ,          B0SIZE / 4    , TroffGame.EAST),
						new PlayerPosInfo(B0SIZE / 2    , B0SIZE - B0SIZE / 4 - 1, TroffGame.EAST),
				}),
				new BoardInfo("BoardInfo.Test3D2", B0SIZE, B0SIZE, new PlayerPosInfo[] {
						new PlayerPosInfo(B0SIZE / 4    , B0SIZE / 4 - 1, TroffGame.NORTH),
						new PlayerPosInfo(B0SIZE / 4    , B0SIZE / 4 + 1, TroffGame.SOUTH),
						new PlayerPosInfo(B0SIZE / 4 - 1, B0SIZE / 4    , TroffGame.WEST),
						new PlayerPosInfo(B0SIZE / 4 + 1, B0SIZE / 4    , TroffGame.EAST),
						new PlayerPosInfo(B0SIZE / 2    , B0SIZE / 2 - 2, TroffGame.SOUTH),
						new PlayerPosInfo(B0SIZE / 2    , B0SIZE / 2 + 3, TroffGame.NORTH),
						new PlayerPosInfo(B0SIZE / 2 - 2, B0SIZE / 2    , TroffGame.EAST),
						new PlayerPosInfo(B0SIZE / 2 + 3, B0SIZE / 2    , TroffGame.WEST),
				}),
			};
		{ // Board 0 gets walls
			final int b0xsize = boardInfos[0].getSize().getWidth();
			final int b0ysize = boardInfos[0].getSize().getHeight();
			for (int x = 0; x < b0xsize; x++) {
				boardInfos[0].setAt(x, b0ysize - 1, BoardInfo.SOLID_WALL);
			}
			for (int y = 0; y < b0ysize; y++) {
				boardInfos[0].setAt(b0xsize - 1, y, BoardInfo.SOLID_WALL);
			}
		}
		{ // Board 1 gets walls
			final int b1xsize = boardInfos[1].getSize().getWidth();
			final int b1ysize = boardInfos[1].getSize().getHeight();
			for (int x = 0; x < b1xsize; x++) {
				boardInfos[1].setAt(x, b1ysize - 1, BoardInfo.SOLID_WALL);
			}
			for (int y = 0; y < b1ysize; y++) {
				boardInfos[1].setAt(b1xsize - 1, y, BoardInfo.SOLID_WALL);
			}
		}
		{ // Board 3 gets some walls for test purposes
			boardInfos[3].setAt(9, 17, BoardInfo.REMOVABLE_WALL);
			for (int i = 1; i < boardInfos[3].getSize().getHeight(); i += 2) {
				boardInfos[3].setAt(4, i, BoardInfo.REMOVABLE_WALL);
			}
		}
	}

	public static BoardInfo[] getBoardInfos() {
		if (null == boardInfos) {
			init();
		}
		return boardInfos;
	}

	public static BoardInfo createBoard(final int i, final int numberOfPlayers) {
		if (null == boardInfos) {
			init();
		}
		return new BoardInfo(boardInfos[i], numberOfPlayers);
	}
}
