package de.ped.troff.server.logic;

import de.ped.tools.MathUtil;
import de.ped.tools.log.Logger;

/**
 * A simple computer player strategy, doing just random moves.
 * 
 * @author Copyright (c) 2008 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ComputerPlayerRandom extends TroffPlayer {
	private static Logger logger = Logger.getRootLogger();

	public ComputerPlayerRandom(final TroffGame game, final byte i, final PlayerPosInfo playerPosInfo, final int playerMode) {
		super(game, i, playerPosInfo, playerMode);
	}

	@Override
	public int decideTurn() {
		final int[] rating = new int[3];
		boolean isMovePossible = false;
		for (int turn = -1; turn <= 1; turn++) {
			int nx = calculateX(turn);
			int ny = calculateY(turn);
			if (getGame().isLegalMove(this, nx, ny)) {
				rating[turn + 1] = 1;
				isMovePossible = true;
				logger.trace("Player " + getIndex() + " could turn " + turn + " to (" + nx + ", " + ny + ").");
			}
		}
		int ret = 0;
		if (isMovePossible) {
			ret = MathUtil.randomDistributed(rating) - 1;
			logger.trace("Player " + getIndex() + " will turn " + ret);
		}
		return ret;
	}
}
