package de.ped.troff.server.logic;

import java.io.Serializable;

import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/**
 * Any one-time event that may result e.g. in a sound.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Event implements Cloneable, Serializable, Marshallable {
	private static final long serialVersionUID = 1L;

	public static final int UNDEFINED       = -1;
	public static final int STEP            =  0;
	public static final int SHOOT_FAILED    =  1;
	public static final int SHOOT_SUCCEEDED =  2;
	public static final int HIT_WALL        =  3;
	public static final int HIT_SOLID_WALL  =  4;
	public static final int NO_HIT          =  5;
	public static final int DEATH           =  6;
	public static final int TRACE_ON        =  7;
	public static final int TRACE_OFF       =  8;
	public static final int INTRO           =  9;

	public Event(final int type) {
		this.type = type;
		this.xpos = UNDEFINED;
		this.ypos = UNDEFINED;
	}

	public Event(final int type, final int x, final int y) {
		this.type = type;
		this.xpos = x;
		this.ypos = y;
	}

	Event(final Marshaller marshaller) {
		marshaller.readMarshallable(this);
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		marshaller.writeInt(type);
		marshaller.writeInt(xpos);
		marshaller.writeInt(ypos);
	}

	@Override
	public Marshallable demarshal(final Marshaller marshaller) {
		type = marshaller.readInt();
		xpos = marshaller.readInt();
		ypos = marshaller.readInt();
		return this;
	}

	public int getType() {
		return type;
	}

	public int getXPos() {
		return xpos;
	}

	public int getYPos() {
		return ypos;
	}

	private int type;
	private int xpos;
	private int ypos;
}