package de.ped.troff.server.logic;

import java.io.Serializable;

import de.ped.tools.Assert;
import de.ped.tools.CollectionUtil;
import de.ped.tools.Marshallable;
import de.ped.tools.Marshaller;

/** 
 * Contains all properties of a player that remain constant at least for a run. 
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class TroffPlayerProperties implements Cloneable, Serializable, Marshallable {

	private static final long serialVersionUID = 1L;

	public TroffPlayerProperties(
			final int type, 
			final int view, 
			final int mode, 
			final int color, 
			final String name, 
			final String clientName) {
		this.typeId = type;
		this.viewId = view;
		this.modeId = mode;
		this.playerColorId = color;
		this.name = name;
		this.clientName = clientName;
	}

	@Override
	public void marshal(final Marshaller marshaller) {
		marshaller.writeInt(typeId);
		marshaller.writeInt(viewId);
		marshaller.writeInt(modeId);
		marshaller.writeInt(playerColorId);
		marshaller.writeString(name);
		marshaller.writeString(clientName);
	}

	@Override
	public Marshallable demarshal(final Marshaller marshaller) {
		typeId = marshaller.readInt();
		viewId = marshaller.readInt();
		modeId = marshaller.readInt();
		playerColorId = marshaller.readInt();
		name = marshaller.readString();
		clientName = marshaller.readString();
		return this;
	}

	@Override
	public TroffPlayerProperties clone() {
		TroffPlayerProperties ret = null;
		try {
			ret = (TroffPlayerProperties) super.clone();
		} catch (final CloneNotSupportedException e) {
			Assert.assertTrue(false);
		}
		return ret;
	}

	public int getTypeId() {
		return typeId;
	}

	public void setTypeId(final int typeId) {
		this.typeId = typeId;
	}

	public int getModeId() {
		return modeId;
	}

	public void setModeId(final int modeId) {
		this.modeId = modeId;
	}

	public int getViewId() {
		return viewId;
	}

	public void setViewId(final int viewId) {
		this.viewId = viewId;
	}

	public int getPlayerColorId() {
		return playerColorId;
	}

	public void setPlayerColorId(final int colorId) {
		this.playerColorId = colorId;
	}

	public String getName() {
		return name;
	}

	public void setName(final String name) {
		this.name = name;
	}

	public String getClientName() {
		return clientName;
	}

	public void setClientName(final String clientName) {
		this.clientName = clientName;
	}

	public String getFullPlayerName() {
		String  ret = name;
		if (!CollectionUtil.isNullOrEmpty(clientName)) {
			ret = name +  "@" + clientName;
		}
		return ret;
	}

	private int typeId;
	private int viewId;
	private int modeId;
	private int playerColorId;
	private String name;
	private String clientName;
}
