package de.ped.troff.test;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;
import de.ped.troff.server.logic.BoardFactory;
import de.ped.troff.server.logic.BoardInfo;
import de.ped.troff.server.logic.PlayerPosInfo;
import de.ped.troff.server.logic.TroffGameProperties;

/**
 * Unit tests for the {@link BoardFactory} class.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class BoardFactoryTest {

	private Logger logger = Logger.getLogger(this.getClass());

	@Before
	public void setup() {
		logger.setLogLevel(Level.INFO);
	}

	protected void checkBoard(final BoardInfo bi) {
		assertNotNull(bi.getKey());
		final int numOfPlayers = bi.getPlayerPosInfos().length;
		assertEquals(TroffGameProperties.MAX_NUMBER_OF_PLAYERS, numOfPlayers);
		assertTrue(numOfPlayers < bi.getSize().getWidth() * bi.getSize().getHeight());
		// Are all start positions free?
		for (final PlayerPosInfo ppi : bi.getPlayerPosInfos()) {
			assertEquals(BoardInfo.FREE, bi.getAt(ppi.getXPos(), ppi.getYPos()));
		}
		// Are all start positions different?
		final PlayerPosInfo[] arr = bi.getPlayerPosInfos();
		for (int i = 0; i < arr.length - 1; i++) {
			for (int j = i + 1; j < arr.length; j++) {
				assertFalse(i == j);
				assertFalse((arr[i].getXPos() == arr[j].getXPos()) && (arr[i].getYPos() == arr[j].getYPos()));
			}
		}
	}

	@Test
	public void testGetBoardInfos() {
		final BoardInfo[] bis = BoardFactory.getBoardInfos();
		for (final BoardInfo bi : bis) {
			checkBoard(bi);
		}
		for (int i = 0; i < bis.length; i++) {
			final BoardInfo bic = BoardFactory.createBoard(i, Integer.MAX_VALUE);
			checkBoard(bic);
		}
	}
}
