package de.ped.troff.test;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import de.ped.tools.log.Level;
import de.ped.tools.log.Logger;
import de.ped.troff.client.logic.Circle;

/**
 * Unit tests for the {@link Circle} class.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class CircleTest {

	private Logger logger = Logger.getLogger(this.getClass());

	@Before
	public void setup() {
		logger.setLogLevel(Level.INFO);
	}

	public static final double EPSILON = 0.000001;

	protected void checkForSymmetry(final Circle circle) {
		final boolean[][] b = new boolean[circle.size() + 1][circle.size() + 1];
		for (int xi = 0; xi < circle.size(); xi++) {
			final int halfCircleX = circle.getX(xi);
			for (int yi = 0; yi < halfCircleX; yi++) {
				b[xi][yi] = true;
			}
		}
		for (int xi = 0; xi < b.length; xi++) {
			for (int yi = 0; yi < b[0].length; yi++) {
				assertEquals("Symmetry broken at " + xi + "," + yi + " - ", b[xi][yi], b[yi][xi]);
			}
		}
	}

	@Test(expected = IllegalArgumentException.class)
	public void testHalfCircle_Negative() {
		new Circle(-0.5);
	}

	@Test
	public void testHalfCircle_0_0() {
		final Circle c = new Circle(0.0);
		assertEquals(0.0, c.getSize(), EPSILON);
		assertEquals(0, c.size());
		assertEquals(Circle.UNDEFINED, c.getX(0));
		assertEquals(Circle.UNDEFINED, c.getX(1));
		assertEquals(Circle.UNDEFINED, c.getX(-1));
		checkForSymmetry(c);
	}

	@Test
	public void testHalfCircle_Half_0() {
		final Circle c = new Circle(0.5);
		assertEquals(0.5, c.getSize(), EPSILON);
		assertEquals(1, c.size());
		assertEquals(1, c.getX(0));
		assertEquals(Circle.UNDEFINED, c.getX(1));
		assertEquals(Circle.UNDEFINED, c.getX(-1));
		checkForSymmetry(c);
	}

	@Test
	public void testHalfCircle_1_1() {
		final Circle c = new Circle(1.0);
		assertEquals(1.0, c.getSize(), EPSILON);
		assertEquals(1, c.size());
		assertEquals(1, c.getX(0));
		assertEquals(Circle.UNDEFINED, c.getX(1));
		assertEquals(Circle.UNDEFINED, c.getX(-1));
		checkForSymmetry(c);
	}

	@Test
	public void testHalfCircle_2_2() {
		final Circle c = new Circle(2.0);
		assertEquals(2.0, c.getSize(), EPSILON);
		assertEquals(2, c.size());
		assertEquals(2, c.getX(0));
		assertEquals(2, c.getX(1));
		assertEquals(2, c.getX(-1));
		assertEquals(Circle.UNDEFINED, c.getX(2));
		assertEquals(Circle.UNDEFINED, c.getX(-2));
		checkForSymmetry(c);
	}

	@Test
	public void testHalfCircle_5_5() {
		final Circle c = new Circle(5.0);
		assertEquals(5.0, c.getSize(), EPSILON);
		assertEquals(5, c.size());
		assertEquals(5, c.getX(0));
		assertEquals(5, c.getX(1));
		assertEquals(5, c.getX(2));
		assertEquals(4, c.getX(3));
		assertEquals(3, c.getX(4));
		assertEquals(Circle.UNDEFINED, c.getX(5));
		checkForSymmetry(c);
	}

	@Test
	public void testHalfCircle_7_7() {
		final Circle c = new Circle(7.0);
		assertEquals(7.0, c.getSize(), EPSILON);
		assertEquals(7, c.size());
		assertEquals(7, c.getX(0));
		assertEquals(Circle.UNDEFINED, c.getX(7));
		checkForSymmetry(c);
	}
}
