package de.ped.troff.test;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.tools.MathMatrix;
import de.ped.tools.MathVector;
import de.ped.troff.client.gui.TroffPlayerViewCanvas;
import de.ped.troff.client.logic.Abstract3DGeometry;
import de.ped.troff.client.logic.Real3DGeometry;
import de.ped.troff.client.logic.Side;

/**
 * Tests {@link Real3DGeometry}.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Real3DGeometryTest extends Real3DGeometry {

	public Real3DGeometryTest() {
		super(
			TroffPlayerViewCanvas.SIZE_IN_PIXEL_X,
			TroffPlayerViewCanvas.SIZE_IN_PIXEL_Y,
			TroffPlayerViewCanvas.PILOTVIEW_HORIZON_DISTANCE);
	}

	public double EPSILON = 1.0e-5;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		// Nothing to do
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// Nothing to do
	}

	@Before
	public void setUp() throws Exception {
		// Nothing to do
	}

	@After
	public void tearDown() throws Exception {
		// Nothing to do
	}

	@Test
	public void testGetHomogenousTranslationRotationMatrix() {
		final MathMatrix tr = getHomogenousTranslationRotationMatrix();
		final double[][] arr = tr.getAll();
		assertEquals(4, arr.length);
		assertEquals(4, arr[0].length);
	}

	public static class TestTransformParameters {
		final MathVector expected;
		final MathVector input;

		public TestTransformParameters(final MathVector expected, final MathVector input) {
			this.expected = expected;
			this.input = input;
		}

		public TestTransformParameters(final double[] expected, final MathVector input) {
			this.expected = new MathVector(expected);
			this.input = input;
		}

		public TestTransformParameters(final double[] expected, final double[] input) {
			this.expected = new MathVector(expected);
			this.input = new MathVector(input);
		}

		@Override
		public String toString() {
			return input.toString();
		}
	}

	public TestTransformParameters[] testTransformParams = new TestTransformParameters[] {
			new TestTransformParameters(new double[]{0.0, 0.0, 0.0}, new double[]{0.0, 0.0, 0.0}),
			new TestTransformParameters(new double[]{1.0, 0.0, 0.0}, new double[]{1.0, 0.0, 0.0}),
			new TestTransformParameters(new double[]{0.0, 0.0, 1.0}, new double[]{0.0, 1.0, 0.0}),
			new TestTransformParameters(new double[]{0.0,-1.0, 0.0}, new double[]{0.0, 0.0, 1.0}),
			new TestTransformParameters(new double[]{1.0,-3.0, 2.0}, new double[]{1.0, 2.0, 3.0}),
			new TestTransformParameters(new double[]{0.5,-3.0, 2.0}, new double[]{0.5, 2.0, 3.0}),
	};

	@Test
	public void testTransform() {
		for (final TestTransformParameters params : testTransformParams) {
			final MathVector v = Real3DGeometry.CAMERA_POSITION.add(params.input);
			final MathVector vt = transform(v);
			assertTrue(params.toString() + ": Expected was " + params.expected + " but was " + vt,
					params.expected.subtract(vt).isNullVector(EPSILON));
		}
	}

	@Test
	public void testGetVertexes() {
		for (int cubeX = -2; cubeX <= 2; cubeX++) {
			for (int cubeY = -2; cubeY <= 2; cubeY++) {
				for (Side side : Side.values()) {
					final MathVector[] vertexes = getVertexes(cubeY, cubeX, side, Abstract3DGeometry.BOX_SIDE);
					for (int v = 0; v < vertexes.length; v++) {
						final MathVector vt = transform(vertexes[v]);
						assertEquals(3, vt.getLength());
						assertEquals(vt.toString(), 0.5 - BORDER_THICKNESS, Math.abs(vt.get(MathVector.X) - cubeX), EPSILON);
						assertEquals(vt.toString(), 0.5 - BORDER_THICKNESS, Math.abs(vt.get(MathVector.Y)), EPSILON);
						assertEquals(vt.toString(), 0.5 - BORDER_THICKNESS, Math.abs(vt.get(MathVector.Z) - cubeY), EPSILON);
					}
				}
			}
		}
	}
}
