package de.ped.troff.test;


import static org.junit.Assert.*;

import java.io.IOException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.ped.tools.gui.ApplicationOrApplet;
import de.ped.tools.test.GenericTest;
import de.ped.troff.client.gui.Troff;
import de.ped.troff.client.gui.TroffMainWindow;
import de.ped.troff.middleware.ServerSelectionModel;

/**
 * Tests {@link ServerSelectionModel}.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ServerSelectionModelTest {

	protected TroffMainWindow window;

	@Before
	public void setUp() throws Exception {
		Troff troff = new Troff();
		ApplicationOrApplet.applicationPreCreate();
		troff.init();
		window = (TroffMainWindow) troff.getFrame();
		assertNotNull(window);
	}

	@After
	public void tearDown() throws Exception {
		window = null;
	}

	public void checkConnection(
			final String name, 
			final boolean isExceptionExpected, 
			final ServerSelectionModel model) {
		try {
			window.updateServerModel(model);
			if (isExceptionExpected) {
				fail("Expected exception, but received none");
			}
		} catch (final IOException ex) {
			if (!isExceptionExpected) {
				ex.printStackTrace();
				fail("Unexpected exception: " + ex);
			}
		}
		window.updateGroupModel(null);
	}

	@Test
	public void testConnectWorksFine() throws IOException {
		checkConnection("Default", false, new ServerSelectionModel());
		final int port = 82;
		assertTrue(ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT != port);
		checkConnection("Change localhost port", false, new ServerSelectionModel(true,
				port, "Newport",
				ServerSelectionModel.HostType.PORT_LOCALHOST, null, port,
				ServerSelectionModel.CLIENTNAME_DEFAULT, 2));
		checkConnection("Back to localhost default", false, new ServerSelectionModel(true,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.SERVERNAME_DEFAULT,
				ServerSelectionModel.HostType.PORT_LOCALHOST, null,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.CLIENTNAME_DEFAULT, 2));
		checkConnection("To external own that leads to localhost", false, new ServerSelectionModel(true,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.SERVERNAME_DEFAULT,
				ServerSelectionModel.HostType.PORT_EXTERNAL_OWN, 
				ServerSelectionModel.HOSTNAME_LOCALHOST,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.CLIENTNAME_DEFAULT, 2));
		checkConnection("Connect locally", false, new ServerSelectionModel(true,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.SERVERNAME_DEFAULT,
				ServerSelectionModel.HostType.LOCALLY, 
				null,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.CLIENTNAME_DEFAULT, 2));
		checkConnection("Connect to a non-running server fails", true, new ServerSelectionModel(false,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.SERVERNAME_DEFAULT,
				ServerSelectionModel.HostType.PORT_EXTERNAL_OWN, 
				ServerSelectionModel.HOSTNAME_LOCALHOST,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.CLIENTNAME_DEFAULT, 2));
		checkConnection("Default as safe ground", false, new ServerSelectionModel());
		checkConnection("Connect to wrong port fails", true, new ServerSelectionModel(true,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT,
				ServerSelectionModel.SERVERNAME_DEFAULT,
				ServerSelectionModel.HostType.PORT_EXTERNAL_OWN, 
				ServerSelectionModel.HOSTNAME_LOCALHOST,
				ServerSelectionModel.PORTNO_LOCALHOST_DEFAULT + 1,
				ServerSelectionModel.CLIENTNAME_DEFAULT, 2));
		checkConnection("Default as safe ground", false, new ServerSelectionModel());
	}

	/**
	 * @see ServerSelectionModel#equals(Object)
	 * @see ServerSelectionModel#clone()
	 */
	@Test
	public void testObjectBasics() {
		GenericTest.checkObjectBasics(false, new ServerSelectionModel(), new ServerSelectionModel());
	}
}
