package de.ped.troff.test;

import static org.junit.Assert.*;

import java.io.IOException;

import org.junit.*;

import de.ped.tools.I18NStringWithFillIns;
import de.ped.troff.middleware.Connector;
import de.ped.troff.middleware.Environment;
import de.ped.troff.server.TroffServerClient;
import de.ped.troff.server.TroffServerGroup;
import de.ped.troff.server.TroffRoundsManagerImpl;

/**
 * Tests {@link TroffRoundsManagerImpl}.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class ServerTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		// Nothing to do
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// Nothing to do
	}

	@Before
	public void setUp() throws Exception {
		// Nothing to do
	}

	@After
	public void tearDown() throws Exception {
		// Nothing to do
	}

	/**
	 * @see TroffRoundsManagerImpl#TroffRoundsManagerImpl()
	 * @see Connector#Connector(Environment, Socket, URL, boolean, Logger)
	 * @see Connector#connect()
	 * @see TroffServerClient#TroffServerClient(Connector)
	 * @see TroffRoundsManagerImpl#getAllClients()
	 * @see TroffRoundsManagerImpl#getAllClientGroups()
	 * @see TroffRoundsManagerImpl#loginClient(TroffServerClient)
	 * @see TroffServerGroup#getName()
	 * @see TroffServerGroup#getClients()
	 * @see TroffServerClient#getTextMessages()
	 * @see TroffRoundsManagerImpl#join(TroffServerClient, TroffServerGroup)
	 * @see TroffRoundsManagerImpl#leave(TroffServerClient)
	 * @see TroffRoundsManagerImpl#logoutClient(TroffServerClient)
	 * @throws IOException
	 *             if communication fails
	 */
	@Test
	public void testNormalFlow() throws IOException {
		final Environment env = new Environment();
		final TroffRoundsManagerImpl server = new TroffRoundsManagerImpl();
		final Connector connector = new Connector(env, null, null, false, null);
		connector.connect();
		final TroffServerClient client1 = new TroffServerClient(connector);
		{
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(0, serverClients.length);
		}
		final TroffServerGroup[] rounds = server.getAllClientGroups();
		final TroffServerGroup round = rounds[0];
		{
			server.loginClient(client1);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			assertTrue(rounds.length >= 1);
			assertEquals(TroffRoundsManagerImpl.DEFAULT_GROUP_NAME, round.getName());
			assertEquals(0, round.getClients().length);
			String[] messages = ClientServerCommunicationTest.convertToStrings(client1.getTextMessages());
			assertEquals(1, messages.length);
			assertEquals("Client %1 connected to server Server.", messages[messages.length - 1]);
		}
		{
			boolean ret = server.join(client1, round);
			assertEquals(true, ret);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			TroffServerClient[] clients = round.getClients();
			assertEquals(1, clients.length);
			assertEquals(client1, clients[0]);
			String[] messages = ClientServerCommunicationTest.convertToStrings(client1.getTextMessages());
			assertEquals(2, messages.length);
			assertEquals("Client %2 joined group default.", messages[messages.length - 1]);
		}
		{
			server.leave(client1);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			assertEquals(0, round.getClients().length);
			String[] messages = ClientServerCommunicationTest.convertToStrings(client1.getTextMessages());
			assertEquals(3, messages.length);
			assertEquals("Client %2 has left group default.", messages[messages.length - 1]);
		}
		server.logoutClient(client1);
		final TroffServerClient[] serverClients = server.getAllClients();
		assertEquals(0, serverClients.length);
		assertEquals(1, server.getAllClientGroups().length);
	}

	@Test
	public void testTwoJoinsSameClientToSameGroup_SecondJoinDoesNothing() throws IOException {
		final Environment env = new Environment();
		final TroffRoundsManagerImpl server = new TroffRoundsManagerImpl();
		final Connector connector = new Connector(env, null, null, false, null);
		connector.connect();
		final TroffServerClient client1 = new TroffServerClient(connector);
		{
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(0, serverClients.length);
		}
		server.loginClient(client1);
		{
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
		}
		final TroffServerGroup[] rounds = server.getAllClientGroups();
		assertTrue(rounds.length >= 1);
		final TroffServerGroup round = rounds[0];
		assertEquals(TroffRoundsManagerImpl.DEFAULT_GROUP_NAME, round.getName());
		assertEquals(0, round.getClients().length);
		{
			boolean ret = server.join(client1, round);
			assertEquals(true, ret);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			final TroffServerClient[] clients = round.getClients();
			assertEquals(1, clients.length);
			assertEquals(client1, clients[0]);
			String[] messages = ClientServerCommunicationTest.convertToStrings(client1.getTextMessages());
			assertEquals(2, messages.length);
			assertEquals("Client %2 joined group default.", messages[messages.length - 1]);
		}
		{
			boolean ret = server.join(client1, round);
			assertEquals(true, ret);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			final TroffServerClient[] clients = round.getClients();
			assertEquals(1, clients.length);
			assertEquals(client1, clients[0]);
			final I18NStringWithFillIns[] messages = client1.getTextMessages();
			assertEquals(2, messages.length);
		}
		server.leave(client1);
		{
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			assertEquals(0, round.getClients().length);
		}
		server.logoutClient(client1);
		final TroffServerClient[] serverClients = server.getAllClients();
		assertEquals(0, serverClients.length);
	}

	@Test
	public void testTwoConnectsOfSameClient_SecondConnectDoesNothing() throws IOException {
		final Environment env = new Environment();
		final TroffRoundsManagerImpl server = new TroffRoundsManagerImpl();
		final Connector connector = new Connector(env, null, null, false, null);
		connector.connect();
		final TroffServerClient client1 = new TroffServerClient(connector);
		{
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(0, serverClients.length);
		}
		{
			server.loginClient(client1);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			final TroffServerGroup[] rounds = server.getAllClientGroups();
			assertTrue(rounds.length >= 1);
			final TroffServerGroup round = rounds[0];
			assertEquals(TroffRoundsManagerImpl.DEFAULT_GROUP_NAME, round.getName());
			assertEquals(0, round.getClients().length);
		}
		{
			server.loginClient(client1);
			final TroffServerClient[] serverClients = server.getAllClients();
			assertEquals(1, serverClients.length);
			final TroffServerGroup[] rounds = server.getAllClientGroups();
			assertTrue(rounds.length >= 1);
			final TroffServerGroup round = rounds[0];
			assertEquals(TroffRoundsManagerImpl.DEFAULT_GROUP_NAME, round.getName());
			assertEquals(0, round.getClients().length);
		}
	}
}
