package de.ped.troff.test;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import de.ped.troff.client.gui.TroffPlayerViewCanvas;
import de.ped.troff.client.logic.Abstract3DGeometry;
import de.ped.troff.client.logic.Side;
import de.ped.troff.client.logic.Simple3DGeometry;

/**
 * Tests {@link Simple3DGeometry}.
 * 
 * @author Copyright (c) 2012 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class Simple3DGeometryTest {

	protected static Simple3DGeometry simple3D = null;

	@BeforeClass
	public static void setUpBeforeClass() {
		simple3D = new Simple3DGeometry(
				TroffPlayerViewCanvas.SIZE_IN_PIXEL_X,
				TroffPlayerViewCanvas.SIZE_IN_PIXEL_Y,
				TroffPlayerViewCanvas.PILOTVIEW_HORIZON_DISTANCE);
	}

	/**
	 * @see Simple3DGeometry#linearTransform(int, int, int, double, double)
	 */
	@Test
	public void testLinearTransformIntIntIntDoubleDouble() {
		assertEquals(0, Simple3DGeometry.linearTransform(0, 0, 0, 0.0, 0.0));
		assertEquals(0, Simple3DGeometry.linearTransform(0, 0, 100, 0.0, 1.0));
		assertEquals(100, Simple3DGeometry.linearTransform(100, 0, 100, 0.0, 1.0));
		assertEquals(100, Simple3DGeometry.linearTransform(100, 50, 100, 0.0, 1.0));
		assertEquals(75, Simple3DGeometry.linearTransform(75, 50, 100, 0.0, 1.0));
		assertEquals(25, Simple3DGeometry.linearTransform(50, 0, 100, 0.0, 0.5));
		assertEquals(25, Simple3DGeometry.linearTransform(50, 0, 100, 0.0, 0.5));
		assertEquals(75, Simple3DGeometry.linearTransform(50, 0, 100, 0.5, 0.5));
		assertEquals(85, Simple3DGeometry.linearTransform(60, 10, 110, 0.5, 0.5));
	}

	/**
	 * @see Simple3DGeometry#linearTransformTrapez(int[][], double, double, double, double)
	 */
	@Test
	public void testLinearTransformPolygon_Null_Null() {
		assertNull(Simple3DGeometry.linearTransformTrapez(null, 0.0, 0.0, 0.0, 0.0));
	}

	/**
	 * @see Simple3DGeometry#getBoxSide(int, int, Side, int)
	 * @see Simple3DGeometry#linearTransformTrapez(int[][], double, double, double, double)
	 */
	@Test
	public void testLinearTransformTrapez_Id() {
		final int[][] poly = simple3D.getBoxSide(1, 0, Side.FRONT, Abstract3DGeometry.BOX_SIDE);
		final int[][] ret = Simple3DGeometry.linearTransformTrapez(poly, 0.0, 0.0, 1.0, 1.0);
		assertEquals(poly.length, ret.length);
		for (int i = 0; i < poly.length; i++) {
			assertEquals(poly[i].length, ret[i].length);
			for (int j = 0; j < poly[i].length; j++) {
				assertEquals(poly[i][j], ret[i][j]);
			}
		}
	}

	/**
	 * @see Simple3DGeometry#getBoxSide(int, int, Side, int)
	 * @see Simple3DGeometry#linearTransformTrapez(int[][], double, double, double, double)
	 */
	@Test
	public void testLinearTransformTrapez_RightEye() {
		final int[][] poly = simple3D.getBoxSide(1, 0, Side.FRONT, Abstract3DGeometry.BOX_SIDE);
		final int[][] ret = Simple3DGeometry.linearTransformTrapez(poly, 3.0 / 16.0, 0.25, 0.125, 0.125);
		assertEquals(poly.length, ret.length);
		for (int i = 0; i < poly.length; i++) {
			assertEquals(poly[i].length, ret[i].length);
			for (int j = 0; j < poly[i].length; j++) {
				assertNotSame(poly[i][j], ret[i][j]);
			}
		}
	}
}
