package de.ped.dsatool.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import de.ped.dsatool.dsa.generated.DiceRollType;
import de.ped.tools.Messages;
import de.ped.tools.gui.ApplicationMainWindow;
import de.ped.tools.gui.FormattedTextFieldInputIntegerVerifier;
import de.ped.tools.gui.FormattedTextFieldInputVerifier;
import de.ped.tools.gui.PedJDialog;
import de.ped.tools.log.Logger;

/**
 * The dialog that asks the user what dices to throw.
 *
 * @author Copyright (c) 2023 Peter Diefenbach (peter@pdiefenbach.de)
 */
public class DiceDialog extends PedJDialog implements ActionListener {
	private static final long serialVersionUID = 1L;

	/**
	 * Constructor
	 * @param parent the parent window
	 * @param diceModel the model to use for the dialog
	 */
	public DiceDialog(final ApplicationMainWindow parent, final DiceRollType diceModel) {
		super(parent, parent);
		model = diceModel;
		final Messages messages = getMessages();
		setTitle(messages.getText("Action.CreateDice"));
		setTitlePanel(messages.getText("DiceDialog.Title")); 

		final JPanel mainPanel = createMainPanel();
		final GridBagLayout gridbag = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST; // All elements are left bound to their space
		mainPanel.setLayout(gridbag);

		final FormattedTextFieldInputVerifier integerVerifier = new FormattedTextFieldInputIntegerVerifier(
				this, FormattedTextFieldInputIntegerVerifier.MIN_DEFAULT, FormattedTextFieldInputIntegerVerifier.MAX_DEFAULT);
		final FormattedTextFieldInputVerifier positiveIntegerVerifier = new FormattedTextFieldInputIntegerVerifier(
				this, 1, FormattedTextFieldInputIntegerVerifier.MAX_DEFAULT);
		textFactor.setInputVerifier(integerVerifier);
		textOffset.setInputVerifier(integerVerifier);
		textNumber.setInputVerifier(positiveIntegerVerifier);
		textEyes.setInputVerifier(positiveIntegerVerifier);
		textFactor.addActionListener(this);
		textOffset.addActionListener(this);
		textNumber.addActionListener(this);
		textEyes.addActionListener(this);

		mainPanel.add(textFactor);
		mainPanel.add(new JLabel(" x ( "));
		mainPanel.add(textNumber);
		mainPanel.add(new JLabel(" W "));
		mainPanel.add(textEyes);
		mainPanel.add(new JLabel(" + "));
		mainPanel.add(textOffset);
		mainPanel.add(new JLabel(" ) "));
		
		getContentPane().add(new JScrollPane(mainPanel), BorderLayout.CENTER);
		setOptionType(JOptionPane.OK_CANCEL_OPTION);
		setDefaultButton(getButtonOk());
		addHelpButton("Action.CreateDice");
		updateUIFieldsBasedOnProperties();
		finishLayout();
	}

	/** 
	 * Invoked when an action occurs.
	 * @param e the event to handle
	 */
	@Override
	public void actionPerformed(final ActionEvent e) {
		logger.debug(e.paramString());
		final Object source = e.getSource();
		if ((source == getButtonOk()) 
				|| (source == textFactor) 
				|| (source == textOffset) 
				|| (source == textNumber) 
				|| (source == textEyes)) {
			// Pressing return in a text field should be handled as pressing the OK button
			readUIFieldsIntoProperties();
			onOk();
		} else if (source == getButtonCancel()) {
			onCancel();
		}
	}

	@Override
	public void readUIFieldsIntoProperties() {
		model.setFactor(((Integer)(textFactor.getValue())).intValue());
		model.setOffset(((Integer)(textOffset.getValue())).intValue());
		model.setNumber(((Integer)(textNumber.getValue())).intValue());
		model.setEyes(((Integer)(textEyes.getValue())).intValue());
		updateUIFieldsBasedOnProperties();
	}

	@Override
	public void updateUIFieldsBasedOnProperties() {
		setValue(textFactor, new Integer(model.getFactor()), 3);
		setValue(textOffset, new Integer(model.getOffset()), 3);
		setValue(textNumber, new Integer(model.getNumber()), 3);
		setValue(textEyes, new Integer(model.getEyes()), 3);
	}

	public DiceRollType getDiceRollType() {
		return model;
	}

	private JFormattedTextField textFactor = new JFormattedTextField();
	private JFormattedTextField textOffset = new JFormattedTextField();
	private JFormattedTextField textNumber = new JFormattedTextField();
	private JFormattedTextField textEyes = new JFormattedTextField();
	private DiceRollType model;
	private final Logger logger = DSATool.getLogger();
}
